/* 
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr 

   This file is part of Polychrony Signal Toolbox (http://www.irisa.fr/espresso/polychrony).

   Polychrony  Signal Toolbox is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal Toolbox is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal Toolbox; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA. 
*/

/********************************************************************************
 Module of management of files realizing the interface with the operating system.
 ********************************************************************************/

#include <ctime>
#include <cctype>
#include <cstdlib>

#ifdef xKHOST
#if (xKHOST == xKwinNT)
/* Windows */
#define fsystemSeparator '\\'
#define CMD_MKDIR "mkdir "
#define fileSuffixSeparator '.'
#define pathSeparator ';'
#define GETDOUBLEQUOTEDSTRING(ud) getDoubleQuotedString(ud, &ud);
#define OS_DEPENDANT_tmpnam(name)   name = _tempnam(NULL, "pk_pk_")
#else
/* xKmacOs, xKsolaris, xKlinux  */
#define fsystemSeparator '/'
#define CMD_MKDIR "mkdir -p "
#define fileSuffixSeparator '.'
#define pathSeparator ':'
#define GETDOUBLEQUOTEDSTRING(ud)
// #define OS_DEPENDANT_tmpnam(name) name = (char *)malloc(L_tmpnam); name = tmpnam(name) replaced, March 2015 LBesnard, by
#define OS_DEPENDANT_tmpnam(name) name = (char *)malloc(22); strncpy(name,"/tmp/pKTmpFile-XXXXXX",21); mkstemp(name)
#endif
#endif

/* ---- environment Variables */
#define isdtversion "pK_VERSION"
#define SIGNAL_BIN_NAME "Signal_bin"
#define CSTLIB_ENV_NAME "SIGNAL_LIBRARY_PATH"

/* CodeGeneratorDefs_LANG (LANG is C, VHDL,...) is a file of the Polychrony distribution (in Signal_bin directory) */
#define CodeGeneratorPredef "CodeGeneratorDefs_"
/* ---- */

#define nullChar '\0'

static ptrepstr userdirname;

typedef FILE *ptFILE;
typedef struct
{
  ptFILE fp;
  boolean eoln;
  boolean eof;
  boolean out;
  boolean init;
  char buf;
} text;

#define Fread(x, f) fread((char *)&x, sizeof(x), 1, f)
#define Get(f) Fread((f).buf, (f).fp)
#define Getx(f) (f).init = 1, (f).eoln = (((f).buf = fgetc((f).fp)) == '\n') ? (((f).buf = ' '), 1) : 0
#define Getchr(f) (f).buf, Getx(f)

static FILE *Tmpfil;
static int Tmplng;

#define MyFlush(f) fflush((f).fp)

#define Fscan(f) (f).init ? ungetc((f).buf, (f).fp) : 0, Tmpfil = (f).fp
#define Scan(p, a) Scanck(fscanf(Tmpfil, p, a))

static void Scanck (int);
static void Getl (text *);

#define EoLine(f) ((f).eoln)
#define Eof(f) ((((f).init == 0) ? (Get(f)) : 0, ((f).eof ? 1 : feof((f).fp))) ? pk_true : pk_false)
#define Fwrite(x, f) fwrite((char *)&x, sizeof(x), 1, f)
#define Put(f) Fwrite((f).buf, (f).fp)
#define Putx(f) (f).eoln = ((f).buf == '\n'), (void)fputc((f).buf, (f).fp)
#define Putchr(c, f) (f).buf = (c), Putx(f)
#define Putl(f, v) (f).eoln = v
#define Finish(f) ((f).out && !(f).eoln) ? (Putchr('\n', f), 0) : 0, !fseek((f).fp, 0L, 0)
#define Fclose(f) (fclose((f).fp))

static char Rmode[] = "r";

#define Reset(f, n, l) (f).init = (f).init ? !fseek((f).fp, 0L, 0) : (((f).fp = Fopen(n, l, Rmode)), 1), (f).eof = (f).out = 0, Get(f)
#define Resetx(f, n, l) (f).init = (f).init ? (Finish(f)) : (((f).fp = Fopen(n, l, Rmode)), 1), (f).eof = (f).out = 0, Getx(f)

#ifdef WRITEONLY
static char Wmode[] = "w";
#else
static char Wmode[] = "w+";
#endif

#define Rewrite(f, n, l) (f).init = (f).init ? !fseek((f).fp, 0L, 0) : (((f).fp = Fopen(n, l, Wmode)), 1), (f).out = (f).eof = 1
#define Rewritex(f, n, l) (f).init = (f).init ? (Finish(f)) : (((f).fp = Fopen(n, l, Wmode)), 1), (f).out = (f).eof = (f).eoln = 1
static ptFILE Fopen (char *n, int l, char *m);

#ifndef MAXFILENAME
#define MAXFILENAME 512
#endif

#define caresp ' '

typedef struct S132		/* A file register : structure assigned to a opened file. It is characterized by :  */
{
  ptrepstr name;		/* The name of the file. */
  ptreg buffer;			/* A buffer containing the informations read or to be write. */
} repfil;

static repfil filreg3, filreg4, filreg5, filreg6;
static text if3, if4, if5, if6;
static char *signaldir;
static boolean nlmode;		/* new line mode */

/**
   Cette fonction lit le fichier ?*ifi jusqu'a rencontrer le caractere ?c et
   insere les caracteres lus dans le buffer associe a ?filreg Rend faux si le
   buffer est plein.
*/
static boolean
rdtoeq (repfil filreg, text * ifi, char c)
{
  ptpage adr;
  int deb, lg, i;

  register struct S132 *W2 = &filreg;
  adr = (ptpage) strphyadr (W2->buffer);
  deb = nregGet_startRank (W2->buffer);
  lg = nregGet_usedSize (W2->buffer);
  i = deb + lg;
  while (pk_true)
    {
      if (Eof (*ifi))
	{
	  strregsetlg (W2->buffer, i - deb);
	  return pk_true;
	}
      if (c == (*ifi).buf)
	{
	  strregsetlg (W2->buffer, i - deb);
	  return pk_true;
	}
      if (i > pagesize)
	{
	  strregsetlg (W2->buffer, i - deb);
	  return pk_false;
	}
      (*adr)[i - 1] = (*ifi).buf;
      i = i + 1;
      Getx (*ifi);
    }
  return pk_false;
}


/**
   Recuperation du nom du fichier ?ifi et ouverture de ce fichier en lecture
   si ?w est faux, en ecriture sinon. ?name est le nom du fichier code dans
   la memoire de chaines.
*/
static void
syslocopen (text * ifi, ptrepstr name, boolean w /* int openingMode */ )
{
  int l, i;
  char *fname;

  l = strsize (name);
  if (l == 0)
    return;
  //  if (l >= maxfname) return;
  fname = allocate_string (l + 1);
  for (i = 1; i <= l; i++)
    fname[i - 1] = strgetchar (name, i);
  fname[l] = '\0';
  if (w)
    {
      (*ifi).fp = Fopen (fname, sizeof fname, Wmode);
      (*ifi).init = (*ifi).out = (*ifi).eof = (*ifi).eoln = 1;
    }
  else
    {
      // Resetx(*ifi, fname.A, sizeof fname.A);
      (*ifi).fp = Fopen (fname, sizeof fname, Rmode);
      (*ifi).eof = (*ifi).out = 0, Getx (*ifi);
    }
  free (fname);
}

/**
   Si on n'est pas en fin de fichier pour ?ifi, met dans !cc le caractere
   courant contenu dans le buffer associe, lit un caractere supplementaire et
   rend vrai. Rend faux sinon.
*/
static boolean
getcha (text * ifi, char *cc)
{
  if (Eof (*ifi))
    return (pk_false);
  *cc = (*ifi).buf;
  Getx (*ifi);
  return (pk_true);
}

/**
  Lit dans le fichier ?ifi jusqu'a rencontrer le caractere ?c 
  ou la fin de fichier
*/
static boolean
sktoeq (text * ifi, char c)
{
  boolean res;
  boolean fini;

  fini = pk_false;
  while (!fini)
    {
      if (Eof (*ifi))
	{
	  fini = pk_true;
	  res = pk_true;
	}
      else
	{
	  if (c == (*ifi).buf)
	    {
	      fini = pk_true;
	      res = pk_true;
	    }
	  Getx (*ifi);
	}

    }
  return res;
}

static boolean
sktone (text * ifi, char c)
{
  boolean b, fini;

  fini = pk_false;
  while (!fini)
    {
      if (Eof (*ifi))
	{
	  fini = pk_true;
	  b = pk_false;
	}
      else if (c != (*ifi).buf)
	{
	  fini = pk_true;
	  b = pk_true;
	}
      else
	Getx (*ifi);
    }
  return b;
}


static boolean
getint (text * ifi, int *val)
{
  if (Eof (*ifi))
    return pk_false;
  Fscan (*ifi), Scan ("%ld", &Tmplng), *val = Tmplng, Getx (*ifi);
  return pk_true;
}


static boolean
newline (text * ifi)
{
  if (Eof (*ifi))
    return pk_false;
  Getl (ifi);
  return pk_true;
}


static void
locwriteln (repfil filreg, text * ifi, ptreg reg)
{
  int i, vto;

  vto = nregGet_usedSize (reg);
  if (vto > 0)
    for (i = 1; i <= vto; i++)
      Putchr (nregGet_char (reg, i), *ifi);
  if (nlmode)
    Putchr ('\n', *ifi);
  MyFlush (*ifi);
}

static void
Scanck (int n)
{
  if (n != 1)
    {
      (void) fprintf (stderr, "Bad input\n");
      exit (1);
    }
}


static void
Getl (text * f)
{
  while (f->eoln == 0)
    Getx (*f);
  Getx (*f);
}


static ptFILE
Fopen (char *n, int l, char *m)
{
  ptFILE f;
  register char *s;
  static char tmp[MAXFILENAME];

  if (n != NULL)
    {
      if (l < 0)
	l = strlen (n);
      if (l >= MAXFILENAME)
	{
	  (void) fprintf (stderr, "Too long filename '%s'\n", n);
	  exit (1);
	}
      else
	strncpy (tmp, n, sizeof (tmp));
    }
  s = tmp;
  if ((f = fopen (s, m)) == NULL)
    {
      (void) fprintf (stderr, "Cannot open: %s\n", s);
      exit (1);
    }
  /* if (n == NULL) unlink(tmp) */ ;
  return f;
}

/** V4.13.10 --> V4.14
   Suppression du phylum ?str en début de fichier */
static void
removeStringBeginOfFile (FILE * pf, const char *str)
{
  int n, i, nbread;
  char c;
  boolean stop;

  n = strlen (str);
  if (n == 0)
    return;
  stop = pk_true;
  nbread = 0;
  while (!stop)
    {
      if (feof (pf))
	return;
      c = fgetc (pf);
      nbread++;
      stop = (c != ' ');
    }
  i = 0;
  while (i < n)
    {
      if (feof (pf))
	return;
      c = fgetc (pf);
      nbread++;
      if (c != str[i])
	return;
      i++;
    }
  fseek (pf, 0, 0);
  for (i = 0; i < nbread; i++)
    fprintf (pf, " ");
}

/**
 * fileName is a file that should be in Polychrony binary directory
 * @returns the full path name of fileName suffixed by suff
 * in a new mallocated string
 * This function may fail
 */
static char *
_sys_suffixedFileInPolyBin (const char *fileName, int suffLength)
{
  char *path = getFilePathName (signaldir, fileName);
  int pathLg = strlen (path);
  char *res = allocate_string (pathLg + suffLength + 2);
  res = strcpy (res, path);
  res[pathLg] = fileSuffixSeparator;
  res[pathLg + 1] = NULL_char;
  return res;
}
