/*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */

//---------------------------------------------------------------
//    Methods called from Java to C
//---------------------------------------------------------------

/// Global initialisation for Signal, called once time for the first graphical window 
JNIEXPORT jlong JNICALL
Java_LiaisonSignal_n_1globalInit (JNIEnv *, jobject)
{
  int i;

  // ptSupervis pSupervis = new supervis((ptLiaisonSignal)null); // the main polychrony supervisor
  ptSupervis pSupervis = new supervis ();	// the main polychrony supervisor
  pGraphArb = pSupervis->getGraphicTrees ();
  the_graphicTablesManager = pSupervis->getTabManager ();
  pMess = pSupervis->getMessage ();
  pSupervis->globalInit ();
  for (i = 0; i < TBS; i++)
    supervis::select_buffer[i] = 0;
  initStack ();
  currentObj = (jobject) null;
  return ((jlong) pSupervis);
}


/// Initialisation for each graphical window execpt the first one
JNIEXPORT jlong JNICALL
Java_LiaisonSignal_n_1localInit (JNIEnv * env, jobject obj)
{
  // ptSupervis pSupervis = new supervis((ptLiaisonSignal)null, pGraphArb, the_graphicTablesManager, pMess); // a not main window polychrony supervisor
  ptSupervis pSupervis = new supervis (pGraphArb, the_graphicTablesManager, pMess);	// a not main window polychrony supervisor
  pSupervis->localInit ();
  return ((jlong) pSupervis);
}


///
JNIEXPORT jlong JNICALL Java_LiaisonSignal_n_1SignalProcCreate
  (JNIEnv * env, jobject obj, jlong pSupervis, jlong noFather, jstring name, jint category, jint index)
{
  int no;
  ntrees p, father;
  ptGraphicTrees pGraphicTrees;

  char *c_name = javaString2CString (env, name);

  pGraphicTrees = ((ptSupervis) pSupervis)->getGraphicTrees ();
  father = pGraphicTrees->getModel (JLONG2INT(noFather));
  p = pGraphicTrees->SignalProcCreate (father, c_name, category, index);
  no = pGraphicTrees->no_boite_proc (p);
  return (jlong) no;
}


///
JNIEXPORT jlong JNICALL
Java_LiaisonSignal_n_1SignalModuleCreate (JNIEnv * env, jobject obj, jlong pSupervis, jlong noFather, jstring name)
{
  int no;
  ntrees p, father;
  ptGraphicTrees pGraphicTrees;

  char *c_name = javaString2CString (env, name);

  pGraphicTrees = ((ptSupervis) pSupervis)->getGraphicTrees ();
  father = pGraphicTrees->getModel (JLONG2INT(noFather));
  p = pGraphicTrees->SignalModuleCreate (father, c_name);
  no = pGraphicTrees->no_boite_proc (p);
  return (jlong) no;
}


///
JNIEXPORT jlong JNICALL
Java_LiaisonSignal_n_1SignalRootCreate (JNIEnv * env, jobject obj, jlong pSupervis, jstring name)
{
  ntrees rac;
  int no;

  char *c_name = javaString2CString (env, name);
  rac = ((ptSupervis) pSupervis)->getBuilder ()->SignalRootCreate (c_name);
  no = ((ptSupervis) pSupervis)->getGraphicTrees ()->no_boite_proc (rac);
  return (jlong) no;
}


///
JNIEXPORT void JNICALL
Java_LiaisonSignal_n_1SignalTreeDelete (JNIEnv * env, jobject obj, jlong pSupervis, jlong node, jboolean detach)
{
  ((ptSupervis) pSupervis)->getBuilder ()->detruire_bte (JLONG2INT(node), detach);
}


/** duplication de l'arbre avec changement de tous les numéros des objets graphiques */
JNIEXPORT jlong JNICALL
Java_LiaisonSignal_n_1SignalTreeCopy (JNIEnv * env, jobject obj, jlong pSupervis, jlong srcNode)
{
  ntrees newTree, srcTree;
  ptGraphicTrees pGraphicTrees;
  long int newNode;

  pGraphicTrees = ((ptSupervis) pSupervis)->getGraphicTrees ();
  srcTree = pGraphicTrees->getModel (JLONG2INT(srcNode));
  newTree = pGraphicTrees->dupl_arbre (srcTree);
  newNode = pGraphicTrees->no_boite_proc (newTree);
  return ((jlong) newNode);
}


///
JNIEXPORT jlong JNICALL
Java_LiaisonSignal_n_1getSignalChildRef (JNIEnv * env, jobject obj, jlong pSupervis, jlong fatherNo, jlong index)
{
  ntrees p, fatherProc;
  ptGraphicTrees pGraphicTrees;
  int no;

  pGraphicTrees = ((ptSupervis) pSupervis)->getGraphicTrees ();
  fatherProc = pGraphicTrees->getModel (JLONG2INT(fatherNo));
  p = STree_getIthModel (fatherProc, JLONG2INT(index));
  if (ISNOT_NILTREE (p))
    no = pGraphicTrees->no_boite_proc (p);
  else
    no = 0;
  return (no);
}


///
JNIEXPORT jlong JNICALL
Java_LiaisonSignal_n_1getSignalTreeArity (JNIEnv * env, jobject obj, jlong fatherProc, jlong index)
{
  return (0);			// not implemented
}


///
// if index < 0 append childNode at the end of the declaration list of fatherNode
// if index >= 0 insert childNode before the node of rank index (rank starts at 0) in fatherNode
JNIEXPORT void JNICALL Java_LiaisonSignal_n_1SignalTreeInsert
  (JNIEnv * env, jobject obj, jlong pSupervis, jlong fatherNode, jlong childNode, jint index)
{
  ptGraphicTrees pGraphicTrees;
  ntrees fatherProc, childProc, ldecl;

  pGraphicTrees = ((ptSupervis) pSupervis)->getGraphicTrees ();
  fatherProc = pGraphicTrees->getModel (JLONG2INT(fatherNode));
  childProc = pGraphicTrees->getModel (JLONG2INT(childNode));
  ldecl = STreeModel_getLocalDeclarations (fatherProc);
  if (index < 0)
    ntreeList_appendLast (ldecl, childProc);
  else
    ntreeList_insertBefore (childProc, nchild (index + 1, ldecl));
}


///
JNIEXPORT void JNICALL
Java_LiaisonSignal_n_1SignalTreeSelect (JNIEnv * env, jobject obj, jlong pSupervis, jlong node)
{
  ntrees p;

  push (env, obj);
  p = ((ptSupervis) pSupervis)->getGraphicTrees ()->getModel (JLONG2INT(node));
  ((ptSupervis) pSupervis)->getCurObjManager ()->selectCurrentTree (p);
  //  C2Java_refresh (pk_false); REMOVED DEC 2015 LBesnard.
  pop ();
}


///
JNIEXPORT void JNICALL
Java_LiaisonSignal_n_1SignalChangeProcName (JNIEnv * env, jobject obj, jlong pSupervis, jlong node, jstring name)
{
  ntrees p;

  char *c_name = javaString2CString (env, name);

  p = ((ptSupervis) pSupervis)->getGraphicTrees ()->getModel (JLONG2INT(node));
  // nchangchild(p, LL_make_nom_atom( string2repstr(c_name)), 1);
  nchangchild (p, string2ident (c_name), 1);
}


///
// JNIEXPORT void JNICALL Java_LiaisonSignal_n_1save_ntrees
//  (JNIEnv *env, jobject obj, jlong pSupervis, jlong node, jstring name)
// {
//    ntrees p;
//    char* c_name= javaString2CString(env, name);
// }

///
JNIEXPORT jboolean JNICALL
Java_LiaisonSignal_n_1sig_1iterate (JNIEnv * env, jobject obj, jlong pSupervis)
{
  boolean b;
  // printf("DEBUG entering Java_LiaisonSignal_n_1sig_1iterate\n"); fflush(stdout);
  push (env, obj);
  mid_C2Java_readMenuButton = currentEnv->GetMethodID (currentEnv->GetObjectClass (currentObj), "readMenuButton", "()I");
  mid_C2Java_razMenuButton = currentEnv->GetMethodID (currentEnv->GetObjectClass (currentObj), "razMenuButton", "()V");
  setInterfSignalC ((ptSupervis) pSupervis);
  b = ((ptSupervis) pSupervis)->superviseurStep ();
  // PLG replaced by above line b = ((ptSupervis)pSupervis)->getSignalSuperv()->SUPERVISEUR_step();
  pop ();
  return ((jboolean) (b));
}


///
JNIEXPORT jlong JNICALL
Java_LiaisonSignal_n_1getRacine (JNIEnv * env, jobject obj, jlong pSupervis)
{
  ntrees p;

  p = ((ptSupervis) pSupervis)->getCurObjManager ()->get_racine ();
  return ((jlong) (((ptSupervis) pSupervis)->getGraphicTrees ()->no_boite_proc (p)));
}


///
JNIEXPORT void JNICALL
Java_LiaisonSignal_n_1getMouseMoveCoord (JNIEnv * env, jobject obj, jlong pSupervis, jlong x, jlong y)
{
  push (env, obj);

  ((ptSupervis) pSupervis)->bouge_souris ((int) x, (int) y);
  pop ();
}


///
JNIEXPORT void JNICALL
Java_LiaisonSignal_n_1set_1COMPILER_1OPTION (JNIEnv * env, jobject obj, jlong pSupervis, jlong i)
{
  push (env, obj);
  ((ptSupervis) pSupervis)->set_COMPILER_OPTION (JLONG2INT(i));
  pop ();
}


///
JNIEXPORT void JNICALL
Java_LiaisonSignal_n_1endAutoComp (JNIEnv * env, jobject obj, jlong pSupervis, jboolean ok, jstring voptions)
{
  char *voptions_c;
  if (ok)
    voptions_c = javaString2CString (env, voptions);
  else
    voptions_c = (char *) 0;
  push (env, obj);
  ((ptSupervis) pSupervis)->endAutoComp (ok, voptions_c);
  pop ();
}



///
JNIEXPORT void JNICALL
Java_LiaisonSignal_n_1WithoutParam (JNIEnv * env, jobject obj, jlong pSupervis)
{
  push (env, obj);
  ((ptSupervis) pSupervis)->WithoutParam ();
  pop ();
}

///
JNIEXPORT void JNICALL
Java_LiaisonSignal_n_1okParam (JNIEnv * env, jobject obj, jlong pSupervis, jstring s)
{
  char *c_name = javaString2CString (env, s);
  push (env, obj);
  ((ptSupervis) pSupervis)->okParam (c_name);
  pop ();
}


///renvoie dans le monde Java du nom d un processus en fonction de sa reference
JNIEXPORT jstring JNICALL
Java_LiaisonSignal_n_1getNameProc (JNIEnv * env, jobject obj, jlong pSupervis, jlong signalRef)
{
  ntrees p;
  jstring chaine;

  p = ((ptSupervis) pSupervis)->getGraphicTrees ()->getModel (JLONG2INT(signalRef));
  chaine = env->NewStringUTF (repstr2string (STreeModelModule_getNameStr (p)));
  return (chaine);
}


///function call from Java with the ok button in the dialog box reset
JNIEXPORT void JNICALL
Java_LiaisonSignal_n_1SignalRaz (JNIEnv * env, jobject obj, jlong pSupervis)
{
  ((ptSupervis) pSupervis)->SignalReset ();
}

/// function call from Java when the user quit the polychrony GUI.
JNIEXPORT void JNICALL
Java_LiaisonSignal_n_1SignalExit (JNIEnv * env, jobject obj, jlong pSupervis)
{
  ((ptSupervis) pSupervis)->SignalExit ();
}


///function call from Java to know if node (in) is a module
JNIEXPORT jboolean JNICALL
Java_LiaisonSignal_n_1isModul (JNIEnv * env, jobject obj, jlong pSupervis, jlong node)
{
  ntrees p;

  p = ((ptSupervis) pSupervis)->getGraphicTrees ()->getModel (JLONG2INT(node));
  return (jboolean) STree_isModule (p);
}


///function call from Java to know if signalRef is a process or a function or a node
JNIEXPORT jboolean JNICALL
Java_LiaisonSignal_n_1STree_isModel (JNIEnv * env, jobject obj, jlong pSupervis, jlong node)
{
  ntrees p;

  p = ((ptSupervis) pSupervis)->getGraphicTrees ()->getModel (JLONG2INT(node));
  return (jboolean) STree_isModel (p);
}


///function call from Java to know if signalRef is a function
JNIEXPORT jboolean JNICALL
Java_LiaisonSignal_n_1isFunc (JNIEnv * env, jobject obj, jlong pSupervis, jlong node)
{
  ntrees p;

  p = ((ptSupervis) pSupervis)->getGraphicTrees ()->getModel (JLONG2INT(node));
  return (jboolean) STree_isFunction (p);
}

///function call from Java to know if signalRef is a node
JNIEXPORT jboolean JNICALL
Java_LiaisonSignal_n_1isNode (JNIEnv * env, jobject obj, jlong pSupervis, jlong node)
{
  ntrees p;

  p = ((ptSupervis) pSupervis)->getGraphicTrees ()->getModel (JLONG2INT(node));
  return (jboolean) STree_isNode (p);
}

///function call from Java to know if signalRef is a node
JNIEXPORT jboolean JNICALL
Java_LiaisonSignal_n_1isAction (JNIEnv * env, jobject obj, jlong pSupervis, jlong node)
{
  ntrees p;

  p = ((ptSupervis) pSupervis)->getGraphicTrees ()->getModel (JLONG2INT(node));
  return (jboolean) STree_isAction (p);
}

///function call from Java to know if it s a process or a function
JNIEXPORT jboolean JNICALL
Java_LiaisonSignal_n_1isProcess (JNIEnv * env, jobject obj, jlong pSupervis, jlong node)
{
  ntrees p;

  p = ((ptSupervis) pSupervis)->getGraphicTrees ()->getModel (JLONG2INT(node));
  return (jboolean) STree_isProcess (p);
}


// Function call from Java to obtain the name of the sub-process number index of the Signal
// library  libRoot (the name is the name of the file.gif which contains the icon which 
//  libRoot the sub-process).
/// Return an empty string if the index sub-process doesn't exist.
JNIEXPORT jstring JNICALL
Java_LiaisonSignal_n_1getNameGif (JNIEnv * env, jobject obj, jlong libRoot, jlong index)
{
  ptrepstr str;
  str = getNameSubProc (JLONG2NTREES(libRoot), JLONG2INT(index));
  return (env->NewStringUTF (repstr2string (str)));
}


// methode appelee par la methode repaint. Cette methode peut etre appelee directement
// par Java, et pas seulement dans l'appli. Il faut donc sauvegarder les variables
// currentEnv et currentObj, sinon une fois le repaint est termine et si l'appli 
// a besoin d'afficher (donc faire appel a des methodes Java), elle n'aura plus les 
/// bonnes variables  et donc plantage.
JNIEXPORT void JNICALL
Java_LiaisonSignal_n_1displayTree (JNIEnv * env, jobject obj, jlong pSupervis)
{
  ptCurrentObjectsManager pManager;
  ntrees p;

  pManager = ((ptSupervis) pSupervis)->getCurObjManager ();
  p = pManager->get_arb_cou_engl ();
  if ((ISNOT_NILTREE (p)) && (p != pManager->get_racine ()) && (!STree_isModule (p)))
    {				// on n'affiche que si on n'est pas à la racine
      push (env, obj);
      ((ptSupervis) pSupervis)->rafraichir_sans_garb ();
      pop ();
    }
}
