/*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */

/**
   It upgrades the operand tree, assumed to be a model or a module saved/loaded by the Signal GUI, to the CONST_NUM_RELEASE.
*/
_PUBLIC_ void
graphicalTrees::upgrade (ntrees vmodel)
{
  long int numVers;

  numVers = TreeGetCurrentVersion (vmodel);
  if (numVers == CONST_NUM_RELEASE) return;
  if (numVers > CONST_NUM_RELEASE)
    {
      printf("In graphicalTrees::upgrade method: the loaded model number version is > current release.\n"); fflush(stdout);
      return;
    }
  initSignalTreeConversions(); 
  while (numVers != (long int) (CONST_NUM_RELEASE))
    {
      if (numVers == 0) upgrade0To1 (vmodel);
      else if (numVers == 1) upgrade1To2 (vmodel);
      else if (numVers == 2) upgrade2To3 (vmodel);
      else if (numVers == 3) upgrade3To4 (vmodel);
      else if (numVers == 4) upgrade4To5 (vmodel);
      else;
      numVers++;
    }
  TreeSetCurrentVersion (vmodel);
}

/**
   Saving a trees (e) into a file (filePathString).
   The tree is annotated by the number of  the current release.
*/
boolean graphicalTrees::GraphicSaving (char *filePathString, ntrees e)
{
  TreeSetCurrentVersion (e);
  return ntrees_save (filePathString, e);
}
