/*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */
///
_PRIVATE_ static char openParenthesis[] = "(";
static char inputBegin[] = "? ";
static char outputBegin[] = "! ";
static char closeParenthesis[] = ")";

/**
   Cette fonction rend vrai si l'arbre ?arb  n'est pas affichable
   (arbre ou liste vide ou contenant des "meta")
*/
boolean
SignalDisplaying::text_non_affichable (ntrees arb)
{
  int oper;

  if (arb == null)
    return pk_true;
  oper = noper (arb);
  if (LL_is_listOp (oper))
    return (nisemptyList (arb));
  return (oper == opnil);
}


/**
   Appele le decompilateur Signal textuel sur l'arbre ?arb_sig
   avec ?holo comme holophraste. La chaine rendue est stockee
   dans stock_chaine pour pouvoir être afficher après.
   Rend le nombre de lignes de la chaîne.
*/
int
SignalDisplaying::appel_decompilo_text (ntrees arb_sig, int holo)
{
  int nl, i, noligne;
  pttab_chaine chaine;

  sysinitgraphchaine ();
  if (pK_SignalPrinter->fileUnparse ((ntrees) arb_sig, filebuf, 0, holo, ographical))
    {
      chaine = sysgetgraphchaine (&noligne);
      for (nl = 0; nl < noligne; nl++)
	{
	  i = 0;
	  while ((*chaine)[nl][i] != '\0')
	    {
	      stock_chaine[nl][i] = (*chaine)[nl][i];
	      i++;
	    };
	  stock_chaine[nl][i] = '\0';
	}
      return (noligne);
    }
  return 0;
}


/**
   Calls the Signal textual unparser for the tree p (in) and displays it in the window w.
   The separator tk is added if defined. It must be defined when the operand tree (p) is not
   an element of a list printed by SignalPrinter::decl1 () method).
*/
void
SignalDisplaying::displayText (text_win_type w, ntrees p, const char *tk)
{
  int j, nbLines;

  nbLines = appel_decompilo_text (p, 1000);
  j = 0;
  while (j != nbLines)
    {
      C2Java_textAppend (w, stock_chaine[j]);
      j++;
      if ((j == nbLines) && (tk != NULL))
	C2Java_textAppend (w, tk);
      C2Java_textAppend (w, "\n ");
    }
}

/**
 * Cette procedure affiche dans la fenetre texte la liste de declarations
 * accrochees aux ports d'entree ou de sortie d'une boite.
 * si ?opeport vaut gpkOp_porte (gpkOp_ports), on affiche la liste des ports
 * d'entree (de sortie)
 * ?lport : liste des ports a editer.
 */
void
SignalDisplaying::decomp_l_decsig (text_win_type win, int opeport, ntrees lport)
{
  int ope;
  ntrees decsig, filsc;

  if (ISNOT_NILTREE (lport))
    {
      filsc = nchild (1, lport);
      while (filsc != null)
	{
	  ope = noper (filsc);
	  decsig = nattrntrees (filsc, SIGNAL_ATTRNUMBER);
	  if (ope == opeport)
	    {
	      if (win == C_CanvasText)
		display_nom_port (filsc);
	      if (decsig != null)
		{
		  if (noper (decsig) == opdecsig)
		    displayText (win, decsig, ";");
		  else
		    displayText (win, decsig, NULL);
		}
	      else
		C2Java_textAppend (win, "~;");
	    }
	  filsc = ntrees_getNext (filsc);
	}
    }
}


/**
   Cette méthode effectue l'affichage de texte dans la fenêtre
   graphique (dans les btes ou à côté des btes pour les noms de ports).
   Le texte se trouve dans le tableau de chaine stock_chaine (var. globale)
   ?x, y : les coordonnees du point ou l'on commence l'affichage
   ?h : hauteur de la boite (h = 0 lorsqu'on affiche un nom de ports)
   ?tyPolice vaut C_box_police lorqu'on affiche le contenu d'une boite,
   vaut C_port_police lorsqu'on affiche un nom de port.
   ?holo : holophrastre avec lequel on appelle le décompilateur Signal;
   si cela ne tient pas dans la bte, on rappelle le décompilateur en
   décrémentant l'holophraste.
*/
void
SignalDisplaying::envoyer_text (ntrees p_sig, int x, int y, int h, int noligne, int tyPol, int holo)
{
  int i, ht;			/* ht : hauteur d'un caractere en pixel */
  boolean coupe;

  car_box[0] = C2Java_getSizeFontL (C_box_police);
  car_box[1] = C2Java_getSizeFontH (C_box_police);

  if (noligne == 1)
    pDraw->dessiner_text ((char *) stock_chaine, x, y, tyPol);
  else
    {
      ht = car_box[1];
      //      if (proport.y != 1.) ht = (int)(ht / proport.y);
      coupe = pk_false;
      if ((h != 0) && (noligne * ht) >= (2 * h))	/* ne concerne que les btes (pas les ports) */
	{
	  coupe = pk_true;
	  while ((noligne * ht) >= (2 * h))
	    noligne--;
	  noligne--;
	};
      if (!coupe || (holo == 0))
	{
	  if (!coupe)
	    {
	      i = (noligne * ht) / 2;
	      y = y - i + ht;
	    }
	  else if (noligne > 0)
	    y = y - h + ht;
	  for (i = 0; i < noligne; i++)
	    {
	      pDraw->dessiner_text ((char *) (stock_chaine + i), x, y, tyPol);
	      y = y + ht;
	    };
	  if (coupe)
	    pDraw->dessiner_text ("...", x + 4, y, tyPol);
	}
      else
	{
	  noligne = appel_decompilo_text (p_sig, holo);
	  holo--;
	  envoyer_text (p_sig, x, y, h, noligne, tyPol, holo);
	}
    }
}


/**
   Decompile l'arbre Signal ?p et l'affiche dans la fenetre graphique dans 
   la boite de coordonnees (?x, ?y) de hauteur  ?h avec la police ?tyPol.
*/
void
SignalDisplaying::decompiler_text (ntrees p, int x, int y, int h, int tyPol)
{
  int holo_text, holo_essai, noligne;

  if (!text_non_affichable (p))
    if (ntreearity (p) > 100)
      //si le texte est trop gros, on n'appelle pas le decompilo Signal, on affiche "..."
      pDraw->dessiner_text (" ... ", x + 20, y + (h / 2), 0);
    else
      {
	if (force)		/* etat du bouton "details" à vrai  */
	  {
	    holo_text = 100;
	    holo_essai = 9;
	  }
	else
	  {
	    holo_text = 4;
	    holo_essai = 3;
	  }
	noligne = appel_decompilo_text (p, holo_text);
	envoyer_text (p, x, y, h, noligne, tyPol, holo_essai);
      }
}


/**
   Calcule les coordonnees de la boite ?g_arb en fonction du niveau de 
   visualisation ?niveau, du facteur d'echelle ?fact_ech et du vecteur 
   de translation ?translat
   rend les coordonnees dans !tcoord  et le nombre de coordonnees (!nbcoord)
   Fonction appelee uniquement de display_rec.
*/
void
SignalDisplaying::calcul_coord_bte (ntrees g_arb, typcoord * tcoord, int *nbcoord, int niveau, typcoord translat, db_reel fact_ech)
{
  int form;
  float val_reel;
  form = the_graphicalTrees->getCoord (g_arb, nbcoord, tcoord, 1);
  if (form == oprect)
    {
      if (niveau == 0)		/* on affiche la boite en tant que cadre */
	{
	  mil_cadre.x = abs_mil_cadre;
	  mil_cadre.y = ord_mil_cadre;
	  val_reel = lg_bte_stand;
	  fact_ech.x = val_reel / tcoord[1].x;
	  val_reel = ht_bte_stand;
	  fact_ech.y = val_reel / tcoord[1].y;
	  lg_cadre = lg_bte_stand;
	  ht_cadre = ht_bte_stand;
	}
      else
	{
	  mil_cadre.x = (translat.x + (int) (tcoord[0].x * fact_ech.x));
	  mil_cadre.y = (translat.y + (int) (tcoord[0].y * fact_ech.y));
	  lg_cadre = (int) (tcoord[1].x * fact_ech.x);
	  ht_cadre = (int) (tcoord[1].y * fact_ech.y);
	}
    }
}


/**
   Calcul de la taille de la boite en nombre de caracteres et positionnement de
   la taille du buffer d'edition (utilise pour la decompilation textuelle)
   Les variables calculees sont globales au module.
   ?p_sig : arbre Signal associe a la boite
   ?fact_ech : facteur d'echelle
   Fonction appelee uniquement de display_rec.
*/
void
SignalDisplaying::calcul_taille_bte (ntrees p_sig, db_reel fact_ech)
{
  car_box[0] = C2Java_getSizeFontL (C_box_police);
  car_box[1] = C2Java_getSizeFontH (C_box_police);
  taille_bte = ((2 * lg_cadre) / car_box[0]) - 1;
  //  if (proport.x != 1.) 
  //    taille_bte =  (int)(taille_bte * proport.x);
  filebuf->setsize (taille_bte / 2, taille_bte);
  if (lg_cadre <= taillepave_x + 2)
    x_bte = mil_cadre.x - lg_cadre;
  else if (fact_ech.x < 1. || STree_isModel (p_sig))
    x_bte = mil_cadre.x - lg_cadre + (taillepave_x / 2) + 1;
  else
    x_bte = mil_cadre.x - lg_cadre + taillepave_x + 2;
  y_bte = mil_cadre.y;
  ht_bte = ht_cadre;
}


/**
   Fonction recursive appelee par la fonction display_chemin ou par la
   fonction display_obj.
   ?couleur :  vaut 0 si on efface, >0 pour dessiner (differentes couleurs)
   ?port_mult : a vrai, alors il s'agit d'un port multiple (le lien est 
   plus gros : 3 pixels d'epaisseur).
   Pour les autres parametres, voir la fonction suivante display_chemin.
*/
void
SignalDisplaying::display_chemin_rec (ntrees p_chemin, typcoord translat, db_reel fact_ech, int couleur, int x, int y, boolean port_mult)
{
  ntrees p, frere, s_chem;
  int x_pat, nbcoord, form, x_halo;
  typcoord tcoord[maxcoord];

  if ((p = nchild (4, p_chemin)) != null)
    {
      x_halo = (int) (halo_bte * fact_ech.x) + 1;
      s_chem = nchild (3, p_chemin);
      p = nchild (1, p);
      while (p != null)
	{
	  form = the_graphicalTrees->getCoord (nchild (2, p), &nbcoord, tcoord, 1);
	  frere = ntrees_getNext (p);
	  x_pat = 0;
	  if ((nb_fois == 0) && (couleur != 0))
	    {
	      tcoord[0].x = x;
	      tcoord[0].y = tcoord[1].y = y;
	    }
	  else
	    {
	      tcoord[0].x = (int) (tcoord[0].x * fact_ech.x) + translat.x;
	      if (nb_fois == 1 && (couleur != 0))
		tcoord[0].y = y;
	      else
		tcoord[0].y = (int) (tcoord[0].y * fact_ech.y) + translat.y;
	      tcoord[1].y = (int) (tcoord[1].y * fact_ech.y) + translat.y;
	    };
	  tcoord[1].x = (int) (tcoord[1].x * fact_ech.x) + translat.x;
	  nb_fois++;
	  if (frere == null && (couleur != 0) && (s_chem == null || ntreearity (s_chem) == 0))
	    if (tcoord[1].x >= abs_mil_cadre + lg_bte_stand - taillepave_x)
	      tcoord[1].x = abs_mil_cadre + lg_bte_stand;
	    else if (tcoord[1].x <= decal_cadre_x + taillepave_x)
	      tcoord[1].x = decal_cadre_x;
	    else if (tcoord[1].x < tcoord[0].x)
	      x_pat = -x_halo;
	    else
	      x_pat = x_halo;

	  tcoord[1].x = tcoord[1].x + x_pat;
	  pDraw->dessiner_lien (2, tcoord, port_mult, couleur);
	  p = frere;
	};
      if (nchild (2, p_chemin) == null)
	{
	  pDraw->dessiner_rect (tcoord[1], 1, 1, couleur);
	}
    };
  if ((p = s_chem) != null)
    {
      p = nchild (1, p);
      while (p != null)
	{
	  display_chemin_rec (p, translat, fact_ech, couleur, x, y, port_mult);
	  p = ntrees_getNext (p);
	};
    }
}


/**
   Returns the color associated with the type vtype (in)
*/
int
SignalDisplaying::conversion_typ_col (int vtype)
{
  int couleur;

  switch (vtype)
    {
    case opinteger:
    case opshort:
      couleur = COL_INT;
      break;
    case oplogical:
    case opevent:
      couleur = COL_BOOL;
      break;
    case opreal:
    case opdpreal:
      couleur = COL_REAL;
      break;
    case opnom: case opident:
      couleur = COL_TYPNOM;
      break;
    case opimplicit:
    default:
      couleur = COL_DEF;
    }
  return (couleur);
}


/**
   Rend la couleur d'un lien (qui dépend du type des ports connectés par ce lien)
   Si le port diffuseur du lien est typé, rend la couleur associée à ce type,
   sinon, on prend la liste des ports récepteurs, et on cherche le premier port
   typé. En principe, tous les ports récepteurs devraient être du même type, 
   sinon il y a incohérence.
*/
int
SignalDisplaying::get_color (ntrees p_chemin)
{
  ntrees p_port, l_ports_recept;
  int type_port, couleur, i, n;

  p_port = the_graphicalTrees->get_port_diff (p_chemin);
  type_port = the_graphicalTrees->get_typesignal_port (p_port);
  if ((type_port == opimplicit) || (type_port == 0))
    {
      l_ports_recept = LL_make_lports_emptyList ();
      the_graphicalTrees->get_ports_recept (p_chemin, l_ports_recept);
      n = ntreearity (l_ports_recept);
      i = 1;
      couleur = COL_DEF;
      while ((i <= n) && (couleur == COL_DEF))
	{
	  p_port = the_graphicTablesManager->graphObj_get_tree ((int) nintof (nchild (i, l_ports_recept)));
	  type_port = the_graphicalTrees->get_typesignal_port (p_port);
	  couleur = conversion_typ_col (type_port);
	  i++;
	}
    }
  else
    couleur = conversion_typ_col (type_port);
  return (couleur);
}


/**
 * dessine l'ensemble des liens qui partent d'un port diffuseur:
 * ?p_chemin : l'arbre des chemins
 * ?translat : vecteur de translation
 * ?fact_ech : facteur d'echelle
 * ?(x,y) : coordonnee du port diffuseur : on prend ces coordonnees-la
 *          comme coordonnees de la premiere extremite du premier segment
 *          (evite les erreurs d'arrondi lorsque ?fact_ech est different
 *          de 1).
 */
void
SignalDisplaying::display_chemin (ntrees p_chemin, typcoord translat, db_reel fact_ech)
{
  ntrees p_port;
  int couleur;
  boolean port_mult;

  if (ISNOT_NILTREE (p_chemin))
    {
      p_port = the_graphicalTrees->get_port_diff (p_chemin);
      couleur = get_color (p_chemin);
      port_mult = the_graphicalTrees->is_port_multi (p_port);
      if (noper (p_port) == gpkOp_porte)
	if (ou == gauche)
	  x_port = x_port + taillepave_x;
	else
	  x_port = x_port - taillepave_x;
      nb_fois = 0;
      display_chemin_rec (p_chemin, translat, fact_ech, couleur, x_port, y_port, port_mult);
    }
}


/**
   Affiche un port et les liens qui partent de ce port. Fonction locale a display_rec.
*/
void
SignalDisplaying::display_port_lien (ntrees p_port, ntrees p_sig, int niveau, typcoord translat,
				     typcoord translat2, db_reel fact_ech, db_reel fact_ech2)
{
  int ope;
  ntrees p_chemin;

  display_port (p_port, COL_DEF, fact_ech);
  if ((STree_isBodyTree (p_sig) || affich_nom_port) && (niveau < 2))
    display_nom_port (p_port);
  ope = noper (p_port);
  p_chemin = nchild (3, p_port);
  if ((ope == gpkOp_ports)
      && (the_graphicalTrees->boite_mere (the_graphicalTrees->tree_get_graphObj (p_port)) != pCurObjManager->get_bte_cou_engl ()))
    display_chemin (p_chemin, translat, fact_ech);
  else if ((ope == gpkOp_porte) && (niveau <= 1))
    display_chemin (p_chemin, translat2, fact_ech2);
}


///
boolean
SignalDisplaying::isTreeWithIcon (ntrees p_sig, ntrees * modelName)
{
  noperators ope;

  if (STree_isUndefinedTree (p_sig))
    return pk_false;
  ope = noper (p_sig);
  if ((ope == opinst) || (ope == opappe))
    {
      if (!findIcon (p_sig))
	return pk_false;
      (*modelName) = STree_getNameCalledModel (p_sig);
      return pk_true;
    }
  if (ope == opdefsig)
    return isTreeWithIcon (nchild (2, p_sig), modelName);
  return pk_false;
}


/**
   Display the box associated with the Signal tree p_sig (in). The box may be a 
   rectangle or an icon.
   mil, lg, ht (in) are the coordinates of the box.
*/
void
SignalDisplaying::display_box (ntrees p_sig, typcoord mil, int lg, int ht)
{
  ntrees name;

  if (isTreeWithIcon (p_sig, &name))
    {
      pDraw->dessiner_rect (mil, lg + 1, ht + 1, COL_DEF);	// Modified Oct 2007 lbesnard.
      pDraw->dessiner_image (repstr2string (nstringof (name)), mil, lg, ht);
    }
  else
    {
      pDraw->dessiner_rect (mil, lg, ht, COL_DEF);
      display_commentInfoWindow (p_sig);
    }
}


/**
   Fonction recursive appelee par display_tree.
   Affiche la boite associée à l'arbre Signal ?p_sig, ses ports, les liens qui
   partent de ses ports et ses boites internes.
   ?p_sig : arbre Signal a partir duquel on fait l'affichage.
   ?holo : niveau d'affichage pour le texte (au depart holo vaut 3)
   ?niveau : au premier appel, vaut 0 ensuite il est incremente de 1.
   ?translat, ?fact_ech : vecteur de translation et facteur d'echelle
   ?avec_cadre : si a faux, alors on ne visualise pas le cadre lorsqu'on est
   au niveau expression de processus (utilise pour l'affichage des
   icones dans la fenetre biblitheque).
*/
void
SignalDisplaying::display_rec (ntrees p_sig, int holo, int niveau, typcoord translat, db_reel fact_ech, boolean avec_cadre)
{
  ntrees g_arb, p, p_port, p_att, x;
  boolean decomp_text;
  typcoord translat2, tcoord[maxcoord];
  int ope, nbcoord;
  float val_reel;
  db_reel fact_ech2;

  g_arb = the_graphicalTrees->get_arb_boite (p_sig);
  if (g_arb != null)
    {
      calcul_coord_bte (g_arb, tcoord, &nbcoord, niveau, translat, fact_ech);
      /* on n'affiche pas le cadre lorsqu'on est au niveau liste de processus */
      if (avec_cadre || niveau != 0)
	display_box (p_sig, mil_cadre, lg_cadre, ht_cadre);
      if (niveau == 0)		// correspond au 1 ier appel de display_rec
	{
	  fact_ech2.x = 1;
	  fact_ech2.y = 1;
	}
      else
	{
	  val_reel = (float) lg_cadre;
	  fact_ech2.x = fact_ech.x * (val_reel / lg_bte_stand);
	  val_reel =  (float) ht_cadre;
	  fact_ech2.y = fact_ech.y * (val_reel / ht_bte_stand);
	};
      translat2.x = mil_cadre.x - lg_cadre;
      translat2.y = mil_cadre.y - ht_cadre;
      p_port = the_graphicalTrees->getFirstPort (g_arb);
      while (p_port != null)
	{
	  display_port_lien (p_port, p_sig, niveau, translat, translat2, fact_ech, fact_ech2);
	  p_port = ntrees_getNext (p_port);
	}
      niveau++;
      holo--;
      calcul_taille_bte (p_sig, fact_ech);
      translat.x = translat2.x;
      translat.y = translat2.y;
      fact_ech.x = fact_ech2.x;
      fact_ech.y = fact_ech2.y;
    }
  // on affiche l'intérieur de la boite qu'on vient d'afficher
  if (holo > 0)
    {
      ope = noper (p_sig);
      if (ope == oplfeed)
	{			// lorsque l'operateur de l'arbre est un oplfeed, on peut avoir des boites internes
	  decomp_text = pk_true;
	  p = nchild (1, p_sig);
	  while (p != null)
	    {
	      p_att = the_graphicalTrees->desc_expr_sig (p);
	      g_arb = nattrntrees (p_att, GRAPHI_ATTRNUMBER);
	      if ((p_att != null) && (g_arb != null))
		{
		  decomp_text = pk_false;
		  if (niveau <= 2)	// on n'affiche qu'un seul niveau de boîtes internes 
		    display_rec (p_att, holo, niveau, translat, fact_ech, avec_cadre);
		};
	      p = ntrees_getNext (p);
	    };
	  if (decomp_text)
	    decompiler_text (p_sig, x_bte, y_bte, ht_bte, C_box_police);
	}
      else if (!isTreeWithIcon (p_sig, &x))
	decompiler_text (p_sig, x_bte, y_bte, ht_bte, C_box_police);
    }
  else if (!isTreeWithIcon (p_sig, &x))
    decompiler_text (p_sig, x_bte, y_bte, ht_bte, C_box_police);
}


/**
 * fonction appelee par la fonction afficher et fait appel a la fonction
 * recursive display_rec. Initialise les parametres d'ordre graphique
 * (vecteur de translation, facteur d'echelle)
 * Memes parametres que la fonction afficher.
 */
void
SignalDisplaying::display_tree (ntrees p, boolean affich_pur)
{
  int holo, niveau;
  typcoord translat;
  db_reel fact_ech;

  if (ISNOT_NILTREE (p))
    if (ISNOT_NILTREE (the_graphicalTrees->get_arb_boite (p)))
      {
	holo = 3;
	niveau = 0;
	translat.x = 0;
	translat.y = 0;
	fact_ech.x = 1.;
	fact_ech.y = 1.;
	display_rec (p, holo, niveau, translat, fact_ech, pk_true);
	if (!affich_pur)
	  pCurObjManager->maj_l_obj_design (p);
      }
}
