/*
 Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
 Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

 Contact: polychronycontact@listes.irisa.fr

 This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

 Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License Version 2 as published by the Free
 Software Foundation.

 Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
 for more details.

 You should have received a copy of the GNU General Public License Version 2
 along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
 http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
 Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
 02110-1301, USA.
 */
/// Function called once time at the beginning of the application
extern void
initBiblio (ptGraphicTrees pGArb, ptGraphicTablesManager pTab, ptMessage pMess)
{
  the_messageTool = pMess;
  the_graphicalTrees = pGArb;
  the_graphicTablesManager = pTab;
  libRoot = LL_make_ldecl_emptyList ();
}

/**
 Function called on user "reset" action.
 */
extern void
resetBiblio (void)
{
  libRoot = LL_make_ldecl_emptyList ();
}

/**
 Function called after the loading of a library.
 @param lib : is the process contained in the file.
 @param filename : (absolute) name of the loaded file without its suffix. 
 The icons will be searched in the "filename"_ICO directory.
 @return true if the operation succeeds, "false" otherwise (the loaded file is not a library).
 */
extern boolean
buildLib (char *filename, ntrees lib, char *filenameWithoutSuffix)
{
  float fcor_x=0.0, fcor_y=0.0;

  if (!STree_isModule (lib))
    return pk_false;
  the_graphicalTrees->reconstruire_tab (lib, pk_false, pk_false, fcor_x, fcor_y);
  ntreeList_insertFirst (lib, libRoot);
  // "Java world" is called to build the window with the icons.
  C2Java_createLibBar (filename, lib, filenameWithoutSuffix, repstr2string (STreeModelModule_getNameStr (lib)));
  return pk_true;
}

///
extern boolean
GetCurrentLibSelectedModel (ntrees * proc)
{
  ntrees rootLib = NILTREE;
  int n;

  rootLib = C2Java_getRefLib ();	// Call to Java to know the library which contains the icon
  n = C2Java_getIconIndex ();	// Call to Java to know the index of the icon
  cout << "GetCurrentLibSelectedModel " << rootLib << " nÂ° =  " << n << endl;
  cout << "operateur rootLib  " << noper (rootLib) << endl;
  *proc = STree_getIthModel (rootLib, n);
  return ISNOT_NILTREE (*proc);
}

/**
 Function called when the user has clicked on an icon and then on the canvas 
 to copy the icon in his program.
 */
extern boolean
copyingIcon (ptBuilder pBuilder, int x, int y)
{
  ntrees proc, pInstance;
  typcoord tcoord[2];

  if (!GetCurrentLibSelectedModel (&proc))
    return pk_false;
  // in principle (IS_NILTREE(proc)) impossible
  tcoord[1].x = (C2Java_getLenghtIcon () / 2);	// C2Java_getLenghtIcon Call to Java 
  tcoord[1].y = (C2Java_getHightIcon () / 2);	// C2Java_getHightIcon Call to Java 
  tcoord[0].x = x + tcoord[1].x;
  tcoord[0].y = y + tcoord[1].y;
  if (STreeModel_getCallToIn (STreeModelModule_getName (proc), C2Java_getRefLib (), &pInstance))	// LB : I don't understand !!!
    pBuilder->createPasteBox (the_graphicalTrees->get_arb_boite (pInstance), pk_false, tcoord);
  else
    pBuilder->createExprBox (STree_mkCompleteModelCall (proc), 2, tcoord);
  return pk_true;
}

/**
 Function called by the SignalDisplaying module to know if the called process (pCall) has an associated icon.
 */
extern boolean
findIcon (ntrees pCall)
{
  ptrepstr nameCall;
  int i, j, n, nbList;
  boolean found;
  ntrees pLib, lDecl, m;

  nameCall = nstringof (STree_getNameCalledModel (pCall));
  nbList = ntreearity (libRoot);
  found = pk_false;
  i = 1;
  while (!found && (i <= nbList))
    {
      pLib = nchild (i, libRoot);
      j = 1;
      lDecl = STreeModel_getLocalDeclarations (pLib);
      n = ntreearity (lDecl);
      while (!found && (j <= n))
	{
	  m = nchild (j, lDecl);
	  if (STree_isModel (m))
	    found = (STreeModelModule_getNameStr (m) == nameCall);
	  j++;
	}
      i++;
    }

  return found;
}

/**
 @return 
 - the name of the index-th element of a library (lib), 
 - nilstr, if it does not exist.
 */
extern ptrepstr
getNameSubProc (ntrees lib, int index)
{
  ptrepstr name;
  ntrees subProc;

  name = nilstr;
  if (STree_isModule (lib))
    {
      subProc = STree_getIthModel (lib, index);
      if (ISNOT_NILTREE (subProc))
	name = STreeModelModule_getNameStr (subProc);
    }
  return (name);
}

/**
 It removes a selected library, given by a call to the "java" world.
 \see {C2Java_getRefLib(), C2Java_getSelectedFilenameLibrary()}
 */
extern void
rm_Library ()
{
  char *fn;
  ntrees vt;
  vt = C2Java_getRefLib ();
  ntreesDestroy (vt);
  fn = C2Java_getSelectedFilenameLibrary ();
  C2Java_removeSelectedLib (fn);
}

/**
 It reload a selected library, given by a call to the "java" world.
 \see {C2Java_getRefLib(), C2Java_install_library()}
 
 */
extern void
install_library (ntrees lib, char *fn)
{
  ntrees vt;
  float fcor_x=0.0, fcor_y=0.0;

  if (STree_isModule (lib))
    {
      /* removing + replacing */
      vt = C2Java_getRefLib ();
      ntreesDestroy (vt);
      the_graphicalTrees->reconstruire_tab (lib, pk_false, pk_false, fcor_x, fcor_y);
      ntreeList_insertFirst (lib, libRoot);
      C2Java_install_library (lib, C2Java_getIndexLibrary (fn));
    }
}
