/*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */
///
_PUBLIC_
// Constructor
builder::builder (ptUndo pUn, ptGraphicTrees pGarb, ptGraphicTablesManager pTab, ptCurrentObjectsManager pObj, ptSignalDisplaying pDec, ptDrawing pDrawing, ptMessage pMess):
pUndo (pUn), the_graphicalTrees (pGarb), the_graphicTablesManager (pTab), pCurObjManager (pObj), pDisplay (pDec), pDraw (pDrawing),
the_messageTool (pMess)
{
  pRoutor = new routor ();
}

/**
  Fonction appelee pour reinitialiser certaines variables graphiques
*/
void
builder::nettoyage (void)
{
  nbcoord = 0;
  premiertour = pk_true;
  sens = horiz;
  tours = 1;
}


/**
  Initialisation des variables necessaires aux constructions graphiques
*/
void
builder::init_construct (void)
{
  int i;

  nbcoord = 0;
  tours = 1;
  sens = horiz;
  mask = 1022;			/*utilise pour construire des boites de hauteur-largeur paires */
  arb_map = null;
  for (i = 0; i < TBM; i++)
    map_buffer[i] = 0;
  coord_bte_engl.x = decal_cadre_x;
  coord_bte_engl.y = decal_cadre_y;
  pRoutor->initRoutor (pCurObjManager);
}


/**
  Rend le type de l'objet contenu dans le buffer utilise pour le "paste".
*/
objet_type
builder::typ_buf_paste (void)
{
  return (the_graphClipBoard.typ);
}


/**
  Cette fonction affiche dans la fenetre texte le prochain identificateur
  contenue dans la liste idente (ou idents si idente est vide).
  Ces listes sont construites par la premiere passe du compilateur apres
  que l'utilisateur ait rentre une expression de processus dans la fenetre
  texte. 
  Rend vrai lorsqu'on a fini de valider tous les ports (listes idente et idents vides)
*/
boolean
builder::passer_port_nom (void)
{
  boolean finished;

  if (idente != null || idents != null)
    {
      finished = pk_false;
      if (porte)
	{
	  idente = next_port (idente, &currentPortName);
	  if (idente != null)
	    pDisplay->editPort (currentPortName);
	  else
	    {
	      porte = pk_false;
	      idents = first_port (idents, &currentPortName);
	      if (idents != null)
		pDisplay->editPort (currentPortName);
	      else
		finished = pk_true;
	    }
	}
      else
	{
	  idents = next_port (idents, &currentPortName);
	  if (idents != null)
	    pDisplay->editPort (currentPortName);
	  else
	    finished = pk_true;
	}
    }
  else
    finished = pk_true;
  return (finished);
}


/**
  Associe la declaration d'un signal d'entree ou de sortie a l'objet ?obj repere en 
  (?x_reticule,y_reticule).
  Si obj est une boite, il y a d'abord creation du port;
  Si obj est un port, on associe directement la declaration
  Si obj ne represente rien.
  Ensuite, affiche dans la fenetre texte l'identificateur suivant de la liste 
  idente ou idents. 
  Rend vrai lorsqu'on a fini de valider tous les ports (listes idente et idents vides)
*/
boolean
builder::valider_port_nom (int obj, int x_reticule, int y_reticule, objet_type typ_obj)
{
  boolean isFinished, recom;

  recom = pk_false;
  if (idente != null || idents != null)
    {
      isFinished = pk_false;
      if (obj != 0)
	recom = assoc_decl_obj (obj, bte_a_text, porte, x_reticule, y_reticule, typ_obj);
      if (porte)
	{			/* cas des ports d'entrée */
	  if (!recom)
	    {			/* cas où l'utilisateur a cliqué comme il faut */
	      idente = next_port (idente, &currentPortName);
	      if (idente != null)
		pDisplay->editPort (currentPortName);
	      else
		{
		  porte = pk_false;
		  idents = first_port (idents, &currentPortName);
		  if (idents != null)
		    pDisplay->editPort (currentPortName);
		  else
		    isFinished = pk_true;
		}
	    }
	}
      else
	{			/* cas des ports de sortie */
	  if (!recom)
	    {			/* cas où l'utilisateur a cliqué comme il faut */
	      idents = next_port (idents, &currentPortName);
	      if (idents != null)
		pDisplay->editPort (currentPortName);
	      else
		isFinished = pk_true;
	    }
	}
    }
  else
    isFinished = pk_true;
  return (isFinished);
}


/**
  Change the type of the model t (in). The different types are process, node, action and function.
  ?ope : the new type
*/
void
builder::chang_typ_model (ntrees t, action_type ty_ch)
{
  int ope;

  ope = noper (t);
  switch (ty_ch)
    {
    case C_ch_typ_proc:
      if (ope == opprocess)
	the_messageTool->info_mess (idem_typ_mod);
      else
	nsetop (t, opprocess);
      break;
    case C_ch_typ_func:
      if (ope == opfunc)
	the_messageTool->info_mess (idem_typ_mod);
      else
	nsetop (t, opfunc);
      break;
    case C_ch_typ_node:
      if (ope == opnode)
	the_messageTool->info_mess (idem_typ_mod);
      else
	nsetop (t, opnode);
      break;
    case C_ch_typ_act:
      if (ope == opact)
	the_messageTool->info_mess (idem_typ_mod);
      else
	nsetop (t, opact);
      break;
    default:
      ;
    }
}

/**
  associate the Signal declaration mpp (in) to the port noPort (in).
  The port is the current graphical object (the user has gone down it).
  Method called by the method CONFIRMING.
 */
boolean
builder::setDeclToPort (ntrees mpp, int noPort)
{
  ntrees arb_gra, pAttr;

  arb_gra = the_graphicTablesManager->graphObj_get_tree (noPort);
  if (arb_gra == null)
    return pk_false;
  pAttr = nattrntrees (arb_gra, SIGNAL_ATTRNUMBER);
  pUndo->get_undo_text (pAttr, pk_false, noPort);	/* sauvegarde UNDO */
  the_graphicalTrees->assoc_decl_port (arb_gra, mpp);
  pCurObjManager->remonter (pk_false);
  return pk_true;
}


/**
  Fonction appelee lorsque l'utilisateur est descendu dans l'interface
  d'une boite ?interface et l'a modifiee :
  elle attribue a chaque port de la boite ?bte sa declaration et remplit
  les entrees et les sorties de l'interface du processus
  (si on est au niveau processus).
*/
boolean
builder::set_profil (int bte, ntrees interface)
{
  ntrees p_sig, port, p_bte, arb_cou, proc_cou, expr_proc_cou;
  int nbport;

  p_bte = the_graphicTablesManager->graphObj_get_tree (bte);
  nbport = the_graphicalTrees->get_nb_ports (p_bte);
  if (nbport > 0)
    {
      arb_cou = pCurObjManager->get_arb_cou_engl ();
      proc_cou = pCurObjManager->getModelInWhich_cou_engl ();
      expr_proc_cou = STreeModel_getBody (proc_cou);
      if (arb_cou == expr_proc_cou)
	p_sig = STree_getInputsOutputs (proc_cou);
      else
	SET_NILTREE (p_sig);
      pUndo->get_undo_profil (nfather (p_bte), p_sig);
      port = the_graphicalTrees->getFirstPort (p_bte);
      the_graphicalTrees->accr_decl_ports_bte (port, interface);
      if (arb_cou == expr_proc_cou)
	STree_replaceInputsOutputs (proc_cou, interface);
    }
  return (pk_true);
}



/**
  Inserts the textual Signal model sigSourceModel 
      if setInAtEnd is true sigSourceModel is inserted at the end gpkTargetModeul declarations  
      if setInAtEnd is false sigSourceModel is inserted before gpkTargetModeul declaration; gpkTargetModeul cannot be a module
  gpkTargetModeul is a model or a module
*/
void
builder::insertTextualModel (ntrees sigSourceModel, ntrees gpkTargetModeul, boolean setInAtEnd)
{
  ntrees lDecl;
  boolean isTop;

  (void) the_graphicalTrees->createbte_proc (sigSourceModel);
  sigSourceModel = the_graphicalTrees->conv_text_gra (sigSourceModel);
  addGraphicalAttributes (sigSourceModel);

  if (setInAtEnd)
    {
      lDecl = STreeModel_getLocalDeclarations (gpkTargetModeul);
      ninsertright (ntreearity (lDecl), sigSourceModel, lDecl);
    }				// March 2007: insertTree(lDecl, sigSourceModel, ntreearity(lDecl)+1);
  else
    ntreeList_insertBefore (sigSourceModel, gpkTargetModeul);
  isTop = (gpkTargetModeul == pCurObjManager->get_racine ());
  the_graphicalTrees->create_modelInJTree (sigSourceModel, isTop, setInAtEnd); // creation of the nodes in the hierarchy window

}


/**
  Inserts a textual Signal modul lModule (in) under gpkTargetModeul (in).
  gpkTargetModeul (in) is a model or a module; lModule (in) is a list of modules. 
*/
void
builder::insertTextualModule (ntrees lModule, ntrees gpkTargetModeul)
{
  ntrees lDecl, lDeclModule, p, decl, pModule;
  int nb, i;
  boolean isTop;

  pModule = nchild (1, lModule);
  lDecl = STreeModel_getLocalDeclarations (gpkTargetModeul);
  STree_addDeclarationToList (lDecl, pModule);
  lDeclModule = STreeModel_getLocalDeclarations (pModule);
  nb = ntreearity (lDeclModule);
  for (i = 1; i <= nb; i++)
    {
      decl = nchild (i, lDeclModule);
      if (STree_isModel (decl))
	p = the_graphicalTrees->conv_text_gra (decl);
    }
  addGraphicalAttributes (pModule);
  isTop = (gpkTargetModeul == pCurObjManager->get_racine ());
  the_graphicalTrees->create_modelInJTree (pModule, isTop, pk_true);	// creation of the nodes in the hierarchy window
}


/**
  Inserts the textual Signal model gpkSourceModel 
      if setInAtEnd is true gpkSourceModel is inserted at the end gpkTargetModeul declarations  
      if setInAtEnd is false gpkSourceModel is inserted before gpkTargetModeul declaration; gpkTargetModeul cannot be a module
  gpkTargetModeul is a model or a module
*/
void
builder::insertGraphicalModel (ntrees gpkSourceModel, ntrees gpkTargetModeul, boolean correction, boolean setInAtEnd)
{
  boolean isTop;
  float fcorX, fcorY;

  // printf("DEBUG entering builder::insertGraphicalModel\n"); fflush(stdout);
  if (correction)
    {
      fcorX = 1.3;
      fcorY = 1.;
    }
  else
    {
      fcorX = 0.0;
      fcorY = 0.0;
    }
  if (setInAtEnd)
    {
      STree_addDeclarationToList (STreeModel_getLocalDeclarations (gpkTargetModeul), gpkSourceModel);
    }
  else
    {
      ntreeList_insertBefore (gpkSourceModel, gpkTargetModeul);
    }
  the_graphicalTrees->reconstruire_tab (gpkSourceModel, pk_false, correction, fcorX, fcorY);
  isTop = (gpkTargetModeul == pCurObjManager->get_racine ());
  the_graphicalTrees->create_modelInJTree (gpkSourceModel, isTop, setInAtEnd);	// creating the nodes in the hierarchy window
  // printf("DEBUG  exiting builder::insertGraphicalModel\n"); fflush(stdout);
}


/**
 Inserts a graphical Signal module p (in) in the local declarations of gpkTargetModeul (in).
 gpkTargetModeul is a model or a module. 
 The graphical objects tables are updated for the new objects contained in p.
*/
void
builder::insertGraphicalModule (ntrees p, ntrees gpkTargetModeul)
{
  ntrees lDecl;
  boolean isTop;

  lDecl = STreeModel_getLocalDeclarations (gpkTargetModeul);
  STree_addDeclarationToList (lDecl, p);

  the_graphicalTrees->reconstruire_tab (p, pk_false, pk_false, 0., 0.);

  isTop = (gpkTargetModeul == pCurObjManager->get_racine ());
  the_graphicalTrees->create_modelInJTree (p, isTop, pk_true);	/* pour gérer l'affichage hierarchie processus */
}


/**
  Creation et dessin d'un port d'entree ou de sortie
  ? (x_ret, y_ret) coordonnees du port
  Rend le numero du port crée si on a pu le créer, rend 0 sinon.
*/
int
builder::cr_port (int x_ret, int y_ret)
{
  int bte, ou, rg, no_port;
  typcoord vcoord;
  boolean en_entree;

  // printf("DEBUG entering builder::cr_port\n"); fflush(stdout);
  rg = 0;
  no_port = 0;
  bte = pCurObjManager->reper_bte (&x_ret, &y_ret, &ou, &rg, 0, &en_entree);
  if (act_autorisee (C_creerporte, bte, typ_bte, null))
    {
      vcoord.x = x_ret;
      vcoord.y = y_ret;
      no_port = the_graphicalTrees->creatept_connex (bte, vcoord, en_entree, rg);
      C2Java_refresh (pk_true);
    }
  else
    the_messageTool->error_mess (actimp);
  // printf("DEBUG  exiting builder::cr_port\n"); fflush(stdout);
  return (no_port);
}


/**
   Equi-repartition d'un ensemble de ports entre 2 points cliques par
   l'utilisateur (les 2 points doivent appartenir au meme bord d'une boite).
   ?(x_ret,y_ret) coordonnees du premier point, puis du deuxieme.
   la procedure est appelee 2 fois, la 1iere fois avec ?debut a vrai, la
   seconde avec ?debut a faux.
   Dans le cas ou on justifie des ports externes, on se remet dans le cas
   normal en faisant le changement d'echelle sur les points cliques.
   Cette procedure appelle trans_ext_chemins, et trans_int_chemins
   pour recalculer les chemins des liens externes et internes qui arrivent
   ou partent des ports deplaces.
*/
void
builder::justifier_ports (boolean debut, int x_ret, int y_ret)
{
  int y2_reticule, xbte, bte2, ou2, rg, nbcoord, compt_ports, y_justifier, pas, c;
  ntrees l_ports, s_arb, p, ieme_port, premier_port, arb_bte, pere;
  typcoord mil, tcoord[2], tcoord_rel[2], bougetail;
  boolean no_pb, undo, b;
  boolean justif_ext /* a vrai lorsque justification des ports externes */ ;

  // printf("DEBUG entering builder_g::justifier_ports\n"); fflush(stdout);
  if (debut)
    {
      y1_reticule = y_ret;
      x1_reticule = x_ret;
    }
  else
    {
      justif_ext = pk_false;
      bte = pCurObjManager->reper_bte (&x1_reticule, &y1_reticule, &ou, &rg, 1, &b);
      bte2 = pCurObjManager->reper_bte (&x_ret, &y_ret, &ou2, &rg, 1, &b);
      y2_reticule = y_ret;
      if ((bte == bte2) && (bte != 0) && (ou == ou2))
	{
	  p = the_graphicTablesManager->graphObj_get_tree (bte);
	  pere = nfather (p);
	  if (pere != null)
	    {
	      pUndo->get_undo_bte (C_justif, bte, pk_false);
	      if (p == pCurObjManager->get_bte_cou_engl ())
		justif_ext = pk_true;
	      the_graphicalTrees->getCoord (p, &nbcoord, tcoord, 0);
	      mil = tcoord[0];
	      if (ou == gauche)
		xbte = mil.x - tcoord[1].x;
	      else
		xbte = mil.x + tcoord[1].x;
	      if (y1_reticule > y_ret)
		{
		  y2_reticule = y1_reticule;
		  y1_reticule = y_ret;
		};
	      if ((y2_reticule - y1_reticule) < taillepave_y)
		{		/* si on clique 2 points rapproches, on justifie sur toute la hauteur */
		  y2_reticule = tcoord[0].y + tcoord[1].y;
		  y1_reticule = tcoord[0].y - tcoord[1].y;
		}
	      else if (justif_ext)
		{
		  pCurObjManager->externCoord2Normal (bte, &x_ret, &y1_reticule, ou);
		  pCurObjManager->externCoord2Normal (bte, &x_ret, &y2_reticule, ou);
		}
	      l_ports = nchild (2, pere);
	      if (l_ports != null)
		s_arb = nchild (1, l_ports);
	      if (ou == droite)
		{
		  if (s_arb != null)
		    the_graphicalTrees->getCoord (s_arb, &nbcoord, tcoord_rel, 1);
		  while (s_arb != null && tcoord_rel[0].x < 0)
		    {
		      s_arb = ntrees_getNext (s_arb);
		      if (s_arb != null)
			the_graphicalTrees->getCoord (s_arb, &nbcoord, tcoord_rel, 1);
		    };
		};
	      if (s_arb != null)
		the_graphicalTrees->getCoord (s_arb, &nbcoord, tcoord, 0);
	      while ((tcoord[0].y <= y1_reticule) && (s_arb != null))
		{
		  s_arb = ntrees_getNext (s_arb);
		  if (s_arb != 0)
		    the_graphicalTrees->getCoord (s_arb, &nbcoord, tcoord, 0);
		};
	      compt_ports = 0;
	      premier_port = s_arb;
	      while (tcoord[0].y <= y2_reticule && s_arb != null && tcoord[0].x == xbte)
		{
		  compt_ports++;
		  s_arb = ntrees_getNext (s_arb);
		  if (s_arb != null)
		    the_graphicalTrees->getCoord (s_arb, &nbcoord, tcoord, 0);
		};
	      pas = (int) (((y2_reticule - y1_reticule) - compt_ports * taillepave_y) / (compt_ports + 1));
	      if (pas >= (taillepave_y / 2))
		{
		  y_justifier = y1_reticule + (taillepave_y / 2);
		  c = 0;
		  ieme_port = premier_port;
		  while (c <= compt_ports - 1)
		    {
		      the_graphicalTrees->getCoord (ieme_port, &nbcoord, tcoord, 0);
		      tcoord_ports[c].x = tcoord[0].x;
		      y_justifier = y_justifier + pas;
		      tcoord_ports[c].y = y_justifier;
		      nmodifvalint (nchild (2, nchild (2, ieme_port)), tcoord_ports[c].y - mil.y);
		      y_justifier = y_justifier + taillepave_y;
		      c++;
		      ieme_port = ntrees_getNext (ieme_port);

		    }
		  if (justif_ext)
		    pCurObjManager->remonter (pk_true);
		  positionne_action (rout_mult_port);
		  bougetail.x = 1;
		  bougetail.y = 1;
		  arb_bte = the_graphicTablesManager->graphObj_get_tree (bte);
		  the_graphicalTrees->annuler_halo_seg_ports (compt_ports, premier_port, pk_true, pk_false, &bougetail, pk_true);
		  no_pb = trans_int_chemins (arb_bte, premier_port, compt_ports, tcoord_ports);
		  if (no_pb)
		    {
		      the_graphicalTrees->annuler_halo_seg_ports (compt_ports, premier_port, pk_false, pk_false, &bougetail, pk_true);
		      no_pb = trans_ext_chemins (premier_port, compt_ports, tcoord_ports, pk_false, &bougetail, pk_true);
		    }
		  if (!no_pb)
		    {
		      pUndo->set_undo_bte ();
		      undo = pk_true;
		    }
		}
	    };
	  if (justif_ext)
	    pCurObjManager->descendre (bte, pk_false);
	  else if (undo)
	    pCurObjManager->setCurrentObjectsList ();
	  C2Java_refresh (pk_true);
	}
    }
  // printf("DEBUG  exiting builder_g::justifier_ports\n"); fflush(stdout);
}



/**
  Deplacement des objets : boite, port et segment
  Le routage est relance des qu'on deplace un port ou une boite
   ?etat_bout x_ret y_ret : meme signification que pour la fonction 
   dessiner_bte
  Sauvegarde de l'arbre avant modification (pour le "undo").
*/
void
builder::depl_obj (int etat_bout, int x_ret, int y_ret)
{
  int rg, pas, nbcoord, form_gra;
  typcoord vcoord[2], tcoord[2], bougetail;
  boolean b, no_pb, undo;
  ntrees arb_bte, port_bis;
  float f_ech_x, f_ech_y;

  // printf("DEBUG entering builder_g::depl_obj\n"); fflush(stdout);
  switch (etat_bout)
    {
    case appuyer:
      obj_desig = pCurObjManager->reper_obj (x_ret, y_ret, &typ_obj);
      if (obj_desig != 0)
	{ 
	  if (act_autorisee (C_move, obj_desig, typ_obj, null))
	    {
	      tours++;
	      switch (typ_obj)
		{
		case typ_bte:
		  pppppppppp = the_graphicTablesManager->graphObj_get_tree (obj_desig);
		  form_gra = the_graphicalTrees->getCoord (pppppppppp, &nbcoord, tcoord, 0);
		  mil.x = tcoord[0].x;
		  mil.y = tcoord[0].y;
		  larg = tcoord[1].x;
		  haute = tcoord[1].y;
		  x_pred = x_ret;
		  y_pred = y_ret;
		  break;
		case typ_seg:
		  pppppppppp = the_graphicTablesManager->acces_segment (obj_desig);
		  form_gra = the_graphicalTrees->getCoord (nchild (2, pppppppppp), &nbcoord, tcoord, 0);
		  x_deb = tcoord[0].x;
		  y_deb = tcoord[0].y;
		  x_fin = tcoord[1].x;
		  y_fin = tcoord[1].y;
		  if (x_deb == x_fin)
		    sens = vertic;
		  else
		    sens = horiz;
		  break;
		case typ_port_ext:
		case typ_port:
		  port = the_graphicTablesManager->graphObj_get_tree (obj_desig);
		  bte = pCurObjManager->reper_bte (&x_ret, &y_ret, &ou, &rg, 1, &b);
		  arb_bte = the_graphicTablesManager->graphObj_get_tree (bte);
		  the_graphicalTrees->getCoord (arb_bte, &nbcoord, tcoord, 0);
		  mil = tcoord[0];
		  larg = tcoord[1].x;
		  y_min = tcoord[0].y - tcoord[1].y + taillepave_y;
		  y_max = tcoord[0].y + tcoord[1].y - taillepave_y;
		  y_pred = y_ret;
		  if (typ_obj == typ_port)
		    {
		      the_graphicalTrees->getCoord (port, &nbcoord, tcoord, 0);
		      tcoord_ports[0].x = tcoord[0].x;
		      tcoord_ports[0].y = tcoord[0].y;
		    }
		  else
		    {
		      calcul_fact_ech (arb_bte, &f_ech_x, &f_ech_y);
		      y_max = (int) ((y_max - mil.y) * f_ech_y) + ord_mil_cadre;
		      y_min = (int) ((y_min - mil.y) * f_ech_y) + ord_mil_cadre;
		      the_graphicalTrees->getCoord (port, &nbcoord, tcoord, 1);
		      tcoord_ports[0].x = (int) (tcoord[0].x * f_ech_x) + abs_mil_cadre;
		      tcoord_ports[0].y = (int) (tcoord[0].y * f_ech_y) + ord_mil_cadre;
		      y_pred = y_ret = tcoord_ports[0].y;
		    }
		  break;
		default:
		  ;
		}
	    }
	  else
	    obj_desig = 0;
	}

      break;
    case tirer:
      if (obj_desig != 0)
	{
	  switch (typ_obj)
	    {
	    case typ_bte:
	      if (tours != 2)
		pDraw->dessiner_rect (mil, larg, haute, COL_EFFACE);
	      mil.x = mil.x + (x_ret & mask) - (x_pred & mask);
	      mil.y = mil.y + (y_ret & mask) - (y_pred & mask);
	      pDraw->dessiner_rect (mil, larg, haute, COL_DEF);
	      x_pred = x_ret;
	      y_pred = y_ret;
	      break;
	    case typ_seg:
	      if (tours != 2)
		pDraw->dessiner_seg (x_deb, y_deb, x_fin, y_fin, COL_EFFACE);
	      if (sens == horiz)
		y_deb = y_fin = y_ret;
	      else
		x_deb = x_fin = x_ret;
	      pDraw->dessiner_seg (x_deb, y_deb, x_fin, y_fin, COL_DEF);
	      break;
	    case typ_port_ext:
	    case typ_port:
	      pas = y_ret - y_pred;
	      if ((tcoord_ports[0].y + pas >= y_min) && (tcoord_ports[0].y + pas <= y_max))
		{
		  pDisplay->display_port_depl (port, typ_obj, tcoord_ports, ou, COL_EFFACE);
		  tcoord_ports[0].y = tcoord_ports[0].y + pas;
		  pDisplay->display_port_depl (port, typ_obj, tcoord_ports, ou, COL_DEF);
		  y_pred = y_ret;
		}
	      /* else  cas ou on change de bord de la boite : non termine 
	         {
	         pDisplay->->afficher_port_depl(port, typ_obj, tcoord_ports, ou, COL_EFFACE);
	         if (ou == gauche) { tcoord_ports[0].x = mil.x + larg; ou = droite; } else { tcoord_ports[0].x = mil.x - larg; ou = gauche; }
	         if (tcoord_ports[0].y + pas < y_min) { y_pred = y_min; tcoord_ports[0].y = y_pred + taillepave_y; }
	         else { y_pred = y_max; tcoord_ports[0].y = y_pred - taillepave_y; }
	         pDisplay->display_port_depl(port, typ_obj, tcoord_ports, ou, COL_DEF);
	         } 
	      */
	    default:
	      ;
	    }
	}
      tours++;
      break;
    case relacher:
      if (obj_desig != 0)
	{
	  undo = pk_false;
	  switch (typ_obj)
	    {
	    case typ_bte:
	      positionne_action (rout_bte);
	      vcoord[0].x = mil.x;
	      vcoord[0].y = mil.y;
	      vcoord[1].x = larg;
	      vcoord[1].y = haute;
	      bougetail.x = 1;
	      bougetail.y = 1;
	      pUndo->get_undo_bte (C_move, obj_desig, pk_false);
	      if (!test_chang_coord (obj_desig, typ_obj, 2, vcoord, pk_true, pk_false, &bougetail))
		{
		  pUndo->set_undo_bte ();
		  undo = pk_true;
		}
	      else
		undo = pk_false;
	      break;
	    case typ_seg:
	      bougetail.x = 1;
	      bougetail.y = 1;
	      vcoord[0].x = x_deb;
	      vcoord[0].y = y_deb;
	      vcoord[1].x = x_fin;
	      vcoord[1].y = y_fin;
	      port = the_graphicalTrees->getMovedPorts (obj_desig, &port_bis);
	      if (port != null)	/* rend NILTREE pour un segment appartenant à un lien multiple */
		{
		  bte = the_graphicalTrees->no_boite_mere (port);
		  pUndo->get_undo_bte (C_move, bte, pk_false);
		  if (!test_chang_coord (obj_desig, typ_obj, 2, vcoord, pk_true, pk_false, &bougetail))
		    {
		      pUndo->set_undo_bte ();
		      undo = pk_true;
		    }
		}
	      else
		the_messageTool->error_mess (linkmoveimp);
	      break;
	    case typ_port_ext:
	    case typ_port:
	      if (typ_obj == typ_port_ext)
		{
		  pCurObjManager->externCoord2Normal (bte, &tcoord_ports[0].x, &tcoord_ports[0].y, ou);
		  pCurObjManager->remonter (pk_true);
		}
	      if (deplac_port (obj_desig, &tcoord_ports[0].x, &tcoord_ports[0].y, &rg))
		{
		  bougetail.x = 1;
		  bougetail.y = 1;
		  positionne_action (rout_port);
		  arb_bte = the_graphicTablesManager->graphObj_get_tree (bte);
		  pUndo->get_undo_bte (C_move, bte, pk_false);
		  the_graphicalTrees->annuler_halo_seg_ports (1, port, pk_true, pk_false, &bougetail, pk_true);
		  no_pb = trans_int_chemins (arb_bte, port, 1, tcoord_ports);
		  if (no_pb)
		    {
		      the_graphicalTrees->annuler_halo_seg_ports (1, port, pk_false, pk_false, &bougetail, pk_true);
		      no_pb = trans_ext_chemins (port, 1, tcoord_ports, pk_false, &bougetail, pk_true);
		      if (no_pb)
			modif_coord (obj_desig, typ_port, vcoord, 1, tcoord_ports, rg);
		      /* le parametre vcoord n'a pas importance quand il s'agit d'un port */
		    }
		  if (!no_pb)
		    {
		      pUndo->set_undo_bte ();
		      undo = pk_true;
		    }
		}
	    default:
	      ;
	    }
	  if (typ_obj == typ_port_ext)
	    pCurObjManager->descendre (bte, pk_false);
	  else if (undo)
	    pCurObjManager->maj_l_obj_design (pCurObjManager->get_arb_cou_engl ());
	  C2Java_refresh (pk_true);
	  tours = 1;
	  obj_desig = 0;
	}
    }
}


/**
  Destruction d'un objet = effacement dans la fenetre, destruction de 
                 l'arbre et destruction dans la tables d'acces.
  ?(x_reticule,y_reticule) : point designe par l'utilisateur.
   si l'objet est un lien, detruit le lien.
   si l'objet est un port :
       detruit le port et les liens qui partent ou arrivent sur ce port.
   si l'objet est une boite :
       detruit la boite, ses ports, ses liens internes, recursivement ses
       boites internes, ainsi que les liens externes avec d'autres boites.
  sauvegarde l'arbre avant modification (pour le undo).
*/
void
builder::detruire_obj (int x_reticule, int y_reticule)
{
  int obj;
  objet_type typ_obj;
  ntrees p_gra, p_sig, chemin, rac_port, p_hyperRoot;

  obj = pCurObjManager->reper_obj (x_reticule, y_reticule, &typ_obj);
  if (obj != 0)
    {
      if (act_autorisee (C_cut_graph, obj, typ_obj, null))
	switch (typ_obj)
	  {
	  case typ_bte:
	    p_gra = the_graphicTablesManager->graphObj_get_tree (obj);
	    p_hyperRoot = nhyperRoot (p_gra);
	    // nDEBUG_treeHyperRoot(0, 20 ,"in builder::detruire_obj bte", p_gra);
	    p_sig = the_graphicalTrees->get_arb_sig (&p_gra);
	    the_graphClipBoard.arb = the_graphicalTrees->sauver_arbre (p_sig, pk_false);
	    the_graphClipBoard.typ = typ_bte;
	    the_graphClipBoard.opproc = STree_isModel (p_sig);
	    pUndo->get_undo_bte (C_cut_graph, obj, pk_false);
	    detruire_bte (obj, pk_false);	// false: destroy not only detach
	    break;
	  case typ_port_ext:
	  case typ_port:
	    p_gra = the_graphicTablesManager->graphObj_get_tree (obj);
	    p_hyperRoot = nhyperRoot (p_gra);
	    // nDEBUG_treeHyperRoot(0, 20 ,"in builder::detruire_obj port", p_gra);
	    the_graphClipBoard.arb = the_graphicalTrees->sauver_arbre (p_gra, pk_false);
	    the_graphClipBoard.typ = typ_obj;
	    the_graphClipBoard.opproc = pk_true;
	    pUndo->get_undo_bte (C_cut_graph, the_graphicalTrees->no_boite_mere (p_gra), pk_false);
	    detruire_port (obj, typ_obj);
	    break;
	  case typ_seg:
	    p_gra = the_graphicTablesManager->acces_segment (obj);
	    p_hyperRoot = nhyperRoot (p_gra);
	    // nDEBUG_treeHyperRoot(0, 20 ,"in builder::detruire_obj segment", p_gra);
	    chemin = the_graphicalTrees->get_chemin (p_gra);
	    rac_port = the_graphicalTrees->get_port_diff (chemin);
	    pUndo->get_undo_bte (C_cut_graph, the_graphicalTrees->no_boite_mere (rac_port), pk_false);
	    detruire_chemin (chemin, pk_false);
	    break;
	  default:;
	  }
      C2Java_refresh (pk_true);
      // nDEBUG_treeHyperRoot(0, 50 ," exiting builder::detruire_obj segment", p_hyperRoot);
    }
}

/**
  Cette fonction selectionne ou deselectionne l'objet designe par l'
  utilisateur, c'est-a-dire qu'elle met ou enleve le numero de l'objet
  dans le buffer selec_buff.
  Si l'objet est deja dans le buffer ?select_buff, alors il est 
  deselectionne (on l'enleve du buffer), s'il n'y est pas, alors on
  l'insere.
  t_buff (in) : size of the buffer
*/
void
builder::select_obj (int select_buff[], int t_buff, int x_ret, int y_ret)
{
  ntrees p_obj;
  int obj, nbcoord;
  typcoord tcoord[2], milieu;

  obj = pCurObjManager->reper_obj (x_ret, y_ret, &typ_obj);
  if ((obj != 0) && (typ_obj == typ_bte))
    {
      p_obj = the_graphicTablesManager->graphObj_get_tree (obj);
      the_graphicalTrees->getCoord (p_obj, &nbcoord, tcoord, 0);
      milieu.x = tcoord[0].x;
      milieu.y = tcoord[0].y;
      if (inserer_buff (select_buff, t_buff, obj))
	{			// la boite est selectionnee */
	  pDraw->dessiner_select (milieu, tcoord[1].x, tcoord[1].y, COL_SEL);
	}
      else
	{			// la boite est déselectionnée
	  pDraw->dessiner_select (milieu, tcoord[1].x, tcoord[1].y, COL_EFFACE);
	  pDisplay->display_bte_avec_ports (p_obj, COL_DEF);
	}
    }
}


/**
  Fonction appelee lorsque l'utilisateur clique sur un icone de la
  fenetre bibliotheque. Elle met a jour le buffer pour le "paste".
*/
// void builder::set_tampon_paste(typ_tampon *tampon_biblio)
// {
// }


/**
  Insertion de l'objet (boite ou port) contenue dans le buffer "the_graphClipBoard"
  a l'endroit designe par l'utilisateur (?x_ret, ?y_ret). Le buffer
  the_graphClipBoard est rempli soit par un "copy", soit par un "cut"
  ?etat_bout : etat du bouton de la souris, soit appuyer (1ier clique
               de l'utilisateur), soit tirer (cliques suivants pour 
	       dessiner la boite car on peut changer la taille de la
	       boite dupliquee), soit relacher (fin de l'action).
*/
void
builder::paste_obj (int etat_bout, int x_ret, int y_ret)
{
  boolean dupl_autorisee;
  int ope;
  ntrees p;
  int l, h, obj_ou_dupl, nbcoord;
  typcoord halo[2], mem_coord[2];
  ntrees arb_ou_dupl, bte_dupl, p_attr, p_attr2;

  typ_obj_dupl = the_graphClipBoard.typ;
  switch (etat_bout)
    {
    case appuyer:
      switch (typ_obj_dupl)
	{
	case typ_bte:
	  bte_dupl = the_graphicalTrees->get_arb_boite (the_graphClipBoard.arb);
	  p = pCurObjManager->get_arb_cou_engl ();
	  if (ISNOT_NILTREE (p))
	    {
	      ope = noper (p);
	      dupl_autorisee = ((the_graphClipBoard.opproc && STree_isLocalDeclList (p)) || (!(the_graphClipBoard.opproc) && STree_isExprProc (ope)));
	    }
	  else
	    dupl_autorisee = pk_false;
	  if (dupl_autorisee)
	    {
	      the_graphicalTrees->getCoord (bte_dupl, &nbcoord, mem_coord, 0);
	      larg = mem_coord[1].x;
	      haute = mem_coord[1].y;
	      x_coin_g = x_ret & mask;
	      y_coin_g = y_ret & mask;
	      x_pred = x_coin_g + 2 * larg;
	      y_pred = y_coin_g + 2 * haute;
	      pDraw->dessiner_rect2 (x_coin_g, y_coin_g, x_pred, y_pred, COL_DEF);
	      x_stop = x_pred;
	      y_stop = y_pred;
	      tours++;
	    }
	  else
	    the_messageTool->error_mess (actimp);
	  break;
	case typ_port:
	case typ_port_ext:
	  obj_ou_dupl = pCurObjManager->reper_obj (x_ret, y_ret, &typ_obj1);
	  if (act_autorisee (C_copy_port, obj_ou_dupl, typ_obj1, null))
	    {
	      p_attr = nattrntrees (the_graphClipBoard.arb, SIGNAL_ATTRNUMBER);
	      if (p_attr != null)
		p_attr = COPY_ATTRIBUTEDSTREE (p_attr);
	      switch (typ_obj1)
		{
		case typ_port:
		case typ_port_ext:
		  arb_ou_dupl = the_graphicTablesManager->graphObj_get_tree (obj_ou_dupl);
		  p_attr2 = nattrntrees (arb_ou_dupl, SIGNAL_ATTRNUMBER);
		  pUndo->get_undo_text (p_attr2, pk_false, obj_ou_dupl);	/* pour le UNDO */
		  if (p_attr != null)
		    the_graphicalTrees->assoc_decl_port (arb_ou_dupl, p_attr);
		  else if (p_attr2 != null)
		    ndestroyattr (arb_ou_dupl, SIGNAL_ATTRNUMBER);
		  break;
		case typ_bte:
		case typ_cadre:
		  cr_port (x_ret, y_ret);
		  arb_ou_dupl = the_graphicTablesManager->graphObj_get_tree (pCurObjManager->reper_obj (x_ret, y_ret, &typ_obj1));
		  if (p_attr != null)
		    the_graphicalTrees->assoc_decl_port (arb_ou_dupl, p_attr);
		  break;
		default:
		  ;
		}
	    }
	  else
	    the_messageTool->error_mess (actimp);
	  break;
	default:
	  the_messageTool->error_mess (actimp);
	  break;
	}
      break;
    case tirer:
      if (typ_obj_dupl == typ_bte)
	dessiner_tirer_souris (x_ret, y_ret);
      break;
    case relacher:
      if (tours > 1)
	switch (typ_obj_dupl)
	  {
	  case typ_bte:
	    bte_dupl = the_graphicalTrees->get_arb_boite (the_graphClipBoard.arb);
	    if (!ecart_port (bte_dupl, mask, (x_coin_g + x_pred) / 2, (y_coin_g + y_pred) / 2, (y_pred - y_coin_g) / 2, pk_true))
	      {
		tours = 2;
		x_pred = x_stop;
		y_pred = y_stop;
	      }
	    else
	      {
		x_pred = x_ret & mask;
		y_pred = y_ret & mask;
	      }
	    if (tours == 2)	/* pas de changement de taille */
	      {
		l = larg;
		h = haute;
		modtail = pk_false;
	      }
	    else
	      {
		modtail = pk_true;
		l = (x_pred - x_coin_g) / 2;
		h = (y_pred - y_coin_g) / 2;
		getSizeMin (&l, &h);	// minimun size for a box
	      }
	    tcoord[0].x = x_coin_g + l;
	    tcoord[0].y = y_coin_g + h;
	    tcoord[1].x = abs (l);
	    tcoord[1].y = abs (h);
	    the_graphicalTrees->calcul_nouv_halo (typ_bte, tcoord, halo);
	    if (pCurObjManager->test_cr_bte (halo))
	      createPasteBox (bte_dupl, modtail, tcoord);
	    else
	      the_messageTool->error_mess (pastebteimp);
	    break;
	  default:
	    ;
	  }
      tours = 1;
      C2Java_refresh (pk_true);
    }
}


/**
  Copie d'un objet dans le buffer "the_graphClipBoard" (en vue de faire un "paste" 
  ensuite).
  ?x_ret,y_ret point designe par l'utilsateur.
*/
void
builder::copy_obj (int x_ret, int y_ret)
{
  graphObj_type obj_copie;
  ntrees p, p_sig;

  // printf("DEBUG entering builder::copy_obj\n"); fflush(stdout);
  obj_copie = pCurObjManager->reper_obj (x_ret & mask, y_ret & mask, &the_graphClipBoard.typ);
  switch (the_graphClipBoard.typ)
    {
    case typ_bte:
      if ((p = nfather (the_graphicTablesManager->graphObj_get_tree (obj_copie))) != null)
	p_sig = nattrcarrier (p);
      else
	p_sig = nattrcarrier (the_graphicTablesManager->graphObj_get_tree (obj_copie));
      the_graphClipBoard.opproc = STree_isModel (p_sig);
      the_graphClipBoard.arb = the_graphicalTrees->sauver_arbre (p_sig, pk_true);
      break;
    default:
      the_graphClipBoard.opproc = pk_false;
      the_graphClipBoard.arb = the_graphicTablesManager->graphObj_get_tree (obj_copie);
    }
  // printf("DEBUG  exiting builder::copy_obj\n"); fflush(stdout);
}


/**
 Visualisation des halos des objets graphiques
  ?force a vrai on visualise aussi les objets dependant de  l'objet 
  designe (?obj)
  ?typ_obj : type de l'objet
*/
void
builder::visualiser_halo (boolean force, int obj, objet_type typ_obj)
{
  ntrees l_ports, arb_port, p;
  boolean trouve;

  switch (typ_obj)
    {
    case typ_bte:
      trouve = pk_false;
      afficher_halo_bte (obj);
      if (force)
	if ((l_ports = ntrees_getNext (the_graphicTablesManager->graphObj_get_tree (obj))) != null)
	  {
	    arb_port = nchild (1, l_ports);
	    while (arb_port != null)
	      {
		afficher_halo_port (arb_port, typ_port);
		arb_port = ntrees_getNext (arb_port);
	      }
	  }
      break;
    case typ_seg:
      if (force)
	{
	  p = the_graphicalTrees->get_chemin (the_graphicTablesManager->acces_segment (obj));
	  afficher_halo_lien (p);
	  afficher_halo_chem (p);
	}
      else
	afficher_halo_seg (obj);
      break;
    case typ_port:
    case typ_port_ext:
      p = the_graphicTablesManager->graphObj_get_tree (obj);
      pDisplay->display_obj (p, COL_HALO);
      if (force)
	afficher_halo_port (p, typ_obj);
      break;
    default:
      ;
    }
}


/**
  search and replace in p (in) every identificator ident1 (in) by ident2 (in)
  p is a module or a model or the Working area.
  The name of p is not changed.
*/
boolean
builder::ReplaceIdent (ntrees p, ptident id1, ptident id2, int select_buff[])
{
  ntrees lDecl;
  int i, n;
  boolean isFinished;

  isFinished = pk_false;
  if ((p == pCurObjManager->get_racine ()) || STree_isModule (p))
    {
      lDecl = STreeModel_getLocalDeclarations (p);
      n = ntreearity (lDecl);
      isFinished = pk_false;
      i = 1;
      while ((i <= n) && !isFinished)
	{
	  isFinished = ReplaceIdent (nchild (i, lDecl), id1, id2, select_buff);
	  i++;
	}
    }
  else 
    if (STree_isModel (p))
      {
	pCurObjManager->descendre (the_graphicalTrees->no_boite_proc (p), pk_false);
	C2Java_refresh (pk_true);
	searchIdentAndReplace (p, pk_false, id1, id2, select_buff);
      }
  return (isFinished);
}
