/*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */

///
_PRIVATE_
#define COPY_ATTRIBUTEDSTREE(ATREE) ncopytree(ATREE, pk_true)
#define OPIS_SIGNALIDENTIFIER(voperator) ( (voperator == opnom) || (voperator == opident) )

typ_tampon builder::the_graphClipBoard ={NILTREE, typ_vide, pk_false};


/*
  Fonction appelee par builder qui positionne la valeur de raction a
  soit rout_mult_port (action justifier), soit rout_tail_bte (changement de
  taille de boite), soit rout_bte (deplacement boite), soit rout_port 
  (deplacement port).
*/
void
builder::positionne_action (int action)
{
  raction = action;
}


/**
   Transformation de coordonnees absolues (par rapport au point gaut-gauche
   de la fenetre graphique en coordonnees relatives a la boite englobante.
*/
void
builder::transfcoord (int nbcoord, typcoord * tcoord, typcoord * nouvtcoord)
{
  int i;

  for (i = 0; i < nbcoord; i++)
    {
      nouvtcoord[i].x = tcoord[i].x - coord_bte_engl.x;
      nouvtcoord[i].y = tcoord[i].y - coord_bte_engl.y;
    }
}


/**
  * cette fonction verifie que l'action effectuee sur un objet
  * graphique par l'utilisateur est autorisee ( vis-a-vis du type
  * de cet objet).
  * Le parametre "objet" n'est pas toujours necessaire (par exemple
  * pour la creation de boites, la verification se fait par rapport
  * au type de la boite englobante).
  */
boolean
builder::act_autorisee (action_type action, int objet, objet_type typ_obj, ntrees p_inser)
{
  boolean result;
  int ope, ope_inser;
  ntrees p_gra, p, p_sig, pere, currentTree;

  // printf("DEBUG entering builder::act_autorisee\n"); fflush(stdout);
  result = pk_true;
  currentTree = pCurObjManager->get_arb_cou_engl ();
  switch (action)
    {
    case C_creerporte:
    case C_creerports:
    case C_copy_port:
      if (objet != 0)
	{			/* objet = bte où on cree ou duplique un port */
	  p_gra = the_graphicTablesManager->graphObj_get_tree (objet);
	  if (IS_NILTREE (nfather (p_gra)))
	    {
	      p_sig = nattrcarrier (p_gra);
	      ope = noper (p_sig);
	      if (OPIS_SIGNALIDENTIFIER (ope))
		result = !STree_isNameOfModel (p_sig);
	      else
		result = STree_isExprProc (ope);
	    }
	}
      else
	result = pk_false;
      break;
    case C_creerproc:
    case C_creerfunc:
    case C_creernode:
    case C_creeraction:
    case C_creerbteexpr:
      break;
    case C_copy:
    case C_copy_graph:
      p_gra = the_graphicTablesManager->graphObj_get_tree (objet);
      if (ISNOT_NILTREE (currentTree))
	{
	  ope = noper (currentTree);
	  p = nfather (p_gra);
	  if (ISNOT_NILTREE (p))
	    p_sig = nattrcarrier (p);
	  else
	    p_sig = nattrcarrier (p_gra);
	  ope_inser = noper (p_sig);
	  result = ((ope == opldecl && STree_isModel (p_sig)) || (STree_isExprProc (ope) && STree_isExprProc (ope_inser)));
	}
      else
	result = pk_false;
      break;
    case C_cut:
    case C_cut_graph:
      if (typ_obj == typ_bte)
	{
	  p_gra = the_graphicTablesManager->graphObj_get_tree (objet);
	  p_sig = the_graphicalTrees->get_arb_sig (&p_gra);
	  if (STree_isNameOfModel (p_sig) || STree_isParametersOfModel (p_sig) || STree_isBodyTree (p_sig) || STree_isSpec (p_sig))
	    result = pk_false;
	}
      break;
    case C_move:
    case C_resize:
      break;
    case C_encaps:
      result = (ISNOT_NILTREE (currentTree) && ntreearity (currentTree) != 0 && !STree_isModel (currentTree) && noper (currentTree) != opldecl);
      break;
    case C_decaps:
      result = (typ_obj == typ_bte && ISNOT_NILTREE (currentTree) && !STree_isModel (currentTree) && noper (currentTree) != opldecl);
      break;
    case C_text_win:
      if (IS_NILTREE (p_inser))
	{			/* si l'arbre courant est NILTREE ou si on est au niveau d'un modèle, on n'a pas
				   le droit insérer du texte */
	  if (IS_NILTREE (currentTree))
	    result = pk_false;
	  else
	    result = !STree_isModel (currentTree);
	}
      else
	{
	  pere = nfather (currentTree);
	  if (ISNOT_NILTREE (pere))
	    ope = noper (pere);
	  ope_inser = noper (p_inser);
	  switch (ope)
	    {
	    case opldecl:
	      result = STree_isModel (p_inser);
	      break;
	    case opprocess:
	    case opfunc:
	    case opnode:
	    case opprocessp:
	    case opfuncp:
	    case opnodep:
	    case opact:
	    case opactp:
	    case opoper:
	      result = (OPIS_SIGNALIDENTIFIER (ope_inser));	// (ope_inser == opnom);
	      break;
	    case opinte:
	      if (ope_inser != opldecs && ope_inser != opnil)
		result = pk_false;
	      break;
	    default:
	      result = STree_isExprProc (ope_inser);
	    }
	}
    default:;
    }
  // printf("DEBUG  exiting builder::act_autorisee\n"); fflush(stdout);
  return (result);
}


/**
  Calcule le facteur d'echelle de la boite ?p_bte lorsqu'on descend dedans.
  Permet (par exemple) de calculer les positions des ports lorsqu'ils
  sont externes.
*/
void
builder::calcul_fact_ech (ntrees p_bte, float *x, float *y)
{
  typcoord tcoord[2];
  int nbcoord;
  float val_reel;

  the_graphicalTrees->getCoord (p_bte, &nbcoord, tcoord, 0);
  val_reel = (float) tcoord[1].x;
  *x = lg_bte_stand / val_reel;
  val_reel = (float) tcoord[1].y;
  *y = ht_bte_stand / val_reel;
}

/**
  Cette procedure modifie les coordonnees (dans l'arbre) et le halo (dans la
  table d'acces) d'un objet ?obj, a la suite du deplacement de cet objet.
   ?typ_obj : type de l'objet
   ?halo : nouveau halo
   ?nbcoord et tcoord : nouvelles coordonnees
   Le parametre "rg" n'est utilise que lorsque l'objet est un port.
*/
void
builder::modif_coord (int obj, int typ_obj, typcoord * halo, int nbcoord, typcoord * tcoord, int rg)
{
  acces_gra *pt1;
  acces_seg *pt2;
  int form;
  ntrees pCoord, p, p_port, pere;
  typcoord vcoord[2];

  switch (typ_obj)
    {
    case typ_bte:
      pt1 = the_graphicTablesManager->getBoxTableElem (obj);
      p = pt1->elem_gra;
      the_graphicTablesManager->setBoxElemWithHalo (pt1, halo);
      transfcoord (1, tcoord, tcoord);
      p = nchild (1, nchild (2, p));
      the_graphicalTrees->setCoord (p, tcoord[0].x, tcoord[0].y);
      break;
    case typ_port:
      form = the_graphicalTrees->getCoord (the_graphicalTrees->boite_mere (obj), &nbcoord, vcoord, 0);
      tcoord[0].y = tcoord[0].y - vcoord[0].y;
      tcoord[0].x = tcoord[0].x - vcoord[0].x;
      p_port = the_graphicTablesManager->graphObj_get_tree (obj);
      p = nchild (2, p_port);
      the_graphicalTrees->setCoord (p, tcoord[0].x, tcoord[0].y);
      /* les ports etant ordonnes, on reinsere le port au bon endroit */
      pere = nfather (p_port);
      ntreesDestroy (p_port);
      if (rg != 0)
	ninsertright (rg, p_port, pere);
      else
	ntreeList_insertFirst (p_port, pere);
      break;
    case typ_seg:
      pt2 = the_graphicTablesManager->getSegTableElem (obj);
      p = pt2->arb_seg;
      the_graphicTablesManager->setSegElemWithHalo (pt2, halo);
      transfcoord (2, tcoord, tcoord);
      pCoord = nchild (2, p);
      the_graphicalTrees->setCoord (nchild (1, pCoord), tcoord[0].x, tcoord[0].y);
      the_graphicalTrees->setCoord (nchild (2, pCoord), tcoord[1].x, tcoord[1].y);
    }
}



/**
  Cette fonction verifie que le deplacement de l'objet est possible
   par rapport aux halos des autres objets, et si oui modifie la
   structure de donnees en consequence.
   les parametres sont fonction de ceux de depl_seg
   ?p_bougetail : utilises lors d'un changement de taille 
   pour ne pas deplacer les chemins des ports qui ne bougent pas
   et represente les cotes deplaces 
   pour x: -1 cote gauche, 1 cote droit et 0 x inchange.
   pour y: -1 cote haut, 1 cote bas et 0 y inchange.
   ?force a vrai si en changement de taille les ports sont deplaces
   lors d'un deplacement sans changement de taille :
    p_bougetail(1,1) et force(vrai)
*/
boolean
  builder::test_chang_coord (int obj, objet_type typ_obj, int nbcoord, typcoord * tcoord, boolean force, boolean reception, typcoord * p_bougetail)
{
  int obj_prec, obj_succ, diff;
  boolean result, intern, prec, chem_simple	/* booleen a vrai si le segment appartient a un
						   chemin simple */ ;
  ntrees pseg, pseg_prec, pseg_succ, arb_gra;
  typcoord halo[2], ocoord[2], vcoord[2], vcoord_prec[2], vcoord_succ[2], bougetail;

  bougetail = *p_bougetail;
  switch (typ_obj)
    {
    case typ_bte:
      arb_gra = the_graphicTablesManager->graphObj_get_tree (obj);
      the_graphicalTrees->getCoord (the_graphicTablesManager->graphObj_get_tree (obj), &nbcoord, ocoord, 0);
      intern = pk_false;
      force = pk_true;
      the_graphicalTrees->annuler_halo_seg_bte (obj, intern, pk_false, &bougetail, force);
      pCurObjManager->enlever_l_bte_d (obj);
      the_graphicalTrees->calcul_nouv_halo (typ_bte, tcoord, halo);
      result = pCurObjManager->test_cr_bte (halo);
      if (result)
	{
	  modif_coord (obj, typ_bte, halo, nbcoord, tcoord, 0);
	  if (ocoord[1].x != tcoord[1].x || ocoord[1].y != tcoord[1].y)
	    the_graphicalTrees->modif_port_bte (arb_gra, ocoord, tcoord, force);
	  pCurObjManager->inser_l_bte_d (obj);
	  if (!force && (ocoord[1].x == tcoord[1].x) && (ocoord[1].y != tcoord[1].y))
	    {
	      the_graphicalTrees->modif_coord_btes_non_prop (arb_gra, ocoord, tcoord);
	      result = pk_true;
	    }
	  else
	    result = depl_bte_chemins (obj, &bougetail, force);
	}
      else
	{
	  the_graphicalTrees->annuler_halo_seg_bte (obj, intern, pk_true, &bougetail, force);
	  pCurObjManager->inser_l_bte_d (obj);
	  if (bougetail.x == 1 && bougetail.y == 1)
	    the_messageTool->error_mess (deplimphalo);
	  else
	    the_messageTool->error_mess (tailimphalo);
	  result = pk_false;
	}
      break;

    case typ_seg:
      pseg = the_graphicTablesManager->acces_segment (obj);
      chem_simple = the_graphicalTrees->chemin_simple (pseg);
      the_graphicalTrees->getCoord (nchild (2, pseg), &nbcoord, vcoord, 0);
      prec = reception;
      if ((pseg_prec = ntrees_getPrevious (pseg)) != null)
	{
	  obj_prec = the_graphicalTrees->tree_get_graphObj (pseg_prec);
	  the_graphicalTrees->getCoord (nchild (2, pseg_prec), &nbcoord, vcoord_prec, 0);
	  prec = pk_true;
	}
      else
	obj_prec = 0;
      if ((pseg_succ = ntrees_getNext (pseg)) != null)
	{
	  obj_succ = the_graphicalTrees->tree_get_graphObj (pseg_succ);
	  the_graphicalTrees->getCoord (nchild (2, pseg_succ), &nbcoord, vcoord_succ, 0);
	  prec = pk_false;
	};
      if (tcoord[0].x == tcoord[1].x)
	{
	  diff = vcoord[0].x - tcoord[0].x;
	  vcoord_prec[1].x = vcoord_prec[1].x - diff;
	  vcoord_succ[0].x = vcoord_succ[0].x - diff;
	}
      else
	{
	  diff = vcoord[0].y - tcoord[0].y;
	  if (pseg_prec != null)
	    vcoord_prec[1].y = vcoord_prec[1].y - diff;
	  if (pseg_succ != null)
	    vcoord_succ[0].y = vcoord_succ[0].y - diff;
	};
      if (pseg_succ == null && chem_simple)
	result = depl_seg (obj, tcoord, obj_prec, vcoord_prec, 0, vcoord_succ, prec, p_bougetail);
      else if (pseg_prec == null && chem_simple)
	result = depl_seg (obj, tcoord, obj_succ, vcoord_succ, 0, vcoord_prec, prec, p_bougetail);

      else if ((chem_simple) || (!chem_simple && pseg_succ != null && pseg_prec != null))
	result = depl_seg (obj, tcoord, obj_prec, vcoord_prec, obj_succ, vcoord_succ, pk_true, p_bougetail);
      else
	result = depl_seg_chem_mult (obj, tcoord);
      break;
    default:
      ;
    }
  return (result);
}


 /**
   Return the first child of the liste ?l_ident and its name in !identName
   if the list is not empty, otherwise return NILTREE.
  */
ntrees
builder::first_port (ntrees l_ident, ptrepstr * identName)
{
  ntrees ident;

  ident = null;
  if (l_ident != null)
    {
      ident = nchild (1, l_ident);
      if (ident != null)
	*identName = ident2repstr (ident);
    }
  return (ident);
}


 /**
  * Rend le premier frere droit de l'identificateur (?ident)
  * et la chaine de caractere representant l'identificateur (!chident)
  * rend null si le frere n'existe pas
  */
ntrees
builder::next_port (ntrees ident, ptrepstr * identName)
{
  if (ident != null)
    {
      ident = ntrees_getNext (ident);
      if (ident != null)
	*identName = ident2repstr (ident);
    }
  return (ident);
}


/**
  Cette fonction associe une declaration de signal a un port. Cette declaration
  est construite a partir de l'identificateur currentPortName.
  isInput a vrai, il s'agit d'un port d'entree
  Si ?typ_obj est une boite : creation d'un port appartenant a la boite ?bte
                              a la position ?x,y avec sa declaration
                 un port   : attribution de sa declaration
  Cette fonction est appelee par valider_port_nom lorsque l'utilisateur, 
  apres avoir associe une expression de processus a une boite, clique dans 
  la boite pour nommer les ports.
  rend faux si tout est ok, rend true s'il faut recommencer pour la meme 
  declaration, (l'utilisateur s'est trompe, par exemple, a clique sur un port 
  d'entree alors qu'on lui demandait de cliquer sur un port de sortie).
*/
boolean
builder::assoc_decl_obj (int obj, int bte, boolean isInput, int x, int y, objet_type typ_obj)
{
  boolean recom, b;
  ntrees sig, pDecl;;
  int no_pt, ou, rg, bord;
  typcoord vcoord;

  // printf("DEBUG entering builder::assoc_decl_obj\n"); fflush(stdout);
  recom = pk_false;
  pDecl = STree_mkImplicitSignalDeclaration (currentPortName);
  switch (typ_obj)
    {
    case typ_bte:
    case typ_cadre:
      if (obj == bte)
	{
	  rg = 0;
	  bord = pCurObjManager->reper_bte (&x, &y, &ou, &rg, 0, &b);
	  vcoord.x = x;
	  vcoord.y = y;
	  no_pt = the_graphicalTrees->creatept_connex (bte, vcoord, isInput, rg);
	  the_graphicalTrees->assoc_decl_port (the_graphicTablesManager->graphObj_get_tree (no_pt), pDecl);
	}
      else
	{
	  the_messageTool->error_mess (cliqbte);
	  recom = pk_true;
	}
      break;
    case typ_port:
    case typ_port_ext:
      if (bte == the_graphicalTrees->tree_get_graphObj (the_graphicalTrees->boite_mere (obj)))
	{
	  sig = the_graphicTablesManager->graphObj_get_tree (obj);
	  if ((isInput && (noper (sig) == gpkOp_porte)) || (!isInput && (noper (sig) == gpkOp_ports)))
	    the_graphicalTrees->assoc_decl_port (sig, pDecl);
	  else
	    {
	      if (isInput)
		the_messageTool->error_mess (cliqpte);
	      else
		the_messageTool->error_mess (cliqpts);
	      recom = pk_true;
	    }
	}
      else
	{
	  the_messageTool->error_mess (cliqbte);
	  recom = pk_true;
	}
      break;
    default:
      ;
    }
  if (!recom)
    C2Java_refresh (pk_true);
  return (recom);
  // printf("DEBUG  exiting builder::assoc_decl_obj\n"); fflush(stdout);
}


/**
   Permet a l'utilisateur de nommer les ports d'une boite apres lui avoir 
   associe du texte .
   initialise les listes d'identificateurs idents et  idente (variables 
   globales au module) et affiche le premier identificateur non vide
   ?bte :  boite a laquelle on associe du texte.
   ?l_idente,l_idents: liste d'identificateurs construites par la premiere
   passe du compilateur qui contiennent respectivement les entrees et les
   sorties de l'expression associee a la boite
*/
void
builder::cliquer_port_nom (int bte, ntrees l_idente, ntrees l_idents)
{
  idents = l_idents;
  idente = first_port (l_idente, &currentPortName);
  bte_a_text = bte;
  if (idente != null)
    {
      pDisplay->editPort (currentPortName);
      porte = pk_true;
    }
  else
    {
      idents = first_port (l_idents, &currentPortName);
      if (idents != null)
	{
	  pDisplay->editPort (currentPortName);
	  porte = pk_false;
	}
    }
  if (bte != the_graphicalTrees->no_boite_proc (pCurObjManager->getModelInWhich_cou_engl ()))
    visualiser_halo (pk_false, bte, typ_bte);
}


/**
  Recursively creates graphical boxes and ports for all the models contained in 
  pMod (in) and for pMod if pMod is a model
  if pMod is a modul, creates a box associated with pMod; the box number is only
  necessary in the Java world.
  Method called when the user inserts a textual Signal program.
*/
void
builder::addGraphicalAttributes (ntrees pMod)
{
  ntrees bte_corp, lDecl, p_fils;
  int i, nb_loc;

  if (STree_isModel (pMod))
    {
      bte_corp = nattrntrees (STreeModel_getBody (pMod), GRAPHI_ATTRNUMBER);
      if (noper (bte_corp) == opschema)
	bte_corp = nchild (1, bte_corp);
      mkInterfacePorts (pMod, bte_corp);
    }
  else if (STree_isModule (pMod))
    {				// creation of a box for the module : use only for Java world
      setDefaultCoord (tcoord);
      the_graphicalTrees->createbte (pMod, tcoord, 2);
    }
  lDecl = STreeModel_getLocalDeclarations (pMod);
  if (lDecl != null && noper (lDecl) != opnil)
    {
      nb_loc = ntreearity (lDecl);
      i = 1;
      while (i <= nb_loc)
	{
	  p_fils = nchild (i, lDecl);
	  if (STree_isModel (p_fils) && !nattrexist (p_fils, GRAPHI_ATTRNUMBER))
	    {
	      the_graphicalTrees->createbte_proc (p_fils);
	      addGraphicalAttributes (p_fils);
	    }
	  i++;
	}
    }
}


/**
 Cette procedure affiche le halo d'une boite dans la fenetre graphique
*/
void
builder::afficher_halo_bte (int bte)
{
  typcoord halo[2];
  ty_halo *pt;

  pt = the_graphicTablesManager->recup_bte_halo (bte);
  halo[0].x = (*pt)[0].x;
  halo[0].y = (*pt)[0].y;
  halo[1].x = (*pt)[1].x;
  halo[1].y = (*pt)[1].y;
  pDraw->dessiner_halo (halo);
}


/**
 Cette procedure affiche le halo d'un segment dans la fenetre graphique
 ?pt : l'element de la table qui contient le segment.
*/
void
builder::afficher_halo_seg (int seg)
{
  typcoord halo[2];
  ty_halo *pt;

  pt = the_graphicTablesManager->recup_segment_halo (seg);
  halo[0].x = (*pt)[0].x;
  halo[0].y = (*pt)[0].y;
  halo[1].x = (*pt)[1].x;
  halo[1].y = (*pt)[1].y;
  pDraw->dessiner_halo (halo);
}


/**
  Cette procedure affiche le halo d'un lien dans la fenetre graphique
  ?lien : arbre representant le sous-chemin.
  On remonte jusqu'au port pour dessiner tout le lien.
*/
void
builder::afficher_halo_lien (ntrees lien)
{
  ntrees p;

  while (noper (lien) != gpkOp_ports && noper (lien) != gpkOp_porte)
    {
      if ((p = nchild (4, lien)) != null)
	{
	  p = nchild (1, p);
	  while (p != null)
	    {
	      afficher_halo_seg (the_graphicalTrees->tree_get_graphObj (p));
	      p = ntrees_getNext (p);
	    }
	}
      lien = nfather (lien);
    }
}


/**
 Cette procedure affiche le halo d'un chemin dans la fenetre graphique
  ?chem : arbre representant le chemin.
*/
void
builder::afficher_halo_chem (ntrees chem)
{
  ntrees p;

  if ((p = nchild (4, chem)) != null)
    {
      p = nchild (1, p);
      while (p != null)
	{
	  afficher_halo_seg (the_graphicalTrees->tree_get_graphObj (p));
	  p = ntrees_getNext (p);
	}
    }
  if ((chem = nchild (3, chem)) != null)
    {
      chem = nchild (1, chem);
      while (chem != null)
	{
	  afficher_halo_chem (chem);
	  chem = ntrees_getNext (chem);
	}
    }
}


/**
 Cette procedure affiche le halo d'un port dans la fenetre graphique
 (c'est-a-dire le halo des liens qui arrivent ou partent de ce port).
  ?arb_port : arbre representant le port.
  ?typ_obj : vaut "typ_port_ext" pour les ports de la boite englobante,
                  "typ_port" pour les autres.
*/
void
builder::afficher_halo_port (ntrees arb_port, objet_type typ_obj)
{
  int ope;
  ntrees p, lien;

  ope = noper (arb_port);
  if ((ope == gpkOp_porte && typ_obj == typ_port) || (ope == gpkOp_ports && typ_obj == typ_port_ext))
    {
      if ((p = nchild (4, arb_port)) != null)
	{
	  lien = the_graphicTablesManager->graphObj_get_tree ((int) nintof (p));
	  afficher_halo_lien (lien);
	}
    }
  else if ((p = nchild (3, arb_port)) != null)
    afficher_halo_chem (p);
}


/**
 Cette fonction cree automatiquement le port d'arrivee lorsqu'on est
  en creation de lien et que l'on clique la boite d'arrivee.
  rend faux si elle ne peut pas creer le port.
  ?x_ret, ?y_ret : point clique par l'utlisateur
  !typ_obj : rend le type de port cree (port ou port externe)
*/
boolean
builder::createport_auto (int x_ret, int y_ret, objet_type * typ_obj)
{
  int x, ope, nbcoord;
  typcoord tcoord[2];

  ope = noper (the_graphicTablesManager->graphObj_get_tree (pt_dep));
  switch (*typ_obj)
    {
    case typ_cadre:
      if (typ_obj1 == typ_port_ext)	/* autorisation des connexions port externe/port externe */
	if (x_ret < abs_mil_cadre)
	  if (ope == gpkOp_porte)
	    x = decal_cadre_x - 1;
	  else
	    x = decal_cadre_x + 1;
	else if (ope == gpkOp_porte)
	  x = abs_mil_cadre + lg_bte_stand + 1;
	else
	  x = abs_mil_cadre + lg_bte_stand - 1;
      else if (x_ret < abs_mil_cadre)
	if (ope == gpkOp_porte)
	  x = decal_cadre_x + 1;
	else
	  x = decal_cadre_x - 1;
      else if (ope == gpkOp_porte)
	x = abs_mil_cadre + lg_bte_stand - 1;
      else
	x = abs_mil_cadre + lg_bte_stand + 1;
      *typ_obj = typ_port_ext;
      break;

    case typ_bte:
      the_graphicalTrees->getCoord (the_graphicTablesManager->graphObj_get_tree (pt_arr), &nbcoord, tcoord, 0);
      if (x_ret < tcoord[0].x)
	if ((ope == gpkOp_porte && typ_obj1 == typ_port) || (ope == gpkOp_ports && typ_obj1 == typ_port_ext))
	  x = tcoord[0].x - tcoord[1].x - 1;
	else
	  x = tcoord[0].x - tcoord[1].x + 1;
      else if ((ope == gpkOp_porte && typ_obj1 == typ_port) || (ope == gpkOp_ports && typ_obj1 == typ_port_ext))
	x = tcoord[0].x + tcoord[1].x + 1;
      else
	x = tcoord[0].x + tcoord[1].x - 1;
      *typ_obj = typ_port;
      break;
    default:
      ;
    }
  pt_arr = cr_port (x, y_ret);
  return (pt_arr != 0);
}


/**
  returns in width (in/out) and height (in/out) the minimum width and height
  for a box. If width(height) is larger than the minimun width(height),
  it doesn't change.
 */
void
getSizeMin (int *width, int *height)
{
  if (abs (*width) < 2 * taillepave_x)
    {
      if (*width > 0)
	*width = 2 * taillepave_x;
      else
	*width = -2 * taillepave_x;
    };
  if (abs (*height) < 2 * taillepave_y)
    {
      if (*height > 0)
	*height = 2 * taillepave_y;
      else
	*height = -2 * taillepave_y;
    }
}


/**
  Fonction locale a cr_lien : dessine les segments intermediaires
  lorsqu'on clique plusieurs points entre 2 ports (routage manuel)
  met a jour des variables globales au module (tcoord, x_pred, y_pred...)
  ?x_reticule, y_reticule coordonnees cliquees par l'utilisateur
*/
void
builder::dessiner_seg_intermediaire (int x_reticule, int y_reticule)
{
  int l, pt;

  if (x_reticule < x_pred + alpha && x_reticule > x_pred - alpha)
    {
      if (depart)
	{
	  if (sens_dep == gauche)
	    if ((x_pred - x_reticule) < halo_segm)
	      l = -halo_segm;
	    else
	      l = x_reticule - x_pred;
	  else if ((x_reticule - x_pred) < halo_segm)
	    l = halo_segm;
	  else
	    l = x_reticule - x_pred;
	  pDraw->dessiner_seg (x_pred, y_pred, x_pred + l, y_pred, COL_DEF);
	  x_pred = x_pred + l;
	  tcoord[nbcoord].x = x_pred;
	  tcoord[nbcoord].y = y_pred;
	  nbcoord++;
	  depart = pk_false;
	}
      if (sens == vertic)
	{
	  pDraw->dessiner_seg (x_pred, y_pred, x_pred, y_reticule, COL_DEF);
	  y_pred = y_reticule;
	  tcoord[nbcoord - 1].y = y_pred;
	}
      else
	{
	  if (y_reticule > y_pred + alpha || y_reticule < y_pred - alpha)
	    {
	      pDraw->dessiner_seg (x_pred, y_pred, x_pred, y_reticule, COL_DEF);
	      y_pred = y_reticule;
	      tcoord[nbcoord].x = x_pred;
	      tcoord[nbcoord].y = y_pred;
	      nbcoord++;
	      sens = vertic;
	    }
	}
    }
  else if (y_reticule < y_pred + alpha && y_reticule > y_pred - alpha)
    {
      depart = pk_false;
      pDraw->dessiner_seg (x_pred, y_pred, x_reticule, y_pred, COL_DEF);
      x_pred = x_reticule;
      if ((nbcoord >= 2) && (y_pred == tcoord[nbcoord - 2].y))
	tcoord[nbcoord - 1].x = x_pred;
      else
	{
	  tcoord[nbcoord].x = x_pred;
	  tcoord[nbcoord].y = y_pred;
	  nbcoord++;
	  sens = horiz;
	}
    }
  else
    {
      depart = pk_false;
      if (sens == vertic)
	{
	  sens = horiz;
	  pt = tcoord[nbcoord - 2].y;
	  if ((y_reticule < y_pred && y_reticule > pt) || (y_reticule > y_pred && y_reticule < pt))
	    pDraw->dessiner_seg (x_pred, y_pred, x_pred, y_reticule, COL_EFFACE);
	  else
	    pDraw->dessiner_seg (x_pred, y_pred, x_pred, y_reticule, COL_DEF);
	  tcoord[nbcoord - 1].y = y_reticule;
	  pDraw->dessiner_seg (x_pred, y_reticule, x_reticule, y_reticule, COL_DEF);
	}
      else
	{
	  sens = vertic;
	  if (nbcoord >= 2)
	    {
	      pt = tcoord[nbcoord - 2].x;
	      if ((x_reticule < x_pred && x_reticule > pt) || (x_reticule > x_pred && x_reticule < pt))
		pDraw->dessiner_seg (x_pred, y_pred, x_reticule, y_pred, COL_DEF);
	      else
		pDraw->dessiner_seg (x_pred, y_pred, x_reticule, y_pred, COL_DEF);
	      tcoord[nbcoord - 1].x = x_reticule;
	      pDraw->dessiner_seg (x_reticule, y_pred, x_reticule, y_reticule, COL_DEF);
	    }
	  else
	    {
	      pDraw->dessiner_seg (x_pred, y_pred, x_reticule, y_pred, COL_DEF);
	      tcoord[nbcoord].x = x_reticule;
	      tcoord[nbcoord].y = y_pred;
	      nbcoord++;
	      pDraw->dessiner_seg (x_reticule, y_pred, x_reticule, y_reticule, COL_DEF);
	    }
	};
      tcoord[nbcoord].x = x_reticule;
      tcoord[nbcoord].y = y_reticule;
      nbcoord++;
      x_pred = x_reticule;
      y_pred = y_reticule;
    }
}


/**
  Fonction locale a cr_lien : dessine le dernier segment d'un lien
  met a jour la variable globale au module : tcoord
    ?x_reticule, y_reticule coordonnees cliquees par l'utilisateur
*/
void
builder::dessiner_dern_seg (int x_reticule, int y_reticule)
{
  if (x_pred == tcoord[nbcoord - 2].x)
    {
      pDraw->dessiner_seg (tcoord[nbcoord - 2].x, tcoord[nbcoord - 2].y, x_pred, y_pred, COL_EFFACE);
      pDraw->dessiner_seg (tcoord[nbcoord - 2].x, tcoord[nbcoord - 2].y, x_pred, y_reticule, COL_DEF);
      tcoord[nbcoord - 1].y = y_reticule;
    }
  else
    {
      pDraw->dessiner_seg (x_pred, y_pred, x_pred, y_reticule, COL_DEF);
      tcoord[nbcoord].x = x_pred;
      tcoord[nbcoord].y = y_reticule;
      nbcoord++;
    };
  pDraw->dessiner_seg (x_pred, y_reticule, x_reticule, y_reticule, COL_DEF);
  tcoord[nbcoord].x = x_reticule;
  tcoord[nbcoord].y = y_reticule;
}


/**
  Fonction utilisee pour afficher un rectangle lorsqu'on tire avec la
  souris. Elle est appelee pour la creation, le paste, l'encapsulage
  et l'insertion de processus.
  ?x_ret et ?y_ret : position du reticule
  utilise les variables globales au module x_pred, y_pred (position
  precedente du reticule) et x_coin_g, y_coin_g : point de depart
  de la boite.
*/
void
builder::dessiner_tirer_souris (int x_ret, int y_ret)
{
  if (tours > 1)
    {
      pDraw->dessiner_rect2 (x_coin_g, y_coin_g, x_pred, y_pred, COL_EFFACE);
      x_pred = x_ret & mask;
      y_pred = y_ret & mask;
      pDraw->dessiner_rect2 (x_coin_g, y_coin_g, x_pred, y_pred, COL_DEF);
      tours++;
    }
}



/**
 Cette procedure est appelee lorsque l'utilisateur a associe du texte
 ?arb_sig a une boite ?p_bte (fonction associer_text dans builder_g.c)
 et si le texte est un appel de processus.
 Si le processus est declare,
 alors les ports de la boite appel sont crees automatiquement (avec
 leur nom) a partir des ports de la declaration.
 S'il n'a pas assez de place pour creer tous les ports, un message est 
 envoye a l'utilisateur. Les ports d'entree sont crees a gauche de la 
 boite et les ports de sortie a droite.
*/
void
builder::gere_appel_proc (ntrees arb_sig, ntrees p_bte)
{
  ntrees l_proc_loc, decl_proc, nom_proc, rac, proc_cou;
  boolean trouve;

  if (noper (p_bte) == gpkOp_bte)
    /* on ne va chercher la declaration du processus que si la boite n'a pas deja des ports */
    {
      nom_proc = STree_getNameCalledModel (arb_sig);
      proc_cou = pCurObjManager->getModelInWhich_cou_engl ();
      rac = pCurObjManager->get_racine ();
      l_proc_loc = STreeModel_getLocalDeclarations (proc_cou);
      trouve = the_graphicalTrees->isModelInList (nom_proc, l_proc_loc, &decl_proc);
      if (!trouve)
	decl_proc = the_graphicalTrees->chercher_decl_proc (nom_proc, proc_cou, rac);
      if (decl_proc != null)
	createports_bte_appel (p_bte, decl_proc);
    }
}


/**
  Insere dans le buffer ?select_buff l'objet ?obj s'il n'y est pas deja
  et rend VRAI.
  Si l'objet se trouve dans le buffer alors il l'enleve en retassant le
  buffer et rend FAUX.
  Remarque : si select_buf[i] = 0, alors, a partir de i, il n'y a plus 
  d'objet.
*/
boolean
builder::inserer_buff (int select_buff[], int t_buf, int obj)
{
  int i, j;
  boolean pas_trouve;

  pas_trouve = pk_true;
  i = 0;
  while (pas_trouve && (select_buff[i] != 0))
    {
      if (select_buff[i] == obj)
	{
	  pas_trouve = pk_false;
	  for (j = i; j < t_buf - 1; j++)
	    select_buff[j] = select_buff[j + 1];
	}
      else
	i++;
    }
  if (pas_trouve)
    select_buff[i] = obj;
  return (pas_trouve);
}



/**   LES FONCTIONS QUI SUIVENT SONT A CHANGER DE PLACE */
ptrepstr
builder::construire_nom (const char *nom, int no)
{
  char buf_aux[20];

  sprintf (buf_aux, "%s%d", nom, no);
  return (string2repstr (buf_aux));
}


/**
  Cree et rend un corps de processus qui contient ?p_expr
*/
ntrees
builder::createcorps (ntrees p_expr)
{
  ntrees p_loc;

  p_loc = LL_make_ldecl_emptyList ();
  return (LL_make_desc_tree (STree_mkEmptyTree (), LL_make_pconf_tree (p_expr, p_loc)));
}


/**
  FONCTION APPELEE POUR LA REPARTITION DE PROGRAMMES (bouton "set archi")

  Construit les objets graphiques associes au modele de processus ?p_proc :
     - creation des 4 boites (nom, param, expr, declarations locales)
     -  creation des ports (autant que de ports de la boite ?p_bte qui
      va etre incluse dans l'expression de ?p_proc)
     - creation des liens (un lien entre un port cree et un port de ?p_bte
     ?num : rang du processus (utilise uniquement pour construire la boite
            dans la liste des sous-processus).
*/
void
builder::build_proc_gra (ntrees p_proc, ntrees p_bte, int num)
{
  typcoord t_port, old_coord, old_coord_rel;
  int no_bte_expr, rg, no_port, no_nouv_port, nbcoord, x1, y1;
  boolean en_entree;
  ntrees p_port, p_decl, p_nouv_port;

  // printf("DEBUG entering builder::build_proc_gra\n"); fflush(stdout);
  the_graphicalTrees->createbte_proc (p_proc);
  no_bte_expr = the_graphicalTrees->no_boite_proc (p_proc);
  p_port = the_graphicalTrees->getFirstPort (p_bte);
  rg = 0;
  while (p_port != null)
    {				/* on cree les ports correspondants aux ports de la boite incluse */
      the_graphicalTrees->getCoord (p_port, &nbcoord, &old_coord_rel, 1);
      the_graphicalTrees->getCoord (p_port, &nbcoord, &old_coord, 0);
      if (old_coord_rel.x < 0)	/* on garde les mêmes coordonnées */
	ou = gauche;
      else
	ou = droite;
      y1 = old_coord.y;
      pCurObjManager->externCoord2Normal (no_bte_expr, &x1, &y1, ou);
      t_port.x = x1;
      t_port.y = y1;
      no_port = the_graphicalTrees->tree_get_graphObj (p_port);
      en_entree = (noper (p_port) == gpkOp_porte);
      no_nouv_port = the_graphicalTrees->creatept_connex (no_bte_expr, t_port, en_entree, rg);
      rg++;
      p_decl = nattrntrees (p_port, SIGNAL_ATTRNUMBER);
      p_nouv_port = the_graphicTablesManager->graphObj_get_tree (no_nouv_port);
      if (p_decl != null)
	the_graphicalTrees->assoc_decl_port (p_nouv_port, COPY_ATTRIBUTEDSTREE (p_decl));
      /* on cree les liens (directement, sans lancer le routage) */
      if (en_entree)
	createlien_direct (p_nouv_port, p_port, typ_port_ext, typ_port);
      else
	createlien_direct (p_port, p_nouv_port, typ_port, typ_port_ext);
      p_port = ntrees_getNext (p_port);
    }
  // printf("DEBUG  exiting builder::build_proc_gra\n"); fflush(stdout);
}


/**
  Fonction utilisee pour la repartion de programmes (l'utilisateur
  a clique sur le bouton "SET ARCHI").
  Construit le pragma "RunOn no" et l'associe a l'arbre ?p_proc
*/
void
builder::build_prag_repart (ntrees p_proc, int no)
{
  ntrees prag_repart, prag_proc, nom, enonce;
  char buf_aux[20];

  sprintf (buf_aux, "\"%d\"", no);
  nom = LL_make_nom_atom (getPredefinedPragma (PRAGMA_RUN_ON));
  enonce = LL_make_chai_list ((LL_make_cha_atom (string2repstr (buf_aux))));
  prag_repart = LL_make_prag_tree (nom, STree_mkEmptyTree (), enonce);
  prag_proc = STree_getPragmas (p_proc);
  if (noper (prag_proc) == opnil)
    STree_setPragmas (p_proc, LL_make_lprag_list (prag_repart));
  else
    ntreeList_appendLast (prag_proc, prag_repart);
}


/**
  Fonction utilisee pour la repartion de programmes (l'utilisateur
  a clique sur le bouton "SET ARCHI"). Elle est appelee par la 
  fonction "traiter_repartition".
  Elle recherche le sous-processus de ?p qui a le pragma "RunOn"
  avec ?no comme numero.
  En principe, il y en a forcemment un.
*/
ntrees
builder::get_modele_same_prag (ntrees p, ptrepstr no)
{
  ntrees l_decl, p_decl, lprag, prag;
  int i, nb;
  boolean trouve;

  trouve = pk_false;
  l_decl = STreeModel_getLocalDeclarations (p);
  if ((ISNOT_NILTREE (l_decl)) && (noper (l_decl) != opnil))
    {
      nb = ntreearity (l_decl);
      i = 1;
      while (!trouve && (i <= nb))
	{
	  p_decl = nchild (i, l_decl);
	  if (STree_isModel (p_decl))
	    if (ISNOT_NILTREE ((lprag = STree_getPragmas (p_decl))))
	      if (noper (lprag) != opnil)
		{
		  prag = nchild (1, lprag);
		  while ((ISNOT_NILTREE (prag)) && !trouve)
		    {
		      if (strcmp (repstr2string (nstringof (nchild (1, prag))), "RUNOn"))
			trouve = (nstringof (nchild (1, nchild (3, prag))) == no);
		      prag = ntrees_getNext (prag);
		    }
		}
	  i++;
	}
    }
  if (trouve)
    return (p_decl);
  return (NILTREE);
}

/**
  Fonction appelée par la fonction "construire_arbre_compil":
  Elle crée les boites associées aux différents s/arbres de
  ?p_corps qui est l'expression de processus rendue par le compilo.
  (permettra de visualiser le résultat de la compilation).
  ATTENTION:
  Suppose qu'il y a au plus 3 boîtes internes dans p_corps.
    - une pour les synchros, (obligatoire)
    - une pour les calculs (obligatoire)
    - et une pour les dépendances (optionnelle)

    Il faut donc vérifier la cohérence avec la méthode qui construit cet arbre: produceSubgraph() .
*/
void
builder::construire_btes_internes_compilees (ntrees p_corps)
{
  ntrees p_synchro, p_calcul, p_dependances;
  int h_inter_bte;
  typcoord tcoord[2];

  /* on récupère les différents s/arbres de l'arbre compilé */
  p_synchro = nchild (1, p_corps);
  p_calcul = nchild (2, p_corps);
  if (ntreearity (p_corps) == 3)
    p_dependances = nchild (3, p_corps);
  else
    SET_NILTREE (p_dependances);
  h_inter_bte = 20;
  tcoord[0].x = abs_mil_cadre - (decal_cadre_x / 2);
  tcoord[1].x = lg_bte_exp - decal_cadre_x;
  if (ISNOT_NILTREE (p_dependances))
    {
      tcoord[1].y = ht_bte_exp / 3;
      tcoord[0].y = mil_bte_exp_y / 3;
    }
  else
    {
      tcoord[1].y = ht_bte_exp / 2;
      tcoord[0].y = mil_bte_exp_y / 2;
    }
  the_graphicalTrees->createbte (p_synchro, tcoord, 2);
  tcoord[0].y = tcoord[0].y + 2 * tcoord[1].y + h_inter_bte;
  the_graphicalTrees->createbte (p_calcul, tcoord, 2);
  if (ISNOT_NILTREE (p_dependances))
    {
      tcoord[0].y = tcoord[0].y + 2 * tcoord[1].y + h_inter_bte;
      the_graphicalTrees->createbte (p_dependances, tcoord, 2);
    }
}


/**
  Fonction appelée par la fonction "construire_arbre_compil" (utilisée lors
  de la compilation d'une architecture).
  ?p_compil est l'arbre processus résultant de la compilation et 
  ?p_appel est l'arbre contenant l'appel à 1 des processeurs.
  On vérifie s'il existe des nouveaux signaux crées par le compilo (booléens
  liés aux horloges) et si oui, on les rajoute à la boîte correspondant à
  ?p_appel.
*/
void
builder::add_new_ports (ntrees p_compil, ntrees p_appel)
{
  ntrees l_entrees, l_sorties, bte_appel, p_signal, l_ports, p_decl;

  bte_appel = nattrntrees (p_appel, GRAPHI_ATTRNUMBER);
  l_entrees = STree_getInputs (p_compil);
  p_signal = the_graphicalTrees->first_list_dec (l_entrees, &p_decl);
  l_ports = nchild (2, bte_appel);
  while (ISNOT_NILTREE (p_signal))
    {
      if (IS_NILTREE (the_graphicalTrees->getPortWithDecl (p_decl, bte_appel, pk_true)))
	the_graphicalTrees->add_signal_proc_appe (p_appel, p_decl, pk_true);
      p_signal = the_graphicalTrees->next_list_dec (p_signal, &p_decl);
    }
  l_sorties = STree_getOutputs (p_compil);
  p_signal = the_graphicalTrees->first_list_dec (l_sorties, &p_decl);
  while (ISNOT_NILTREE (p_signal))
    {
      if (IS_NILTREE (the_graphicalTrees->getPortWithDecl (p_decl, bte_appel, pk_false)))
	the_graphicalTrees->add_signal_proc_appe (p_appel, p_decl, pk_false);
      p_signal = the_graphicalTrees->next_list_dec (p_signal, &p_decl);
    }
}


typChoice
builder::displayAndGetUserChoice (ntrees * currentP, ptident id2, boolean inAPort, int select_buff[])
{
  typChoice choice;
  ntrees father, pPort;
  int pos, noObj;

  pppppppppp = *currentP;
  if (inAPort)
    {
      pPort = the_graphicalTrees->get_arb_sig (&pppppppppp);
      noObj = the_graphicalTrees->tree_get_graphObj (pPort);
      pppppppppp = the_graphicalTrees->get_arb_sig (&pPort);
    }
  else
    noObj = the_graphicalTrees->getMotherBox (pppppppppp);
// we go down in the process to display it
  pCurObjManager->descendre (the_graphicalTrees->no_boite_proc (STree_getModelInWhich (pppppppppp)), pk_true);
  select_buff[0] = noObj;
  C2Java_refresh (pk_false);
  choice = C2Java_getUserChoice ();
  switch (choice)
    {
    case OK:
      father = nfather (*currentP);
      pos = nposit (*currentP);
      // *currentP = LL_make_nom_atom (id2);
      printf("DEBUG replacing ptident\n"); fflush(stdout);
      *currentP = copy_ident(id2);
      nchangchild (father, *currentP, pos);
      C2Java_refresh (pk_true);
      break;
    case CANCEL:
    case SKIP:
      break;
    }
  return (choice);
}


/**
   search and replace in the ports of a box (pBox) (in) every identificator id1 (in) by id2 (in).
   @return true is the operation succeeds, false if the user cancels the operation.
 */
boolean
builder::searchIdentAndReplaceInPorts (ntrees pBox, ptident id1, ptident id2, int select_buff[])
{
  boolean userCancel;
  ntrees pPort, pDecl;

  userCancel = pk_false;
  pPort = the_graphicalTrees->getFirstPort (pBox);
  while ((ISNOT_NILTREE (pPort)) && !userCancel)
    {
      pDecl = nattrntrees (pPort, SIGNAL_ATTRNUMBER);
      if (ISNOT_NILTREE (pDecl))
	userCancel = searchIdentAndReplace (pDecl, pk_true, id1, id2, select_buff);
      pPort = ntrees_getNext (pPort);
    }
  return (userCancel);
}


/**
   search and replace in the model p (in) every identificator ident1 (in) by ident2 (in) except the model name.
*/
boolean
builder::searchIdentAndReplace (ntrees p, boolean inAPort, ptident id1, ptident id2, int select_buff[])
{
  int ope, pos;
  boolean finished, userCancel;
  ntrees currentP, pSav, pBox;

  finished = pk_false;
  userCancel = pk_false;
  currentP = p;
  pos = 1;
  while (!finished)
    {
      if (STree_isModel (currentP)) currentP = nchild (2, currentP);	// we skip the model name
      pBox = the_graphicalTrees->get_arb_boite (currentP);
      if (ISNOT_NILTREE(pBox)) searchIdentAndReplaceInPorts (pBox, id1, id2, select_buff);
      pSav = currentP;
      ope = noper (currentP);
      if (OPIS_SIGNALIDENTIFIER(ope))
	{
	  if (eq_idents(currentP, id1)) userCancel = (displayAndGetUserChoice (&currentP, id2, inAPort, select_buff) == CANCEL);
	  pSav = currentP;
	  currentP = ntrees_getNext (currentP);
	  pos++;
	}
      else
	{
	  currentP = nchild (1, currentP);
	  pos = 1;
	}
      finished = userCancel;
      if ((IS_NILTREE (currentP)) && (pos < ntreearity (pSav)))
	{			/* in some case, we have a NILTREE and it has a brother, not normal! */
	  pos++;
	  currentP = nchild (pos, pSav);
	}
      while ((IS_NILTREE (currentP)) && !finished)
	{
	  currentP = ntrees_getNext (pSav);
	  pSav = nfather (pSav);
	  finished = (IS_NILTREE (nfather (pSav)));
	}
      pos = nposit (currentP);
    }
  return userCancel;
}
