/*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */
///
_PUBLIC_ currentObjectsManager::currentObjectsManager (ptGraphicTrees pGarb, ptGraphicTablesManager pTab):
the_graphicalTrees (pGarb), the_graphicTablesManager (pTab)
{

}

/**
  Initialisation des structures des structures d'acces aux objets graphiques
  et de certaines variables.
*/
void
currentObjectsManager::initCurObjManager (void)
{
  proc_cou_engl = arb_cou_engl = bte_cou_engl = null;
  f_ech_bte_engl.x = 1.;
  f_ech_bte_engl.y = 1.;
  l_bte_design = null;
  l_seg_design = null;
}


/**
  Gets the root of the Signal program
*/
ntrees
currentObjectsManager::get_racine (void)
{
  return racine;
}


/**
  Sets the root of the Signal program with p (in)
*/
void
currentObjectsManager::set_racine (ntrees p)
{
  racine = p;
}


/**
  Sets the root of the Signal program with p (in)
*/
boolean
currentObjectsManager::is_racine (ntrees p)
{
  return (racine == p);
}


/**
  Rend le modèle de processus courant
*/
ntrees
currentObjectsManager::getModelInWhich_cou_engl (void)
{
  return (proc_cou_engl);
}


/**
   Positionne le modèle de processus courant à ?p
*/
void
currentObjectsManager::set_proc_cou_engl (ntrees p)
{
  proc_cou_engl = p;
}


/**
  Rend l'arbre Signal courant
*/
ntrees
currentObjectsManager::get_arb_cou_engl (void)
{
  return (arb_cou_engl);
}


/**
   Positionne l'arbre Signal courant à ?p
*/
void
currentObjectsManager::set_arb_cou_engl (ntrees p)
{
  arb_cou_engl = p;
}


///
void
currentObjectsManager::set_arb_complet (ntrees p)
{
  arb_cou_engl = racine = proc_cou_engl = p;
}


/**
  Rend la boite courante englobante
*/
ntrees
currentObjectsManager::get_bte_cou_engl (void)
{
  return (bte_cou_engl);
}


/**
   Positionne la boite courante englobante
*/
void
currentObjectsManager::set_bte_cou_engl (ntrees p)
{
  bte_cou_engl = p;
}


/** Positionne des variables globales au module concernant des coordonnées
   graphiques (facteur d'échelle, et coordonnées de la boite englobante).
   Fonction appelée lorqu'on descend dans une boite, qu'on en remonte, ou
   lorsqu'on sélectionne un nouveau modèle.
*/
void
currentObjectsManager::setGraphicCoord (ntrees pBox)
{
  int form, nbcoord;
  typcoord tcoord[2];

  form = the_graphicalTrees->getCoord (pBox, &nbcoord, tcoord, 1);
  f_ech_bte_engl.x = ((float) lg_bte_stand) / tcoord[1].x;
  f_ech_bte_engl.y = ((float) ht_bte_stand) / tcoord[1].y;
}


/// TEMPORAIRE TEMPORAIRE
void
currentObjectsManager::setGraphicCoordSimu ()
{
  f_ech_bte_engl.x = 1;
  f_ech_bte_engl.y = 1;
}


/**
  A partir de coordonnees d'un point (*x, *y) appartenant au cadre de la 
  boite ?bte, cette fonction calcule les coordonnees reelles de ce point
  (c'est-a-dire dans le repere ou la boite est creee).
  cette fonction est appelee lorsqu'on cree un port directement sur un
  bord externe (sur le cadre) et lorsqu'on fait du mapping (afin 
  d'aligner des ports de niveaux differents).
  ?ou : position du port, a gauche ou a droite de la boite.
  Les nouvelles coordonnees sont rendues dans !x et !y.
*/
void
currentObjectsManager::externCoord2Normal (int bte, int *x, int *y, int ou)
{
  float val_reel, fact_ech_y;
  typcoord bte_coord[2], bte_coord_rela[2];
  int nbcoord;

  the_graphicalTrees->getCoord (the_graphicTablesManager->graphObj_get_tree (bte), &nbcoord, bte_coord, 0);
  if (ou == gauche)
    *x = bte_coord[0].x - bte_coord[1].x;
  else
    *x = bte_coord[0].x + bte_coord[1].x;
  the_graphicalTrees->getCoord (the_graphicTablesManager->graphObj_get_tree (bte), &nbcoord, bte_coord_rela, 1);
  val_reel = ht_bte_stand;
  fact_ech_y = val_reel / bte_coord_rela[1].y;
  *y = (int) ((*y - ord_mil_cadre) / fact_ech_y) + bte_coord[0].y;
}


/*
   Gestion des listes des objets designables : un objet peut etre invisible,
   visible, visible et designable. Ces listes permettent d'acceder plus 
   rapidement aux objets designables par l'utilisateur.Il existe 2 listes :
      - l_bte_design : liste des boites (les ports sont accessibles via
                       leurs boites)
      - l_seg_design : liste des segements
*/


/**
  Remise a zero de la liste l_bte_design
*/
void
currentObjectsManager::raz_l_bte (void)
{
  acces_gra *p, *succ;

  p = l_bte_design;
  while (p != null)
    {
      p->pred_d = null;
      succ = p->suiv_d;
      p->suiv_d = null;
      p = succ;
    };
  l_bte_design = null;
}


/**
  Recuperation de la liste l_bte_design (appelee par le routage)
*/
acces_gra *
currentObjectsManager::recup_l_bte_design (void)
{
  return (l_bte_design);
}


/**
   returns true is the current graphical object is a port.
   When the user has gone down a port, only the variable arb_cou_engl
   has changed, bte_cou_engl and proc_cou_engl have not changed)
*/
boolean
currentObjectsManager::isCurrentObjAPort (void)
{
  ntrees p;
  boolean b;

  p = bte_cou_engl;
  if (IS_NILTREE (bte_cou_engl))
    b = (arb_cou_engl != proc_cou_engl);
  else
    b = (arb_cou_engl != the_graphicalTrees->get_arb_sig (&p));
  return (b);
}


/**
  Rend vrai si la boite ?obj est dans la liste des boites designables,
  faux sinon.
*/
boolean
currentObjectsManager::is_bte_design (graphObj_type obj)
{
  acces_gra *pt;
  boolean trouve;

  pt = l_bte_design;
  trouve = pk_false;
  while ((pt != null) && !trouve)
    {
      trouve = (pt->graphObj == obj);
      pt = pt->suiv_d;
    }
  return (trouve);
}


/**
  Insertion d'un objet ?obj dans la liste l_bte_design
*/
void
currentObjectsManager::inser_l_bte_d (int obj)
{
  acces_gra *pt, *pred, *e;
  boolean trouve;
  int xx1, x1, tours;

  pt = the_graphicTablesManager->getBoxTableElem (obj);
  if (pt != null)		// en principe, pt toujours different de null
    {
      x1 = pt->ext1_halo.x;
      e = l_bte_design;
      trouve = pk_false;
      tours = 0;
      while ((e != null) && (!trouve))
	{
	  xx1 = e->ext1_halo.x;
	  if (xx1 > x1)
	    trouve = pk_true;
	  else
	    {
	      pred = e;
	      e = e->suiv_d;
	    }
	  tours++;
	};
      if (trouve)
	{
	  if (tours == 1)
	    {
	      pt->suiv_d = l_bte_design;
	      pt->pred_d = null;
	      if (l_bte_design != null)
		l_bte_design->pred_d = pt;
	      l_bte_design = pt;
	    }
	  else
	    {
	      pt->suiv_d = e;
	      pt->pred_d = e->pred_d;
	      pred->suiv_d = pt;
	      e->pred_d = pt;
	    };
	}
      else
	{
	  if (tours == 0)
	    {
	      pt->suiv_d = null;
	      pt->pred_d = null;
	      l_bte_design = pt;
	    }
	  else
	    {
	      pt->suiv_d = null;
	      pt->pred_d = pred;
	      pred->suiv_d = pt;
	    }
	}
    }
}


/**
  Retrait d'un objet ?obj de la liste d'objets désignables l_bte_design
  Attention, ?obj peut ne pas faire partie de la liste; dans ce
  cas, on ne fait rien.
*/
void
currentObjectsManager::enlever_l_bte_d (int obj)
{
  acces_gra *pt;

  pt = the_graphicTablesManager->getBoxTableElem (obj);
  if (pt != null)
    if ((pt->pred_d != null) || (pt->suiv_d != null))
      if (pt->pred_d == null)
	{
	  l_bte_design = pt->suiv_d;
	  if (l_bte_design != null)
	    (l_bte_design)->pred_d = null;
	}
      else
	{
	  pt->pred_d->suiv_d = pt->suiv_d;
	  if (pt->suiv_d != null)
	    pt->suiv_d->pred_d = pt->pred_d;
	}
    else			/* cas où suiv_d et pred_d sont à null, 2 cas possibles :
				   pt == l_bte_design, c'est qu'il n'y a qu'un seul élément dans la liste,
				   sinon, c'est que ?obj ne fait pas partie de la liste des objets désignables */ if (pt == l_bte_design)
      l_bte_design = null;
}


/**
  Recuperation de la liste l_seg_design (appelee par decompgraph)
*/
acces_seg *
currentObjectsManager::recup_l_seg_design (void)
{
  return (l_seg_design);
}


/**
  Remise a zero de la liste des segments designables
*/
void
currentObjectsManager::raz_l_seg (void)
{
  acces_seg *p, *succ;

  p = l_seg_design;
  while (p != null)
    {
      p->pred_d = null;
      succ = p->suiv_d;
      p->suiv_d = null;
      p = succ;
    };
  l_seg_design = null;
}


/**
  Insertion du segment ?seg dans la liste l_seg_design
*/
void
currentObjectsManager::inser_l_seg (int seg)
{
  acces_seg *pt, *pred, *e;
  boolean trouve;
  int xx1, x1, tours;

  pt = the_graphicTablesManager->getSegTableElem (seg);
  if (pt != null)
    {
      x1 = pt->ext1_halo.x;
      e = l_seg_design;
      trouve = pk_false;
      tours = 0;
      while (e != null && !trouve)
	{
	  xx1 = e->ext1_halo.x;
	  if (xx1 > x1)
	    trouve = pk_true;
	  else
	    {
	      pred = e;
	      e = e->suiv_d;
	    }
	  tours++;
	};
      if (trouve)
	if (tours == 1)
	  {
	    pt->suiv_d = l_seg_design;
	    pt->pred_d = null;
	    if (l_seg_design != null)
	      l_seg_design->pred_d = pt;
	    l_seg_design = pt;
	  }
	else
	  {
	    pt->suiv_d = e;
	    pt->pred_d = e->pred_d;
	    pred->suiv_d = pt;
	    e->pred_d = pt;
	  }
      else if (tours == 0)
	{
	  pt->suiv_d = null;
	  pt->pred_d = null;
	  l_seg_design = pt;
	}
      else
	{
	  pt->suiv_d = null;
	  pt->pred_d = pred;
	  pred->suiv_d = pt;
	}
    }
}


/**
  Retrait du segment ?seg dans la liste l_seg_design
*/
void
currentObjectsManager::enlever_l_seg (int seg)
{
  acces_seg *pt;

  pt = the_graphicTablesManager->getSegTableElem (seg);
  if (pt != null)
    if (pt->pred_d == null)
      {
	l_seg_design = pt->suiv_d;
	if (l_seg_design != null)
	  l_seg_design->pred_d = null;
      }
    else
      {
	pt->pred_d->suiv_d = pt->suiv_d;
	if (pt->suiv_d != null)
	  pt->suiv_d->pred_d = pt->pred_d;
      }
}


/**
  Fonction appellee par recouv_lien et recouv_lien_auto lorsqu'on cree
  un lien partant ou arrivant sur un port emetteur deja connecte : on
  cherche a se raccorder sur un segment deja existant. Pour cela,
    on cherche dans la liste des segments designables le segment dont les
    caracteristiques sont :
    - direction opposee a ?direc_seg
    - le plus proche du segment de coordonnees ?ext
    - appartient a un lien qui part du port ?pt_neg
   Si un tel segment existe, la fonction rend un pointeur sur ce dernier,
   et rend dans !coupe : vrai, si les deux segments se coupent par un
   prolongement, faux sinon,
   sinon la fonction rend  null.
*/
pt_seg
currentObjectsManager::cherch_seg (int pt_neg, typcoord * ext, int direc_seg, boolean * coupe)
{
  int sens, min1, max1, nbcoord;
  boolean trouve, inter, non_vide;
  float d, dmin, d1, d2;
  pt_seg seg_trouve, s;
  typcoord scoord[2];
  ntrees arb_gra;

  if (direc_seg == horiz)
    sens = vertic;
  else
    sens = horiz;
  trouve = *coupe = pk_false;
  s = l_seg_design;
  seg_trouve = null;
  dmin = dist_max;
  while (s != null && !trouve)
    {
      if ((s->port_ == pt_neg) && (s->direc == sens))
	{
	  arb_gra = s->arb_seg;
	  the_graphicalTrees->getCoord (arb_gra, &nbcoord, scoord, 0);
	  d = fmini (dist (ext[1], scoord[0]), dist (ext[1], scoord[1]));
	  inter = non_vide = pk_false;
	  if (sens == horiz)
	    {
	      min1 = mini (scoord[0].x, scoord[1].x);
	      max1 = maxi (scoord[0].x, scoord[1].x);
	      if (ext[1].x > min1 && ext[1].x < max1)
		{
		  min1 = mini (ext[0].y, ext[1].y);
		  max1 = maxi (ext[0].y, ext[1].y);
		  inter = pk_true;
		  if (scoord[0].y >= min1 && scoord[0].y <= max1)
		    non_vide = pk_true;
		  d1 = (float) ((min1 - scoord[0].y) * (min1 - scoord[0].y)) / 100;
		  d2 = (float) ((max1 - scoord[0].y) * (max1 - scoord[0].y)) / 100;
		  d = fmini (d1, d2);
		}
	    }
	  else
	    {
	      min1 = mini (scoord[0].y, scoord[1].y);
	      max1 = maxi (scoord[0].y, scoord[1].y);
	      if (ext[1].y > min1 && ext[1].y < max1)
		{
		  min1 = mini (ext[0].x, ext[1].x);
		  max1 = maxi (ext[0].x, ext[1].x);
		  inter = pk_true;
		  if (scoord[0].x >= min1 && scoord[0].x <= max1)
		    non_vide = pk_true;
		  d1 = (float) ((min1 - scoord[0].x) * (min1 - scoord[0].x)) / 100;
		  d2 = (float) ((max1 - scoord[0].x) * (max1 - scoord[0].x)) / 100;
		  d = fmini (d1, d2);
		}
	    };
	  if (non_vide)
	    {
	      trouve = pk_true;
	      seg_trouve = s;
	      *coupe = inter;
	    }
	  else
	    {
	      if ((d < dmin) || (d == dmin && inter))
		{
		  dmin = d;
		  seg_trouve = s;
		  *coupe = inter;
		}
	    }
	};
      s = s->suiv_d;
    };
  return (seg_trouve);
}



/**
  Cette fonction insere dans la liste (?l_n_seg_c) composee de numeros
   de segments les numeros de segments qui coupent le segment 
   de coordonnees ?ext et de direction ?direc_seg.
*/
void
currentObjectsManager::cherch_segs_coupe_seg (typcoord * ext, int direc_seg, ntrees l_n_seg_c)
{
  int sens, nbcoord;
  boolean entre_x, entre_y;
  pt_seg s;
  typcoord scoord[2];
  ntrees arb_gra;

  if (direc_seg == horiz)
    sens = vertic;
  else
    sens = horiz;
  s = l_seg_design;
  while (s != null)
    {
      if (s->direc == sens)
	{
	  arb_gra = s->arb_seg;
	  the_graphicalTrees->getCoord (nchild (2, arb_gra), &nbcoord, scoord, 0);
	  if (sens == horiz)
	    {
	      entre_x = (scoord[0].x <= ext[0].x && scoord[1].x >= ext[0].x) || (scoord[0].x >= ext[0].x && scoord[1].x <= ext[0].x);
	      entre_y = scoord[0].y <= ext[1].y && scoord[0].y >= ext[0].y;
	    }
	  else
	    {
	      entre_x = scoord[0].x <= ext[1].x && scoord[0].x >= ext[0].x;
	      entre_y = (scoord[0].y <= ext[0].y && scoord[1].y >= ext[0].y) || (scoord[0].y >= ext[0].y && scoord[1].y <= ext[0].y);
	    }
	  if (entre_x && entre_y)
	    ntreeList_appendLast (l_n_seg_c, LL_make_design_atom (s->id));
	}
      s = s->suiv_d;
    }
}


/**
  Insertion dans la liste des segments désignables des segments composant
  le chemin ?p_chemin
*/
void
currentObjectsManager::maj_l_seg_design (ntrees p_chemin)
{
  ntrees p, s_chem, p_seg;

  if ((p = nchild (4, p_chemin)) != null)
    {
      p_seg = nchild (1, p);
      while (p_seg != null)
	{
	  inser_l_seg (the_graphicalTrees->tree_get_graphObj (p_seg));
	  p_seg = ntrees_getNext (p_seg);
	}
    }
  s_chem = nchild (3, p_chemin);
  if (s_chem != null)
    {
      s_chem = nchild (1, s_chem);
      while (s_chem != null)
	{
	  maj_l_seg_design (s_chem);
	  s_chem = ntrees_getNext (s_chem);
	}
    }
}


/**
  Insere dans les 2 listes d'objets designables tous les objets graphiques
  (boites et segments) du même niveau trouves à partir de l'arbre 
  Signal ?p_sig.
  Si ?p_sig est une composition (oplfeed), alors les boites internes et 
  les segments internes sont insérés dans les listes.
*/
void
currentObjectsManager::maj_l_obj_design (ntrees p_sig)
{
  ntrees p_att, g_arb, p_port, p_chemin, p;

  raz_l_seg ();
  raz_l_bte ();
  p_att = the_graphicalTrees->desc_expr_sig (p_sig);
  g_arb = nattrntrees (p_att, GRAPHI_ATTRNUMBER);
  if (g_arb != null)
    {
      p_port = the_graphicalTrees->getFirstPort (g_arb);
      while (p_port != null)
	{
	  p_chemin = nchild (3, p_port);
	  if ((ISNOT_NILTREE (p_chemin)) && (noper (p_port) == gpkOp_porte))
	    maj_l_seg_design (p_chemin);
	  p_port = ntrees_getNext (p_port);
	}
    }
  if (noper (p_sig) == oplfeed)
    {
      p = nchild (1, p_sig);
      while (p != null)
	{
	  p_att = the_graphicalTrees->desc_expr_sig (p);
	  g_arb = nattrntrees (p_att, GRAPHI_ATTRNUMBER);
	  if (g_arb != null)
	    {
	      inser_l_bte_d (the_graphicalTrees->tree_get_graphObj (g_arb));
	      p_port = the_graphicalTrees->getFirstPort (g_arb);
	      while (p_port != null)
		{
		  p_chemin = nchild (3, p_port);
		  if ((ISNOT_NILTREE (p_chemin)) && (noper (p_port) == gpkOp_ports))
		    maj_l_seg_design (p_chemin);
		  p_port = ntrees_getNext (p_port);
		}
	    }
	  p = ntrees_getNext (p);
	}
    }
}


/**
  Updates the list of the graphical clicked objects from the current Signal tree.
 */
void
currentObjectsManager::setCurrentObjectsList ()
{
  maj_l_obj_design (get_arb_cou_engl ());
}


/**
  Cette fonction teste si on peut ajouter un nouveau segment
   ?halo : halo du segment
   ?sens : sens (horizontal ou vertical) du segment
   On compare le halo du segment avec les halos des boites et des segments
   de direction opposee deja existants.
*/
boolean
currentObjectsManager::test_cr_seg (typcoord * halo, int sens)
{
  int x1, x2, y1, y2, xx1, xx2, yy1, yy2;
  acces_gra *pt;
  acces_seg *pt_s;
  typcoord boxHalo[2];

  x1 = halo[0].x;
  x2 = halo[1].x;
  y1 = halo[0].y;
  y2 = halo[1].y;
  if (bte_cou_engl != null)
    if (!isInFrame (x1, y1, x2, y2))
      return (pk_false);
  pt = l_bte_design;
  while (pt != null)
    {
      boxHalo[0].x = pt->ext1_halo.x;
      boxHalo[0].y = pt->ext1_halo.y;
      boxHalo[1].x = pt->ext2_halo.x;
      boxHalo[1].y = pt->ext2_halo.y;
      if (boxHalo[0].x > x2)
	pt = null;
      else if (!isIntersectionEmpty (halo, boxHalo))
	return (pk_false);
      if (pt != null)
	pt = pt->suiv_d;
    };
  pt_s = l_seg_design;
  while (pt_s != null)
    {
      if (pt_s->direc == sens)
	{
	  xx1 = pt_s->ext1_halo.x;
	  yy1 = pt_s->ext1_halo.y;
	  xx2 = pt_s->ext2_halo.x;
	  yy2 = pt_s->ext2_halo.y;
	  if (xx1 > x2)
	    pt_s = null;
	  else /* cas des segments qui se touchent qui appartiennent au meme lien */ if (((abs (x1 - xx1) <= 1) && (abs (x2 - xx2) <= 1) &&
											  (abs (y1 - yy2) <= 1 || abs (yy1 - y2) <= 1))
											 || ((abs (y1 - yy1) <= 1) && (abs (y2 - yy2) <= 1) &&
											     (abs (x1 - xx2) <= 1 || abs (xx1 - x2) <= 1)))
	    return (pk_true);
	  else if (x1 <= xx2 && y2 >= yy1 && yy2 >= y1)
	    return (pk_false);
	};
      if (pt_s != null)
	pt_s = pt_s->suiv_d;
    };
  return (pk_true);
}


/**
  Cette fonction teste si on peut ajouter une nouvelle boite
   On compare le halo de la boite avec les halos des boites et des segments
   deja existants.
*/
boolean
currentObjectsManager::test_cr_bte (typcoord * halo)
{
  acces_gra *pt;
  acces_seg *pt_s;
  typcoord curHalo[2];

  if (bte_cou_engl != null)
    if (!isInFrame (halo[0].x, halo[0].y, halo[1].x, halo[1].y))
      return (pk_false);
  pt = l_bte_design;
  while (pt != null)
    {
      curHalo[0].x = pt->ext1_halo.x;
      curHalo[0].y = pt->ext1_halo.y;
      curHalo[1].x = pt->ext2_halo.x;
      curHalo[1].y = pt->ext2_halo.y;
      if (curHalo[0].x > halo[1].x)
	pt = null;
      else if (!isIntersectionEmpty (halo, curHalo))
	return (pk_false);
      if (pt != null)
	pt = pt->suiv_d;
    };
  pt_s = l_seg_design;
  while (pt_s != null)
    {
      curHalo[0].x = pt_s->ext1_halo.x;
      curHalo[0].y = pt_s->ext1_halo.y;
      curHalo[1].x = pt_s->ext2_halo.x;
      curHalo[1].y = pt_s->ext2_halo.y;
      if (curHalo[0].x >= halo[1].x)
	return (pk_true);
      else if (!isIntersectionEmpty (halo, curHalo))
	return (pk_false);
      if (pt_s != null)
	pt_s = pt_s->suiv_d;
    };
  return (pk_true);
}


/**
   Fonction appelee (entre autre) lors de la creation d'un nouveau port.
   Retrouve la boite (si elle existe) dont le bord gauche ou le
   bord droit a pour abcisse la valeur de ?x_ret (approximativement),
   la valeur de ?y_ret devant etre comprise entre les ordonnees du coin
   bas-gauche et du coin haut-droit.
   !ou le cote sur lequel sera dessine le port (gauche ou droite)
   si ?f = 0, on recherche uniquement le numero de la boite.
         = 1, on recherche en plus l'endroit ou sera place le port
                     (!rg sera positionne et ?y_ret pourra etre modifie)
   si le point clique est dans le halo de la boite mais en dehors de la
   boite, il s'agit d'un port de sortie (!en_entree positionne a faux),
   sinon il s'agit d'un port d'entree (!en_entree positionne a  vrai).
   *x_ret et *y_ret peuvent etre des points appartenant aux bords de la
   boite englobante; dans ce cas, la fonction rend dans *x_ret et *y_ret
   les coordonnees du point relativement a la boite (comme si on etait
   remonte d'un niveau).
   Retourne le numero de la boite.
*/
int
currentObjectsManager::reper_bte (int *x_ret, int *y_ret, int *ou, int *rg, int f, boolean * en_entree)
{
  int trouve, nbcoord, form, abs_mil, ord_mil, haute, larg, num;
  ntrees g_arb;
  typcoord tcoord[maxcoord];
  acces_gra *pt;
  int y_ext_b, y_ext_h;

  pt = l_bte_design;
  trouve = pk_false;
  while (pt != null && !trouve)
    {
      g_arb = pt->elem_gra;
      if (noper (g_arb) == gpkOp_bte)
	{
	  form = the_graphicalTrees->getCoord (g_arb, &nbcoord, tcoord, 0);
	  if (form == oprect)
	    {
	      larg = tcoord[1].x;
	      haute = tcoord[1].y;
	      abs_mil = tcoord[0].x;
	      ord_mil = tcoord[0].y;
	      if (*x_ret < abs_mil - larg + (2 * taillepave_x) &&
		  *x_ret > abs_mil - larg - 2 * (taillepave_x) && *y_ret < ord_mil + haute && *y_ret > ord_mil - haute)
		{
		  *ou = gauche;
		  *en_entree = (*x_ret > abs_mil - larg);
		  *x_ret = abs_mil - larg;
		  trouve = pk_true;
		}
	      else
		if (*x_ret < abs_mil + larg + 2 * (taillepave_x) &&
		    *x_ret > abs_mil + larg - 2 * (taillepave_x) && *y_ret < ord_mil + haute && *y_ret > ord_mil - haute)
		{
		  *ou = droite;
		  *en_entree = (*x_ret <= abs_mil + larg);
		  *x_ret = abs_mil + larg;
		  trouve = pk_true;
		}
	    }
	};
      pt = pt->suiv_d;
    };
  if (trouve)
    {
      num = the_graphicalTrees->tree_get_graphObj (g_arb);
      if (f == 1)
	return (num);
      else
	{
	  if ((*ou == gauche) || (*ou == droite))
	    {
	      y_ext_b = ord_mil + haute;
	      y_ext_h = ord_mil - haute;
	    }
	  else
	    {
	      y_ext_b = abs_mil + larg;
	      y_ext_h = abs_mil - larg;
	    }
	  if (the_graphicalTrees->bte_avec_port (g_arb))
	    {
	      if (the_graphicalTrees->insert_port (g_arb, 0, *ou, y_ret, y_ext_b, y_ext_h, rg))
		return (num);
	    }
	  else if ((*ou == gauche) || (*ou == droite))
	    {
	      if (*y_ret < y_ext_h + taillepave_y)
		*y_ret = y_ext_h + taillepave_y;
	      else if (*y_ret > y_ext_b - taillepave_y)
		*y_ret = y_ext_b - taillepave_y;
	    }
	  else /* cas haut-bas */ if (*x_ret < y_ext_h + taillepave_x)
	    *x_ret = y_ext_h + taillepave_x;
	  else if (*x_ret > y_ext_b - taillepave_x)
	    *x_ret = y_ext_b - taillepave_x;
	  return (num);
	}
    }
  else if (bte_cou_engl != null)	/*on regarde si le point clique est sur la bte 
					   englobante (utilise pour creer un port directement sur le cadre) */
    {
      if ((*x_ret <= decal_cadre_x + taillepave_x && *x_ret >= decal_cadre_x - taillepave_x) &&
	  (*y_ret <= ord_mil_cadre + ht_bte_stand) && (*y_ret >= ord_mil_cadre - ht_bte_stand))
	{
	  *ou = gauche;
	  *en_entree = (*x_ret > decal_cadre_x);
	  trouve = pk_true;
	}
      else if ((*x_ret >= abs_mil_cadre + lg_bte_stand - taillepave_x) &&
	       (*x_ret <= abs_mil_cadre + lg_bte_stand + taillepave_x) &&
	       (*y_ret <= ord_mil_cadre + ht_bte_stand) && (*y_ret >= ord_mil_cadre - ht_bte_stand))
	{
	  *ou = droite;
	  *en_entree = (*x_ret <= abs_mil_cadre + lg_bte_stand);
	  trouve = pk_true;
	}
      if (trouve)
	{
	  num = the_graphicalTrees->tree_get_graphObj (bte_cou_engl);
	  if (f == 1)
	    return (num);
	  else
	    {
	      form = the_graphicalTrees->getCoord (bte_cou_engl, &nbcoord, tcoord, 0);
	      y_ext_b = tcoord[0].y + tcoord[1].y;
	      y_ext_h = tcoord[0].y - tcoord[1].y;
	      externCoord2Normal (num, x_ret, y_ret, *ou);
	      if (the_graphicalTrees->bte_avec_port (bte_cou_engl))
		{
		  if (the_graphicalTrees->insert_port (bte_cou_engl, 0, *ou, y_ret, y_ext_b, y_ext_h, rg))
		    return (num);
		}
	      else
		{
		  if (*y_ret < y_ext_h + taillepave_y)
		    *y_ret = y_ext_h + taillepave_y;
		  else if (*y_ret > y_ext_b - taillepave_y)
		    *y_ret = y_ext_b - taillepave_y;
		  return (num);
		}
	    }
	}
    }
  return (0);
}


/**
  Recherche si le point ?(x_ret,y_ret) appartient a un port de la boite
  ?p_bte.
  rend l'arbre representant le port.
*/
ntrees
currentObjectsManager::recherche_port (ntrees p_bte, int x_ret, int y_ret)
{
  ntrees p;
  int form, nbcoord;
  boolean trouve;
  typcoord tcoord[2];

  trouve = pk_false;
  p = the_graphicalTrees->getFirstPort (p_bte);
  while (p != null && !trouve)
    {
      if (p_bte == get_bte_cou_engl ())
	{
	  form = the_graphicalTrees->getCoord (p, &nbcoord, tcoord, 1);
	  tcoord[0].x = (int) (tcoord[0].x * f_ech_bte_engl.x) + abs_mil_cadre;
	  tcoord[0].y = (int) (tcoord[0].y * f_ech_bte_engl.y) + ord_mil_cadre;
	}
      else
	form = the_graphicalTrees->getCoord (p, &nbcoord, tcoord, 0);
      if (x_ret < tcoord[0].x + taillepave_x && x_ret > tcoord[0].x - taillepave_x &&
	  y_ret > tcoord[0].y - taillepave_y && y_ret < tcoord[0].y + taillepave_y)
	trouve = pk_true;
      else
	p = ntrees_getNext (p);
    }
  return (p);
}


/**
   A partir du point designe par l'utilisateur, reper_obj retrouve l'objet
   concerne (rend le numero de l'objet).
   ?x_ret, ?y_ret : coordonnees du point designe par l'utilisateur
   !typ_obj : type de l'objet (s'il existe)
   Rend le numero de l'objet retrouve (boite, port ou segment), 0 sinon.
*/
int
currentObjectsManager::reper_obj (int x_ret, int y_ret, objet_type * typ_obj)
{
  ntrees g_arb, p;
  acces_gra *pt;
  acces_seg *pt_s;
  typcoord tcoord[2];

  pt_s = l_seg_design;		/* on recherche d'abord dans les segments */
  while (pt_s != null)
    {
      g_arb = pt_s->arb_seg;
      the_graphicTablesManager->setHaloWithSegElem (pt_s, tcoord);
      if (!isInRectangle (x_ret, y_ret, tcoord))
	pt_s = pt_s->suiv_d;
      else
	{
	  *typ_obj = typ_seg;
	  return (the_graphicalTrees->tree_get_graphObj (g_arb));
	}
    }
  if (bte_cou_engl != null)	/* on recherche s'il s'agit d'un port externe */
    {
      g_arb = recherche_port (bte_cou_engl, x_ret, y_ret);
      if (g_arb != null)
	{
	  *typ_obj = typ_port_ext;
	  return (the_graphicalTrees->tree_get_graphObj (g_arb));
	}
      else
	{			/* on regarde si c'est les bords de la boite englobante */
	  if (((x_ret <= decal_cadre_x + taillepave_x && x_ret >= decal_cadre_x - taillepave_x)
	       || ((x_ret <= abs_mil_cadre + lg_bte_stand + taillepave_x &&
		    x_ret >= abs_mil_cadre + lg_bte_stand - taillepave_x))) &&
	      ((y_ret <= ord_mil_cadre + ht_bte_stand) && (y_ret >= ord_mil_cadre - ht_bte_stand)))
	    {
	      *typ_obj = typ_cadre;
	      return (the_graphicalTrees->tree_get_graphObj (bte_cou_engl));
	    }
	}
    }
  pt = l_bte_design;		/* on regarde dans la liste des boites courantes */
  while (pt != null)
    {
      g_arb = pt->elem_gra;
      the_graphicTablesManager->setHaloWithBoxElem (pt, tcoord);
      if (isInRectangle (x_ret, y_ret, tcoord))
	{
	  p = recherche_port (g_arb, x_ret, y_ret);
	  if (p != null)
	    {
	      *typ_obj = typ_port;
	      return (the_graphicalTrees->tree_get_graphObj (p));
	    }
	  else
	    {
	      *typ_obj = typ_bte;
	      return (the_graphicalTrees->tree_get_graphObj (g_arb));
	    };
	}
      pt = pt->suiv_d;
    }
  *typ_obj = typ_vide;
  return (0);
}


/**
  Cette fonction descend dans la boite ou le port designé par l'utilisateur. 
  Les variables arb_cou_engl, proc_cou_engl et bte_cou_engl sont modifiées
  et provoque le reaffichage de la fenetre.
  ATTENTION, si on descend dans un port, on ne change ni bte_cou_engl, ni
  proc_cou_engl.
  ?obj : objet dans lequel on descend
  ?simu : booleen vrai si c'est une descente simulee (on fait comme si on
  descendait dans la boite mais sans faire d'affichage : utilise par exemple
  pour faire du routage sur des liens internes);
*/
void
currentObjectsManager::descendre (int obj, boolean simu)
{
  ntrees arb;

  arb = the_graphicTablesManager->graphObj_get_tree (obj);
  if (the_graphicalTrees->isAPort (obj))
    arb_cou_engl = nattrntrees (arb, SIGNAL_ATTRNUMBER);
  else
    {
      bte_cou_engl = arb;
      arb_cou_engl = the_graphicalTrees->get_arb_sig (&arb);
      setGraphicCoord (bte_cou_engl);
      if (the_graphicalTrees->isBoxProc (obj))
	proc_cou_engl = STree_getModelInWhich (arb_cou_engl);
      maj_l_obj_design (arb_cou_engl);
    }
}


/**
  Go up from the current graphical object. The object may be a port or a box.
  The method is called with simu (in) equal to true during the routing
  when it is necessary to do routing at a upper level.
*/
void
currentObjectsManager::remonter (boolean simu)
{
  ntrees graphTree;
  boolean isAPort;

  isAPort = isCurrentObjAPort ();
  if (isAPort)
    {				// to go up a port, it is only necessary to set arb_cou_engl 
      graphTree = bte_cou_engl;
      arb_cou_engl = the_graphicalTrees->get_arb_sig (&graphTree);
    }
  else
    {				// the current object is a box
      if (arb_cou_engl != STreeModel_getBody (proc_cou_engl))
	{			// it is possible to go up in the boxes hierarchy
	  arb_cou_engl = nfather (arb_cou_engl);
	  bte_cou_engl = the_graphicalTrees->get_arb_boite (arb_cou_engl);
	  setGraphicCoord (bte_cou_engl);
	  maj_l_obj_design (arb_cou_engl);
	}
      else if (simu)
	{
	  setGraphicCoordSimu ();
	  SET_NILTREE (bte_cou_engl);
	  arb_cou_engl = proc_cou_engl;
	  maj_l_obj_design (arb_cou_engl);
	}
    }
}


/**
  Method called when the user clicks on a Signal node in the Jtree window;
  The selected node becomes the current node.
  When the user clicks on a module or on the root, there is no box associated.
*/
void
currentObjectsManager::selectCurrentTree (ntrees p)
{
  // nDEBUG_subtree(0, 8, "currentObjectsManager::selectCurrentTree racine is", racine);
  // nDEBUG_subtree(0, 4, "currentObjectsManager::selectCurrentTree p is", p);
  if (p == racine)
    {
      set_arb_complet (p);
      set_bte_cou_engl (NILTREE);
    }
  else if (STree_isModule (p))
    {
      set_proc_cou_engl (p);
      set_arb_cou_engl (p);
      set_bte_cou_engl (NILTREE);
    }
  else
    {
      set_proc_cou_engl (p);
      arb_cou_engl = STreeModel_getBody (p);
      bte_cou_engl = the_graphicalTrees->get_arb_boite (arb_cou_engl);
      if (ISNOT_NILTREE (bte_cou_engl))
	{
	  setGraphicCoord (bte_cou_engl);
	  maj_l_obj_design (arb_cou_engl);
	}
    }
}


/**
  Cette fonction teste si on peut encapsuler une ou plusieurs boites
   On compare le halo de la boite avec les halos des boites encapsulees, 
   non encapsulees et des segments n'appartenant pas aux chemins issus
   des ports encapsules.
   !l_bte_enc: la liste des numeros des boites encapsulees
   !no_bte a vrai si la liste est vide.
   rend faux si on ne peut pas encapsuler (soit a cause des halos, soit
   parce qu'il n'y a pas de boite a encapsuler)
   annule tous les halos des segments qui font partie des liens a rerouter
*/
boolean
currentObjectsManager::test_encaps_bte (typcoord * halo, ntrees * l_bte_enc, boolean * no_bte)
{
  int x1, x2, y1, y2, xx1, xx2, yy1, yy2;
  acces_gra *pt;
  acces_seg *pt_s;
  ntrees arb_no_bte;
  boolean cont;
  typcoord bougetail;

  cont = pk_true;
  *no_bte = pk_true;
  bougetail.x = 1;
  bougetail.y = 1;
  x1 = halo[0].x;
  x2 = halo[1].x;
  y1 = halo[0].y;
  y2 = halo[1].y;
  if (bte_cou_engl != null)
    cont = isInFrame (x1, y1, x2, y2);
  if (cont)
    {
      *l_bte_enc = LL_make_design_emptyList ();
      pt = l_bte_design;
      while (pt != null && cont)
	{
	  xx1 = pt->ext1_halo.x;
	  yy1 = pt->ext1_halo.y;
	  xx2 = pt->ext2_halo.x;
	  yy2 = pt->ext2_halo.y;
	  if (xx1 > x2)
	    pt = null;
	  else if (x1 <= xx2 && y2 >= yy1 && yy2 >= y1)
	    if (isInRectangle (xx1, yy1, halo) && isInRectangle (xx2, yy2, halo))
	      {
		ntreeList_insertFirst (LL_make_design_atom (pt->graphObj), *l_bte_enc);
		*no_bte = pk_false;
	      }
	    else
	      cont = pk_false;
	  if (pt != null)
	    pt = pt->suiv_d;
	}
      if (cont)
	{
	  arb_no_bte = nchild (1, *l_bte_enc);
	  while (arb_no_bte != null)
	    {
	      the_graphicalTrees->annuler_halo_seg_bte ((int) nintof (arb_no_bte), pk_false, pk_false, &bougetail, pk_true);
	      arb_no_bte = ntrees_getNext (arb_no_bte);
	    }
	  pt_s = l_seg_design;
	  while (pt_s != null && cont)
	    {
	      xx1 = pt_s->ext1_halo.x;
	      yy1 = pt_s->ext1_halo.y;
	      xx2 = pt_s->ext2_halo.x;
	      yy2 = pt_s->ext2_halo.y;
	      if (xx1 > x2)
		pt_s = null;
	      else if (x1 <= xx2 && y2 >= yy1 && yy2 >= y1)
		cont = pk_false;
	      if (pt_s != null)
		pt_s = pt_s->suiv_d;
	    }
	  if (!cont)
	    {
	      arb_no_bte = nchild (1, *l_bte_enc);
	      while (arb_no_bte != null)
		{
		  the_graphicalTrees->annuler_halo_seg_bte ((int) nintof (arb_no_bte), pk_false, pk_true, &bougetail, pk_true);
		  arb_no_bte = ntrees_getNext (arb_no_bte);
		}
	    }
	}
    }
  if (cont && *no_bte)
    cont = pk_false;
  else
    *no_bte = pk_false;
  return (cont);
}
