/*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */
//_PUBLIC_


/**
  Hash function
*/
int
graphicTablesManager::hashnum (int num)
{
  int h;

  h = num % taille_t_hash;
  if (h < 0)
    h = -h;
  return (h);
}


/**
  Creates a new element with its number num (in) in the hash table and 
  inserts it at the head of suivants 
  Returns the new element.
*/
t_hash_num *
graphicTablesManager::crconvnum (int num, t_hash_num * suivants)
{
  t_hash_num *pt;
  int t;

  t = sizeof (t_hash_num);
  pt = (t_hash_num *) calloc (1, t);
  pt->ovovovov = num;
  pt->ovovnvnv = the_freeGraphObjIdentity;
  pt->suivant = suivants;
  the_freeGraphObjIdentity++;
  return (pt);
}


/**
  Returns the new unique number which corresponds to the number num (in)
  tab (in) : 0 for a box or a port, 1 for a segment, 2 for a path.
*/
graphObj_type
graphicTablesManager::crtconvnum (int tab, int num, int h, t_hash_num * l)
{
  t_hash_num *pt;

  pt = crconvnum (num, l);
  switch (tab)
    {
    case 0:
      this_boxHashTable[h] = pt;
      break;
    case 1:
      this_segmentHashTable[h] = pt;
      break;
    case 2:
      this_pathHashTable[h] = pt;
      break;
    }

  return the_freeGraphObjIdentity - 1;
}


/**
  Creates a new element in the hash table and inserts it via precedent (in) and
  suivants (in).
  Returns the new number.
*/
graphObj_type
graphicTablesManager::crlconvnum (int num, t_hash_num * precedent, t_hash_num * suivants)
{
  t_hash_num *pt;

  pt = crconvnum (num, suivants);
  precedent->suivant = pt;
  return the_freeGraphObjIdentity - 1;
}


/**
  Returns the conversion of the number num (in) in the list l (in)

*/
int
graphicTablesManager::numinlconv (int num, t_hash_num * targetList)
{
  t_hash_num *listTail;

  if (num == targetList->ovovovov)
    return targetList->ovovnvnv;
  listTail = targetList->suivant;
  if (listTail == null)
    return crlconvnum (num, targetList, listTail);
  if (num < listTail->ovovovov)
    return crlconvnum (num, targetList, listTail);
  return numinlconv (num, listTail);
}
