/*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */
//_PRIVATE_

#define bas 3
#define haut 4
#define oplroutage 100
#define LL_make_lroutage_list(ARG) nmakelist(oplroutage, ARG)
#define opcout 101
#define opsens 102
#define oproutage 103
#define LL_make_routage(ARG1, ARG2, ARG3) nmaketernary(oproutage, ARG1, ARG2, ARG3)

///
struct ty_l_project
{
  typcoord ext1_halo, ext2_halo;
  ptListProjections suivant;
} XXXX;


///
typedef struct l_arb
{
  ntrees chemin;
  int cout;
  int sens;
  int longueur;
  struct l_arb *suivant;
} ty_l_chemins;
///

#define PLUSOUMOINS1(X,Y) ((X<=Y+1 && X>=Y-1)||(Y<=X+1 && Y>=X-1)) \
                        ? (pk_true) : (pk_false)
#define MAXI(A,B) ((A) > (B) ? (A) : (B))
#define MINI(A,B) ((A) < (B) ? (A) : (B))

/**
   Initialisations des variables necessaires a l'algorithme de routage.
*/
void
routor::init_routage (int xa, int ya, int sensa, int xb, int yb, int sensb)
{
  cout_cour = cout_chem (xa, ya, sensa, xb, yb, sensb);
  racine_routage = LL_make_routage (LL_make_abs_atom (xa), LL_make_ord_atom (ya), null);
  routage_cour = racine_routage;
  sens_cour = sensa;
  l_cour = 0;
  x_cour = xa;
  y_cour = ya;
  x_arr = xb;
  y_arr = yb;
  sens_arr = sensb;
  l_chemins = null;
  seg_recouv = 0;
}


/**
  Met a jour l'arbre des chemins calcules en inserant un nouveau segment
  ? (x,y) coordonnees du segment, ?sens : vertical ou horizontal
  maj_chem est appelee uniquement a la fin lorsqu'on a trouve un
  chemin complet entre A et B pour inserer les dernieres coordonnees
  dans l'arbre des chemins.
*/
void
routor::maj_chem (int x, int y, int sens)
{
  ntrees s3, p;

  p = LL_make_routage (LL_make_abs_atom (x), LL_make_ord_atom (y), null);
  if ((s3 = nchild (3, routage_cour)) == null)
    nchangchild (routage_cour, LL_make_lroutage_list (p), 3);
  else
    ninsertright (1, p, s3);
  routage_cour = p;
}


/**
   * cette procedure est appelee lorsqu'on fait du routage vers un port
   * diffuseur deja connecte. 
   * ?obst contient les coordonnees du segment le plus proche de
   * ?(xa,ya) et qui appartient a un lien qui va sur le port. 
   * Cette procedure calcule les coordonnees (x',y') du segment 
   * qui arrive sur le segment trouve et les place  dans l'arbre des 
   * chemins..
   * ?sens_horiz est egal a vrai lorsque le segment ((xa,ya),(x',y')) 
   * est horizontal, faux sinon.
   */
void
routor::calcul_seg_recouv (int xa, int ya, boolean sens_horiz, typcoord * obst)
{
  int x0, y0, x1, y1, x, y;

  x0 = obst[0].x;
  y0 = obst[0].y;
  x1 = obst[1].x;
  y1 = obst[1].y;
  if (sens_horiz)
    {
      if (x1 <= x0 + (2 * halo_segm))	/* l'obstacle rencontre est vertical */
	{
	  maj_chem (x0, ya, gauche);
	  x_arr = x0;
	  y_arr = ya;
	}
      else
	{

	  if (abs (xa - x0) < abs (xa - x1))
	    x_arr = x = x0;
	  else
	    x_arr = x = x1;
	  maj_chem (x, ya, gauche);
	  y_arr = y0;
	}
    }
  else
    {
      if (y1 <= y0 + (2 * halo_segm))	/* l'obstacle rencontre est horizontal */
	{
	  x_arr = xa;
	  y_arr = y0;
	  maj_chem (xa, y0, haut);
	}
      else
	{
	  if (abs (ya - y0) < abs (ya - y1))
	    y_arr = y = y0;
	  else
	    y_arr = y = y1;
	  maj_chem (xa, y, haut);
	  y_arr = y0;
	}
    }
}


/**
 * cette fonction regarde si il y a un chemin direct entre A et B :
 * le chemin passe par le milieu de A et B. Il faut que :
 * sensa = gauche et sensb = droite et xa > xb
 * ou
 * sensa = droite et sensb = gauche et xa < xb
 */
boolean
routor::chem_direct (int xa, int ya, int sensa, int xb, int yb, int sensb)
{
  int x;
  typcoord obst[2];
  boolean res;

  x = (xa + xb) / 2;
  res = pk_false;
  /*    if ((xa>(xb + halo_bte) && sensa==gauche && sensb==droite) ||
     (xa < (xb-halo_bte) && sensa==droite && sensb==gauche)) */
  if ((xa > xb && sensa == gauche && sensb == droite) || (xa < xb && sensa == droite && sensb == gauche))
    {
      if (!obstacle_x (xa, x, ya, obst))
	if (!obstacle_y (ya, yb, x, obst))
	  {
	    if (!obstacle_x (x, xb, yb, obst))
	      res = pk_true;
	    else
	      {
		if (seg_recouv != 0)
		  {
		    maj_chem (x, ya, sensa);
		    maj_chem (x, yb, sensa);
		    calcul_seg_recouv (x, yb, pk_true, obst);
		    res = pk_true;
		  }
	      }
	  }
	else
	  {
	    if (seg_recouv != 0)
	      {
		maj_chem (x, ya, sensa);
		calcul_seg_recouv (x, ya, pk_false, obst);
		res = pk_true;
	      }
	  }
      else
	{
	  if (seg_recouv != 0)
	    {
	      calcul_seg_recouv (xa, ya, pk_true, obst);
	      res = pk_true;
	    }
	}
    }
  return (res);
}


/**
 * insere dans la liste des chemins intermediaires un nouveau point 
 * represente sous forme d'arbre ?p, avec son cout ?ct et son sens ?sens.
 */
void
routor::inser_l_chem (ntrees p, int ct, int sens, int l)
{
  ty_l_chemins *pt_chem;

  pt_chem = (ty_l_chemins *) calloc (1, sizeof (ty_l_chemins));
  pt_chem->cout = ct;
  pt_chem->sens = sens;
  pt_chem->longueur = l;
  pt_chem->chemin = p;
  if (l_chemins == null)
    {
      l_chemins = pt_chem;
      l_chemins->suivant = null;
    }
  else
    {
      pt_chem->suivant = l_chemins;
      l_chemins = pt_chem;
    }
}


/**
  fonction qui rend le cout du chemin optimal pour aller de ?(xa,ya) 
  en partant dans la direction ?sensa (gauche,droite,haut ou bas) en
  ?(xb,yb) en arrivant par la direction ?sensb (gauche,droite,haut ou bas)
*/
int
routor::cout_chem (int xa, int ya, int sensa, int xb, int yb, int sensb)
{
  switch (sensb)
    {
    case gauche:
      switch (sensa)
	{
	case gauche:
	  break;
	case droite:
	  break;
	case haut:
	  break;
	case bas:
	  break;
	};
    case droite:
      switch (sensa)
	{
	case gauche:
	  break;
	case droite:
	  break;
	case haut:
	  break;
	case bas:
	  break;
	}
    }
  return (0);
}


/**
   Cette fonction verifie qu'entre x1 et x2, il y a assez de place pour
   un chemin (a cause de la largeur du halo) ; rend vrai si oui, 
   faux sinon.
*/
boolean
routor::pass_chem (int x1, int x2)
{
  if (x2 - x1 >= halo_segm)
    return (pk_true);
  else
    return (pk_false);
}


/**
   cette fonction rend vrai si l'intersection de l'intervalle
   ?[x1,x2] avec l'intervalle ?[borne1,borne2] est non vide, faux sinon.
   attention : x1 (borne1) peut etre plus grand que x2 (borne2).
*/
boolean
routor::alinterieur (int x1, int x2, int borne1, int borne2)
{
  if (borne1 <= borne2)
    if ((x1 <= borne2 && x1 >= borne1) || (x1 <= borne1 && x2 >= borne2) || (x1 <= borne1 && x2 >= borne1))
      return (pk_true);
    else
      return (pk_false);
  else if ((x2 <= borne1 && x2 >= borne2) || (x1 <= borne1 && x1 >= borne2) || (x1 <= borne2 && x2 >= borne1))
    return (pk_true);
  else
    return (pk_false);
}


/**
   cette fonction rend vrai si l'intersection de l'intervalle
   ?[x1,x2] avec l'intervalle ?[borne1,borne2] est non vide, faux sinon.
   attention : borne1 peut etre plus grand que borne2.
   Si les intervalles se touchent, rend faux.
   Utilisee pour le calcul des projections.
*/
boolean
routor::alinterieur_strict (int x1, int x2, int borne1, int borne2)
{
  if (borne1 <= borne2)
    if ((x1 < borne2 && x1 >= borne1) || (x1 <= borne1 && x2 >= borne2) || (x1 <= borne1 && x2 >= borne1))
      return (pk_true);
    else
      return (pk_false);
  else if ((x2 <= borne1 && x2 >= borne2) || (x1 < borne1 && x1 >= borne2) || (x1 <= borne2 && x2 >= borne1))
    return (pk_true);
  else
    return (pk_false);
}


/**
  Cette fonction verifie si on peut passer en ?x sans etre dans l'
  obstacle ou trop pres de l'obstacle ?(borne1,borne2).
  La fonction rend vrai si on touche l'obstacle, faux sinon; elle est
  appelee par obstacle_x et obstacle_y.
*/
boolean
routor::touche_obst (int borne1, int borne2, int x)
{
  if (borne1 <= x && borne2 >= x)
    return (pk_true);
  else if (borne2 < x && borne2 > x - halo_segm)
    return (pk_true);
  else if (borne1 > x && borne1 < x + halo_segm)
    return (pk_true);
  else
    return (pk_false);
}


/**
 * construit un nouvel element de projection represente par ses
 *coordonnees ?(abs1,ord1,abs2,ord2) et l'insere dans
 * la liste des projections courantes l_proj.
 * La liste est triee par ordre croissant des abscisses si ?en_absc
 * est vrai, des ordonnees sinon.
 * cette procedure est appelee par projection_x et projection_y.
 */
void
routor::construire_proj (int abs1, int ord1, int abs2, int ord2, boolean en_absc)
{
  ty_l_project *pt_proj, *pt, *pt_pred;
  boolean fini;

  pt_proj = (ty_l_project *) calloc (1, sizeof (ty_l_project));
  pt_proj->ext1_halo.x = abs1;
  pt_proj->ext1_halo.y = ord1;
  pt_proj->ext2_halo.x = abs2;
  pt_proj->ext2_halo.y = ord2;
  pt_proj->suivant = null;
  if (l_proj == null)
    l_proj = pt_proj;
  else
    {
      fini = pk_false;
      pt = l_proj;
      pt_pred = null;
      while (!fini && pt != null)
	if (en_absc)
	  if (abs1 < pt->ext1_halo.x)
	    {
	      pt_proj->suivant = pt;
	      if (pt_pred == null)
		l_proj = pt_proj;
	      else
		pt_pred->suivant = pt_proj;
	      fini = pk_true;
	    }
	  else
	    {
	      pt_pred = pt;
	      pt = pt->suivant;
	    }
	else if (ord1 < pt->ext1_halo.y)
	  {
	    pt_proj->suivant = pt;
	    if (pt_pred == null)
	      l_proj = pt_proj;
	    else
	      pt_pred->suivant = pt_proj;
	    fini = pk_true;
	  }
	else
	  {
	    pt_pred = pt;
	    pt = pt->suivant;
	  };
      if (!fini)
	pt_pred->suivant = pt_proj;

    }
}


/**
 * calcule les projections des obstacles sur l'axe des abscisses
 *  entre ?xa et ?xb, les obstacles devant se trouver au-dessus de 
 * l'ordonnee ?y  si ?au-dessus est a vrai, en-dessous sinon.
 * La fonction rend dans l_project la liste des projections trouvees.
 */
void
routor::projection_x (int xa, int xb, int y, boolean au_dessus, ptListProjections * l_project)
{

  boolean fini, b, en_absc;
  int abs1, abs2, ord2, ord1;
  acces_gra *pt_obj;
  acces_seg *pt_s;

  l_proj = null;
  fini = pk_false;
  en_absc = pk_true;
  pt_obj = pCurObjManager->recup_l_bte_design ();
  while (pt_obj != null && !fini)
    {
      if (pt_obj->ext1_halo.x > xb)
	fini = pk_true;
      else
	{
	  abs1 = pt_obj->ext1_halo.x;
	  abs2 = pt_obj->ext2_halo.x;
	  ord1 = pt_obj->ext1_halo.y;
	  ord2 = pt_obj->ext2_halo.y;
	  b = alinterieur_strict (abs1, abs2, xa, xb);
	  if (b && ((au_dessus && (ord1 <= y) && (ord2 > y_arr)) || (!au_dessus && (ord2 >= y) && (ord1 < y_arr))))
	    construire_proj (abs1, ord1, abs2, ord2, en_absc);
	  pt_obj = pt_obj->suiv_d;
	}
    };
  pt_s = pCurObjManager->recup_l_seg_design ();
  fini = pk_false;
  while (pt_s != null && fini == pk_false)
    {
      if (pt_s->direc == vertic)
	{
	  if (pt_s->ext1_halo.x > xb)
	    fini = pk_true;
	  else
	    {
	      abs1 = pt_s->ext1_halo.x;
	      abs2 = pt_s->ext2_halo.x;
	      ord1 = pt_s->ext1_halo.y;
	      ord2 = pt_s->ext2_halo.y;
	      b = alinterieur (abs1, abs2, xa, xb);
	      if (b && ((au_dessus && ord1 <= y && ord2 > y_arr) || (!au_dessus && ord2 >= y && ord1 < y_arr)))
		construire_proj (abs1, ord1, abs2, ord2, en_absc);
	    };
	}
      pt_s = pt_s->suiv_d;
    };
  *l_project = l_proj;
}


/**
 * projection sur l'axe des ordonnees entre ?ya et ?yb ; si ?a_droite
   est a vrai, on ne prend que les obstacles qui sont a droite de la ligne
   d'abscisse ?x, sinon on prend les obstacles qui sont a gauche.
   Le resultat est rendu dans l_project.
*/
void
routor::projection_y (int ya, int yb, int x, boolean a_droite, ptListProjections * l_project)
{
  int abs1, ord1, ord2, abs2;
  boolean b, en_absc;
  acces_gra *pt_obj;
  acces_seg *pt_s;

  l_proj = null;
  en_absc = pk_false;
  pt_obj = pCurObjManager->recup_l_bte_design ();
  while (pt_obj != null)
    {
      abs1 = pt_obj->ext1_halo.x;
      abs2 = pt_obj->ext2_halo.x;
      ord1 = pt_obj->ext1_halo.y;
      ord2 = pt_obj->ext2_halo.y;
      b = alinterieur_strict (ord1, ord2, ya, yb);
      if (b && ((a_droite && (abs2 >= x) && (abs1 < x_arr)) || (!a_droite && (abs1 <= x) && (abs2 > x_arr))))
	construire_proj (abs1, ord1, abs2, ord2, en_absc);
      pt_obj = pt_obj->suiv_d;
    };
  pt_s = pCurObjManager->recup_l_seg_design ();
  while (pt_s != null)
    {
      if (pt_s->direc == horiz)
	{
	  abs1 = pt_s->ext1_halo.x;
	  abs2 = pt_s->ext2_halo.x;
	  ord1 = pt_s->ext1_halo.y;
	  ord2 = pt_s->ext2_halo.y;
	  b = alinterieur (ord1, ord2, ya, yb);
	  if (b && ((a_droite && abs2 >= x && abs1 < x_arr) || (!a_droite && abs1 <= x && abs2 > x_arr)))
	    construire_proj (abs1, ord1, abs2, ord2, en_absc);
	};
      pt_s = pt_s->suiv_d;
    };
  *l_project = l_proj;
}


/**
  rend le premier obstacle rencontre entre ?xa et ?xb, a partir de xa,
   l'ordonnee des points etant ?y. s'il y a un obstacle, rend vrai, 
   faux sinon. L'obstacle est rendu dans ?obst.
   Attention xa peut etre inferieur ou superieur a xb.
*/
boolean
routor::obstacle_x (int xa, int xb, int y, typcoord * obst)
{

  boolean b, fini, trouve, envers, prem_obst;
  int abs1, abs2, ord1, ord2;
  acces_gra *pt_obj;
  acces_seg *pt_s;

  fini = pk_false;
  trouve = pk_false;
  prem_obst = pk_true;
  envers = xa > xb;
  pt_obj = pCurObjManager->recup_l_bte_design ();
  while (pt_obj != null && !fini)
    {
      abs1 = pt_obj->ext1_halo.x;
      if (abs1 > MAXI (xa, xb))
	fini = pk_true;
      else
	{
	  abs2 = pt_obj->ext2_halo.x;
	  ord1 = pt_obj->ext1_halo.y;
	  ord2 = pt_obj->ext2_halo.y;
	  b = alinterieur (abs1, abs2, xa, xb);
	  if (b && touche_obst (ord1, ord2, y))
	    {
	      trouve = pk_true;
	      if (!envers)
		fini = pk_true;
	      obst[0].x = abs1;
	      obst[0].y = ord1;
	      obst[1].x = abs2;
	      obst[1].y = ord2;
	      prem_obst = pk_false;
	    };
	  pt_obj = pt_obj->suiv_d;
	}
    };
  pt_s = pCurObjManager->recup_l_seg_design ();
  fini = pk_false;
  while (pt_s != null && !fini)
    {
      if (pt_s->direc == horiz)
	{
	  abs1 = pt_s->ext1_halo.x;
	  abs2 = pt_s->ext2_halo.x;
	  ord1 = pt_s->ext1_halo.y;
	  ord2 = pt_s->ext2_halo.y;
	  b = alinterieur (abs1, abs2, xa, xb);
	  if (b && touche_obst (ord1, ord2, y))
	    {
	      if (prem_obst)
		{
		  prem_obst = pk_false;
		  trouve = pk_true;

		  obst[0].x = abs1;
		  obst[0].y = ord1;
		  obst[1].x = abs2;
		  obst[1].y = ord2;
		  if (!envers)
		    fini = pk_true;
		  if (port_recouv != 0 && port_recouv == pt_s->port_)
		    seg_recouv = pt_s->id;
		  else
		    seg_recouv = 0;

		}
	      else if ((!envers && abs1 < obst[0].x) || (envers && abs1 > obst[0].x))
		{
		  obst[0].x = abs1;
		  obst[0].y = ord1;
		  obst[1].x = abs2;
		  obst[1].y = ord2;
		  if (!envers)
		    fini = pk_true;
		  if (port_recouv != 0 && port_recouv == pt_s->port_)
		    seg_recouv = pt_s->id;
		  else
		    seg_recouv = 0;
		}
	    }
	}
      else if (port_recouv != 0 && pt_s->port_ == port_recouv)
	{
	  abs1 = pt_s->ext1_halo.x;
	  abs2 = pt_s->ext2_halo.x;
	  ord1 = pt_s->ext1_halo.y;
	  ord2 = pt_s->ext2_halo.y;
	  b = alinterieur (abs1, abs2, xa, xb);
	  if (b && touche_obst (ord1, ord2, y))
	    {
	      if (prem_obst)
		{
		  obst[0].x = abs1;
		  obst[0].y = ord1;
		  obst[1].x = abs2;
		  obst[1].y = ord2;
		  prem_obst = pk_false;
		  trouve = pk_true;
		  if (!envers)
		    fini = pk_true;
		  seg_recouv = pt_s->id;
		}
	      else 
		if ((!envers && abs1 < obst[0].x) || (envers && abs1 > obst[0].x))
		  {
		    obst[0].x = abs1;
		    obst[0].y = ord1;
		    obst[1].x = abs2;
		    obst[1].y = ord2;
		    if (!envers)
		      fini = pk_true;
		    seg_recouv = pt_s->id;
		  }
	    }
	}
      pt_s = pt_s->suiv_d;
    }
  return (trouve);
}


/**
  idem que la fonction ci-dessus, mais sur l'axe des ordonnees.
*/
boolean
routor::obstacle_y (int ya, int yb, int x, typcoord * obst)
{
  boolean prem_obst, trouve, b, envers;
  int abs1, abs2, ord1, ord2;
  acces_gra *pt_obj;
  acces_seg *pt_s;

  trouve = pk_false;
  prem_obst = pk_true;
  envers = ya > yb;
  pt_obj = pCurObjManager->recup_l_bte_design ();
  while (pt_obj != null)
    {
      abs1 = pt_obj->ext1_halo.x;
      abs2 = pt_obj->ext2_halo.x;
      ord1 = pt_obj->ext1_halo.y;
      ord2 = pt_obj->ext2_halo.y;
      b = alinterieur (ord1, ord2, ya, yb);
      if (b && touche_obst (abs1, abs2, x))
	{
	  if (prem_obst)
	  {
	    obst[0].x = abs1;
	    obst[0].y = ord1;
	    obst[1].x = abs2;
	    obst[1].y = ord2;
	    trouve = pk_true;
	    prem_obst = pk_false;
	  }
	else
	  {
	    if ((ord2 > obst[1].y && envers) || (ord2 < obst[1].y && !envers))
	      {
		obst[0].x = abs1;
		obst[0].y = ord1;
		obst[1].x = abs2;
		obst[1].y = ord2;
	      }
	  }
	}
      pt_obj = pt_obj->suiv_d;
    }
  pt_s = pCurObjManager->recup_l_seg_design ();
  while (pt_s != null)
    {
      if (pt_s->direc == vertic)
	{
	  abs1 = pt_s->ext1_halo.x;
	  abs2 = pt_s->ext2_halo.x;
	  ord1 = pt_s->ext1_halo.y;
	  ord2 = pt_s->ext2_halo.y;
	  b = alinterieur (ord1, ord2, ya, yb);
	  if (b && touche_obst (abs1, abs2, x))
	    {
	      if (prem_obst)
		{
		  obst[0].x = abs1;
		  obst[0].y = ord1;
		  obst[1].x = abs2;
		  obst[1].y = ord2;
		  trouve = pk_true;
		  prem_obst = pk_false;
		  if (port_recouv != 0 && port_recouv == pt_s->port_)
		    seg_recouv = pt_s->id;
		  else
		    seg_recouv = 0;
		}
	      else
		{
		  if ((envers && ord2 > obst[1].y) || (!envers && ord2 < obst[1].y))
		    {
		      obst[0].x = abs1;
		      obst[0].y = ord1;
		      obst[1].x = abs2;
		      obst[1].y = ord2;
		      trouve = pk_true;
		      if (port_recouv != 0 && port_recouv == pt_s->port_)
			seg_recouv = pt_s->id;
		      else
			seg_recouv = 0;
		    }
		}
	    }
	}
      else if (port_recouv != 0 && pt_s->port_ == port_recouv)
	{
	  abs1 = pt_s->ext1_halo.x;
	  abs2 = pt_s->ext2_halo.x;
	  ord1 = pt_s->ext1_halo.y;
	  ord2 = pt_s->ext2_halo.y;
	  b = alinterieur (ord1, ord2, ya, yb);
	  if (b && touche_obst (abs1, abs2, x))
	    { 
	      if (prem_obst)
		{
		  obst[0].x = abs1;
		  obst[0].y = ord1;
		  obst[1].x = abs2;
		  obst[1].y = ord2;
		  trouve = pk_true;
		  prem_obst = pk_false;
		  seg_recouv = pt_s->id;
		}
	      else
		{
		  if ((envers && ord2 > obst[1].y) || (!envers && ord2 < obst[1].y))
		    {
		      obst[0].x = abs1;
		      obst[0].y = ord1;
		      obst[1].x = abs2;
		      obst[1].y = ord2;
		      trouve = pk_true;
		      seg_recouv = pt_s->id;
		    }
		}
	    }
	}
      pt_s = pt_s->suiv_d;
    };
  return (trouve);
}


/**
  construit un nouveau bout de chemin (jusqu'en ?(x,y)) et l'insere
   dans l'arbre des chemins comme un des fils du chemin courant.
   (procedure appellee par maj_routage).
*/
void
routor::ajout_chem (int x, int y, int sens)
{
  int ct, l;
  ntrees s3, p;

  /*    ct = cout_chem(x,y,sens,x_arr,y_arr,sens_arr); */
  ct = cout_cour + 2;
  p = LL_make_routage (LL_make_abs_atom (x), LL_make_ord_atom (y), null);
  if ((s3 = nchild (3, routage_cour)) == null)
    nchangchild (routage_cour, LL_make_lroutage_list (p), 3);
  else
    ninsertright (1, p, s3);
  l = l_cour + abs (x - x_cour) + abs (y - y_cour);
  inser_l_chem (p, ct, sens, l);

}


/**
   recherche quel est le futur chemin courant; pour cela, on
   parcourt la liste des chemins et on prend le chemin dont le cout est
   minimal ; a cout egal, on garde le chemin courant.
   Les variables x_cour,y_cour,sens_cour et cout_cour sont mises a jour.
   Si la liste des chemins est vide, c'est qu'on n'a pas pu trouver de
   chemin entre A et B.
 */
void
routor::maj_chem_cour (void)
{
  ty_l_chemins *pt, *pt_pred, *pt_pred_cour, *pt_cour;

  if (l_chemins != null)
    {
      cout_cour = l_chemins->cout;
      l_cour = l_chemins->longueur;
      pt_pred = pt_pred_cour = pt_cour = l_chemins;
      pt = l_chemins->suivant;
      while (pt != null)
	{
	  if (pt->cout < cout_cour || (pt->cout == cout_cour && pt->longueur < l_cour))
	    {
	      pt_cour = pt;
	      cout_cour = pt->cout;
	      l_cour = pt->longueur;
	      pt_pred_cour = pt_pred;
	    }
	  pt_pred = pt;
	  pt = pt->suivant;
	};
      if (pt_cour == l_chemins)
	l_chemins = l_chemins->suivant;
      else
	pt_pred_cour->suivant = pt_cour->suivant;
      routage_cour = pt_cour->chemin;
      sens_cour = pt_cour->sens;
      x_cour = (int) nintof (nchild (1, routage_cour));
      y_cour = (int) nintof (nchild (2, routage_cour));
    }
}


/**
  A partir de la liste des projections ?l_projections entre ?borne1 et 
  ?borne2 et du  nouvelle direction ?sens_nouv, calcule tous les nouveaux
  chemins partiels.
  Attention, cas particuliers au niveau des bornes.
*/
void
routor::maj_routage (ty_l_project * l_projections, int borne1, int borne2, int sens_nouv)
{
  boolean premier_tour, traite;
  int x1, x2, y1, y2, x1_suiv, x2_suiv, y1_suiv, y2_suiv;

  if (l_projections == null)
    {
      if (sens_nouv == haut || sens_nouv == bas)
	ajout_chem ((borne1 + borne2) / 2, y_cour, sens_nouv);
      else
	ajout_chem (x_cour, (borne1 + borne2) / 2, sens_nouv);
    }
  else
    {
      premier_tour = pk_true;
      while (l_projections != null)
	{
	  traite = pk_false;
	  if (premier_tour)
	    {
	      x1 = l_projections->ext1_halo.x;
	      x2 = l_projections->ext2_halo.x;
	      y1 = l_projections->ext1_halo.y;
	      y2 = l_projections->ext2_halo.y;
	    }
	  else
	    {
	      x1 = x1_suiv;
	      x2 = x2_suiv;
	      y1 = y1_suiv;
	      if (y2 < y2_suiv)
		y2 = y2_suiv;
	    };
	  l_projections = l_projections->suivant;
	  if (l_projections != null)
	    {
	      x1_suiv = l_projections->ext1_halo.x;
	      x2_suiv = l_projections->ext2_halo.x;
	      y1_suiv = l_projections->ext1_halo.y;
	      y2_suiv = l_projections->ext2_halo.y;
	      if (sens_nouv == bas || sens_nouv == haut)
		{
		  if (premier_tour)
		    {
		      premier_tour = pk_false;
		      if (x1 <= borne1 && x2 > borne1)
			{
			  if (pass_chem (x1_suiv, x2))
			    ajout_chem ((x1_suiv + x2) / 2, y_cour, sens_nouv);
			  else if ((sens_nouv == haut && y1_suiv < y1 &&
				    pass_chem (x2, x2_suiv)) || (sens_nouv == bas && y1 < y1_suiv && pass_chem (x2, x2_suiv)))
			    ajout_chem ((x2 + x2_suiv) / 2, y_cour, sens_nouv);
			  else
			    if ((sens_nouv == haut && y1_suiv > y1 &&
				 pass_chem (borne1, x1_suiv)) || (sens_nouv == bas && y1 > y1_suiv && pass_chem (borne1, x1_suiv)))
			    ajout_chem ((borne1 + x1_suiv) / 2, y_cour, sens_nouv);
			  traite = pk_true;
			}
		      else if (pass_chem (borne1, x1))
			ajout_chem ((borne1 + x1) / 2, y_cour, sens_nouv);
		    }
		  if (!traite)
		    if (pass_chem (x2, x1_suiv))
		      ajout_chem ((x1_suiv + x2) / 2, y_cour, sens_nouv);
		    else if ((sens_nouv == haut && y1_suiv < y1 &&
			      pass_chem (x2, x2_suiv)) || (sens_nouv == bas && y1 < y1_suiv && pass_chem (x2, x2_suiv)))
		      ajout_chem ((x2 + x2_suiv) / 2, y_cour, sens_nouv);
		    else if (pass_chem (x1, x1_suiv))
		      ajout_chem ((x1 + x1_suiv) / 2, y_cour, sens_nouv);
		}
	      else
		{		/* cas ou on se dirige vers la gauche ou la droite */
		  if (premier_tour)
		    {
		      premier_tour = pk_false;
		      if (borne1 > y1 && borne2 < y2)
			/* cas ou l'obstacle deborde des 2 cotes des bornes */
			{
			  if (sens_cour == bas)
			    ajout_chem (x_cour, y2 + taille_virage, sens_nouv);
			  else
			    ajout_chem (x_cour, y1 - taille_virage, sens_nouv);
			  traite = pk_true;
			}
		      else if (y1 <= borne1 && y2 > borne1)
			{
			  if (pass_chem (y2, y1_suiv))
			    ajout_chem (x_cour, (y1_suiv + y2) / 2, sens_nouv);
			  else
			    if ((sens_nouv == droite && x1_suiv > x1 &&
				 pass_chem (y2, y2_suiv)) || (sens_nouv == gauche && x1 > x1_suiv && pass_chem (y2, y2_suiv)))
			    ajout_chem (x_cour, (y2 + y2_suiv) / 2, sens_nouv);
			  else
			    if ((sens_nouv == droite && x1_suiv < x1 &&
				 pass_chem (borne1, y1_suiv)) || (sens_nouv == gauche && x1 < x1_suiv && pass_chem (borne1, y1_suiv)))
			    ajout_chem (x_cour, (borne1 + y1_suiv) / 2, sens_nouv);
			  traite = pk_true;
			}
		      else if (pass_chem (borne1, y1))
			ajout_chem (x_cour, (borne1 + y1) / 2, sens_nouv);
		    }
		  if (!traite)
		    if (pass_chem (y2, y1_suiv))
		      ajout_chem (x_cour, (y1_suiv + y2) / 2, sens_nouv);
		    else
		      if ((sens_nouv == droite && x1_suiv > x1 &&
			   pass_chem (y2, y2_suiv)) || (sens_nouv == gauche && x1 > x1_suiv && pass_chem (y2, y2_suiv)))
		      ajout_chem (x_cour, (y2 + y2_suiv) / 2, sens_nouv);
		    else if (pass_chem (y1, y1_suiv))
		      ajout_chem (x_cour, (y1 + y1_suiv) / 2, sens_nouv);
		}
	    }
	  else /* cas ou il n'y a plus (ou pas) de projection */ if (sens_nouv == haut || sens_nouv == bas)
	    if (premier_tour)
	      {
		if (pass_chem (borne1, x1))
		  ajout_chem ((borne1 + x1) / 2, y_cour, sens_nouv);
		if (pass_chem (x2, borne2))
		  ajout_chem ((x2 + borne2) / 2, y_cour, sens_nouv);
		else		/* rajout 11/9/98 cas d'un seul obstacle qui deborde
				   des 2 cotes des bornes   A REVOIR */ if (pass_chem (borne1, borne2))
		  ajout_chem ((borne1 + borne2) / 2, y_cour, sens_nouv);
	      }
	    else
	      {
		if (pass_chem (x2, borne2))
		  ajout_chem ((x2 + borne2) / 2, y_cour, sens_nouv);
		else if (x2 > borne2 && pass_chem (x1, borne2))
		  ajout_chem ((x1 + borne2) / 2, y_cour, sens_nouv);
	      }
	  else
	    {
	      if (borne1 > y1 && borne2 < y2)
		if (sens_cour == bas)
		  ajout_chem (x_cour, y2 + taille_virage, sens_nouv);
		else
		  ajout_chem (x_cour, y1 - taille_virage, sens_nouv);
	      else
		{
		  if (premier_tour)
		    {
		      if (pass_chem (borne1, y1))
			ajout_chem (x_cour, (borne1 + y1) / 2, sens_nouv);
		      if (pass_chem (y2, borne2))
			ajout_chem (x_cour, (y2 + borne2) / 2, sens_nouv);
		    }
		  else
		    {
		      if (pass_chem (y2, borne2))
			ajout_chem (x_cour, (y2 + borne2) / 2, sens_nouv);
		      if (y2 > borne2 && pass_chem (y1, borne2))
			ajout_chem (x_cour, (y1 + borne2) / 2, sens_nouv);
		    }
		}
	    }
	}
    };
  maj_chem_cour ();
}



/**
  fonction recursive qui cherche un chemin entre A et B.
   Le point d'arrivee est fixe ?(xb,yb), c'est le point de depart qui
   avance ?(xa,ya). Si, entre 2 appels a calcul_routage, (xa,ya) n'a
   pas bouge, la fonction rend faux, elle n'a pas trouve de chemin.
   Sinon, la fonction s'arrete lorsqu'elle atteint (xb,yb) et rend vrai.
*/
boolean
routor::calcul_routage (int xa, int ya, int sensa, int xb, int yb, int sensb)
{
  boolean sens_horiz, b, au_dessus, a_droite;
  int x, y, nouv_sens, x_min, y_min, x_max, y_max;
  typcoord obst[2];
  ty_l_project *l_project;

  if (sensa == gauche || sensa == droite)
    {
      sens_horiz = pk_true;
      au_dessus = (ya > yb);
      if (au_dessus)
	nouv_sens = haut;
      else
	nouv_sens = bas;
    }
  else
    {
      sens_horiz = pk_false;
      a_droite = (xa < xb);
      if (a_droite)
	nouv_sens = droite;
      else
	nouv_sens = gauche;
    }
  if (sens_horiz)
    {
      if ((xa > xb && sensa == droite) || (xa < xb && sensa == gauche))
	{
	  if (sensa == droite)
	    {
	      x_min = xa;
	      x_max = abs_mil_cadre + lg_bte_stand - taillepave_x;
	      x = taille_virage;
	      b = obstacle_x (x_min, x_max, ya, obst);
	    }
	  else
	    {
	      x_min = decal_cadre_x + taillepave_x;
	      x_max = xa;
	      x = -taille_virage;
	      b = obstacle_x (x_max, x_min, ya, obst);
	    };
	  if (!b)
	    {
	      projection_x (x_min, x_max, ya, au_dessus, &l_project);
	      if (l_project == null)
		{
		  ajout_chem (xa + x, ya, nouv_sens);
		  maj_chem_cour ();
		}
	      else
		maj_routage (l_project, x_min, x_max, nouv_sens);
	    }
	  else
	    {
	      if (seg_recouv != 0)
		{
		  calcul_seg_recouv (xa, ya, sens_horiz, obst);
		  return (pk_true);
		}
	      else
		{
		  if (xa < xb)
		    x = obst[1].x;
		  else
		    x = obst[0].x;
		  x_min = MINI (xa, x);
		  x_max = MAXI (xa, x);
		  projection_x (x_min, x_max, ya, au_dessus, &l_project);
		  maj_routage (l_project, x_min, x_max, nouv_sens);
		}
	    };
	  if (x_cour == xa && y_cour == ya)
	    return (pk_false);
	  else
	    return (calcul_routage (x_cour, y_cour, sens_cour, xb, yb, sensb));
	}
      else
	b = obstacle_x (xa, xb, ya, obst);
    }
  else
    b = obstacle_y (ya, yb, xa, obst);

  if (!b)			/* cas ou il n'y a pas d'obstacle entre a et b  */
    {
      if (sens_horiz)
	b = obstacle_y (ya, yb, xb, obst);
      else
	b = obstacle_x (xa, xb, yb, obst);
      if (!b)
	{
	  if (sens_horiz)
	    maj_chem (xb, ya, sensa);
	  else
	    maj_chem (xa, yb, sensa);
	  return (pk_true);
	}
      else if (seg_recouv != 0)	/* cas ou l'obstacle est un segment qui 
				   arrive sur le port diffuseur */
	{
	  if (sens_horiz)
	    {
	      maj_chem (xb, ya, sensa);
	      calcul_seg_recouv (xb, ya, pk_false, obst);
	    }
	  else
	    {
	      maj_chem (xa, yb, sensa);
	      calcul_seg_recouv (xa, yb, pk_true, obst);
	    }
	  return (pk_true);
	}
      else
	{
	  if (sens_horiz)
	    {
	      if (sensa == droite)
		{
		  x_min = xa;
		  x_max = abs_mil_cadre + lg_bte_stand - taillepave_x;
		  x = taille_virage;
		  b = obstacle_x (x_min, x_max, ya, obst);
		}
	      else
		{
		  x_min = decal_cadre_x + taillepave_x;
		  x_max = xa;
		  x = -taille_virage;
		  b = obstacle_x (x_max, x_min, ya, obst);
		};
	      if (b)
		{
		  if (sensa == gauche)
		    x = obst[1].x;
		  else
		    x = obst[0].x;
		  x_min = MINI (xa, x);
		  x_max = MAXI (xa, x);
		};
	      projection_x (x_min, x_max, ya, au_dessus, &l_project);
	      maj_routage (l_project, x_min, x_max, nouv_sens);
	    }
	  else
	    {
	      if (sensa == haut)
		{
		  y_max = ya;
		  y_min = decal_cadre_y + taillepave_y;
		  y = -taille_virage;
		}
	      else
		{
		  y_min = ya;
		  y_max = ord_mil_cadre + ht_bte_stand - taillepave_y;
		  y = taille_virage;
		};
	      b = obstacle_y (y_min, y_max, xa, obst);
	      if (!b)
		{
		  projection_y (y_min, y_max, xa, a_droite, &l_project);
		  if (l_project == null)
		    {
		      ajout_chem (xa, ya + y, nouv_sens);
		      maj_chem_cour ();
		    }
		  else
		    maj_routage (l_project, y_min, y_max, nouv_sens);
		}
	      else
		{
		  if (ya < yb)
		    y = obst[0].y;
		  else
		    y = obst[1].y;
		  y_min = MINI (ya, y);
		  y_max = MAXI (ya, y);
		  projection_y (y_min, y_max, xa, a_droite, &l_project);
		  maj_routage (l_project, y_min, y_max, nouv_sens);
		}
	    };
	  if (x_cour == xa && y_cour == ya)
	    return (pk_false);
	  else
	    return (calcul_routage (x_cour, y_cour, sens_cour, xb, yb, sensb));
	}
    }
  else /* cas ou il y  a un obstacle  */ if (seg_recouv != 0)	/* cas ou l'obstacle est un segment qui arrive
								   sur le port diffuseur */
    {
      calcul_seg_recouv (xa, ya, sens_horiz, obst);
      return (pk_true);
    }
  else
    {
      if (sens_horiz)
	{
	  if (xa < xb)
	    x = obst[0].x;
	  else
	    x = obst[1].x;
	  x_min = MINI (xa, x);
	  x_max = MAXI (xa, x);
	  projection_x (x_min, x_max, ya, au_dessus, &l_project);
	  maj_routage (l_project, x_min, x_max, nouv_sens);
	}
      else
	{
	  if (ya < yb)
	    y = obst[0].y;
	  else
	    y = obst[1].y;
	  y_min = MINI (ya, y);
	  y_max = MAXI (ya, y);
	  projection_y (y_min, y_max, xa, a_droite, &l_project);
	  maj_routage (l_project, y_min, y_max, nouv_sens);
	};
      if (x_cour == xa && y_cour == ya)
	return (pk_false);
      else
	return (calcul_routage (x_cour, y_cour, sens_cour, xb, yb, sensb));
    }
}
