/*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */

_PUBLIC_
///
  void
supervis::endBuild ()
{
  BC = the_pKBatchCompiler;
  OM = the_pKBatchCompiler->getOptionManager ();	// Option manager

  this_SUPERVISEUR = new SUPERVISEUR ();
  pGraph_text = new graph_text (the_graphicalTrees, the_graphicTablesManager);
  pDraw = new drawing ();
  pCurObjManager = new currentObjectsManager (the_graphicalTrees, the_graphicTablesManager);
  pDisplay = new SignalDisplaying (pDraw, the_graphicalTrees, the_graphicTablesManager, pCurObjManager, the_messageTool);
  pUndo = new undo (the_graphicalTrees, the_graphicTablesManager, pCurObjManager);
  pBuilder = new builder (pUndo, the_graphicalTrees, the_graphicTablesManager, pCurObjManager, pDisplay, pDraw, the_messageTool);
  ok_pre_comp = pk_false;
  debut_pre_comp = pk_true;
  err_nom_vide = pk_false;
  obj_cour = 0;
  is_prag = pk_false;
  is_param = pk_false;
  is_loc_decl = pk_false;
  is_embedded = pk_false;


}

/**
   Constructor for the main window; 
   Should be called only once
   create the_graphicTablesManager, the_graphicalTrees, 
*/
supervis::supervis ()
{
  // ntrees root;
  // pLiaison = pLs;                               // pLS is null
  the_graphicTablesManager = new graphicTablesManager ();
  the_graphicalTrees = new graphicalTrees (the_graphicTablesManager);
  the_messageTool = new message ();

  the_pKBatchCompiler = new BatchCompiler ();
  the_pKBatchCompiler->BC_initialize (pKGUI);
//  the_graphClipBoard.arb=NILTREE;
//  the_graphClipBoard.typ=typ_vide;
//  the_graphClipBoard.opproc=false;
  endBuild ();
  // root = the_graphicalTrees->getRoot(); NOT A ROOT ???
  // pCurObjManager->set_arb_complet(root);
  // pCurObjManager->set_bte_cou_engl(NILTREE); // for the root, no associated box
  // nDEBUG_subtree(4, 8," exiting supervis::supervis(ptLiaisonSignal pLs) root is", NILTREE);
}


/**
   Constructor for all the windows, except the first one.
   pGraphArb and pMess are common for all the windows
*/
supervis::supervis (ptGraphicTrees pGraphArb, ptGraphicTablesManager pTablesManager, ptMessage pMess)
{
  ntrees root;

  // pLiaison = pLs;
  the_graphicTablesManager = pTablesManager;
  the_graphicalTrees = pGraphArb;
  the_messageTool = pMess;
  endBuild ();
  root = the_graphicalTrees->getRoot ();
  pCurObjManager->set_arb_complet (root);
  pCurObjManager->set_bte_cou_engl (NILTREE);	// for the root, no associated box
  // nDEBUG_subtree(4, 8," exiting supervis::supervis(ptLiaisonSignal pLs, ...) root is", root);
}



/**
  Les fonctions suivantes sont appelées du monde Java (par 
  l'intermédiaire de LiaisonSignal.java et  LiaisonSignal.c
*/
ptGraphicTrees
supervis::getGraphicTrees (void)
{
  return (the_graphicalTrees);
}


///
ptBuilder
supervis::getBuilder (void)
{
  return (pBuilder);
}


///
ptGraphicTablesManager
supervis::getTabManager (void)
{
  return (the_graphicTablesManager);
}

///
ptCurrentObjectsManager
supervis::getCurObjManager (void)
{
  return (pCurObjManager);
}

///
ptMessage
supervis::getMessage (void)
{
  return (the_messageTool);
}


// PLG replaced by the following call
// ptSUPERVISEUR supervis::getSignalSuperv(void)
// {
//   return(this_SUPERVISEUR);
// }


///
boolean
supervis::superviseurStep (void)
{
  return (this_SUPERVISEUR->SUPERVISEUR_step ());
}


///
void
supervis::bouge_souris (int x, int y)
{
  switch (en_cours)
    {
    case creation:
      pBuilder->buildBox (tirer, x, y);
      break;
    case encapsulage:
      pBuilder->encapsuler_bte (tirer, x, y);
      break;
    case dimension:
      pBuilder->taille_bte (tirer, x, y, pk_false);
      break;
    case deplacement:
      pBuilder->depl_obj (tirer, x, y);
      break;
    case collage:
      pBuilder->paste_obj (tirer, x, y);
      break;
    }
}


/**
   Fonction appelee directement par JAVA lorsqu'il faut rafraichir la
   fenetre (evenement d'exposure : mise au premier plan de la fenetre par exemple )
 */
void
supervis::rafraichir_sans_garb (void)
{
  pDisplay->display (pk_true);
  pDisplay->display_buff_select (select_buffer, TBS);
}


/**
  Initialisations for one window
  should return true if it succeeds
*/
void
supervis::localInit ()
{
  ptrepstr signalstr;
  char *dirsignal;
  static char adresse[TBUF];

  dirsignal = getEnvDef ("Signal_bin");
  signalstr = string2repstr ("SIGNAL");
  pBuilder->init_construct ();
  pCurObjManager->initCurObjManager ();
  pDisplay->init_decomp ();
  filebuf = new buffer (10, 60);
  appel_comp = dirsignal;
  SigaliCmdForPolychrony = getEnvDef ("SigaliCmdForPolychrony");
  strcpy (adresse, dirsignal);
  strcpy (nom_fich_load, "");

  nom_fich_parser = sysGetnameTempfile ();
  nom_fich_compil = sysGetnameTempfile ();

  this_SUPERVISEUR->SUPERVISEUR_initialize ();	/* initialisation de l'automate ecrit en Signal */

  waitForParameters = pk_false;
  debut_pre_comp = pk_true;
  CurrentOutputDirectory = NIL_String;
  strcpy (previous_options, "signal -POLY");
}


/// Global Initialisations : one time called (for the first window)
void
supervis::globalInit ()
{
  pK_SignalPrinter->setPrinterContext (pk_true);
  the_graphicalTrees->initGraphicTrees ();
  ntrees_setExistingDataPersistent ();
  localInit ();
  initInterfSignalC (the_messageTool);
  initBiblio (the_graphicalTrees, the_graphicTablesManager, the_messageTool);
}


/// Call when the user clicks on the button "reset"
void
supervis::SignalReset ()
{
  the_pKBatchCompiler->reset ();
  pCurObjManager->initCurObjManager ();
  pBuilder->init_construct ();
  resetBiblio ();
  CurrentOutputDirectory = NIL_String;
}

/**
   Called when the user quit the Polychrony GUI.
   The temporary files are deleted.
*/
void
supervis::SignalExit ()
{
  remove (nom_fich_parser);
  remove (nom_fich_compil);
  pK_messenger->gmessend ();
}

/** Eclipse --
    include the model or module filePathString in this initial environment
    @param filePathString is assumed to be a readable file path  s
    @return statusOK or if the action failed a status ERR_readableFile or ERR_gpkFileContent 
    int supervis::loadFileInitPk(int action, char* filePathString)
{ 
  if ( isReadableFile(filePathString) ) {
    CurrentOutputDirectory = sys_extract_dirPath(filePathString, pk_true);
    return load_model(filePathString, action, pk_true);  // @return status among ERR_gpkFileContent, statusOK
  }
  return ERR_readableFile;
}
 */
