/*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */

_PRIVATE_
#define GM_ERR 1
/*  constantes definissant les differents types de compilation */
#define ty_norm_comp 1
#define ty_embed_comp 2
#define ty_archi_comp   3
// codage des boutons de la souris
#define J_left 0
#define J_middle 1
#define J_right 2
#define J_ctrl_middle 3
/**
   @return 
   - 1 by default and when vsuff is SIG or sig
   - 2 when vsuff is SSME or ssme
*/
int
supervis::getUnparseOperation (ptrepstr vsuff)
{
  if (vsuff == nilstr)
    return 1;
  if (streq (vsuff, string2repstr ("SIG")))
    return 1;
  if (streq (vsuff, string2repstr ("sig")))
    return 1;
  if (streq (vsuff, string2repstr ("SSME")))
    return 2;
  if (streq (vsuff, string2repstr ("ssme")))
    return 2;
  return 1;
}

/**
  Cette fonction rend vrai si la chaine ?filePathString contient la chaine ".gpk", rend faux sinon.
*/
boolean
supervis::isGraphicalFile (char *filePathString)
{
  return (strstr (filePathString, ".gpk") != NULL);
}

boolean
supervis::isCorrectFileName (char *filePathString)
{
  if (filePathString == null)
    return pk_false;
  return filePathString[strlen (filePathString) - 1] != '/';
}

/** check for existing readable file filePathString 
    @return false for (ERR_readableFile error)
 */
boolean
supervis::isReadableFile (char *filePathString)
{
  fich_p = fopen (filePathString, "r");
  if (fich_p == null)
    return pk_false;
  fclose (fich_p);
  return pk_true;
}

boolean
supervis::is_dotSuffixed (const char *stringModel, int lgString, const char *suffix, int lgSuffix)
{
  char *stringTail;
  if (lgString <= lgSuffix)
    return pk_false;
  stringTail = (char *) (stringModel + lgString - lgSuffix - 1);
  if (stringTail[0] == '.')
    return (strcmp ((char *) (stringTail + 1), suffix) == 0);
  return pk_false;


}

/**
   Positionnement l'action en cours (ne concerne que les actions ou on 
   bouge la souris : creation, deplacement, changement de taille...
*/
void
supervis::set_action_en_cours (int act_cour)
{
  en_cours = act_cour;
}

/* ===========================================================================
 * MAIN IO ACTIONS CALLED IN LoadSaveAction
 */

/** return a file pathname such that
    let SUFFIX =   '.' followed by suffix
    if pathString is a directory pathname (empty or terminated by '/') 
         if suffix is empty the file pathname is pathString followed by the name of proc
	 if suffix is not empty 
             the file pathname is pathString followed by the name of proc followed by SUFFIX
    if pathString is a file pathname 
         if suffix is empty the file pathname is pathString 
	 if suffix is not empty 
             if pathString is suffixed by SUFFIX the file pathname is pathString 
	     if pathString is not suffixed by SUFFIX the file pathname is pathString followed by SUFFIX
 */
char *
supervis::build_filePathName (const char *pathString, const char *suffix, ntrees proc)
{
  char *string2;
  int lgPath = (int) strlen (pathString);
  int lgSuffix = (int) strlen (suffix);
  if (lgPath > 0)
    {
      if (pathString[lgPath - 1] != '/')
	{
	  if (lgSuffix == 0)
	    {
	      sprintf (fich_sig, "%s", pathString);
	      return fich_sig;
	    }
	  if (is_dotSuffixed (pathString, lgPath, suffix, lgSuffix))
	    {
	      sprintf (fich_sig, "%s", pathString);
	      return fich_sig;
	    }
	  sprintf (fich_sig, "%s.%s", pathString, suffix);
	  return fich_sig;
	}
      string2 = repstr2string (STreeModelModule_getNameStr (proc));
      if (lgSuffix == 0)
	sprintf (fich_sig, "%s%s", pathString, string2);
      else
	sprintf (fich_sig, "%s%s.%s", pathString, string2, suffix);
      return fich_sig;
    }
  string2 = repstr2string (STreeModelModule_getNameStr (proc));
  if (lgSuffix == 0)
    sprintf (fich_sig, "%s", string2);
  else
    sprintf (fich_sig, "%s.%s", string2, suffix);
  return fich_sig;
}

/**
   sauvegarde de l'arbre partiel sous forme graphique dans le fichier de 
   nom ?filePathString. On sauvegarde l'arbre proc_cou_engl qui est le processus 
   courant.
*/
/** @return status among ERR_fileSaving, statusOK
 */
int
supervis::save_model (char *filePathString, ntrees sourceTree)
{
  char *nom;

  // nDEBUG_subtree(0, 10, "entering supervis::save_model", sourceTree);
  if (STree_isModel (sourceTree) || STree_isModule (sourceTree))
    {
      nom = build_filePathName (filePathString, "gpk", sourceTree);
      if (!the_graphicalTrees->GraphicSaving (nom, sourceTree)) return ERR_fileSaving;
    }
  // nDEBUG_subtree(0, 10, " exiting supervis::save_model", sourceTree);
  return statusOK;
}

/**
   sauvegarde de l'arbre partiel sous forme graphique dans le fichier de 
   nom ?filePathString. On sauvegarde l'arbre proc_cou_engl qui est le processus 
   courant.
   @return status among ERR_fileSaving, statusOK
*/
int
supervis::save_model (char *filePathString)
{
  ntrees sourceTree;
  int res;

  sourceTree = pCurObjManager->getModelInWhich_cou_engl ();
  res = save_model (filePathString, sourceTree);	// @return status among ERR_fileSaving, statusOK
  pBuilder->nettoyage ();
  return res;
}

/**
   Sauvegarde de l'arbre partiel sous forme textuelle dans le fichier de nom ?filePathString
   Avant d'appeler le decompilateur Signal, on appelle la fonction
   "pGraph_text->completeTree" qui rajoute les renommages, masquages...
*/
int
supervis::unparseAs_model (char *filePathString)
{
  ntrees toBeUnparsed, copyToBeUnparsed;
  char *nom;
  ptrepstr str_nom_prog;
  int codeOp;
  boolean res;

  toBeUnparsed = pCurObjManager->getModelInWhich_cou_engl ();
  ntrees_memoryStoreIndex ();
  copyToBeUnparsed = the_graphicalTrees->dupl_arbre (toBeUnparsed);
  nom = build_filePathName (filePathString, "SIG", copyToBeUnparsed);
  str_nom_prog = string2repstr (nom);
  pGraph_text->completeTree (copyToBeUnparsed);
  filebuf->setsize (margeGaucheMax, lgligne);
  // The isf format will be used to save the graphical program (it is not an unparse operation, but the "save".
  codeOp = getUnparseOperation (sysGetFileExtension (filePathString));
  if (codeOp == 1)
    res = pK_SignalPrinter->fileUnparse (copyToBeUnparsed, filebuf, str_nom_prog);
  else if (codeOp == 2)
    res = pK_SSME_Printer->fileUnparse (copyToBeUnparsed, filebuf, str_nom_prog);
  else
    res = pk_false;
  ntrees_memoryRestoreIndex ();
  if (res) return statusOK;
  return ERR_unparseFile;
}

/**
  Chargement d'un programme Signal sous forme graphique contenu dans le 
  fichier ?nom_fich. Le fichier peut contenir soit un processus, soit une 
  liste de  processus.
  Au moment du chargement,
        si la fenetre est vide et si le fichier contient un processus, alors
	il est charge en tant que programme.
	si la fenetre est vide et si le fichier contient une liste de 
	processus, alors on cree un processus vide et on insere la liste de 
	processus dans la zone de ses processus locaux.
	si la fenetre est non vide (il y a deja un programme), on insere le
	processus ou la liste de processus dans la zone des processus locaux
	du processus courant. S'il s'agit d'un seul processus, l'utilisateur
	clique dans la zone pour construire la boite qui contiendra le 
	processus.
	S'il s'agit d'une liste, les processus sont inseres dans les  
	positions ou ils avaient ete sauvegardes.
*/

#include "C_string.h"

/** include the model or module filePathString in this environment
    @param filePathString is assumed to be a readable file path 
    @return statusOK or if the action failed a status ERR_gpkFileContent 
 */
int
supervis::load_model (char *filePathString, int action, boolean setInAtEnd)
{
  ntrees father, loaded_ntree;

  // printf("DEBUG entering supervis::load_model file name is <<%s>>, action is %i, J_load is %i\n", filePathString, action, J_load); fflush(stdout);
  strcpy (nom_fich_load, filePathString);
  SET_NILTREE (loaded_ntree);
  if (ntrees_load (nom_fich_load, &loaded_ntree))
    {
      switch (action)
	{
	case J_load:
	  the_graphicalTrees->upgrade (loaded_ntree);
	  father = pCurObjManager->getModelInWhich_cou_engl ();
	  if (STree_isModel (loaded_ntree))
	    {
	      pBuilder->insertGraphicalModel (loaded_ntree, father, pk_false, setInAtEnd);
	    }
	  else if (STree_isModule (loaded_ntree))
	    if (pCurObjManager->is_racine (father))
	      {  // PLG Nov 3 message missing
		pBuilder->insertGraphicalModule (loaded_ntree, father);
	      }

	    else
	      return ERR_insertModule;
	}
      first_save (nom_fich_load, loaded_ntree);
      // printf("DEBUG  exiting supervis::load_model 1\n"); fflush(stdout);
      return statusOK;
    }
  // printf("DEBUG  exiting supervis::load_model *\n"); fflush(stdout);
  return ERR_gpkFileContent;
}

///
//  
/** include the library filePathString in this environment
    @param filePathString is assumed to be a readable file path
    @return statusOK or if the action failed a status among ERR_gpkFileContent, ERR_LibraryContent, 
 */
int
supervis::load_library (char *filePathString)
{
  char *fn;
  ntrees p_inser;

  strcpy (nom_fich_load, filePathString);
  fn = sys_extract_dirPath (filePathString, pk_false);
  if (C2Java_isLoadedLibrary (filePathString))
    return statusOK;
  if (ntrees_load (nom_fich_load, &p_inser))
    {
      if (buildLib (filePathString, p_inser, fn))
	return statusOK;
      return ERR_LibraryContent;
    }
  return ERR_gpkFileContent;
}

/** include the parsing of filePathString in this environment
    @param filePathString is assumed to be a readable file path
    this method apply parsing to module, model, library, solver, list of modules and/or models
    @return statusOK or if the action failed a status among ERR_noGarbage, ERR_parsing 
 */
int
supervis::parse_model (char *filePathString, int action, boolean setInAtEnd)
{
  // PLG to be completed wrt errors
  ntrees mpp, locTree;
  strcpy (nom_fich_load, filePathString);
  if (treeparser (nom_fich_load, pk_false, &mpp))
    {
      if (STree_isModel (mpp) || STree_isModulesList (mpp))
	{
	  if (mpp == (ntrees) GM_ERR)
	    {
	      printf ("Garbage not implemented\n");
	      fflush (stdout);
	      lancer_garbage ();
	      return ERR_noGarbage;
	    }
	  else
	    {
	      locTree = pCurObjManager->getModelInWhich_cou_engl ();
	      if (STree_isModel (mpp))
		{		// le fichier contient un modèle
		  pBuilder->insertTextualModel (mpp, locTree, setInAtEnd);
		  return statusOK;
		}
	      /* le fichier contient une liste de modèles */
	      pBuilder->insertTextualModule (mpp, locTree);
	      return statusOK;
	    }
	}
      // PLG ERROR ??
      // nDEBUG_subtree(4, 100," exiting supervis::parse_model 1FTTF, parsed tree is", mpp);
    }
  return ERR_parsing;
}

/* ===========================================================================
 * MAIN IO ACTION SET
 */

/**
  Fonction appelee apres le lancement d'une compilation ou apres une 
  sauvegarde textuelle  : comme l'arbre a ete modifie, on reinitialise
  tout et on recharge l'arbre tel qu'il etait avant sauf si la racine est
  nulle (dans ce cas, on a appele "compile" alors que la fenetre etait vide).
*/
void
supervis::postcomp (void)
{
  if (!err_nom_vide)
    {
      pK_SignalPrinter->setPrinterContext (pk_true);	/* obligatoire si on ne veut pas voir tous les commentaires du compilateur affiches dans l'arbre */
      pGraph_text->cleanMarkedTree (pCurObjManager->getModelInWhich_cou_engl ());
    }
}

/**
  Des le chargement d'un programme Signal, on fait une sauvegarde de ce 
  programme dans "nom_fichier.bak"
  Fonction appelee par les fonctions load_model et parse_model.
*/
void
supervis::first_save (char *filePathString, ntrees vt)
{
  if (!the_graphicalTrees->GraphicSaving (filePathString, vt))
    the_messageTool->error_mess (ERR_fileSaving);
}

/**
  Lancement du garbage collector pour recuperer de la place dans la memoire
  d'arbres : cette fonction est appelee soit lorsque le chargement d'un 
  programme a echoue par manque de place (fonctions load_model, parse_model), 
  soit lorsque l'utilisateur clique sur le bouton "refresh" et si le 
  garbage est necessaire.
*/
void
supervis::lancer_garbage ()
{
  ntrees p1, p2, p3, p4, p5;

  // printf("DEBUG entering supervis::lancer_garbage\n"); fflush(stdout);
  p1 = pCurObjManager->get_racine ();
  p2 = pCurObjManager->get_arb_cou_engl ();
  p3 = pCurObjManager->get_bte_cou_engl ();
  p4 = pCurObjManager->getModelInWhich_cou_engl ();
  SET_NILTREE (p5);
  the_messageTool->error_mess (recupmemnch);
  ntrees_recup (&p1, &p2, &p3, &p4, &p5);
  if (p1 != null) the_graphicalTrees->reconstruire_tab (p1, pk_false, pk_false, 0, 0);
  BC->reset ();
  pCurObjManager->set_racine (p1);
  pCurObjManager->set_arb_cou_engl (p2);
  pCurObjManager->set_bte_cou_engl (p3);
  pCurObjManager->set_proc_cou_engl (p4);
  // printf("DEBUG  exiting supervis::lancer_garbage\n"); fflush(stdout);
}


/**
  Fonction qui appelle l'analyseur syntaxique sur le fichier de nom 
  ?nom_fich. L'arbre cree s'appelle mpp (variable globale).
  La fonction rend vrai si l'analyse est correcte, faux sinon.
  ATTENTION: parfois l'analyseur ne rend pas d'erreur mais un arbre vide!
*/
boolean
supervis::treeparser (char *nom_fich, boolean addedPhylum, ntrees * mpp)
{
  boolean not_err, b;
  integer vph;

  if (addedPhylum)
    vph = currentPhylum;
  else
    vph = without_phylum;
  not_err = sysparserGUI (vph, nom_fich, mpp);
  b = !not_err || (IS_NILTREE (*mpp));
  if (b)
    {
      char *x = pK_messenger->getInfoParser (gStringErrorParser, PARSER_MESSAGE_SIZE);
      the_messageTool->ParsingError (x);
    }
  pK_messenger->parsing_logFileReset ();
  return !b;
}


/**
  Sauvegarde du contenu de la fenetre texte dans le fichier de nom
  tmpparse."no de processus". C'est ce fichier qui est passe a 
  l'analyseur syntaxique.
*/
boolean
supervis::mkParserFile (text_win_type w)
{
  char *chaine;

  // printf("DEBUG entering supervis::mkParserFile"); fflush(stdout);
  fich_p = fopen (nom_fich_parser, "w");
  if (fich_p == null)
    {
      the_messageTool->error_mess (ERR_fileOpening);
      return (pk_false);
    }

  chaine = C2Java_getText (w);
  fwrite (chaine, 1, strlen (chaine), fich_p);
  fprintf (fich_p, "\n");
  fclose (fich_p);
  // printf("DEBUG exiting supervis::mkParserFile"); fflush(stdout);
  return (pk_true);
}

/**
  Method called by the method CONFIRMING when the user has entered text in the
  textual window associated with the hierarchy window.
 */
void
supervis::manageHierarTextWin (ntrees proc_cour, ntrees mpp)
{
  switch (memAction)
    {
    case C_pragmaDeclaration:
      STree_setPragmas (proc_cour, mpp);
      break;
    case C_formalParameters:
      // ATTENTION IL MANQUE LE PUNDO->UNDO
      STree_setParameters (proc_cour, mpp);
      break;
    case C_inOutDeclaration:
      pBuilder->set_profil (the_graphicalTrees->no_boite_proc (proc_cour), mpp);
      C2Java_refresh (pk_true);
      break;
    case C_specification:
      // ATTENTION IL MANQUE LE PUNDO->UNDO
      STree_setExternalGraph (proc_cour, mpp);
      break;
    case C_localDeclaration:
      pUndo->get_undo_sigloc (STreeModel_getLocalDeclarations (proc_cour));
      STree_replaceLocalDecl (proc_cour, mpp);
      break;
    default:
      ;
    }
  // C2Java_textHide(C_HierarText);
}

/** 
   Display the program parameters in the appropriate window.
   If the parameter file exists, the file is displayed.
*/
void
supervis::displayParam (char *parameterFileName)
{
  int c, i;
  char s[6000];			//??? (size)
  FILE *fich;
  char *fichPar;
  // printf("DEBUG entering supervis::displayParam\n"); fflush(stdout);
  fichPar = getCompleteNameCurrentDir (parameterFileName);
  sysUpdateParametersFile (fichPar);
  fich = fopen (fichPar, "r");
  // C2Java_resetText(C_ParamText);
  C2Java_textShow (C_ParamText);
  // Diplay file content
  if (fich != NULL)
    {
      i = 0;
      while ((c = getc (fich)) != EOF)
	{
	  s[i] = c;
	  i++;
	}
      s[i] = '\00';
      C2Java_textAppend (C_ParamText, s);
      fclose (fich);
    }
  waitForParameters = pk_true;
}



///
void
supervis::setPhylum (integer phylum)
{
  currentPhylum = phylum;
}

/** Saves the current waited phylum and prints the tree p in w window. */
void
supervis::decompTextWin (text_win_type w, ntrees p)
{
  int ope;
  boolean toutdec;
  ntrees p_fils;

  // printf("DEBUG entering supervis::decompTextWin, w is %i\n", (int)w); fflush(stdout);

  toutdec = pk_true;
  if (IS_NILTREE (p))
    setPhylum (ph_ol_dec_s);
  else
    {
      ope = noper (p);
      switch (ope)
	{
	case opldecl:
	  setPhylum (ph_ol_decl);
	  break;
	case opnil:		//when ope is opnil, it may be an external graph or a declaration (a port)
	  if (STree_isSpec (p))
	    setPhylum (ph_o_spec);
	  else
	    setPhylum (ph_ol_dec_s);
	  break;
	case opldecs:
	case opdecsig:
	  setPhylum (ph_ol_dec_s);
	  break;
	case opspec:
	  setPhylum (ph_o_spec);
	  break;
	default:
	  if (STree_isExprProc (ope))
	    {
	      if (ope == oplfeed)
		{
		  p_fils = nchild (1, p);
		  if (p_fils != null)
		    if (nattrntrees (p_fils, GRAPHI_ATTRNUMBER) != null)
		      toutdec = pk_false;
		}
	      if (toutdec)
		setPhylum (ph_e_proc);
	    }
	}
    }
  if (toutdec)
    pDisplay->decompTextWin (w, p);
  // printf("DEBUG  exiting supervis::decompTextWin, w is %i\n", (int)w); fflush(stdout);
}


 /** affiche le profil d'une boite dans la fenêtre texte */
void
supervis::edit_profil (text_win_type CanvasText, int obj)
{
  setPhylum (ph_ent_sort);
  pDisplay->edit_profil (CanvasText, obj);
}

/** It returns "true" if the text of the w window is a SIGNAL text, in this 
   case !mpp is the abstract tree of this text */
boolean
supervis::mkTreeFromWindowText (text_win_type w, ntrees * mpp)
{
  // printf("DEBUG entering supervis::mkTreeFromWindowText \n"); fflush(stdout);
  if (mkParserFile (w))
    {
      // printf("DEBUG  exiting supervis::mkTreeFromWindowText as treeparser \n"); fflush(stdout);
      return treeparser (nom_fich_parser, pk_true, mpp);
    }
  the_messageTool->error_mess (seminccor);
  // printf("DEBUG  exiting supervis::mkTreeFromWindowText FALSE \n"); fflush(stdout);
  return pk_false;
}

///
char *
supervis::getCompleteNameCurrentDir (char *vname)
{
  char *dirName;

  if (CurrentOutputDirectory == NIL_String)
    dirName = allocate_string ( (int) strlen (vname) + 1);
  else
    {
      dirName = allocate_string ( (int) strlen (CurrentOutputDirectory) + (int) strlen (vname) + 1);
      strcpy (dirName, CurrentOutputDirectory);
    }
  strcat (dirName, vname);
  return dirName;
}

///
void
supervis::setCompilerOutputDir (ntrees p)
{
  ptrepstr vstr;

  srcTree = p;
  if (ISNOT_NILTREE (p))
    {
      vstr = STreeModelModule_getNameStr (p);
      OM->set_outputDirectory (getCompleteNameCurrentDir (repstr2string (vstr)), pk_false);
      sys_set_userOutputDir (vstr);
    }
}

///
void
supervis::DebugPrint (ntrees p)
{
  ptrepstr str_nom_prog;

  return;
  str_nom_prog = string2repstr ("toto");
  filebuf->setsize (margeGaucheMax, lgligne);
  pK_SignalPrinter->fileUnparse (p, filebuf, str_nom_prog);
}


/** @return true if the model pComp is ready for compiling, false otherwise
   Currently, only some verifications on the name of the model are realized.
*/
boolean
supervis::isNameReadyForCompiling (ntrees pComp)
{
  ptrepstr x;
  x = STreeModelModule_getNameStr (pComp);
  if (x == nilstr)
    return pk_false;
  return (x != the_graphicalTrees->chaine_vide ());
}
