/*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */

_PUBLIC_
/**
   @return le mode de compilation : automatique/defaut, automatique/options ou interactif:
   - pour interactif : bauto=faux , bdirect=faux
   - pour automatique/defaut : bauto=vrai , bdirect=vrai
   - pour automatique/option : bauto=vrai , bdirect=faux
*/
  void
supervis::GET_COMPILING_MOD (boolean * bauto, boolean * bdirect)
{
  // printf("DEBUG entering supervis::GET_COMPILING_MOD valbut is %i\n", valBut); fflush(stdout);
  if (valBut == J_inter_comp)
    {
      *bauto = pk_false;
      *bdirect = pk_false;
    }
  else if (valBut == J_auto_comp)
    {
      *bauto = pk_true;
      *bdirect = pk_false;
    }
  else
    {
      *bauto = pk_true;
      *bdirect = pk_true;
    }
}

/**
   Méthode appelée du monde Java lorsque l'utilisateur clique sur le bouton "OK" de la fenêtre des paramètres.
   Elle recopie le contenu de la fenêtre contenant les parametres 
   dans un fichier et appelle l'analyseur syntaxique de Signal.
   Un message est émis sur une erreur de syntaxe.
*/
void
supervis::okParam (char *str)
{
  char c;
  int i;
  char *fichPar;

  C2Java_textHide (C_ParamText);
  fichPar = getCompleteNameCurrentDir (fich_par);
  fich_p = fopen (fichPar, "w");
  i = 0;
  while ((c = str[i]) != '\00')
    {
      i++;
      putc (c, fich_p);
    }
  fprintf (fich_p, "\n");
  fclose (fich_p);

  setPhylum (ph_ol_e_para);
  err_pre_comp = !treeparser (fichPar, pk_true, &p_param);
  // on regarde si l'arbre est bien du bon type
  if (err_pre_comp)
    the_messageTool->error_mess (syntinccor);
  else
    err_pre_comp = !STree_isEffectiveParameter (p_param);
  ok_pre_comp = pk_true;
  this_SUPERVISEUR->SUPERVISEUR_step ();	// Why here and not in Java Code ?
  waitForParameters = pk_false;
}

/**
   Méthode appelée du monde Java lorsque l'utilisateur clique sur le
   bouton "NO PARAMETER" de la fenêtre parametres.
*/
void
supervis::WithoutParam ()
{
  ok_pre_comp = pk_true;
  SET_NILTREE (p_param);
  this_SUPERVISEUR->SUPERVISEUR_step (); // Why here and not in Java Code ?
  waitForParameters = pk_false;
}


/**
   Cette méthode est appelée par les superviseur (Signal) avant la compilation d'un programme représenté par un arbre (pComp).
   Elle vérifie que le programme peut être compilé (arbre non vide, ...) puis appelle la méthode ok_comp() qui complète l'arbre 
   en vue de la compilation.
   @param pComp (in) : arbre Signal devant être compilé.
   @param typ_comp (in) : type de compilation  (normale, embedded, architecture)
   @param p_copie (out) : pComp modifié par ok_comp().
   @param err (out) : vrai si erreur, faux sinon
   @param ok (out) : vrai quand la pré-compilation est terminée, "faux" sinon (?).
*/
void
supervis::PRE_COMP (ntrees pComp, integer typ_comp, ntrees * p_copie, boolean * err, boolean * ok)
{
  if (debut_pre_comp)
    {
      debut_pre_comp = pk_false;
      waitForParameters = pk_false;
      err_nom_vide = pk_false;
      if ((pComp != null) && (pComp != pCurObjManager->get_racine ()))
	if (isNameReadyForCompiling (pComp))
	  {
	    *p_copie = ok_comp (pComp, typ_comp);
	    *ok = ok_pre_comp;
	    *err = err_pre_comp;
	  }
	else
	  {
	    the_messageTool->error_mess (nommproc);
	    *err = pk_true;
	    err_nom_vide = pk_true;
	    *ok = pk_true;
	  }
      else
	{
	  the_messageTool->error_mess (procinexist);
	  *err = pk_true;
	  *ok = pk_true;
	}
    }
  else
    {
      *ok = ok_pre_comp;
      *err = err_pre_comp;
    }
}


/**
   Construction du graphe associé à l'arbre (p) compilé.
   @param p (in): arbre à compiler
   @param igraph (out) graphe construit si err est à "faux"
   @param err (out) à vrai s'il y a eu  une erreur, "faux" sinon.
*/
void
supervis::GRAPH_BUILD (ntrees p, ptDCG_basic * igraph, boolean * err)
{
  int i, ii;
  ptDCG vgraph;

  i = pK_messenger->gmessload (systmess);
  C2Java_displayCompilerDiag ();
  setTraceabilityMode (OM->req_gen_cosimuTable_opt ());
  setCompilerOutputDir (p);
  ii = SIGNALTree2DCG (p, p_param, pk_false, &vgraph);
  *err = (ii != 0);

  if (!(*err))
    (*igraph) = new DCG_basic (vgraph);
  else
    STree_resetVMAnnotations (p);
}


/**
   Fin de la compilation : fonction appelée lorsque 
   - l'utilisateur a cliqué sur le bouton "end" de la fenêtre de compilation,
   - ou dès qu'il y a eu une erreur dans la compilation.
*/
void
supervis::END_COMP (void)
{
  ok_pre_comp = pk_false;
  debut_pre_comp = pk_true;
  postcomp ();
  is_embedded = pk_false;		/* pour le TEST */
}


/**
   Fin de la compilation : fonction appelée lorsque qu'une erreur est détectée en cours de compilation.
   Elle appelle END_COMP() avant de fermer les fenêtres de compilation (C2Java_compilerWinClose()).
*/
void
supervis::END_COMP_ERR (void)
{
  END_COMP ();
  C2Java_compilerWinClose ();
}


/**
   Fin de la compilation d'une architecture (succès): le résultat est restitué à l'utilisateur dans la fenêtre graphique
   @see END_COMP(), PartitioningRunOn().
*/
void
supervis::END_COMP_ARCHI (ptDCG_bool vgraph)
{
  END_COMP ();
  PartitioningRunOn (vgraph);
}


/**
   Fin de la compilation d'une architecture (échec): fonction appelée lorsque qu'une erreur est détectée en cours de compilation.
   @see END_COMP().
*/
void
supervis::END_COMP_ERR_ARCHI ()
{
  END_COMP ();
  C2Java_compilerWinClose ();
  /*
     if (!err_nom_vide) 
     { // restauration de l'arbre
     pUndo->set_undo_archi(pCurObjManager->getModelInWhich_cou_engl());
     C2Java_refresh();
     }
   */
}

/**
   Fin de la compilation embarquée (succès): compilation embarquée "non implémentée".
*/
void
supervis::END_COMP_EMBED (ntrees p_proc_embed)
{
  END_COMP ();
}


/**
   Fin de la compilation embarquée (échec): compilation embarquée "non implémentée".
*/
void
supervis::END_COMP_ERR_EMBED (ntrees p_proc_embed)
{
  END_COMP ();
  C2Java_compilerWinClose ();
}


/**
   Compilation automatique (mode option): préparation à la sélection des options pour application sur un graphe (g).
   Affichage des options (WIN_DISPLAY_AUTO()).
   @return toujours "vrai".
*/
boolean
supervis::AUTO_COMPILING_ACTIONS (ptDCG_basic g)
{
  vgraph_auto = g;
  WIN_DISPLAY_AUTO ();
  return pk_true;
}


/**
   Compilation automatique (mode option) : affichage de la fenêtre présentant les différentes options du compilateur "batch" Signal.
*/
boolean
supervis::WIN_DISPLAY_AUTO (void)
{
  C2Java_displayAutoCompilerFunct ();
  return (pk_true);
}


/**
   Compilation automatique (mode option) : méthode appelée lorsque l'utilisateur a cliqué 
   de la fenêtre de compilation automatique (compile->option access") sur
        - sur le bouton "GO" (ok=true)
        - ou sur le bouton "QUIT" (ok=false) 
   
   ou bien sur le  "compile->default"
*/
void
supervis::endAutoComp (boolean ok, char *voptions)
{
  if (ok)
    {
      BC->SignalBatchCompilerGUIPop (STreeModelModule_getNameStr (srcTree), (ptDCG_basic) vgraph_auto, voptions);
      // the_pKBatchCompiler->SignalBatchCompilerGUIPop(STreeModelModule_getNameStr(srcTree), (ptDCG_basic) vgraph_auto, voptions);
      strcpy (previous_options, voptions);
    }
  END_COMP ();
}

/** 
    Compilation automatique (mode options):
    cette méthode positionne l'option du compilateur "batch" selon numBut.
*/
void
supervis::set_COMPILER_OPTION (integer numBut)
{
  integer numOption;
    
  numOption = numBut;
  switch (numOption)
    {
    case 0:			/* Boolean2events */
      OM->switch_ctrl2event_opt ();
      break;

    case 1:			/* Retiming       */
      OM->switch_run_retiming_opt ();
      break;

    case 2:			/* Abstraction    */
      OM->switch_run_abstraction_opt ();
      break;

    case 3:			/* Clusters      */
      OM->switch_clustering_opt ();
      break;

    case 4:			/* Separate booleans/others */
      OM->switch_run_booleanSeparation_opt ();
      break;

    case 5:			/* Delay unMulptiplexing */
      OM->switch_run_delayMultiplexing_opt ();
      break;

    case 6:			/* Unifications  */
      OM->switch_run_signalUnification_opt ();
      break;

    case 7:			/* Substitute */
      OM->switch_run_signalSubstitution_opt ();
      break;

    case 8:			/* Don't apply clock calc */
      OM->switch_solvingDepth_opt ();
      break;

    case 9:			// booleanizing: boolean format (bdc+)
      OM->switch_make_bool_opt ();
      break;

    case 10:			// flattening the hierarchy (sts) 
      OM->switch_make_flat_opt ();
      break;

    case 11:			// Serialization (sbDC+) 
      OM->switch_make_sequential_opt ();
      break;

    case 12:			// C code  
      OM->switch_export_c_modules_opt ();
      break;

    case 13:			// C++ code
      OM->switch_export_cpp_modules_opt ();
      break;

    case 14:			/*  Java code                 */
      OM->switch_export_java_modules_opt ();
      break;

    case 15:			/*  check  Add code for assertions */
      OM->switch_export_check_subopt ();
      break;

    case 16:			/*  force code generation (if no cycle) */
      OM->switch_force_subopt ();
      break;

    case 17:			/*  Add code for simulation (C/C++ generator) */
      OM->switch_gen_cosimuTable_opt ();
      break;

    case 18:			/*  Gen Code with threads (C/C++ generator) */
      OM->switch_export_threads_subopt ();
      break;

    case 19:			/* Signal Code */
      OM->switch_trace_opt ();
      break;

    case 20:			/* z3z (sigali) */
      OM->switch_export_z3z_opt ();
      break;

    case 21:			/* Syndex */
      OM->switch_export_syndex_opt ();
      break;

    case 22:			/* profiling */
      OM->switch_export_profiling_opt ();
      break;

    case 23:			/* udo :Don't apply delay optimizations" */
      OM->switch_run_delayOptimizing_opt ();
      break;
    }
}


/**
   Compilation automatique (mode défaut).
   Appel du compilateur en utilisant les options précédentes.
   Par default (ou -POLY par default).
*/
boolean
supervis::DIRECT_COMPILING (ptDCG_basic g)
{
  // printf("DEBUG entering supervis::DIRECT_COMPILING\n"); fflush(stdout);
  vgraph_auto = g;
  C2Java_displayCompilerDiag ();
  // printf("DEBUG      in  supervis::DIRECT_COMPILING\n"); fflush(stdout);
  endAutoComp (pk_true, previous_options);
  // printf("DEBUG  exiting supervis::DIRECT_COMPILING\n"); fflush(stdout);
  return (pk_true);
}


/**
   Not yet implemented.
   @return "true"
 */
boolean
supervis::EMBEDDED_COMPILING (ntrees vt)
{
  return pk_true;
}
