/*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */

///
_PUBLIC_

/* valeur rendue lorsque l'utilisateur n'a rien cliqué dans les fenêtres de compilation */
#define NOTHING -2

/* valeur rendue lorsque l'utilisateur a cliqué sur le bouton "END"  */
#define END -1

/**
  Les fonctions qui suivent sont des fonctions d'entree appelees par l'automate
  ecrit en Signal pour savoir sur quel bouton a clique l'utilisateur. 
  valBut (lu dans le monde Java) correspond au dernier bouton du menu cliqué 
  par l'utilisateur.
*/
boolean supervis::MOUSE_DOWN_IN_GE (void)
{
  boolean B;

  B = C2Java_isMouseButtonClicked ();	/* les boutons de la souris geres en JAVA */
  return (B);
}


/**
   returns the last clicked Button
*/
int
supervis::ReadMenuButton (void)
{
  if (waitForParameters)
    return valBut;

  valBut = C2Java_readMenuButton ();	// lecture du dernier bouton cliqué (vient du monde Java)
  C2Java_razMenuButton ();
  return (valBut);
}


///
boolean
supervis::SAVE_DOWN (void)
{
  return ((valBut == J_saveAllRoots) || (valBut == J_saveAllRootsAs) ||
	  (valBut == J_saveAs) || (valBut == J_save) || (valBut == J_unparseAs) || (valBut == J_save_PS));
}


///
boolean
supervis::BUILD_RESOLV_DOWN (void)
{
  return (valBut == J_incl_prov);
}


/**
   Cette fonction rend le numero de l'objet dans !obj et son type !typ_obj
   qui se trouve a la position ?x,?y (s'il en existe un)
*/
void
supervis::GET_OBJ (int x, int y, int *obj, objet_type * typ_obj)
{
  objet_type temp_typ_obj;

  *obj = pCurObjManager->reper_obj (x, y, &temp_typ_obj);
  *typ_obj = temp_typ_obj;
}


/**
   Cette fonction rend dans !x,!y les positions du reticule
*/
void
supervis::GET_XY (mouse_type e, int *x, int *y)
{
  *x = C2Java_getX ();
  *y = C2Java_getY ();
}


/**
  Cette fonction rend le processus sélectionné dans la fenÃªtre de hiérarchie
*/
ntrees
supervis::getSelectedProc ()
{
  return (pCurObjManager->getModelInWhich_cou_engl ());
}


/**
   Cette fonction rend le type du bouton de la souris sur lequel a clique l'utilisateur : 
   - left : bouton gauche, 
   - middle: bouton du milieu,
   - right : bouton de droite, 
   - crtl_middle : bouton du milieu et touche crtl enfoncé.
*/
mouse_type
supervis::GET_BUTTON (void)
{
  int val;
  mouse_type button;

  val = C2Java_getMouseButton (); // vient du monde JAVA
  switch (val)
    {
    case J_left:
      button = left;
      break;
    case J_middle:
      button = middle;
      break;
    case J_right:
      button = right;
      break;
    case J_ctrl_middle:
      button = ctrl_middle;
      break;
    }
  return (button);
}


///
boolean
supervis::PASTE_BOITE (void)
{
  return (pBuilder->typ_buf_paste () == typ_bte);
}


/**
   procedures de sortie appelees par SIGNAL :
   execution des actions associees aux boutons
*/

void
supervis::PASTING (int action, int x, int y)
{
  set_action_en_cours (collage);
  switch (action)
    {
    case 1:
      pUndo->init_undo ();
      pBuilder->paste_obj (appuyer, x, y);
      C2Java_setMoveMouseEvent ();
      break;
    case 2:
      C2Java_resetMoveMouseEvent ();
      pBuilder->paste_obj (relacher, x, y);
      break;
    }
}

///
void
supervis::COPYING (int x, int y)
{
  pUndo->init_undo ();
  pBuilder->copy_obj (x, y);
}


///
void
supervis::COPYING_ICON (int x, int y)
{
  copyingIcon (pBuilder, x, y);
  // C2Java_refresh(pk_false);
}

///
void
supervis::DISPLAYINGINFO_ICON ()
{
  pDisplay->display_CurrentLibSelectedModel ();
}

///
void
supervis::removeLibrary ()
{
  rm_Library ();
}

/** return the status among ERR_readableFile, ERR_gpkFileContent, statusOK
 */
int
supervis::reloadLibrary ()
{
  ntrees vt;
  char *fn = C2Java_getSelectedFilenameLibrary ();

  if (isReadableFile (fn))
    {
      if (ntrees_load (fn, &vt))
	{
	  install_library (vt, fn);
	  return statusOK;
	}
      return ERR_gpkFileContent;
    }
  return ERR_readableFile;

}


///
void
supervis::CREATION_BOITE (int x, int y, int action)
{
  set_action_en_cours (creation);
  switch (action)
    {
    case 1:
      pUndo->init_undo ();
      pBuilder->buildBox (appuyer, x, y);
      C2Java_setMoveMouseEvent ();
      break;
    case 2:
      C2Java_resetMoveMouseEvent ();
      pBuilder->buildBox (relacher, x, y);
      break;
    }
}


///
void
supervis::CREATION_LIEN (int x, int y)
{
  pUndo->init_undo ();
  pBuilder->cr_lien (x, y);
}


///
void
supervis::CREATION_PORT (int x, int y)
{
  pUndo->init_undo ();
  pBuilder->cr_port (x, y);
}


///
void
supervis::JUSTIFYING (int but, int etat, int x, int y)
{
  switch (but)
    {
    case 1:
      pUndo->init_undo ();
      pBuilder->justifier_ports (etat, x, y);
      break;
    case 2:
      pBuilder->justifier_ports (etat, x, y);
      break;
    }
}


///
void
supervis::GLUING (int action, int x, int y)
{
  set_action_en_cours (encapsulage);
  switch (action)
    {
    case 1:
      pUndo->init_undo ();
      pBuilder->encapsuler_bte (appuyer, x, y);
      C2Java_setMoveMouseEvent ();
      break;
    case 2:
      C2Java_resetMoveMouseEvent ();
      pBuilder->encapsuler_bte (relacher, x, y);
      break;
    }
}


///
void
supervis::RESIZING (int action, int x, int y)
{
  set_action_en_cours (dimension);
  switch (action)
    {
    case 1:
      pUndo->init_undo ();
      pBuilder->taille_bte (appuyer, x, y, force);
      C2Java_setMoveMouseEvent ();
      break;
    case 2:
      C2Java_resetMoveMouseEvent ();
      pBuilder->taille_bte (relacher, x, y, force);
      break;
    }
}


///
void
supervis::MOVING (int action, int x, int y)
{
  set_action_en_cours (deplacement);
  switch (action)
    {
    case 1:
      pUndo->init_undo ();
      pBuilder->depl_obj (appuyer, x, y);
      C2Java_setMoveMouseEvent ();
      break;
    case 2:
      C2Java_resetMoveMouseEvent ();
      pBuilder->depl_obj (relacher, x, y);
      break;
    }
}


///
void
supervis::BREAKING (int x, int y)
{
  pUndo->init_undo ();
  pBuilder->decapsuler_bte (x, y);
}


///
void
supervis::CUTTING (int x, int y)
{
  pUndo->init_undo ();
  pBuilder->detruire_obj (x, y);
}


///
void
supervis::razSelectBuf (void)
{
  int i;
  for (i = 0; i < TBS; i++)
    select_buffer[i] = 0;
  // C2Java_refresh(pk_false);
}


///
void
supervis::SELECTING (int x, int y)
{
  set_action_en_cours (selectionnage);
  pBuilder->select_obj (select_buffer, TBS, x, y);
}


///
void
supervis::MAPPING_BTE (int x, int y)
{
  if (select_buffer[0] == 0)
    the_messageTool->error_mess (warmap);	/* le buffer de selection est vide */
  else
    {
      pUndo->init_undo ();
      pBuilder->mapper_bte (select_buffer, x, y);
    }
}


///
void
supervis::END_MAPPING (void)
{
  pBuilder->end_mapper_bte ();
}


///
void
supervis::MAPPING_LIEN (int but, int etat, int x, int y)
{
  pBuilder->mapper_lien (etat, x, y);
}


///
void
supervis::changeModelCat (ntrees t, action_type ty_ch)
{
  pUndo->init_undo ();
  pBuilder->chang_typ_model (t, ty_ch);
}

void
supervis::RECUP_IDENTS (ptident * ident1, ptident * ident2)
{
  C2Java_getIdentsForReplace (ident1, ident2);
}


void
supervis::REPLACING_IDENT (ntrees p, ptident ident1, ptident ident2)
{
  if ((ident1 != nilstr) && (ident2 != nilstr))
    {
      pBuilder->ReplaceIdent (p, ident1, ident2, select_buffer);
      razSelectBuf ();
    }
}


///
void
supervis::SEE_PROFIL (int obj)
{
  obj_cour = obj;
  edit_profil (C_CanvasText, obj); /* affiche le profil d'une boite dans la fenÃªtre texte */
}


///
void
supervis::SHOW_HALO (int obj, objet_type typ_obj)
{
  pBuilder->visualiser_halo (force, obj, typ_obj);
}


// Association d'un nom à un port : fonction appelée aprÃ¨s que l'utilisateur ait associé une
// expression de processus Ã  une boite; pour chaque identificateur de l'expression, 
// l'utilisateur peut cliquer sur un port ou sur le bord de la boite en (x, y) et ainsi nommer
// son port. Les 2 listes d'identificateurs, en entrée et en sortie sont gérées dans builder.
/// rend vrai s'il n'y a plus de ports Ã  nommer.
boolean
supervis::VALIDER_PORT (int x, int y, int obj, objet_type typ_obj)
{
  boolean b;
  C2Java_showPortHide ();
  b = pBuilder->valider_port_nom (obj, x, y, typ_obj);
  return (b);
}


// Association d'un nom Ã  un port : fonction appelée lorsque l'utilisateur clique avec le
// bouton du milieu --> on saute cet identificateur et on passe au suivant.
/// rend vrai s'il n'y a plus de ports Ã  nommer.
boolean
supervis::PASSER_PORT (int passe)
{
  C2Java_showPortHide ();
  return (pBuilder->passer_port_nom ());
}


/// Fin de l'association de nom aux ports : on cache les fenêtres texte utilisées
void
supervis::CACHE_TEXTE (void)
{
  C2Java_textHide (C_CanvasText);
  C2Java_showPortHide ();
  pDisplay->restitue_nomport ();
}


///
void
supervis::GO_UP (int remonte)
{
  ntrees p;
  // printf("DEBUG entering supervis::GO_UP \n"); fflush(stdout);
  pBuilder->nettoyage ();
  pUndo->init_undo ();
  pCurObjManager->remonter (pk_false);
  C2Java_refresh (pk_false);
  pDisplay->display_buff_select (select_buffer, TBS);
  C2Java_textHide (C_CanvasText);
  C2Java_resetText (C_CanvasText);
  p = pCurObjManager->get_arb_cou_engl ();
  if (ISNOT_NILTREE (p))
    decompTextWin (C_CanvasText, p);
  obj_cour = 0;
  // printf("DEBUG  exiting supervis::GO_UP \n"); fflush(stdout);
}


///
boolean
supervis::GO_DOWN (int obj)
{
  // printf("DEBUG entering supervis::GO_DOWN \n"); fflush(stdout);
  obj_cour = obj;
  pUndo->init_undo ();
  pCurObjManager->descendre (obj, pk_false);
  if (the_graphicalTrees->isAPort (obj))
    pDisplay->display_nom_port (the_graphicTablesManager->graphObj_get_tree (obj));
  else
    C2Java_refresh (pk_false);
  pDisplay->display_buff_select (select_buffer, TBS);
  C2Java_resetText (C_CanvasText);
  decompTextWin (C_CanvasText, pCurObjManager->get_arb_cou_engl ());
  // printf("DEBUG  exiting supervis::GO_DOWN \n"); fflush(stdout);
  return (pk_false);
}

/// Affiche les pragmas du processus p dans la fenetre texte dédiée
void
supervis::DISPLAYING_PRAGMAS (void)
{
  ntrees p;
  memAction = C_pragmaDeclaration;
  p = pCurObjManager->getModelInWhich_cou_engl ();
  if (ISNOT_NILTREE (p))
    {
      setPhylum (ph_ol_prag);
      pDisplay->edit_pragmas (p);
    }
}

/// Affiche les paramÃ¨tres du processus p dans la fenetre texte dédiée
void
supervis::seeingParam (ntrees p)
{
  // printf("DEBUG entering supervis::seeingParam \n"); fflush(stdout);
  memAction = C_formalParameters;
  decompTextWin (C_ParamText, STree_getParameters (p));
  // printf("DEBUG  exiting supervis::seeingParam \n"); fflush(stdout);
}


///
void
supervis::CREATING_SPEC (ntrees p)
{
  ntrees t, tspec;

  // printf("DEBUG entering supervis::CREATING_SPEC \n"); fflush(stdout);
  memAction = C_specification;
  SET_NILTREE (tspec);
  t = STree_getInterface (p);
  if (ISNOT_NILTREE (t))
    {
      tspec = nchild (3, t);
      if (IS_NILTREE (tspec))
	{
	  tspec = STree_mkEmptyTree ();
	  STree_setExternalGraph (p, tspec);
	}
    }
  decompTextWin (C_HierarText, tspec);
  // printf("DEBUG  exiting supervis::CREATING_SPEC \n"); fflush(stdout);
  //decompTextWin(C_HierarText, STree_getSpec(p));
}


/// Affiche les déclarations locales du processus p dans la fenetre texte associée aux processus
void
supervis::seeingLocDecl (ntrees p)
{
  memAction = C_localDeclaration;
  decompTextWin (C_HierarText, STreeModel_getLocalDeclarations (p));
}

/// Affiche les entrées/sorties du processus p dans la fenetre texte associée aux processus
void
supervis::seeingInOut (ntrees p)
{
  memAction = C_inOutDeclaration;
  edit_profil (C_HierarText, the_graphicalTrees->no_boite_proc (p));
}


void
supervis::TRACE (char *message, action_type action)
{
  int act = C_no_action;
  switch (action)
    {
    case C_creer:
      act = J_creer;
      break;
    case C_cut:
      act = J_cut;
      break;
    case C_cut_graph:
      act = J_cut_graph;
      break;
    case C_copy:
      act = J_copy;
      break;
    case C_copy_graph:
      act = J_copy_graph;
      break;
    case C_paste:
      act = J_paste;
      break;
    case C_paste_graph:
      act = J_paste_graph;
      break;
    case C_encaps:
      act = J_encaps;
      break;
    case C_decaps:
      act = J_decaps;
      break;
    case C_bte_map:
      act = J_bte_map;
      break;
    case C_link_map:
      act = J_link_map;
      break;
    case C_justif:
      act = J_justif;
      break;
    case C_resize:
      act = J_resize;
      break;
    case C_move:
      act = J_move;
      break;
    case C_selection:
      act = J_selection;
      break;
    case C_halo:
      act = J_halo;
      break;
    case C_confirm:
      act = J_confirm;
      break;
    case C_embed:
      act = J_embed;
      break;
    case C_copy_Icon:
      act = J_copy_Icon;
      break;
      // J_info_Icon ???
    case C_not_impl:
      act = J_not_impl;
      break;
    case C_nop:
      act = J_nop;
      break;
    case C_no_action:
      act = J_no_action;
      break;
    default:
      printf ("TRACE %s is %i, NOT a canvas action \n", message, act);
      fflush (stdout);
      return;

    }
  printf ("TRACE %s is %i\n", message, act);
  fflush (stdout);
}

/**
  Affichage de l'action courante ?action dans le bandeau 
  pour envoyer Ã  JAVA, on est obligé de passer du type énuméré aux
  cstes entiÃ¨res.
*/
void
supervis::AFFICHAGE (action_type action)
{
  int act;
  switch (action)
    {
    case C_creer:
      act = J_creer;
      break;
    case C_cut_graph:
      act = J_cut_graph;
      break;
    case C_copy_graph:
      act = J_copy_graph;
      break;
    case C_paste_graph:
      act = J_paste_graph;
      break;
    case C_encaps:
      act = J_encaps;
      break;
    case C_decaps:
      act = J_decaps;
      break;
    case C_bte_map:
      act = J_bte_map;
      break;
    case C_link_map:
      act = J_link_map;
      break;
    case C_justif:
      act = J_justif;
      break;
    case C_resize:
      act = J_resize;
      break;
    case C_move:
      act = J_move;
      break;
    case C_selection:
      act = J_selection;
      break;
    case C_halo:
      act = J_halo;
      break;
    case C_confirm:
      act = J_confirm;
      break;
    case C_embed:
      act = J_embed;
      break;
    case C_copy_Icon:
      act = J_copy_Icon;
      break;
      // J_info_Icon ???
    case C_not_impl:
      act = J_not_impl;
      break;
    case C_nop:
    case C_no_action:
      act = J_nop;
      break;
    default:
      act = -1;
    }
  C2Java_displayCurrentAction (act);

}

///
void
supervis::UNDOING (void)
{
  pUndo->applyUndo ();
  pCurObjManager->setCurrentObjectsList ();
  C2Java_refresh (pk_false);
}


///
#define SEUILGARBAGE 524288	/* 0.5 Mo */
void
supervis::REFRESHING (void)
{
  boolean no_garbage = pk_true;

  if (ntrees_notEnoughSpace (SEUILGARBAGE))
    {
      lancer_garbage ();
      no_garbage = pk_false;
    }
  // ATTENTION PB LORSQU'IL Y A GARBAGE : NON TRAITE !!!
  C2Java_refresh (pk_false);
}


///
void
supervis::WINDOW_TEXTING (void)
{
  // printf("DEBUG entering supervis::WINDOW_TEXTING\n"); fflush(stdout);
  C2Java_textShow (C_CanvasText);
}


/** It Parses the text of the "last" active window (Signal text), then 
   the abstract tree is attached to the contextual/current tree.
*/
action_type
supervis::CONFIRMING ()
{
  ntrees proc_cour, mpp;
  boolean ClickNameNecessary, ok;
  text_win_type w;

  // printf("DEBUG entering supervis::CONFIRMING \n"); fflush(stdout);
  ClickNameNecessary = pk_false;
  ok = pk_false;
  w = C2Java_getLastActiveTextWin ();
  // printf("DEBUG      in  supervis::CONFIRMING 0 w is %i \n", w); fflush(stdout);
  if (mkTreeFromWindowText (w, &mpp))
    {
      pUndo->init_undo ();
      proc_cour = pCurObjManager->getModelInWhich_cou_engl ();
      if (w == C_ParamText)
	{
	  manageHierarTextWin (proc_cour, mpp);
	  C2Java_textHide (C_ParamText);
	}
      else if ((w == C_HierarText) || (w == C_IOText))
	{
	  manageHierarTextWin (proc_cour, mpp);
	}
      else if (STree_isInputOutput (mpp))
	{
	  ok = pBuilder->set_profil (obj_cour, mpp);
	  C2Java_textHide (C_CanvasText);
	}
      else
	{
	  if (pCurObjManager->isCurrentObjAPort ())
	    ok = pBuilder->setDeclToPort (mpp, obj_cour);
	  else
	    {
	      obj_cour = the_graphicalTrees->tree_get_graphObj (pCurObjManager->get_bte_cou_engl ());
	      if (obj_cour == 0)
		ok = pk_false;
	      else
		ok = pBuilder->setTextToBox (mpp, obj_cour, &ClickNameNecessary);
	      if (!ok)
		the_messageTool->error_mess (seminccor);
	    }
	  obj_cour = 0;
	  if (!ClickNameNecessary)
	    {
	      C2Java_resetText (C_CanvasText);
	      C2Java_textHide (C_CanvasText);
	    }
	}
    }
  if (ok)
    C2Java_refresh (pk_true);
  if (ClickNameNecessary)
    return (C_cliqportnom);
  // printf("DEBUG  exiting supervis::CONFIRMING \n"); fflush(stdout);
  return (C_confirm);
}

///
void
supervis::DISPLAYING_POLICE (action_type act)
{
  C2Java_displayFont ();
}


///
void
supervis::DISPLAYING_NAMES (boolean nom_port)
{
  pDisplay->misajour_nomport (nom_port);
  C2Java_refresh (pk_false);
}


///
void
supervis::DISPLAYING_DETAILS (boolean detail)
{
  force = detail;
  pDisplay->misajour_detail (detail);
  C2Java_refresh (pk_false);
}


///
void
supervis::EMBEDDED_CREATING (integer bte, ntrees * t, boolean * ok)
{
  ntrees p_proc;

  pUndo->init_undo ();
  p_proc = pBuilder->build_embedded_proc (bte);
  *ok = (ISNOT_NILTREE (p_proc));
  *t = p_proc;
}


///
ntrees
supervis::GET_RACINE (void)
{
  return (pCurObjManager->get_racine ());
}


///
ntrees
supervis::GET_PROC_COU (void)
{
  return (pCurObjManager->getModelInWhich_cou_engl ());
}

///
boolean
supervis::IS_ROOT (void)
{
  return (pCurObjManager->getModelInWhich_cou_engl () == pCurObjManager->get_racine ());
}

///
boolean
supervis::IS_MODULE (void)
{
  return (STree_isModule (pCurObjManager->getModelInWhich_cou_engl ()));
}


/**
  Les 8 fonctions qui suivent concernent la compilation d'un modele Signal.

  Cette fonction est appelee avant de lancer la compilation proprement dite :
  elle complete l'arbre syntaxique (fonction ok_comp : rajout des affectations
  correspondant aux liens, rajout de declarations...)
*/

ntrees
supervis::ARCHI_BUILD (ntrees p_proc)
{
  ntrees p;
  p = pBuilder->buildArchi (p_proc);
  return (p);
}


///
void
supervis::PRINTING (boolean b)
{
  C2Java_PostscriptPrint ();
}

///
/**
   if SAVING fails an error message is printed: ERR_fileSaving, ERR_gpkFileContent, ERR_LibraryContent, ERR_noGarbage, ERR_parsing
*/
void
supervis::LOCAL_SAVING (int act, boolean isGraphic)
{
  char **filePathString;
  int status;

  filePathString = new (char *);
  if (C2Java_getFileNameFromBrowser (isGraphic, pk_false, filePathString) == 0) return;
  if (isGraphic)
    status = save_model (*filePathString);
  else
    status = unparseAs_model (*filePathString);
  if (status != statusOK) C2Java_displayError (status);
}

/**
   if LOADING fails an error message is printed: ERR_readableFile, ERR_gpkFileContent, ERR_LibraryContent, ERR_noGarbage, ERR_parsing
*/
boolean
supervis::LOADING (int action, boolean isGraphic)
{
  int status;
  char **filePathString;
  boolean setInAtEnd;

  filePathString = new (char *);
  status = C2Java_getFileNameFromBrowser (isGraphic, pk_true, filePathString);	// 0: canceled or error; -1: load/parse before; 1: load/parse after or save/unparse
  if (status == 0) return pk_false;

  setInAtEnd = (status == 1);
  if (isReadableFile (*filePathString))
    {
      // *filePathString is a nonempty string not terminated by  '/'
      switch (action)
	{
	case J_load:
	  status = load_model (*filePathString, action, setInAtEnd);	// @return status among ERR_gpkFileContent, ERR_insertModule, statusOK
	  break;
	case J_load_Lib:
	  status = load_library (*filePathString);	//  @return status among ERR_gpkFileContent, ERR_LibraryContent, statusOK
	  break;
	case J_parse:
	  status = parse_model (*filePathString, action, setInAtEnd);	// @return status among ERR_noGarbage, ERR_parsing, statusOK
	  break;
	default:
	  status = unexpectedAction;
	}
      if (status == statusOK) return pk_true;
    }
  else
    status = ERR_readableFile;
  C2Java_displayError (status);
  return pk_false;
}

/**
   Call to SIGALI using the SigaliCmdForPolychrony shell variable.
*/
void
supervis::CALLING_PROVER ()
{
  char appel[TBUF];
  
  if (NIL_String == SigaliCmdForPolychrony)
    the_messageTool->error_mess (actNotAllowed);
  else
    {
      sprintf (appel, SigaliCmdForPolychrony);
      system (appel);
    }
}


///
void
supervis::BUILDING_RESOLV ()
{
  ntrees p_proc;

  p_proc = pCurObjManager->getModelInWhich_cou_engl ();
  if (p_proc != pCurObjManager->get_racine ())
    // on construit le nouveau programme incluant le résolveur
    buildResolv (p_proc, the_graphicalTrees, pGraph_text, the_messageTool, filebuf);
  else
    the_messageTool->error_mess (actNotAllowed);
}


/**
  A REVOIR AVEC INTERFACE JAVA     PEUT-ETRE INUTILE
  Fonction appelee par Signal dans 2 cas :
   a chaque fois que l'utilisateur clique sur un bouton du menu (dans ce
   cas, ?on_efface vaut 0 et on ne reaffiche pas.
   a chaque fois qu'une action multiple est annulee (creation, deplacement,
   changement de taille...); dans ce cas, on reaffiche.
*/
void
supervis::ANNULATION (int on_efface)
{
  C2Java_resetMoveMouseEvent ();
  pBuilder->nettoyage ();
  if (on_efface != 0)
    C2Java_refresh (pk_false);
}
