 /*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */

import java.awt.Container;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.TextArea;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 * A window to display the different functionalities of the Signal compiler when
 * the user asks the "option access".
 */
public class CompilerOptions extends JFrame {
    private TextArea text = new TextArea();
    private static final long serialVersionUID = 1L;

    private PolychronyWindow mainWin; // reference of the main window
    private CompilerWin compWin; // reference of the compiling window
    private LiaisonSignal LS; // Connection to the C world


    private void endMake(Container frame, String but1Label, String but2Label, OptionsCheckBoxListener myListener) {
        JPanel panel;
        panel = new JPanel();
        JButton button1 = new JButton(but1Label);
        JButton button2 = new JButton(but2Label);
        button1.addActionListener(myListener);
        button2.addActionListener(myListener);
        panel.add(button1, BorderLayout.WEST);
        panel.add(button2, BorderLayout.EAST);
        frame.add(panel, BorderLayout.SOUTH);

	//
       	text.setBackground(Color.white);
        frame.add(text, BorderLayout.CENTER);
        text.setFont(new Font("Helvetica", Font.PLAIN, 14));
	resetText();
	appendText("-POLY"); // Default options.
    }

    private Container Common(String vLabel, JPanel funcPanel) {
        Container frame = getContentPane();
        frame.setLayout(new BorderLayout());
        // Build of the label
        JPanel title = new JPanel();
        title.setLayout(new BorderLayout());
        JLabel title1 = new JLabel(vLabel);
        title1.setForeground(Color.black);
        title.add(title1, BorderLayout.NORTH);
        title1.setFont(new Font("Helvetica", Font.PLAIN, 20));
        title.setBorder(BorderFactory.createMatteBorder(20, 20, 20, 20, pKGUIColors.Color1));

        // Builting the checkboxes
        funcPanel.setLayout(new GridLayout(0, 1));
        funcPanel.setBackground(Color.white);
        funcPanel.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
        frame.add(title, BorderLayout.NORTH);
        frame.add(funcPanel, BorderLayout.CENTER);
        return (frame);
    }

    public CompilerOptions(PolychronyWindow gw, CompilerWin cw, int x0, int y0) {

        String but1Label = "GO";
        String but2Label = "QUIT";
        String vJLabel = "Batch options";

        JPanel panel;
        this.mainWin = gw;
        this.compWin = cw;
        panel = new JPanel();
        Container frame = Common(vJLabel, panel);
        OptionsCheckBoxListener myListener = new OptionsCheckBoxListener(this);
        endMake(frame, but1Label, but2Label, myListener);
        setLocation(x0, y0);
        pack();
    }

    // Setting the current Java-C link (options panel is shared by all the windows).
    public void setCurrentWindow(LiaisonSignal l) 
    {
        LS = l;
    }

    public void see(boolean b) 
    {
        super.setVisible(b);
    }

    /**
       @return the options as a string (one line) and "signal" as first element (batch compiler command analysis).
     */
    public String getBatchCompilerOptions() 
    {
	String vstr;
	vstr = text.getText();
	vstr= vstr.replaceAll("\n", " ");
	vstr= vstr.replaceAll("\t", " ");
	vstr= vstr.replaceAll("\f", " ");
	vstr= vstr.replaceAll("\r", " ");
	vstr= vstr.replaceAll("  ", " ");
	vstr= "signal " + vstr;
	return vstr;
    }
    public void appendText(String s) 
    {
        text.append(s);
    }
    public void resetText()
    {
        text.setText("");
    }


    public void actionPerformed(String label) 
    {
	String voptions;
        setVisible(false);
        if (label == "GO") // button GO of the functionalities window in automatic mode
	    {
		voptions=getBatchCompilerOptions();
		LS.endAutoComp(true, voptions);
		mainWin.reset_selectedAction();
	    }
	else 
	    if (label == "QUIT") // button "QUIT" of the window in automatic mode
		{
		    LS.endAutoComp(false, null);
		    compWin.compilerDiagHide();
		    mainWin.reset_selectedAction();
		}
    }
}

class OptionsCheckBoxListener implements ItemListener, ActionListener {
    CompilerOptions parent;

    public OptionsCheckBoxListener(CompilerOptions w) {
        parent = w;
    }

    public void itemStateChanged(ItemEvent e) {}

    public void actionPerformed(ActionEvent e) {
        JButton source = (JButton) e.getSource();
        String label = source.getText();
        parent.actionPerformed(label);
    }

}
