 /*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */


 

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.net.URL;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;

class Help extends JFrame implements HyperlinkListener {
    private static final long serialVersionUID = 1L;
    private JEditorPane area;
    private static String htmlDir; // contains the name of the directory of all the html files

    private Color getMyColor1() {
        return pKGUIColors.Color1; 
    }

    public Help(String htmlDirectory) {
        htmlDir = htmlDirectory;
	//Global Panel
        setTitle("POLYCHRONY GUI HELP");
        JPanel content = (JPanel) getContentPane();
        //Color of the window's background
        content.setBackground(getMyColor1());
        content.setLayout(new BorderLayout());
        //Lay out the label from top to bottom.
        JPanel labelPanel = new JPanel();
        labelPanel.setLayout(new BoxLayout(labelPanel, BoxLayout.Y_AXIS));
        labelPanel.setBackground(getMyColor1());
	// JLabel label = new JLabel("HELP");
        labelPanel.add(Box.createRigidArea(new Dimension(0, 5)));
        area = createEditorPane();
        area.setEditable(false);
        area.addHyperlinkListener(this);
        area.setFont(new Font("Helvetica", Font.PLAIN, 10));
        //Adds a scroll to the text zone of the example
        JScrollPane scrollArea = new JScrollPane(area);
        labelPanel.add(scrollArea);
        //Lay out the buttons from left to right.
        JPanel buttonPane = new JPanel();
        buttonPane.setBackground(getMyColor1());
        buttonPane.setLayout(new BoxLayout(buttonPane, BoxLayout.X_AXIS));
        buttonPane.setBorder(BorderFactory.createEmptyBorder(0, 5, 5, 5));
        buttonPane.add(Box.createHorizontalGlue());
        JButton sommaire = new JButton("Table of contents");
        buttonPane.add(sommaire);
	ActionListener actSommaire = new ActionListener() {
             public void actionPerformed(ActionEvent e) {
                 showSommaire();
             }
	 };
        sommaire.addActionListener(actSommaire);
        buttonPane.add(sommaire);
        buttonPane.add(Box.createRigidArea(new Dimension(10, 0)));
        // JButton close = new JButton("Close");
        // ActionListener actClose = new ActionListener() {
	// 	public void actionPerformed(ActionEvent e) {
        //         setVisible(false);
        //     }
        // };
        // close.addActionListener(actClose);
        // buttonPane.add(close);
        //Put everything together, using the content pane's BorderLayout.
        content.add(labelPanel, BorderLayout.CENTER);
        content.add(buttonPane, BorderLayout.SOUTH);
        pack();
        setLocation(100, 100);
        setSize(600, 600);
        setVisible(true);
    }

    public void showSommaire() {
        String s = null;
        try {
            s = "file:" + htmlDir + "sommaire.html";
            URL helpURL = new URL(s);
            displayURL(helpURL, area);
        } catch (Exception e) {
            System.err.println("Couldn't create help URL: " + s);
        }
    }

    private JEditorPane createEditorPane() {
        JEditorPane editorPane = new JEditorPane();
        editorPane.setEditable(false);
        String s = null;
        try {
            s = "file:" + htmlDir + "help.html";
            URL helpURL = new URL(s);
            displayURL(helpURL, editorPane);
        } catch (Exception e) {
            System.err.println("Couldn't create help URL: " + s);
        }
        return editorPane;
    }

    private void displayURL(URL url, JEditorPane edit) {
        try {
            edit.setPage(url);
        } catch (IOException e) {
            System.err.println("Attempted to read a bad URL: " + url);
        }
    }

    /** Necessary to implement HyperlinkListener */
    public void hyperlinkUpdate(HyperlinkEvent ev) {
        try {
            if (ev.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
                URL u = ev.getURL();
                displayURL(u, area);
            }
        } catch (Exception ex) {
            ex.printStackTrace(System.err);
        }
    }

}
