/*
  Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
  Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

  Contact: polychronycontact@listes.irisa.fr

  This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

  Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License Version 2 as published by the Free
  Software Foundation.

  Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
  for more details.

  You should have received a copy of the GNU General Public License Version 2
  along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
  http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
  02110-1301, USA.
*/

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.lang.System;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JSplitPane;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.SwingConstants;
import javax.swing.filechooser.FileNameExtensionFilter;

import javax.swing.JFrame;
import java.awt.event.WindowListener;
import java.awt.Container;

// ============================== Polychrony window Listener belongs to (2-1) this_PwindowMenu
class PwindowListener implements ActionListener
{
    PolychronyWindow this_Pwindow;
    int PwindowListener_userAction;
    int PwindowListener_complement;

    PwindowListener (PolychronyWindow G, int noAct)
    {
	this_Pwindow = G;
	PwindowListener_userAction = noAct;
	PwindowListener_complement = UserActions.J_no_action;
    }

    PwindowListener (PolychronyWindow G, int noAct, int complement)
    {
	this_Pwindow = G;
	PwindowListener_userAction = noAct;
	PwindowListener_complement = complement;
    }

    public void actionPerformed (ActionEvent event)
    {
	this_Pwindow.actionPerformedPwindowListener (event, PwindowListener_userAction, PwindowListener_complement);
    }
}

// ============================== GEwindow Grid Listener ???
class GEwindowGridListener implements ActionListener
{
    PolychronyWindow this_Pwindow;
    int userAction;

    GEwindowGridListener (PolychronyWindow G, int noAct)
    {
	this_Pwindow = G;
	userAction = noAct;
    }

    public void actionPerformed (ActionEvent e)
    {
	this_Pwindow.actionPerformedGrid (userAction);
    }
}

// ============================== Undo Listener ???
class GEwindowUndoListener implements ActionListener
{
    PolychronyWindow this_Pwindow;

    GEwindowUndoListener (PolychronyWindow G)
    {
	this_Pwindow = G;
    }

    public void actionPerformed (ActionEvent e)
    {
	this_Pwindow.actionPerformedundo ();
    }
}

// ============================== GEwindow Zoom Listener ???
class GEwindowZoomListener implements ActionListener
{
    PolychronyWindow this_Pwindow;
    double scaleVal;

    GEwindowZoomListener (PolychronyWindow this_Pwindow_, double val)
    {
	this_Pwindow = this_Pwindow_;
	scaleVal = val;
    }

    public void actionPerformed (ActionEvent e)
    {
	this_Pwindow.actionPerformedZoom (scaleVal);
    }
}

// ============================== GEwindow Stroke Listener ???
class GEwindowStrokeWidthListener implements ActionListener
{
    PolychronyWindow this_Pwindow;
    int userAction;

    GEwindowStrokeWidthListener (PolychronyWindow G, int noAct)
    {
	this_Pwindow = G;
	userAction = noAct;
    }

    public void actionPerformed (ActionEvent e)
    {
	this_Pwindow.actionPerformedStrokeWidth (userAction);
    }
}

// ============================== reset current action Listener ???
class CurrentActionListener implements ActionListener
{
    PolychronyWindow this_Pwindow;

    CurrentActionListener (PolychronyWindow this_Pwindow_)
    {
	this_Pwindow = this_Pwindow_;
    }

    public void actionPerformed (ActionEvent e)
    {
	this_Pwindow.resetCurrentActionListener ();
    }
}

// ============================== Graph Editor Listener
class GEwindowListener implements ActionListener
{
    PolychronyWindow this_Pwindow;
    int GEwindowListener_userAction;

    GEwindowListener (PolychronyWindow G, int noAct)
    {
	this_Pwindow = G;
	GEwindowListener_userAction = noAct;
    }

    public void actionPerformed (ActionEvent event)
    {
	this_Pwindow.actionPerformedGEwindowListener (event, GEwindowListener_userAction);
    }
}

// ========================================================================================================
// ========================================================================================================
// ========================================================================================================
public class PolychronyWindow extends JFrame
{

    /** this(_PolychronyWindow) is a JFrame extension  made of
	(0) required variables
	(1) its (SWING standard String) PwindowTitle, that displays 
	the used Polychrony version
	a tag -**- when its content has been modified (unsafe)
	its status in the hierarchy of Pwindows
	its default file path
	(2) its PwindowMenuPanel JPanel 
	(3) its PWindowEditPanels JSplitPane and
	(4) other floatting windows
	this PwindowMenuPanel and this PWindowEditPanels are assembled as the top level window
	
	(2) this_PwindowMenuPanel is a JFrame that only contains 
	(2-1) a PwindowMenu JMenuBar this_PwindowMenu, defined with PwindowListeners, that contains
	(2-1-1) a File menu JMenu
	(2-1-2) a Window menu JMenu
	(2-1-3) a Tools menu JMenu
	(2-1-2) a Help menu JMenuItem
	(3) this_PWindowEditPanels is a JSplitPane made of 
	(3-1) a PwindowTreePanel JPanel, this_PwindowTreePanel that contains
	(3-1-1) an icon menu bar JToolBar this_TreeIconBar
	(3-1-2) an extended JPanel this_PwindowTree
	(3-2) a PwindowGraphPanel JPanel,this_PwindowGraphPanel that contains
	(3-2-1) this_GraphMenuPanel JPanel that contains
	(3-2-1-1) this_GraphIconBar JToolBar contains
	(3-2-1-1-1) a "cut" JButton
	(3-2-1-1-...) 
	(3-2-1-1-8)  a "print" JButton
	(3-2-1-2) this_GraphMenuBar JMenuBar contains
	(3-2-1-2-1) an Edit menu JMenu to modify the signal program
	(3-2-1-2-2) a Draw menu JMenu to modify the graphical view of the signal program
	(3-2-1-2-3) a View menu JMenu to adapt the graph appearance
	(3-2-1-2-2) a variable current action JMenuItem this_currentActionMenuItem
	(3-2-2) this_GEwindowPanel  JScrollPane that contains
	(3-2-2-1) this_GEwindow an extended JPanel
	(3-2-3) this_scrollPanelBib JScrollPane
	(3-3) a PwindowTextPanel (TODO)
	the PwindowGraphPanel and the PwindowTextPanel overlap
    */

    private static final long serialVersionUID = 1L;



    /* (0) =====================================================================================
       Pwindows tree
       =====================================================================================
    */
    /* (0-1) =====================================================================================
       Pwindows tree hierarchy
       =====================================================================================
    */
    public static PolychronyWindow the_mainPwindow = null;	/* contains polychrony main window */
    public static PolychronyWindow[] the_RwindowList;	/* contains all Rwindows but the_mainPwindow */
    private int this_PwindowIndex;	/* -1 for main, 0 to Constants.MAX_OPENED_RWINDOWS-1 for other roots, 
					   0 to Constants.MAX_OPENED_RWINDOWS-1 for children windows */
    private boolean this_isRwindow;	/* true for Rwindows including main window, false for Cwindows */
    private PolychronyWindow this_PwindowAncestor;	/* this for a root window */

    // ----- dependent avant...

    /* Function is called on an exit menu choice or delete from the WM menu*/
    protected void Destroy() {
        dispose();
    } 

    protected void assembleInTopWindow(JPanel globalMenuPanel, JSplitPane signalEditPanel ) {
        Container frame;

        frame = getContentPane();
        frame.setLayout(new BorderLayout());
        frame.add(globalMenuPanel, BorderLayout.NORTH);
        frame.add(signalEditPanel, BorderLayout.CENTER);
        validate();
        setContentPane(frame);
        pack();

    }

    protected void CloseOperation()
    {
	setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    }


    protected void addWindowListener()
    {	
	// to close the window
	WindowListener closeWin = new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    PolychronyWindow source = (PolychronyWindow) e.getSource();
		    source.close_Pwindow();
		}
	    };
	addWindowListener(closeWin);
    }

    // ----- dependent avant...



    // requires this_PwindowIndex, this_isRwindow, this_PwindowAncestor
    public String get_thisPwindowIdent ()
    {
	if (this_PwindowIndex == -1)
	    return "main";
	if (this_isRwindow)
	    return "root " + (this_PwindowIndex + 1);
	return this_PwindowAncestor.get_thisPwindowIdent () + "::" + (this_PwindowIndex + 1);
    }

    /* ======================================================================================
       main polychrony window (this)
       the_mainPwindow == this
       this_PwindowIndex == -1
       it is a Rwindow
       ======================================================================================
    */

    /** Initialize the root window list
     * 
     */
    private void create_theRwindowList ()
    {
	the_RwindowList = new PolychronyWindow[Constants.MAX_OPENED_RWINDOWS];
	for (int i = 0; i < Constants.MAX_OPENED_RWINDOWS; i++)
	    the_RwindowList[i] = null;
    }

    /** create the Rwindow list
	apply common root window initialization
    */
    private void set_PolychronySessionVariables ()
    {
	this_SignalVersion = System.getenv ("SignalGUI_VERSION");
	the_mainPwindow = this;
	create_theRwindowList ();
	this_PwindowIndex = -1;
    }

    /* ======================================================================================
       root polychrony windows
       this_isRwindow == true
       this_PwindowAncestor == this
       this_sourceSTNode == null
       ======================================================================================
    */

    /** Adds this window in a free slot of the_RwindowList
	@return true: set this_PwindowIndex
	@return false: -1 is assigned to this_PwindowIndex, an error message is shown
    */
    private boolean register_thisRwindow ()
    {
	for (int i = 0; i < Constants.MAX_OPENED_RWINDOWS; i++)
	    {
		if (the_RwindowList[i] == null)
		    {
			the_RwindowList[i] = this;
			this_PwindowIndex = i;
			return true;
		    }
	    }
	JOptionPane.showMessageDialog (this, "Too much root open (limit  " + Constants.MAX_OPENED_RWINDOWS + ")", "Polychrony alert 2", JOptionPane.ERROR_MESSAGE);
	this_PwindowIndex = -1;
	return false;
    }

    /** @return the first window root if startIndex is -1
	@return the first window root that follows position startIndex >= 0
	@return this if startIndex is the last root window index
    */
    private PolychronyWindow find_rightRwindow (int startIndex)
    {
	for (int i = startIndex + 1; i < Constants.MAX_OPENED_RWINDOWS; i++)
	    {
		if (the_RwindowList[i] != null)
		    return the_RwindowList[i];
	    }
	return this;
    }

    /** @return the first window root
	@return the_mainPwindow if it is the single root
    */
    private PolychronyWindow find_firstRwindow ()
    {
	return find_rightRwindow (-1);
    }

    /** @return the last window root that precedes position startIndex
	@return the_mainPwindow if startIndex is the first window index
    */
    private PolychronyWindow find_leftRwindow (int startIndex)
    {
	for (int i = startIndex - 1; i >= 0; i--)
	    {
		if (the_RwindowList[i] != null)
		    return the_RwindowList[i];
	    }
	return the_mainPwindow;
    }

    /** @return the last window root
	@return the_mainPwindow if it is the single root
    */
    private PolychronyWindow find_lastRwindow ()
    {
	return find_leftRwindow (Constants.MAX_OPENED_RWINDOWS);
    }

    /* ======================================================================================
       child polychrony windows
       this_isRwindow == false
       this_PwindowAncestor != this
       this_sourceSTNode != null;
       ======================================================================================
    */

    private PolychronyWindow[] this_CwindowList;	/* contains all children windows of this */
    private SignalTreeNode this_sourceSTNode;	/* for a Cwindow: the signal tree node represented in this Cwindow */

    // Initialize this children window list
    private void create_thisCwindowList ()
    {
	this_CwindowList = new PolychronyWindow[Constants.MAX_OPENED_CWINDOWS];
	for (int i = 0; i < Constants.MAX_OPENED_CWINDOWS; i++)
	    this_CwindowList[i] = null;
    }

    /** Adds aChildWindow in a free slot of this_CwindowList
	@return aChildWindow slot number
	@return -1 if the operation failed
    */
    public int register_aCwindow (PolychronyWindow aChildWindow)
    {
	for (int i = 0; i < Constants.MAX_OPENED_CWINDOWS; i++)
	    {
		if (this_CwindowList[i] == null)
		    {
			this_CwindowList[i] = aChildWindow;
			return i;
		    }
	    }
	JOptionPane.showMessageDialog (this, "Too much children open (limit  " + Constants.MAX_OPENED_CWINDOWS + ")", "Polychrony alert 3", JOptionPane.ERROR_MESSAGE);
	return -1;
    }

    public void cancel_aCwindow (int index)
    {
	this_CwindowList[index] = null;
    }

    /** @return this first window child that follows position startIndex
	@return this if startIndex is the last child window index
    */
    private PolychronyWindow find_rightCwindow (int startIndex)
    {
	for (int i = startIndex + 1; i < Constants.MAX_OPENED_CWINDOWS; i++)
	    {
		if (this_CwindowList[i] != null)
		    return this_CwindowList[i];
	    }
	if (startIndex >= 0)
	    return this_CwindowList[startIndex];
	return this;
    }

    /** @return the first window child
	@return this if this does not have children
    */
    private PolychronyWindow find_firstCwindow ()
    {
	return find_rightCwindow (-1);
    }

    /** @return the last window child that precedes position startIndex
	@return this if startIndex is the first window index
    */
    private PolychronyWindow find_leftCwindow (int startIndex)
    {
	for (int i = startIndex - 1; i >= 0; i--)
	    {
		if (this_CwindowList[i] != null)
		    return this_CwindowList[i];
	    }
	if (startIndex >= 0)
	    return this_CwindowList[startIndex];
	return this;
    }

    /** @return the last window child
	@return this if it is the single root
    */
    private PolychronyWindow find_lastCwindow ()
    {
	return find_leftCwindow (Constants.MAX_OPENED_CWINDOWS);
    }

    /* (0-2) =====================================================================================
       Pwindows tree context attributes
       =====================================================================================
    */
    private String this_SignalVersion;

    private String this_defaultDirectoryString;
    private File this_defaultFile = null;
    private String this_defaultFileString = null;	/* used in Title */

    private boolean this_fileContextSaved = false;
    private String this_initialDirectoryString;	/* expected as first parameter of the main method, reset after browsing */
    private String this_saveDirectoryString;

    private boolean this_hasBeenChanged;
    private boolean this_saveChanged;
    private File this_saveFile = null;

    // temporary attributes for save/load multi step action
    private String this_currentDirectoryString = null;
    private String this_currentFileString = null;
    private File this_currentDirectory = null;
    private File this_currentFile = null;
    private boolean this_actionSafe;
    public static boolean the_silentSave;	/* is true if saving is done in silent mode */

    /** reset this Pwindow file context
	assume this_initialDirectoryString defined
    */
    private void new_nullFileContextVariables (boolean init)
    {
	if (init)
	    this_defaultDirectoryString = this_initialDirectoryString;
	else
	    this_defaultDirectoryString = this_saveDirectoryString;
	this_defaultFile = null;
	this_defaultFileString = this_defaultDirectoryString + File.separator;
    }

    /** reset this Pwindow file context after loading or saving this tree root
	assume non null file name
    */
    private void new_fileContextVariables (String fileName)
    {
	this_defaultFileString = fileName;
	this_defaultFile = new File (this_defaultFileString);
	this_defaultDirectoryString = this_defaultFile.getParent ();
    }

    /** reset this Pwindow file context after loading or saving this tree root
	assume non null reference file
    */
    private void new_fileContextVariables (File referenceFile)
    {
	this_defaultFile = referenceFile;
	this_defaultDirectoryString = this_defaultFile.getParent ();
	this_defaultFileString = this_defaultFile.getPath ();
    }

    /** reset this Pwindow file context
	assume this_initialDirectoryString defined
    */
    private void save_fileContextVariables ()
    {
	// System.out.println("DEBUG entering PolychronyWindow::save_fileContextVariables" );
	if (!this_fileContextSaved)
	    {
		this_fileContextSaved = true;
		this_saveDirectoryString = this_defaultDirectoryString;
		this_saveChanged = this_hasBeenChanged;
		this_saveFile = this_defaultFile;
	    }
	// System.out.println("DEBUG  exiting PolychronyWindow::save_fileContextVariables" );
    }

    private void restore_fileContextVariables ()
    {
	// System.out.println("DEBUG entering PolychronyWindow::restore_fileContextVariables" );
	if (this_fileContextSaved)
	    {
		this_hasBeenChanged = this_saveChanged;
		if (this_saveFile == null)
		    new_nullFileContextVariables (false);
		else
		    new_fileContextVariables (this_saveFile);
		this_fileContextSaved = false;
	    }
	// System.out.println("DEBUG  exiting PolychronyWindow::restore_fileContextVariables" );
    }

    /* ======================================================================================
       polychrony window selection
       ======================================================================================
    */
    public void selectPwindow (PolychronyWindow aPwindow)
    {
	aPwindow.setVisible (true);
	aPwindow.this_PwindowGraphPanel.revalidate ();	// Modif LBesnard.
    }

    public void selectPwindow ()
    {
	setVisible (true);
	this_PwindowGraphPanel.revalidate ();	// Modif LBesnard.
    }

    /** select this main window;
     */
    public void goto_Mwindow ()
    {
	the_mainPwindow.selectPwindow ();
    }

    /** select this right window; 
	select the_mainPwindow if this is the last root
	select this if this is a last child
    */
    public void goto_rightPwindow ()
    {
	if (this_isRwindow)
	    {
		if (the_mainPwindow == this)
		    {
			find_firstRwindow ().selectPwindow ();
			return;
		    }
		find_rightRwindow (this_PwindowIndex).selectPwindow ();
		return;
	    }
	(this_PwindowAncestor.find_rightCwindow (this_PwindowIndex)).selectPwindow ();
    }

    /** select this first child window; this when this is does not have children
     */
    public void goto_firstCwindow ()
    {
	find_firstCwindow ().selectPwindow ();
    }

    private LiaisonSignal this_C_interface;


    /* (1) =====================================================================================
       this_PwindowTitle
       =====================================================================================
    */
    String this_PwindowTitle;

    // requires this_C_interface, this_PwindowIndex, this_isRwindow, this_PwindowAncestor, this_defaultFileString
    // 1- PwindowTitle
    private void set_thisPwindowTitle ()
    {
	String vstr;
	if (this_hasBeenChanged) vstr = " -**-"; else  vstr = " ";
	this_PwindowTitle = "Polychrony Signal GUI " + this_SignalVersion + vstr + get_thisPwindowIdent () + "::" + this_defaultFileString;
	setTitle (this_PwindowTitle);
    }

    // (1) this_PwindowTitle completed

    /* (2) =====================================================================================
       this_PwindowMenuPanel
       =====================================================================================
    */
    JPanel this_PwindowMenuPanel;	// (2) create- with unique component this_PwindowMenu
    private JMenuBar this_PwindowMenu;	// (2) create- (File, Windows, Tools, Help popups)

    /* (2-1) =====================================================================================
       this_PwindowMenu
       =====================================================================================
    */
    private void add_JMenuItem (JMenu aMenu, String itemLabel, int noAct)
    {
	JMenuItem aMenuItem = new JMenuItem (itemLabel);
	aMenuItem.addActionListener (new PwindowListener (this, noAct));
	aMenu.add (aMenuItem);
    }

    private JMenu new_fileMenu ()
    {
	JMenu thisMenu = new JMenu ("File");

	add_JMenuItem (thisMenu, "load(cut)", UserActions.J_load);
	add_JMenuItem (thisMenu, "parse(cut)", UserActions.J_parse);
	add_JMenuItem (thisMenu, "save root", UserActions.J_save);
	add_JMenuItem (thisMenu, "save root as", UserActions.J_saveAs);
	add_JMenuItem (thisMenu, "unparse root as", UserActions.J_unparseAs);
	thisMenu.addSeparator ();
	add_JMenuItem (thisMenu, "save all roots", UserActions.J_saveAllRoots);
	add_JMenuItem (thisMenu, "save all roots as", UserActions.J_saveAllRootsAs);
	thisMenu.addSeparator ();
	add_JMenuItem (thisMenu, "load library (cut)", UserActions.J_load_Lib);
	thisMenu.addSeparator ();
	add_JMenuItem (thisMenu, "reset Polychrony", UserActions.J_raz);
	add_JMenuItem (thisMenu, "exit Polychrony", UserActions.J_the_end);
	return thisMenu;
    }

    private JMenu new_WindowManagerMenu ()
    {
	JMenu thisMenu = new JMenu ("Windows");
	add_JMenuItem (thisMenu, "new", UserActions.J_new_window);
	add_JMenuItem (thisMenu, "reset", UserActions.J_reset_window);
	add_JMenuItem (thisMenu, "close", UserActions.J_close_window);
	add_JMenuItem (thisMenu, "refresh", UserActions.J_refresh);
	thisMenu.addSeparator ();
	add_JMenuItem (thisMenu, "main", UserActions.J_main_window);
	add_JMenuItem (thisMenu, "up", UserActions.J_up_window);
	add_JMenuItem (thisMenu, "down", UserActions.J_down_window);
	add_JMenuItem (thisMenu, "left", UserActions.J_left_window);
	add_JMenuItem (thisMenu, "right", UserActions.J_right_window);
	return thisMenu;
    }

    private JMenu new_compileMenu ()
    {
	JMenu compileMenu = new JMenu ("Compile");
	add_JMenuItem (compileMenu, "batch options", UserActions.J_auto_comp);
	add_JMenuItem (compileMenu, "interactive", UserActions.J_inter_comp);
	return compileMenu;
    }

    private JMenu new_proveMenu ()
    {
	JMenu proveMenu = new JMenu ("Prove");
	add_JMenuItem (proveMenu, "call prover", UserActions.J_call_prov);
	add_JMenuItem (proveMenu, "build resolv", UserActions.J_incl_prov);
	return proveMenu;
    }

    private JMenu new_ToolsMenu ()
    {
	JMenu thisMenu = new JMenu ("Tools");

	thisMenu.add (new_compileMenu ());
	add_JMenuItem (thisMenu, "Allocate", UserActions.J_archi_compil);
	thisMenu.add (new_proveMenu ());
	return thisMenu;
    }

    private void create_this_PwindowMenu ()
    {
	this_PwindowMenu = new JMenuBar ();
	this_PwindowMenu.setBackground (getMyColor1 ());
	this_PwindowMenu.setForeground (Color.red);

	this_PwindowMenu.add (new_fileMenu ());
	this_PwindowMenu.add (new_WindowManagerMenu ());
	this_PwindowMenu.add (new_ToolsMenu ());

	JMenuItem helpItem = new JMenuItem ("Help");
	helpItem.addActionListener (new PwindowListener (this, UserActions.J_help));
	this_PwindowMenu.add (helpItem);
    }

    private boolean checkNonEmptyTree (String actionLabel)
    {
	if (this_PwindowTree.isEmpty_nTrees ())
	    {
		show ();
		JOptionPane.showMessageDialog (this, "Empty window", actionLabel, JOptionPane.INFORMATION_MESSAGE);
		cancel_action ();
		return false;
	    }
	return true;
    }

    public void actionPerformedPwindowListener (ActionEvent event, int userAction, int complement)
    {
	/* Pwindow FileMenu
	 */
	boolean changed = false;
	// System.out.println("DEBUG entering PolychronyWindow::actionPerformedPwindowListener userAction is "+userAction);

	// inner call to sig_iterate or local action
	if (userAction == UserActions.J_load)
	    {
		load_root (true);
		return;
	    }
	if (userAction == UserActions.J_parse)
	    {
		load_root (false);
		return;
	    }
	if (userAction == UserActions.J_save)
	    {
		if (checkNonEmptyTree ("Save"))	// guarantee: this_PwindowTree has a root and one or more children
		    save_root (true);
		return;
	    }
	if (userAction == UserActions.J_saveAs)
	    {
		if (checkNonEmptyTree ("Save as"))	// guarantee: this_PwindowTree has a root and one or more children
		    save_root (false);
		return;
	    }
	if (userAction == UserActions.J_unparseAs)
	    {
		if (checkNonEmptyTree ("Unparse"))	// guarantee: this_PwindowTree has a root and one or more children
		    unparse_root ();
		return;
	    }
	if (userAction == UserActions.J_saveAllRoots)
	    {
		save_allRoots (true);
		return;
	    }
	if (userAction == UserActions.J_saveAllRootsAs)
	    {
		save_allRoots (false);
		return;
	    }
	if (userAction == UserActions.J_refresh)
	    {
		refresh ();
		return;
	    }
	if (userAction == UserActions.J_up_window)
	    {
		goto_parentPwindow ();
		return;
	    }
	if (userAction == UserActions.J_down_window)
	    {
		goto_firstCwindow ();
		return;
	    }
	if (userAction == UserActions.J_left_window)
	    {
		goto_leftPwindow ();
		return;
	    }
	if (userAction == UserActions.J_right_window)
	    {
		goto_rightPwindow ();
		return;
	    }
	if (userAction == UserActions.J_main_window)
	    {
		goto_Mwindow ();
		return;
	    }

	// call to sig_iterate 
	if (userAction == UserActions.J_raz)
	    {
	    }				// refresh(false)
	else if (userAction == UserActions.J_the_end)
	    {
	    }				// refresh(false)
	else
	    {
		// call to sig_iterate and refresh(), unchanged
		if (userAction == UserActions.J_load_Lib)
		    {
		    }			// this_C_interface.sig_iterate();
		/* Pwindow WindowManagerMenu
		 */
		else if (userAction == UserActions.J_new_window)
		    {
			if (this_PwindowTree.hasActiveSTNode ())
			    {
				this_PwindowTree.mutateActiveToSelectedSTNode ();
				new PolychronyWindow (this);
			    }
			else
			    {
				new PolychronyWindow (this, this_defaultDirectoryString, this_gifHtmlDir, this_docHtmlDir);
			    }
		    }
		else if (userAction == UserActions.J_reset_window)
		    {
			reset_Pwindow ();
			return;
		    }
		else if (userAction == UserActions.J_close_window)
		    {
			close_Pwindow ();
			return;
		    }
		/* Pwindow ToolsMenu
		 */
		else if (userAction == UserActions.J_auto_comp)
		    {
		    }			// this_C_interface.sig_iterate();
		else if (userAction == UserActions.J_inter_comp)
		    {
		    }			// this_C_interface.sig_iterate();
		else if (userAction == UserActions.J_archi_compil)
		    {
		    }			// this_C_interface.sig_iterate();
		else if (userAction == UserActions.J_call_prov)
		    {
		    }			// this_C_interface.sig_iterate();
		else if (userAction == UserActions.J_incl_prov)
		    {
		    }			// this_C_interface.sig_iterate();
		else if (userAction == UserActions.J_save_PS)
		    {
		    }			// this_C_interface.sig_iterate();
		/* Pwindow Help
		 */
		else if (userAction == UserActions.J_help)
		    {
		    }			// this_C_interface.sig_iterate();
		set_clickedAction (userAction);
		this_C_interface.sig_iterate ();
		refresh ();
		return;
	    }
	set_clickedAction (userAction);
	this_C_interface.sig_iterate ();
	refresh (false);
    }

    // (2-1) this_PwindowMenu completed

    private void create_this_PwindowMenuPanel ()
    {
	this_PwindowMenuPanel = new JPanel ();
	FlowLayout layoutMenu = new FlowLayout (FlowLayout.LEFT);
	this_PwindowMenuPanel.setLayout (layoutMenu);
	create_this_PwindowMenu ();	//  File, Windows, Tools, Help popups
	this_PwindowMenuPanel.add (this_PwindowMenu, BorderLayout.WEST);
	this_PwindowMenuPanel.setBackground (getMyColor1 ());
    }

    // (2) this_PwindowMenuPanel completed





    // Polychrony session variables

    // Polychrony window main permanent and lasting attributes


    private String this_gifHtmlDir;   /** Second parameter of the main method */
    private String this_docHtmlDir;   /** Third parameter of the main method */

    // Polychrony window multi action attributes


    // Polychrony window, main components


    final static int innerBodyEditDefaultWidth = 1000;	// to be moved in Graph ?
    final static int innerBodyEditDefaultHeight = 800;	// to be moved in Graph ?

    /* (3) =====================================================================================
       this_PWindowEditPanels
       =====================================================================================
    */
    JSplitPane this_PwindowEditPanels;	// (3) create- (PwindowTreePanel, PwindowGraphPanel) should contain PwindowTextPanel to be created
    // 3---1 this_PWindowTreePanel
    private JPanel this_PwindowTreePanel;	// (3-1) create- contains the treePanel and its menus
    private JToolBar this_TreeIconBar;	// (3-1-1) create- contains the tree icon menus
    private JTree2 this_PwindowTree;	// (3-1-2) create- contains the tree edit panel   JTree2 extends JPanel

    // 3---2 this_PWindowGraphPanel
    private JPanel this_PwindowGraphPanel;	// (3-2) contains the GraphPanel and its menus
    private JPanel this_GraphMenuPanel;	// (3-2-1)
    private JToolBar this_GraphIconBar;	// (3-2-1-1)
    private JMenuBar this_GraphMenuBar;	// (3-2-1-1)
    private JMenuItem this_currentActionMenuItem;	// (3-2-1-2)


    private CompilerWin this_compilerPopup;
    /** Contains the current String to search */
    private String searchString = "";
    private double scale = 1.0;
    private int this_clickedAction = UserActions.J_no_action;
    private int this_selectedAction = UserActions.J_nop;
    private int this_selectedComplement = UserActions.J_nop;

    /** True if the names of the ports are displayed */
    private boolean this_portNameVisibility;	// PLG move to canvas ?
    /** True if all the text is displayed */
    private boolean this_textFullVisibility;

    /** True if the main bar is displayed */
    private boolean mainBarreState = true;
    /** True if the JTree action bar is displayed */
    private boolean jtreeBarreState = true;

    /** Item corresponding to the libraries display */
    private JCheckBoxMenuItem biblioBar;
    /** True if the library bar is displayed */
    private boolean biblioBarreState = false;
    /** List of the Library bars */
    private static Vector < LibBar > this_LibBarList = new Vector < LibBar > ();
    /** Toolbar containing the main actions */
    private JToolBar barreIcon;
    /** Toolbar containing the actions related to the JTree */
    private JToolBar jtreeBar;
    /** Scroll containing the library bars */
    private JScrollPane this_scrollPanelBib;
    /** Panel containing the libraries */
    private JPanel panelBib;

    private void show_DEBUG (String message)
    {
	JOptionPane.showMessageDialog (this, message, "DEBUG", JOptionPane.INFORMATION_MESSAGE);
    }

    /* ======================================================================================
       ======================================================================================
       polychrony window methods
       ======================================================================================
       ======================================================================================
    */
    // ============================================================= This file context low level functions


    private void ack_defaultDirectoryAndFile (File referenceFile)
    {
	// System.out.println("DEBUG entering PolychronyWindow::ack_defaultDirectoryAndFile" );
	new_fileContextVariables (referenceFile);
	refresh (false);
    }

    private void unset_defaultDirectoryAndFile (File referenceFile)
    {
	// System.out.println("DEBUG entering PolychronyWindow::unset_defaultDirectoryAndFile" );
	new_fileContextVariables (referenceFile);
	refresh ();
    }

    /* ======================================================================================
       ======================================================================================
       polychrony methods for tree of windows
       ======================================================================================
       ======================================================================================
    */

    /* ======================================================================================
       ======================================================================================
       text windows this_TwindowList
       this text edit window the_activeTwindow, this_activeTwindow, 
       ======================================================================================
    */
    private TextWindow[] this_TwindowList;	/* contains all children windows of this */
    private static TextWindow the_activeTwindow;
    private TextWindow this_activeTwindow;
    private TextWindow this_signalExpressionTwindow;	// used by grapWindow 
    private TextWindow this_hierarTwindow;	// generic
    private TextWindow this_parameterDeclarationTwindow;
    private TextWindow this_InputOutputDeclarationTwindow;
    private TextWindow this_signalDeclarationTwindow;

    static final int C_CanvasText = 1;	// OK this_signalExpressionTwindow
    static final int C_HierarText = 2;	// OK this_hierarTwindow
    static final int C_DiagComp = 3;
    static final int C_ParamText = 4;	// OK this_parameterDeclarationTwindow
    static final int C_IOText = 5;	// OK this_InputOutputDeclarationTwindow
    static final int C_SignalDeclText = 6;	// OK
    // Initialize this text window list
    private void create_thisTwindowList ()
    {
	this_TwindowList = new TextWindow[Constants.MAX_OPENED_TWINDOWS];
	for (int i = 0; i < Constants.MAX_OPENED_TWINDOWS; i++)
	    this_TwindowList[i] = null;
    }

    /** @return the index of the first free slot in this_TwindowList
	@return -1 if the operation failed
	assume this_TwindowList contains MAX_OPENED_TWINDOWS slots
    */
    public int get_firstFreeTextwindow ()
    {
	for (int i = 0; i < Constants.MAX_OPENED_TWINDOWS; i++)
	    {
		if (this_TwindowList[i] == null)
		    return i;
	    }
	JOptionPane.showMessageDialog (this, "Too much text windows open (limit  " + Constants.MAX_OPENED_TWINDOWS + ")", "Polychrony alert 3", JOptionPane.ERROR_MESSAGE);
	return -1;
    }

    /** Assign aTextWindow to position index of this_TwindowList
	assume this_TwindowList contains MAX_OPENED_TWINDOWS slots
	assume 0 <= index < MAX_OPENED_TWINDOWS
    */
    public void register_aTwindow (int index, TextWindow aTextWindow)
    {
	this_TwindowList[index] = aTextWindow;
    }

    /** create and register a new text window
	@return false if there is not free slot in this_TwindowList
    */
    private TextWindow create_Twindow (int textCategory, int x0, int y0, int width, int height, String stringOfTextCategory)
    {
	int index = get_firstFreeTextwindow ();
	if (index >= 0)
	    {
		TextWindow aTextWindow = new TextWindow (this, textCategory, 20, 40, 380, 120, get_thisPwindowIdent () + "::" + stringOfTextCategory);
		register_aTwindow (index, aTextWindow);
		return aTextWindow;
	    }
	return null;
    }

    /** @return this first text window that follows position startIndex
	@return null if startIndex is the last text window index
    */
    private TextWindow find_rightTwindow (int startIndex)
    {
	for (int i = startIndex + 1; i < Constants.MAX_OPENED_TWINDOWS; i++)
	    {
		if (this_TwindowList[i] != null)
		    return this_TwindowList[i];
	    }
	if (startIndex >= 0)
	    return this_TwindowList[startIndex];
	return null;
    }

    /** @return the text window at position index in this_TwindowList
	assume this_TwindowList contains MAX_OPENED_TWINDOWS slots
	assume 0 <= index < MAX_OPENED_TWINDOWS
    */
    private TextWindow get_Twindow (int index)
    {
	if (this_TwindowList[index] != null)
	    return this_TwindowList[index];
	return null;
    }

    /** @return the first text window 
     */
    private TextWindow find_firstTwindow ()
    {
	return find_rightTwindow (-1);
    }

    /** @return the last text window that precedes position startIndex
	@return null if startIndex is the first window index
    */
    private TextWindow find_leftTwindow (int startIndex)
    {
	for (int i = startIndex - 1; i >= 0; i--)
	    {
		if (this_TwindowList[i] != null)
		    return this_TwindowList[i];
	    }
	return null;
    }

    /** @return the last window child
	@return this if it is the single root
    */
    private TextWindow find_lastTwindow ()
    {
	return find_leftTwindow (Constants.MAX_OPENED_TWINDOWS);
    }

    // creates this_signalExpressionTwindow, this_signalDeclarationTwindow, this_parameterDeclarationTwindow
    private void create_initialTextWindows ()
    {
	this_signalExpressionTwindow = create_Twindow (C_CanvasText, 600, 670, 400, 300, "Equations");
	this_hierarTwindow = create_Twindow (C_HierarText, 600, 670, 400, 300, "");
	this_parameterDeclarationTwindow = create_Twindow (C_ParamText, 20, 40, 380, 120, "Parameters");
	this_InputOutputDeclarationTwindow = create_Twindow (C_IOText, 20, 40, 380, 120, "Input Ouput Signals");
	this_signalDeclarationTwindow = create_Twindow (C_SignalDeclText, 20, 670, 400, 200, "Signal declarations");
    }

    /** destroy the TextWindow (aTextWindow) at position index in this_TwindowList
	set to null current text popup equal to aTextWindow
	assume this_TwindowList contains MAX_OPENED_TWINDOWS slots
	assume 0 <= index < MAX_OPENED_TWINDOWS
    */
    private void doClose_Twindow (int index)
    {
	TextWindow aTextWindow = this_TwindowList[index];
	if (this_parameterDeclarationTwindow == aTextWindow)
	    this_parameterDeclarationTwindow = null;
	else if (this_InputOutputDeclarationTwindow == aTextWindow)
	    this_InputOutputDeclarationTwindow = null;
	else if (this_signalExpressionTwindow == aTextWindow)
	    this_signalExpressionTwindow = null;
	else if (this_signalDeclarationTwindow == aTextWindow)
	    this_signalDeclarationTwindow = null;
	aTextWindow.setVisible (false);
	remove (aTextWindow);	// destroys and unallocate the window
	this_TwindowList[index] = null;
    }

    private void doClose_thisTwindowList ()
    {
	for (int i = Constants.MAX_OPENED_TWINDOWS - 1; i >= 0; i--)
	    if (this_TwindowList[i] != null)
		doClose_Twindow (i);
    }

    /* ======================================================================================
       Text window methods
       ======================================================================================
    */
    private void set_activeTwindow (TextWindow aTextWindow)
    {
	this_activeTwindow = aTextWindow;
	the_activeTwindow = aTextWindow;
	aTextWindow.setVisible (true);
    }

    public void hide_Twindow (int w)
    {
	switch (w)
	    {
	    case C_CanvasText:
		this_signalExpressionTwindow.setVisible (false);
		break;
	    case C_HierarText:
		this_hierarTwindow.setVisible (false);
		break;
	    case C_DiagComp:
		break;
	    case C_ParamText:
		this_parameterDeclarationTwindow.setVisible (false);
		break;
	    }
    }

    public void show_Twindow (int w)
    {
	// System.out.println("DEBUG entering PolychronyWindow::show_Twindow w is " + w);
	switch (w)
	    {
	    case C_CanvasText:
		set_activeTwindow (this_signalExpressionTwindow);
		break;
	    case C_HierarText:
		set_activeTwindow (this_hierarTwindow);
		break;
	    case C_DiagComp:
		break;
	    case C_ParamText:
		set_activeTwindow (this_parameterDeclarationTwindow);
		break;
	    }
    }

    public void append_text2Twindow (int w, String s)
    {
	// System.out.println("DEBUG ===== entering PolychronyWindow::appendText w is " + w + ", string is <<" + s + ">>");
	switch (w)
	    {
	    case C_CanvasText:
		// System.out.println("DEBUG      in  PolychronyWindow::appendText C_CanvasText");
		this_signalExpressionTwindow.append (s);
		break;
	    case C_IOText:
		// System.out.println("DEBUG      in  PolychronyWindow::appendText C_IOText");
		this_hierarTwindow.append (s);
		this_hierarTwindow.setCaretPosition (0);
		break;
	    case C_HierarText:
		// System.out.println("DEBUG      in  PolychronyWindow::appendText C_HierarText");
		this_hierarTwindow.append (s);
		this_hierarTwindow.setCaretPosition (0);
		break;
	    case C_DiagComp:
		// System.out.println("DEBUG      in  PolychronyWindow::appendText C_DiagComp");
		this_compilerPopup.append (s);
		break;
	    case C_ParamText:
		// System.out.println("DEBUG      in  PolychronyWindow::appendText C_ParamText");
		this_parameterDeclarationTwindow.append (s);
		this_parameterDeclarationTwindow.setCaretPosition (0);
		break;
	    }
	// System.out.println("DEBUG =====  exiting PolychronyWindow::appendText");
    }

    public String get_TwindowContent (int w)
    {
	// System.out.println("DEBUG entering PolychronyWindow::getText ");
	switch (w)
	    {
	    case C_CanvasText:
		return this_signalExpressionTwindow.getText ();
	    case C_HierarText:
		return this_hierarTwindow.getText ();
	    case C_DiagComp:
		break;
	    case C_ParamText:
		{
		    // System.out.println("DEBUG  in  PolychronyWindow::getText C_ParamText");
		    return this_parameterDeclarationTwindow.getText ();
		}
	    }
	return "error";
    }

    public int get_TwindowCharWidth (int w)
    {
	switch (w)
	    {
	    case C_CanvasText:
		return this_signalExpressionTwindow.getColumns ();
	    case C_HierarText:
		return Constants.MAX_LENGTH_STRING_SIGNAL_PARSER;
	    case C_DiagComp:
		break;
	    case C_ParamText:
		return Constants.MAX_LENGTH_STRING_SIGNAL_PARSER;
	    }
	return -1;
    }

    public void clear_Twindow (int w)
    {
	switch (w)
	    {
	    case C_CanvasText:
		this_signalExpressionTwindow.resetText ();
		break;
	    case C_HierarText:
		this_hierarTwindow.resetText ();
		break;
	    case C_DiagComp:
		break;
	    case C_ParamText:
		this_parameterDeclarationTwindow.resetText ();
		break;
	    }
    }

    public int get_activeTwindowType ()
    {
	if (this_activeTwindow == null)
	    set_activeTwindow (this_hierarTwindow);
	return the_activeTwindow.get_textCategory ();
    }

    /* ======================================================================================
       ======================================================================================
       this graph edit window:      this_GEwindow
       ======================================================================================
    */
    private JScrollPane this_GEwindowPanel;	// (3-2-2)
    private GraphCanvas this_GEwindow;

    // =======================================================================================================
    // This Graph Editor full Window
    // =======================================================================================================
    // this_GEwindow 0
    // create this Graph Edit Window
    private void create_GEwindow ()
    {
	this_GEwindow = new GraphCanvas (this, this_C_interface);
	this_GEwindow.setPreferredSize (new Dimension (innerBodyEditDefaultWidth, innerBodyEditDefaultHeight));
	this_GEwindow.setBackground (Color.white);
    }

    // this_GEwindow 0
    private void createGraphMenuPanel ()
    {
	Dimension minimumSize = new Dimension (5, 100);
	this_GraphMenuPanel = new JPanel ();
	this_GraphMenuPanel.setLayout (new BorderLayout ());

	createGraphIconBar ();
	createGraphMenuBar ();

	this_GraphMenuPanel.add (this_GraphIconBar, BorderLayout.WEST);
	this_GraphMenuPanel.add (this_GraphMenuBar, BorderLayout.CENTER);

	this_GraphMenuPanel.setBackground (getMyColor1 ());
    }

    /**
     * Builds the graph viewing canvas with scroll bars
     *
     * @return
     */
    // 3-2-2 this PwindowGraphPanel
    // requires this_C_interface
    private void createPgraphDrawingPanel ()
    {
	create_GEwindow ();
	createGraphMenuPanel ();
	this_GEwindowPanel = new JScrollPane (this_GEwindow);
	this_GEwindowPanel.setVerticalScrollBarPolicy (JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
	this_GEwindowPanel.setHorizontalScrollBarPolicy (JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
	this_GEwindowPanel.setPreferredSize (new Dimension (700, 500));
	this_GEwindow.setVisible (true);
    }

    /**
     * builds the panel which contains the Canvas and the panel of the libraries
     * icons
     */
    // 3-2 this PwindowGraphPanel
    // requires this_C_interface
    private void createPwindowGraphPanel ()
    {
	// subWindow that contains this_GEwindowPanel and the future LibBar
	Dimension minimumSize = new Dimension (5, 100);
	this_PwindowGraphPanel = new JPanel ();
	this_PwindowGraphPanel.setLayout (new BorderLayout ());
	this_PwindowGraphPanel.setBackground (getMyColor1 ());

	// requires this_C_interface
	createPgraphDrawingPanel ();	// creates inner drawing bars this_GEwindow and this_GEwindowPanel
	this_GEwindowPanel.setMinimumSize (minimumSize);

	panelBib = new JPanel ();
	panelBib.setLayout (new FlowLayout (FlowLayout.LEFT));
	panelBib.setBackground (getMyColor1 ());
	this_scrollPanelBib = new JScrollPane (panelBib);
	this_scrollPanelBib.setVerticalScrollBarPolicy (JScrollPane.VERTICAL_SCROLLBAR_NEVER);
	this_scrollPanelBib.setVisible (false);

	this_PwindowGraphPanel.add (this_GraphMenuPanel, BorderLayout.NORTH);
	this_PwindowGraphPanel.add (this_GEwindowPanel, BorderLayout.CENTER);
	this_PwindowGraphPanel.add (this_scrollPanelBib, BorderLayout.SOUTH);
    }


    /* ======================================================================================
       graph window methods
       ======================================================================================
    */
    public void actionPerformedundo ()
    {
	set_clickedAction (UserActions.J_undo);
	this_C_interface.sig_iterate ();
	this_GEwindow.repaint ();
    }

    public void actionPerformedZoom (double scaleVal)
    {
	int w, h;

	if (scaleVal < 0)
	    {
		scale = scale / (-scaleVal);
		w = (int) (this_GEwindow.getWidth () / -scaleVal);
		h = (int) (this_GEwindow.getHeight () / -scaleVal);
	    }
	else if (scaleVal > 0)
	    {
		scale = scale * scaleVal;
		w = (int) (this_GEwindow.getWidth () * scaleVal);
		h = (int) (this_GEwindow.getHeight () * scaleVal);
	    }
	else
	    {				// default values
		scale = 1.0;
		w = innerBodyEditDefaultWidth;
		h = innerBodyEditDefaultHeight;
	    }

	this_GEwindow.setScale (scale);
	this_GEwindow.setPreferredSize (new Dimension (w, h));
	this_GEwindowPanel.setViewportView (this_GEwindow);
	this_GEwindow.revalidate ();
    }

    public void actionPerformedGrid (int userAction)
    {
	set_clickedAction (userAction);
	this_GEwindow.actionPerformedGrid (userAction);
    }

    public void actionPerformedStrokeWidth (int userAction)
    {
	set_clickedAction (userAction);
	this_GEwindow.actionPerformedStrokeWidth (userAction);
    }

    public void print_GEwindow ()
    {
	String nomProcess = new String (this_PwindowTree.getSelectedName ());
	new PrintWindow (this_GEwindow, nomProcess);
    }

    public void drawPolyline (int[]x_points, int[]y_points, int nbPoints, int coul, int ep)
    {
	this_GEwindow.drawPolyline (x_points, y_points, nbPoints, coul, ep);
    }

    public void drawLine (int x1, int y1, int x2, int y2, int coul, int ep)
    {
	this_GEwindow.drawLine (x1, y1, x2, y2, coul, ep);
    }

    public void fillPolygon (int[]xPoints, int[]yPoints, int nPoints, int coul, int ep)
    {
	this_GEwindow.fillPolygon (xPoints, yPoints, nPoints, coul, ep);
    }

    public void drawPolygon (int[]xPoints, int[]yPoints, int nPoints, int coul, int ep)
    {
	this_GEwindow.drawPolygon (xPoints, yPoints, nPoints, coul, ep);
    }

    public void drawRectangle (int x, int y, int w, int h, int coul, int ep)
    {
	this_GEwindow.drawRectangle (x, y, w, h, coul, ep);
    }

    public void fillRectangle (int x, int y, int w, int h, int coul)
    {
	this_GEwindow.fillRectangle (x, y, w, h, coul);
    }

    public void drawString (String chaine, int x, int y, int coul, int typePolice)
    {
	this_GEwindow.drawString (chaine, x, y, coul, typePolice);
    }

    public void drawImage (String name, int x, int y)
    {
	this_GEwindow.drawImage (name, x, y);
    }

    public int get_GEwindowMouseButton ()
    {
	return this_GEwindow.getMouseButton ();
    }

    public boolean is_GEwindowMouseButtonClicked ()
    {
	if (this_GEwindow == null)
	    return false;
	return (this_GEwindow.isMouseButtonClicked ());
    }

    public int getXOrd ()
    {
	return (this_GEwindow.getXOrd ());
    }

    public int getYOrd ()
    {
	return (this_GEwindow.getYOrd ());
    }

    public void set_GEwindowMoveMouseEvent ()
    {
	this_GEwindow.setMoveMouseEvent ();
    }

    public void reset_GEwindowMoveMouseEvent ()
    {
	this_GEwindow.resetMoveMouseEvent ();
    }

    public int getSizeFontH (int type)
    {
	return this_GEwindow.getSizeFontH (type);
    }

    public int getSizeFontL (int type)
    {
	return this_GEwindow.getSizeFontL (type);
    }


    /* ======================================================================================
       ======================================================================================
       this tree edit window:      this_PwindowTree
       ======================================================================================
    */

    public String getSelectedObjectName ()
    {
	// System.out.println("DEBUG entering/exitingPolychronyWindow::getSelectedObjectName");
	return this_PwindowTree.getSelectedName ();
    }

    public void deleteSelectedNode ()
    {
	if (this_PwindowTree.deleteSelectedNode ())
	    refresh (true);
    }

    public JTree2 getTreeWin ()
    {
	return this_PwindowTree;
    }

    public void initSelectedNode (boolean afterCompil)
    {
	this_PwindowTree.initSelectedNode (afterCompil);
    }

    public void upTempSelectedNode ()
    {
	this_PwindowTree.upTempSelectedNode ();
    }

    public void setCurrentTree (int iBox)
    {
	this_PwindowTree.setCurrentObject (iBox);
    }

    // 3-1-2 this PwindowTree
    private void createPwindowTree (boolean rootWindow)
    {
	if (rootWindow)
	    this_PwindowTree = new JTree2 (this_C_interface, this);
	else
	    this_PwindowTree = new JTree2 (this_PwindowAncestor.this_PwindowTree, this_C_interface, this);
    }

    public String getString1 ()
    {
	String str = "";
	// using "this_PwindowTree" instead of "this" as first parameter to have a good position of the window
	str = (String) JOptionPane.showInputDialog (this_PwindowTree, "find", "Polychrony find and replace", JOptionPane.QUESTION_MESSAGE, null, null, searchString);
	if (str != null)
	    if (!str.equals (searchString))
		searchString = str;
	return (str);
    }

    public String getString2 ()
    {
	String mess = "Replace with ?";
	// using "this_PwindowTree" instead of "this" as first parameter to have a good position of the window
	return JOptionPane.showInputDialog (this_PwindowTree, "replace by", "Polychrony find and replace", JOptionPane.QUESTION_MESSAGE);
    }

    public int getUserChoice ()
    {
	Object[]options =
	    {
		"Replace", "Skip", "Cancel", "replace all"};
	int stop;

	stop = JOptionPane.showOptionDialog (this_PwindowTree, "\"" + searchString + "\" found", "Polychrony find and replace", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE, null, options, options[0]);
	if (stop == JOptionPane.OK_OPTION)
	    return 1;
	if (stop == JOptionPane.CANCEL_OPTION)
	    return 3;
	if (stop == JOptionPane.CLOSED_OPTION)
	    return 3;
	return 2;
    }

    // ------------------------------------------------------------- This file context
    /**
     * Function is called 
     */
    public void notImplemented (String message)
    {
	JOptionPane.showMessageDialog (this, message, "Not implemented", JOptionPane.INFORMATION_MESSAGE);
    }

    /** display amessage dialog box dedicated to file errors
     *  assume: this_currentFileString contains the file path string
     */
    private void displayFileError (int no)
    {
	JOptionPane.showMessageDialog (this, this_currentFileString + "\n" + Constants.ERRORS_TAB[no], "Polychrony alert", JOptionPane.INFORMATION_MESSAGE);
	this_actionSafe = false;
    }

    public void displayError (int no)
    {
	String mess;
	this_actionSafe = false;
	if (no == Constants.J_ERR_gpkFileContent)
	    {
		mess = new String (this_currentFileString + "\n" + Constants.ERRORS_TAB[Constants.J_ERR_gpkFileContent]);
	    }
	if (no == Constants.J_ERR_parsing)
	    {
		mess = new String (this_currentFileString + "\n" + Constants.ERRORS_TAB[Constants.J_ERR_parsing]);
	    }
	else
	    mess = new String (Constants.ERRORS_TAB[no]);
	JOptionPane.showMessageDialog (this, mess, "Polychrony alert 1 " + "numero " + no, JOptionPane.ERROR_MESSAGE);
	if (this_GEwindow != null)
	    this_GEwindow.repaint ();	// after an error, it may be necessary to refresh the canvas
	restore_fileContextVariables ();
	refresh ();
    }

    private void doClose_theRwindowList (boolean saveChanged)
    {
	for (int i = Constants.MAX_OPENED_RWINDOWS - 1; i >= 0; i--)
	    {
		if (the_RwindowList[i] != null)
		    {
			the_RwindowList[i].doClose_Pwindow (saveChanged);
			the_RwindowList[i] = null;
		    }
	    }
    }

    private void doClose_thisCwindowList (boolean saveChanged)
    {
	for (int i = Constants.MAX_OPENED_CWINDOWS - 1; i >= 0; i--)
	    if (this_CwindowList[i] != null)
		{
		    this_CwindowList[i].doClose_Pwindow (saveChanged);
		    this_CwindowList[i] = null;
		}
    }

    private void doClose_innerWindows (boolean saveChanged)
    {
	doClose_thisTwindowList ();
	doClose_thisCwindowList (saveChanged);
	if (this_GEwindow != null)
	    {
		this_GEwindow.setVisible (false);
		remove (this_GEwindow);
		this_GEwindow = null;
	    }
	if (this_PwindowTree != null)
	    {
		this_PwindowTree.setVisible (false);
		remove (this_PwindowTree);
		this_PwindowTree = null;
	    }
	if (this_compilerPopup != null)
	    {
		this_compilerPopup.endInteractCompiling ();
		// remove(this_compilerPopup);
		this_compilerPopup = null;
	    }
	if (this_sourceSTNode != null)
	    this_sourceSTNode.detach_dedicatedWindow ();
    }

    // Method to close a polychrony window PLG TO BE CHANGED
    private void doClose_Pwindow (boolean saveChanged)
    {
	doClose_innerWindows (saveChanged);
	if (saveChanged)
	    querySave_root ();
	setVisible (false);
	Destroy ();			// destroys and unallocate the window
    }

    /* ======================================================================================
       polychrony miscellenaous windows
       ======================================================================================
    */
    /** select this left window; 
	select the_mainPwindow if this is the first root
	select this if this is a first child
    */
    public void goto_leftPwindow ()
    {
	if (this_isRwindow)
	    {
		if (the_mainPwindow == this)
		    {
			find_lastRwindow ().selectPwindow ();
			return;
		    }
		find_leftRwindow (this_PwindowIndex).selectPwindow ();
		return;
	    }
	(this_PwindowAncestor.find_leftCwindow (this_PwindowIndex)).selectPwindow ();
    }

    /** select this ancestor window; this when this is a root
     */
    public void goto_parentPwindow ()
    {
	this_PwindowAncestor.selectPwindow ();
    }

    //////////////////////////////   



    public LiaisonSignal get_C_interface ()
    {
	return this_C_interface;
    }

    public void cancel_action ()
    {
	this_clickedAction = UserActions.J_no_action;
    }

    public void refresh_theseGwindows (boolean value)
    {
	// System.out.println("DEBUG entering PolychronyWindow::refresh_theseGwindows");
	if (this_PwindowGraphPanel != null)
	    {
		if (value)
		    {
			this_PwindowGraphPanel.setEnabled (true);
			this_PwindowGraphPanel.setVisible (true);
			this_PwindowGraphPanel.repaint ();
			this_PwindowEditPanels.resetToPreferredSizes ();
		    }
		else
		    {
			this_PwindowGraphPanel.setEnabled (false);
			this_PwindowGraphPanel.setVisible (false);
		    }
		this_PwindowGraphPanel.revalidate ();
	    }				// modif LBesnard.
	// System.out.println("DEBUG  exiting PolychronyWindow::refresh_theseGwindows");
    }

    /** Display current Tree and its selected node
	Display selected node content in its graph 
    */
    public void refresh ()
    {
	// System.out.println("DEBUG entering PolychronyWindow::refresh()");
	set_thisPwindowTitle ();
	if (this_PwindowTree != null)
	    {
		this_PwindowTree.refresh ();
		refresh_theseGwindows (this_PwindowTree.isSelected_model ());
	    }
	// System.out.println("DEBUG  exiting PolychronyWindow::refresh()");
    }

    public void set_clickedAction (int userAction)
    {
	this_clickedAction = userAction;
    }

    /** call C interface with "action" as current action
     */
    public boolean sig_iterate (int action)
    {
	set_clickedAction (action);
	return this_C_interface.sig_iterate ();
    }

    /** Display current Tree and its selected node
	Display selected node content in its graph 
    */
    public void refresh (boolean changed)
    {
	this_hasBeenChanged = changed;
	refresh ();
    }

    /** load a tree 
	if this_selectedComplement is J_before the tree is inserted before the selected node
	if this_selectedComplement is J_in the tree is inserted after the last declaration of the selected node
    */
    public void load_node_in (boolean graphic)
    {
	// System.out.println("DEBUG entering PolychronyWindow::load_node_in" );
	this_actionSafe = true;
	if (graphic)
	    sig_iterate (UserActions.J_load);
	else
	    sig_iterate (UserActions.J_parse);
	if (this_actionSafe)
	    refresh (true);
	else
	    refresh ();
	cancel_action ();
	// System.out.println("DEBUG  exiting PolychronyWindow::load_node_in" );
    }

    /** load a tree as the new root of this window
	cut to clipboard the current root if any
	this default file is reset, this Pwindow becomes not changed
    */
    public void load_root (boolean graphic)
    {
	// System.out.println("DEBUG entering PolychronyWindow::load_root" );
	this_actionSafe = true;
	if (!this_PwindowTree.isEmpty_nTrees ())
	    {
		// guarantee: this_PwindowTree has a root and one or more children
		this_PwindowTree.setSelected_ntreesRoot ();
		if (sig_iterate (UserActions.J_cut))
		    {
			this_PwindowTree.select_JTreeRoot ();
			if (sig_iterate (UserActions.J_nop));	// to inhibit current action in graph
			this_clickedAction = UserActions.J_load;
			this_selectedComplement = UserActions.J_root;
		    }
		else
		    {
			refresh ();
			// System.out.println("DEBUG  exiting PolychronyWindow::load_root 1" );
			return;
		    }
	    }
	this_selectedComplement = UserActions.J_root;
	if (graphic)
	    {
		if (sig_iterate (UserActions.J_load));
	    }
	else
	    {
		if (sig_iterate (UserActions.J_parse));
	    }
	if (this_actionSafe)
	    refresh (false);
	else
	    refresh ();
	cancel_action ();
	// System.out.println("DEBUG  exiting PolychronyWindow::load_root *" );
    }

    /** ASSUME: this window tree is not empty
	if this window has changed, dialog save this window tree
	if the operation succeeds set this_hasBeenChanged to false
    */
    private void save_nTreesRoot ()
    {
	// System.out.println("DEBUG entering PolychronyWindow::save_nTreesRoot");
	if (this_PwindowTree.isEmpty_nTrees ())
	    {
		show ();
		JOptionPane.showMessageDialog (this, "Empty window", "Save " + get_thisPwindowIdent (), JOptionPane.INFORMATION_MESSAGE);
	    }
	else if (this_hasBeenChanged || (!the_silentSave))
	    {
		SignalTreeNode currentNode = this_PwindowTree.get_selectedJNode ();
		// System.out.println("DEBUG      in  PolychronyWindow::save_nTreesRoot 1");
		this_PwindowTree.setSelected_ntreesRoot ();
		set_clickedAction (UserActions.J_save);
		if (this_C_interface.sig_iterate ());
		this_PwindowTree.select_node (currentNode);
		if (this_actionSafe)
		    {
			// System.out.println("DEBUG      in  PolychronyWindow::save_nTreesRoot 2");
			refresh (false);
		    }
		else
		    {
			// System.out.println("DEBUG      in  PolychronyWindow::save_nTreesRoot 3");
			restore_fileContextVariables ();
			refresh ();
		    }
	    }
	// System.out.println("DEBUG  exiting PolychronyWindow::save_nTreesRoot");
    }

    /** ASSUME: a node is selected and it is not the root
	save this node with the hep of browsing 
	see also save_root
    */
    // called in JTree2::save_JNode ==> from events
    //           (modelPopupMenu) "save as"
    public void save_innerModel ()
    {
	// System.out.println("DEBUG entering PolychronyWindow::save_innerModel");
	the_silentSave = false;
	this_selectedAction = UserActions.J_save;
	this_C_interface.sig_iterate ();
	cancel_action ();
	// System.out.println("DEBUG  exiting PolychronyWindow::save_innerModel");
    }

    /** ASSUME: this_PwindowTree is not empty 
	save the root
	if silent = true, browsing is proposed only if filename is missing
	see also save_innerModel
    */
    // called in File menu, menu Item "save(root)", button saveFile
    // called in JTree2::save_JNode ==> from events
    //           (modulePopupMenu, modelPopupMenu) "save as"
    //           (noSelectionPopupMenu) "save(root)"
    public void save_root (boolean silent)
    {
	// System.out.println("DEBUG entering PolychronyWindow::save_root");
	the_silentSave = silent;
	save_nTreesRoot ();		// if the operation succeeds set this_hasBeenChanged to false
	cancel_action ();
	the_silentSave = false;
	// System.out.println("DEBUG  exiting PolychronyWindow::save_root");
    }

    /** if this window has changed, dialog save this window 
	selectedNtrees is assumed to be a root
    */
    public void querySave_root ()
    {
	if (this_hasBeenChanged)
	    {
		// myOptionPane = new JOptionPane("Save window");
		int value = JOptionPane.showConfirmDialog (this, "save window in file", "Save window", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
		if (value == JOptionPane.OK_OPTION)
		    {
			the_silentSave = true;
			save_nTreesRoot ();
		    }			// if the operation succeeds set this_hasBeenChanged to false
		the_silentSave = false;
		cancel_action ();
	    }
    }

    /** ASSUME: this_PwindowTree is not empty 
	save the root
	if silent = true, browsing is proposed only if filename is missing
    */
    // called in File menu, menu Item "unparse(root) as"
    // other unparse are directly made in sig_iterate
    public void unparse_root ()
    {
	// System.out.println("DEBUG entering PolychronyWindow::unparse_root");
	SignalTreeNode currentNode = this_PwindowTree.get_selectedJNode ();
	this_PwindowTree.setSelected_ntreesRoot ();
	set_clickedAction (UserActions.J_unparseAs);
	if (this_C_interface.sig_iterate ());
	this_PwindowTree.select_node (currentNode);
	refresh ();
	cancel_action ();
    }


    /** recursively save children windows and then 
     *  if this window has changed or if saving in query mode, dialog save this window 
     */
    private void save_fullPwindow ()
    {
	for (int i = Constants.MAX_OPENED_CWINDOWS - 1; i >= 0; i--)
	    {
		if (this_CwindowList[i] != null)
		    {
			this_CwindowList[i].save_fullPwindow ();
		    }
	    }				// inner sig_iterate
	save_nTreesRoot ();		// inner sig_iterate  // if the operation succeeds set this_hasBeenChanged to false

    }

    /** recursively save all root windows but the main one
     *  if this window has changed or if saving in query mode, dialog save this window 
     */
    private void save_allRwindows ()
    {
	for (int i = Constants.MAX_OPENED_RWINDOWS - 1; i >= 0; i--)
	    {
		if (the_RwindowList[i] != null)
		    {
			the_RwindowList[i].save_fullPwindow ();
		    }
	    }
    }

    /** recursively save all root windows including the main one
     *  if this window has changed or if saving in query mode, dialog save this window 
     */
    public void save_allRoots (boolean silent)
    {
	the_silentSave = silent;
	if (the_mainPwindow == this)
	    save_allRwindows ();
	save_fullPwindow ();
	cancel_action ();
	the_silentSave = false;
    }


    /** set this Pwindow as a (root) Rwindow
	apply common parameter window initialization
    */
    private void set_RwindowVariables (String WDPath, String imageDirectory, String docDirectory)
    {
	this_isRwindow = true;
	this_PwindowAncestor = this;
	this_sourceSTNode = null;

	this_gifHtmlDir = imageDirectory + File.separator;
	this_docHtmlDir = docDirectory + File.separator;

	this_initialDirectoryString = WDPath;
	new_nullFileContextVariables (true);
	set_Visibility ();
    }


    private void doReset_Pwindow (boolean saveChanged)
    {
	// System.out.println("DEBUG entering PolychronyWindow::doReset_Pwindow" );
	if (the_mainPwindow == this)
	    doClose_theRwindowList (saveChanged);
	doClose_thisTwindowList ();
	doClose_thisCwindowList (saveChanged);
	new_nullFileContextVariables (true);
	set_Visibility ();
	this_PwindowTree.select_JTreeRoot ();
	this_PwindowTree.cut_selectedJNode ();
	reset_selectedAction ();
	for (int i = this_LibBarList.size () - 1; i >= 0; i--)
	    removeIthLibrary (i);
	this_PwindowTree.clear ();
	this_scrollPanelBib.setVisible (false);
	if (biblioBar != null)
	    {
		biblioBar.setEnabled (true);
		biblioBar.setState (false);
		biblioBarreState = false;
	    }
	this_PwindowGraphPanel.revalidate ();
	refresh (false);
	// System.out.println("DEBUG  exiting PolychronyWindow::doReset_Pwindow" );
    }

    private void removeIthLibrary (int i)
    {
	LibBar x = (LibBar) (this_LibBarList.get (i));
	x.setFloatable (false);
	x.setVisible (false);
	panelBib.remove (x);
	this_LibBarList.remove (i);
    }

    private boolean has_someNonEmptyCwindowChanged ()
    {
	for (int i = 0; i < Constants.MAX_OPENED_CWINDOWS; i++)
	    if (this_CwindowList[i] != null)
		if (this_CwindowList[i].has_someNonEmptyCwindowChanged ())
		    return true;
	return false;
    }

    private boolean has_thisNonEmptyPwindowChanged ()
    {
	if (this_hasBeenChanged && (!this_PwindowTree.isEmpty_nTrees ()))
	    // guarantee: this_PwindowTree has a root and one or more children
	    return true;
	return has_someNonEmptyCwindowChanged ();
    }

    private boolean has_someNonEmptyRWindowChanged ()
    {
	for (int i = 0; i < Constants.MAX_OPENED_RWINDOWS; i++)
	    if (the_RwindowList[i] != null)
		if (the_RwindowList[i].has_thisNonEmptyPwindowChanged ())
		    return true;
	return false;
    }

    private void set_Visibility ()
    {
	this_portNameVisibility = true;
	this_textFullVisibility = true;
    }

    public boolean get_portNameVisibility ()
    {
	return this_portNameVisibility;
    }

    public boolean getdetailsState ()
    {
	return this_textFullVisibility;
    }

    private Color getMyColor1 ()
    {
	return pKGUIColors.Color1;
    }

    private Color getMyColor2 ()
    {
	return pKGUIColors.Color2;
    }

    /* ======================================================================================
       ======================================================================================
       polychrony window menu 
       ======================================================================================
       ======================================================================================
    */
    private JButton createJButton (String iconFileName, String tipText, int noAct)
    {
	ImageIcon imageNew = new ImageIcon (this_gifHtmlDir + iconFileName);
	JButton newButton = new JButton (imageNew);
	newButton.setToolTipText (tipText);
	newButton.addActionListener (new PwindowListener (this, noAct));
	return newButton;
    }

    private JButton createJButton (String iconFileName, String tipText, int noAct, int complement)
    {
	ImageIcon imageNew = new ImageIcon (this_gifHtmlDir + iconFileName);
	JButton newButton = new JButton (imageNew);
	newButton.setToolTipText (tipText);
	newButton.addActionListener (new PwindowListener (this, noAct, complement));
	return newButton;
    }

    private boolean is_thisWindowFileNameCompleted ()
    {
	return (!this_defaultFileString.endsWith (File.separator));
    }

    // 2-1- PwindowMenu (File popup)
    // 2-2- PwindowMenu (Windows popup)

    // 2-2- PwindowMenu (Tools popup)
    // 2- PwindowMenu (File, Windows, Tools, Help popups)
    // requires NONE

    // requires NONE
    // 2- PwindowMenuPanel
    /**
     * Builds the panel who contain JTree and his menuBar
     *
     * @return
     */
    // 3-1-1 this TreeIconBar
    private void createTreeIconBar ()
    {
	this_TreeIconBar = new JToolBar ();
	this_TreeIconBar.setFloatable (false);
	this_TreeIconBar.setBackground (getMyColor1 ());

	JButton loadFile = createJButton ("Open.gif", "load a new root, the current one is cut", UserActions.J_load);
	this_TreeIconBar.add (loadFile);

	JButton saveFile = createJButton ("Save.gif", "silent save this window root", UserActions.J_save);
	this_TreeIconBar.add (saveFile);

	JButton refreshButton = createJButton ("Refresh.gif", "refresh", UserActions.J_refresh);
	this_TreeIconBar.add (refreshButton);




	//      JButton find = createJButton("Find.gif", "select a model according to a name in the previous selected one", UserActions.J_search_Process);
	//        this_TreeIconBar.add(find);
    }

    // 3-1 this PwindowTreePanel
    private void createPwindowTreePanel (boolean rootWindow)
    {
	// subWindow that contains tree model hierarchy (this_PwindowTree) and the tree menu bar
	Dimension minimumSize = new Dimension (5, 100);
	this_PwindowTreePanel = new JPanel ();
	this_PwindowTreePanel.setLayout (new BorderLayout ());
	this_PwindowTreePanel.setBackground (getMyColor1 ());
	this_PwindowTreePanel.setMinimumSize (minimumSize);

	createTreeIconBar ();
	this_PwindowTreePanel.add (this_TreeIconBar, BorderLayout.NORTH);
	createPwindowTree (rootWindow);
	this_PwindowTreePanel.add (this_PwindowTree, BorderLayout.CENTER);
    }

    // 3-1 this PwindowGraphPanel
    private JButton createGEJButton (String iconFileName, String tipText, int noAct)
    {
	ImageIcon imageNew = new ImageIcon (this_gifHtmlDir + iconFileName);
	JButton newButton = new JButton (imageNew);
	newButton.setToolTipText (tipText);
	newButton.addActionListener (new PwindowListener (this, noAct));
	return newButton;
    }

    // 3-2-1-1 graph icon menu
    private void createGraphIconBar ()
    {

	this_GraphIconBar = new JToolBar ();
	this_GraphIconBar.setFloatable (true);
	this_GraphIconBar.setBackground (getMyColor1 ());
	this_GraphIconBar.validate ();

	// ImageIcon imageSelect = new ImageIcon(this_gifHtmlDir + "select.jpg");
	// JButton select = new JButton(imageSelect);
	// select.setToolTipText("select");
	// select.setDefaultCapable(true);   //PLG ESSAIS
	// select.addActionListener(new GEwindowListener(this, UserActions.J_cut_graph)); ////
	// this_GraphIconBar.add(select);

	JButton aButton;

	aButton = createGEJButton ("Cut.gif", "cut", UserActions.J_cut_graph);
	this_GraphIconBar.add (aButton);

	aButton = createGEJButton ("Copy.gif", "copy", UserActions.J_copy_graph);
	this_GraphIconBar.add (aButton);

	aButton = createGEJButton ("Paste.gif", "paste", UserActions.J_paste_graph);
	this_GraphIconBar.add (aButton);

	ImageIcon imageUndo = new ImageIcon (this_gifHtmlDir + "Undo.gif");
	JButton undo = new JButton (imageUndo);
	undo.setToolTipText ("undo");
	undo.addActionListener (new GEwindowUndoListener (this));
	this_GraphIconBar.add (undo);

	ImageIcon imageLoupeMoins = new ImageIcon (this_gifHtmlDir + "ZoomOut.gif");
	JButton zom1 = new JButton (imageLoupeMoins);
	zom1.setToolTipText ("zoom -");
	zom1.addActionListener (new GEwindowZoomListener (this, -1.25));
	this_GraphIconBar.add (zom1);

	ImageIcon imageNormal = new ImageIcon (this_gifHtmlDir + "Zoom.gif");
	JButton zomn = new JButton (imageNormal);
	zomn.setToolTipText ("normal");
	zomn.addActionListener (new GEwindowZoomListener (this, 0));
	this_GraphIconBar.add (zomn);

	ImageIcon imageLoupePlus = new ImageIcon (this_gifHtmlDir + "ZoomIn.gif");
	JButton zom2 = new JButton (imageLoupePlus);
	zom2.setToolTipText ("zoom +");
	zom2.addActionListener (new GEwindowZoomListener (this, 1.25));
	this_GraphIconBar.add (zom2);

	aButton = createGEJButton ("Print.gif", "print", UserActions.J_save_PS);
	this_GraphIconBar.add (aButton);

	this_GraphIconBar.add (new JSeparator (SwingConstants.VERTICAL));
	// this_GraphIconBar.setDefaultButton(aButton);
    }

    // creates this_GraphMenuPanel 
    // 3-2-1-2
    private void setCurrentActionMenuItem (String label)
    {
	this_currentActionMenuItem.setText (label);
	this_currentActionMenuItem.setForeground (Color.red);
	this_currentActionMenuItem.setBackground (Color.white);
    }

    private void resetCurrentActionMenuItem ()
    {
	this_currentActionMenuItem.setText ("nop");
	this_currentActionMenuItem.setForeground (Color.black);
	this_selectedAction = UserActions.J_nop;
    }

    public void resetCurrentActionListener ()
    {
	set_clickedAction (UserActions.J_nop);
	resetCurrentActionMenuItem ();
	this_C_interface.sig_iterate ();
    }

    private JMenu createGraphEditMenu ()
    {
	JMenu thisMenu = new JMenu ("Edit");
	JMenuItem item;

	item = new JMenuItem ("undo");
	item.addActionListener (new GEwindowListener (this, UserActions.J_undo));
	thisMenu.add (item);

	item = new JMenuItem ("create");
	item.addActionListener (new GEwindowListener (this, UserActions.J_creer));
	item.setAccelerator (KeyStroke.getKeyStroke (KeyEvent.VK_C, ActionEvent.CTRL_MASK));
	thisMenu.add (item);

	item = new JMenuItem ("cut");
	item.addActionListener (new GEwindowListener (this, UserActions.J_cut_graph));
	item.setAccelerator (KeyStroke.getKeyStroke (KeyEvent.VK_D, ActionEvent.CTRL_MASK));
	thisMenu.add (item);

	item = new JMenuItem ("copy");
	item.addActionListener (new GEwindowListener (this, UserActions.J_copy_graph));
	item.setAccelerator (KeyStroke.getKeyStroke (KeyEvent.VK_Y, ActionEvent.CTRL_MASK));
	thisMenu.add (item);

	item = new JMenuItem ("paste");
	item.addActionListener (new GEwindowListener (this, UserActions.J_paste_graph));
	item.setAccelerator (KeyStroke.getKeyStroke (KeyEvent.VK_P, ActionEvent.CTRL_MASK));
	thisMenu.add (item);

	item = new JMenuItem ("glue");
	item.addActionListener (new GEwindowListener (this, UserActions.J_encaps));
	item.setAccelerator (KeyStroke.getKeyStroke (KeyEvent.VK_G, ActionEvent.CTRL_MASK));
	thisMenu.add (item);

	item = new JMenuItem ("break");
	item.addActionListener (new GEwindowListener (this, UserActions.J_decaps));
	item.setAccelerator (KeyStroke.getKeyStroke (KeyEvent.VK_B, ActionEvent.CTRL_MASK));
	thisMenu.add (item);

	JMenu map = new JMenu ("map");
	item = new JMenuItem ("box mapping");
	item.addActionListener (new GEwindowListener (this, UserActions.J_bte_map));
	item.setAccelerator (KeyStroke.getKeyStroke (KeyEvent.VK_A, ActionEvent.CTRL_MASK));
	map.add (item);
	item = new JMenuItem ("link mapping");
	item.addActionListener (new GEwindowListener (this, UserActions.J_link_map));
	map.add (item);
	item = new JMenuItem ("end mapping");
	item.addActionListener (new GEwindowListener (this, UserActions.J_end_map));
	map.add (item);
	thisMenu.add (map);

	item = new JMenuItem ("replace");
	item.addActionListener (new GEwindowListener (this, UserActions.J_replaceIdent));
	thisMenu.add (item);

	return thisMenu;
    }

    private JMenu createDrawMenu ()
    {
	JMenu thisMenu = new JMenu ("Draw");
	JMenuItem item;

	item = new JMenuItem ("justify");
	item.addActionListener (new GEwindowListener (this, UserActions.J_justif));
	item.setAccelerator (KeyStroke.getKeyStroke (KeyEvent.VK_J, ActionEvent.CTRL_MASK));
	thisMenu.add (item);

	item = new JMenuItem ("size");
	item.addActionListener (new GEwindowListener (this, UserActions.J_resize));
	item.setAccelerator (KeyStroke.getKeyStroke (KeyEvent.VK_S, ActionEvent.CTRL_MASK));
	thisMenu.add (item);

	item = new JMenuItem ("move");
	item.addActionListener (new GEwindowListener (this, UserActions.J_move));
	item.setAccelerator (KeyStroke.getKeyStroke (KeyEvent.VK_M, ActionEvent.CTRL_MASK));
	thisMenu.add (item);

	return thisMenu;
    }

    private JMenu createViewMenu ()
    {
	JMenu thisMenu = new JMenu ("View");
	JMenuItem item;

	item = new JMenuItem ("refresh");
	item.addActionListener (new GEwindowListener (this, UserActions.J_refresh));
	item.setAccelerator (KeyStroke.getKeyStroke (KeyEvent.VK_R, ActionEvent.CTRL_MASK));
	thisMenu.add (item);

	item = new JMenuItem ("select");
	item.addActionListener (new GEwindowListener (this, UserActions.J_selection));
	item.setAccelerator (KeyStroke.getKeyStroke (KeyEvent.VK_L, ActionEvent.CTRL_MASK));
	thisMenu.add (item);

	item = new JMenuItem ("halo");
	item.addActionListener (new GEwindowListener (this, UserActions.J_halo));
	item.setAccelerator (KeyStroke.getKeyStroke (KeyEvent.VK_H, ActionEvent.CTRL_MASK));
	thisMenu.add (item);

	item = new JMenuItem ("box font");
	item.addActionListener (new GEwindowListener (this, UserActions.J_box_police));
	thisMenu.add (item);

	item = new JMenuItem ("port font");
	item.addActionListener (new GEwindowListener (this, UserActions.J_port_police));
	thisMenu.add (item);

	JCheckBoxMenuItem box1 = new JCheckBoxMenuItem ("details", this_textFullVisibility);
	box1.addActionListener (new GEwindowListener (this, UserActions.J_displayDetails));
	thisMenu.add (box1);

	JCheckBoxMenuItem box2 = new JCheckBoxMenuItem ("names", this_portNameVisibility);
	box2.addActionListener (new GEwindowListener (this, UserActions.J_displayPortNames));
	thisMenu.add (box2);

	JMenu grid = new JMenu ("grid");
	// -----
	item = new JMenuItem ("small");
	item.addActionListener (new GEwindowGridListener (this, UserActions.J_grid_1));
	grid.add (item);
	item = new JMenuItem ("medium");
	item.addActionListener (new GEwindowGridListener (this, UserActions.J_grid_2));
	grid.add (item);
	item = new JMenuItem ("large");
	item.addActionListener (new GEwindowGridListener (this, UserActions.J_grid_3));
	grid.add (item);
	item = new JMenuItem ("none");
	item.addActionListener (new GEwindowGridListener (this, UserActions.J_grid_none));
	grid.add (item);
	// -----
	thisMenu.add (grid);

	thisMenu.addSeparator ();

	JMenu StrokeWidth = new JMenu ("Stroke width");
	// -----
	item = new JMenuItem ("small");
	item.addActionListener (new GEwindowStrokeWidthListener (this, UserActions.J_StrokeWidth_1));
	StrokeWidth.add (item);
	item = new JMenuItem ("medium");
	item.addActionListener (new GEwindowStrokeWidthListener (this, UserActions.J_StrokeWidth_2));
	StrokeWidth.add (item);
	item = new JMenuItem ("large");
	item.addActionListener (new GEwindowStrokeWidthListener (this, UserActions.J_StrokeWidth_3));
	StrokeWidth.add (item);
	// -----
	thisMenu.add (StrokeWidth);

	return thisMenu;
    }

    // 3-2-1-3
    private void createGraphMenuBar ()
    {
	this_GraphMenuBar = new JMenuBar ();
	this_GraphMenuBar.setBackground (getMyColor1 ());
	this_GraphMenuBar.setForeground (Color.red);

	// menuPanel::this_GraphMenuBar
	JMenu editMenu = createGraphEditMenu ();
	this_GraphMenuBar.add (editMenu);

	JMenu drawMenu = createDrawMenu ();
	this_GraphMenuBar.add (drawMenu);

	JMenu viewMenu = createViewMenu ();
	this_GraphMenuBar.add (viewMenu);

	this_currentActionMenuItem = new JMenuItem ("nop");
	this_currentActionMenuItem.addActionListener (new CurrentActionListener (this));
	this_GraphMenuBar.add (this_currentActionMenuItem);

    }

    // 3- this PwindowEditPanels
    // requires this_C_interface
    private void createPwindowEditPanels (boolean rootWindow)
    {
	createPwindowTreePanel (rootWindow);	// creates inner tree bars and this_PwindowTreePanel, this_PwindowTree
	// requires this_C_interface
	createPwindowGraphPanel ();	// creates inner drawing bars and this_PwindowGraphPanel, inner this_GEwindow
	this_PwindowEditPanels = new JSplitPane (JSplitPane.HORIZONTAL_SPLIT, this_PwindowTreePanel, this_PwindowGraphPanel);	// 3
	this_PwindowEditPanels.setOneTouchExpandable (true);
	this_PwindowEditPanels.setDividerLocation (200);
    }

    // 0- this PolychronyWindow (Polychrony window)
    // requires this_C_interface

    private void createTopPanel (int xx, int yy, boolean rootWindow)
    {
	setLocation (xx, yy);
	setVisible (true);
	// set_thisPwindowTitle(); // 1

	create_this_PwindowMenuPanel ();	// 2

	// requires this_C_interface
	createPwindowEditPanels (rootWindow);	// 3

	assembleInTopWindow (this_PwindowMenuPanel, this_PwindowEditPanels);
    }

    /* --------------------------------------------------------------------------------------
       --------------------------------------------------------------------------------------
       main polychrony window menu completed
       --------------------------------------------------------------------------------------
       --------------------------------------------------------------------------------------
    */

    /* ======================================================================================
       ======================================================================================
       main polychrony window edit panel
       ======================================================================================
       ======================================================================================
    */
    /* ======================================================================================
       polychrony graph edit window 
       ======================================================================================
    */

    /* ======================================================================================
       polychrony tree edit window 
       ======================================================================================
    */

    /* ======================================================================================
       polychrony graph edit window 
       ======================================================================================
    */

    public void actionPerformedGEwindowListener (ActionEvent event, int userAction)
    {
	/* Pwindow FileMenu
	 */
	boolean changed = false;
	// System.out.println("DEBUG entering PolychronyWindow::actionPerformedGEwindowListener, user action is "+userAction);

	/* Graph EditMenu ======================================================================
	 */
	if (userAction == UserActions.J_refresh)
	    {
		refresh ();
		return;
	    }
	else if (userAction == UserActions.J_undo)
	    {
	    }				// this_C_interface.sig_iterate();
	else if (userAction == UserActions.J_creer)
	    {
		changed = true;
	    }				// this_C_interface.sig_iterate(); *
	else if (userAction == UserActions.J_cut_graph)
	    {
		changed = true;
	    }				// this_C_interface.sig_iterate(); *
	else if (userAction == UserActions.J_copy_graph)
	    {
		changed = true;
	    }				// this_C_interface.sig_iterate(); *
	else if (userAction == UserActions.J_paste_graph)
	    {
		changed = true;
	    }				// this_C_interface.sig_iterate(); *
	else if (userAction == UserActions.J_encaps)
	    {
		changed = true;
	    }				// this_C_interface.sig_iterate(); *
	else if (userAction == UserActions.J_decaps)
	    {
		changed = true;
	    }				// this_C_interface.sig_iterate(); *
	else if (userAction == UserActions.J_bte_map)
	    {
		changed = true;
	    }				// this_C_interface.sig_iterate(); *
	else if (userAction == UserActions.J_link_map)
	    {
		changed = true;
	    }				// this_C_interface.sig_iterate(); *
	else if (userAction == UserActions.J_end_map)
	    {
		changed = true;
	    }				// this_C_interface.sig_iterate();
	else if (userAction == UserActions.J_replaceIdent)
	    {
		changed = true;
	    }				// this_C_interface.sig_iterate();
	/* Graph DrawMenu
	 */
	else if (userAction == UserActions.J_justif)
	    {
		changed = true;
	    }				// this_C_interface.sig_iterate(); *
	else if (userAction == UserActions.J_resize)
	    {
		changed = true;
	    }				// this_C_interface.sig_iterate(); *
	else if (userAction == UserActions.J_move)
	    {
		changed = true;
	    }				// this_C_interface.sig_iterate(); *
	/* Graph ViewMenu
	 */
	else
	    {
		if (userAction == UserActions.J_selection)
		    {
		    }			// this_C_interface.sig_iterate(); *
		else if (userAction == UserActions.J_halo)
		    {
		    }			// this_C_interface.sig_iterate(); *
		else if (userAction == UserActions.J_box_police)
		    {
			this_GEwindow.selectTypeFont (UserActions.J_box_police);
		    }			// this_C_interface.sig_iterate();
		else if (userAction == UserActions.J_port_police)
		    {
			this_GEwindow.selectTypeFont (UserActions.J_port_police);
		    }			// this_C_interface.sig_iterate();
		else if (userAction == UserActions.J_displayDetails)
		    {
			JCheckBoxMenuItem source = (JCheckBoxMenuItem) event.getSource ();
			this_textFullVisibility = source.getState ();
		    }			// this_C_interface.sig_iterate();
		else if (userAction == UserActions.J_displayPortNames)
		    {
			JCheckBoxMenuItem source = (JCheckBoxMenuItem) event.getSource ();
			this_portNameVisibility = source.getState ();
		    }			// this_C_interface.sig_iterate();
		else if (userAction == UserActions.J_grid_1)
		    {
		    }			// this_C_interface.sig_iterate();
		else if (userAction == UserActions.J_grid_2)
		    {
		    }			// this_C_interface.sig_iterate();
		else if (userAction == UserActions.J_grid_3)
		    {
		    }			// this_C_interface.sig_iterate();
		else if (userAction == UserActions.J_grid_none)
		    {
		    }			// this_C_interface.sig_iterate();
		else if (userAction == UserActions.J_StrokeWidth_1)
		    {
		    }			// this_C_interface.sig_iterate();
		else if (userAction == UserActions.J_StrokeWidth_2)
		    {
		    }			// this_C_interface.sig_iterate();
		else if (userAction == UserActions.J_StrokeWidth_3)
		    {
		    }			// this_C_interface.sig_iterate();
		// else if (userAction == UserActions.J_nop) {} // this_C_interface.sig_iterate();

		set_clickedAction (userAction);
		// System.out.println("DEBUG ==>> sig_iterate in PolychronyWindow::actionPerformedGEwindowListener 1"); 
		this_C_interface.sig_iterate ();
		// System.out.println("DEBUG ==>> sig_iterate in PolychronyWindow::actionPerformedGEwindowListener 1 DONE"); 
		refresh ();
	    }
	set_clickedAction (userAction);
	// System.out.println("DEBUG ==>> sig_iterate in PolychronyWindow::actionPerformedGEwindowListener 2"); 
	this_C_interface.sig_iterate ();
	// System.out.println("DEBUG ==>> sig_iterate in PolychronyWindow::actionPerformedGEwindowListener 2 DONE"); 
	refresh (true);
    }

    public void appendNewJNode (String nom, int SignalTree, boolean setInAtEnd)
    {
	// System.out.println("DEBUG entering PolychronyWindow::appendNewJNode; nom is "+nom);
	boolean res;
	if (setInAtEnd)
	    res = this_PwindowTree.insertJNodeLastIn (nom, SignalTree);
	else
	    res = this_PwindowTree.insertJNodeBefore (nom, SignalTree);
	if (res)
	    refresh (true);		// PAS FORCEMENT mettre un argument
	// System.out.println("DEBUG  exiting PolychronyWindow::appendNewJNode; nom is "+nom);
    }

    // 
    /** return the clicked value if it has been set since previous call
	return J_no_action otherwise
    */

    public int read_clickedAction ()
    {				// Modified LBesnard
	int prevValue = this_clickedAction;
	if ((this_clickedAction != UserActions.J_inter_comp))
	    this_clickedAction = UserActions.J_no_action;
	return prevValue;
    }

    public void set_selectedAction (int userAction, int complement)
    {
	this_clickedAction = userAction;
	this_selectedAction = userAction;
	this_selectedComplement = complement;
    }

    public void set_selectedAction (int userAction)
    {
	this_clickedAction = userAction;
	this_selectedAction = userAction;
	this_selectedComplement = UserActions.J_nop;
    }

    /**
     * this method is called by the Signal supervisor
     */
    public void raz_selectedAction ()
    {				// Modified LBesnard
	if ((this_clickedAction != UserActions.J_inter_comp))
	    {
		this_selectedAction = UserActions.J_nop;
		this_clickedAction = UserActions.J_no_action;
	    }
    }

    /**
     * method called by CompilerFunct and CompilerDiag
     */
    public void reset_selectedAction ()
    {
	this_clickedAction = UserActions.J_no_action;
	this_selectedAction = UserActions.J_nop;
    }

    /**
       require this_defaultDirectoryString
       set this_currentDirectory, this_currentFileString
    */
    private JFileChooser create_fileChooser ()
    {
	JFileChooser chooser = new JFileChooser (this_defaultDirectoryString);
	chooser.setMultiSelectionEnabled (true);
	this_currentDirectory = chooser.getCurrentDirectory ();
	this_currentFileString = this_currentDirectoryString + File.separator;
	chooser.setLocation (1000, 1000);
	return chooser;
    }

    private void set_GUIFileChooserFilter (JFileChooser chooser)
    {
	FileNameExtensionFilter sigFilter;
	sigFilter = new FileNameExtensionFilter ("Polychrony GUI Files", "gpk");
	chooser.addChoosableFileFilter (sigFilter);
	chooser.setFileFilter (sigFilter);
    }

    private void set_SignalFileChooserFilter (JFileChooser chooser, boolean openDialog)
    {
	FileNameExtensionFilter sigFilter;
	if (!openDialog)
	    {
		// saving.
		sigFilter = new FileNameExtensionFilter ("Signal Eclipse files", "SSME", "ssme");
		chooser.addChoosableFileFilter (sigFilter);
	    }
	sigFilter = new FileNameExtensionFilter ("SIGNAL language Files", "SIG", "sig");
	chooser.addChoosableFileFilter (sigFilter);
	chooser.setFileFilter (sigFilter);
    }

    /** check readbility of this_currentFile
	@return true: if this_fileContextSaved is true default have been overwritten
    */
    private boolean check_gpkIn ()
    {
	if (this_currentFile.canRead ())
	    {
		if (this_PwindowTree.isSelected_JTreeRoot () || this_PwindowTree.isSelected_ntreesRoot ())
		    {
			save_fileContextVariables ();
			ack_defaultDirectoryAndFile (this_currentFile);
		    }
		return true;
	    }
	displayFileError (Constants.J_ERR_readableFile);
	return false;
    }

    /** check writability of this_currentFile if it exists
	check if it can be created otherwise
	@return true: if this_fileContextSaved is true default have been overwritten
    */
    private boolean check_fileOut (int errorNum)
    {
	// System.out.println("DEBUG entering PolychronyWindow::check_fileOut" );
	if (this_currentFile.exists ())
	    {
		if (!this_currentFile.canWrite ())
		    {
			displayFileError (errorNum);
			// System.out.println("DEBUG  exiting PolychronyWindow::check_fileOut FALSE 1" );
			return false;
		    }
	    }
	else
	    {
		try
		    {
			if (this_currentFile.createNewFile ())
			    this_currentFile.delete ();
		    }
		catch (Exception e)
		    {
			displayFileError (errorNum);
			// System.out.println("DEBUG  exiting PolychronyWindow::check_fileOut FALSE 2" );
			return false;
		    }
	    }
	return true;
    }

    /** check writability of this_currentFile if it exists
	check if it can be created otherwise
	@return true: if this_fileContextSaved is true default have been overwritten
    */
    private boolean check_gpkOut ()
    {
	// System.out.println("DEBUG entering PolychronyWindow::check_gpkOut" );
	if (check_fileOut (Constants.J_ERR_fileSaving))
	    {
		if (this_PwindowTree.isSelected_JTreeRoot () || this_PwindowTree.isSelected_ntreesRoot ())
		    {
			save_fileContextVariables ();
			ack_defaultDirectoryAndFile (this_currentFile);
		    }
		return true;
	    }
	return false;
    }

    private boolean browserSet_gpkFile ()
    {
	JFileChooser chooser;
	int Val;
	// System.out.println("DEBUG entering PolychronyWindow::browserSet_gpkFile" );
	chooser = create_fileChooser ();
	set_GUIFileChooserFilter (chooser);
	if (this_defaultFile != null)
	    {
		chooser.setSelectedFile (this_defaultFile);
	    }
	Val = chooser.showOpenDialog (this);
	if (Val != JFileChooser.APPROVE_OPTION)
	    {
		// System.out.println("DEBUG  exiting PolychronyWindow::browserSet_gpkFile 1" );
		return (false);
	    }
	this_currentFile = chooser.getSelectedFile ();
	if (this_currentFile == null)
	    {
		// System.out.println("DEBUG  exiting PolychronyWindow::browserSet_gpkFile 2" );
		return (false);
	    }
	this_currentFileString = this_currentFile.getPath ();	// return the directory path of this_currentFile
	// System.out.println("DEBUG  exiting PolychronyWindow::browserSet_gpkFile * this_currentFileString is \n" + this_currentFileString);
	return true;
    }


    /** load a gpk file this_selectedComplement is (J_cut ?), J_before, J_in, J_root
	@return true: if this_fileContextSaved is true: restore when failure
    */
    private boolean browserIn_gpkFilePath ()
    {
	// System.out.println("DEBUG entering PolychronyWindow::browserIn_gpkFilePath" );
	if (this_defaultFile != null)
	    {
		if (this_selectedComplement == UserActions.J_root)
		    {
			int value = JOptionPane.showConfirmDialog (this, "Confirm reload " + this_defaultFileString, "Polychrony load", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
			if (value == JOptionPane.YES_OPTION)
			    {
				save_fileContextVariables ();
				this_hasBeenChanged = false;	// PLG if load succeed, if the root has been loaded
				// System.out.println("DEBUG  exiting PolychronyWindow::browserIn_gpkFilePath true YES" );
				return true;
			    }
			if (value == JOptionPane.CANCEL_OPTION)
			    {
				// System.out.println("DEBUG  exiting PolychronyWindow::browserIn_gpkFilePath false CANCEL" );
				return false;
			    }
			// value == JOptionPane.NO_OPTION
		    }
	    }
	if (browserSet_gpkFile ())
	    return check_gpkIn ();	// if true and if this_fileContextSaved is true: restore when failure
	return false;
    }

    /** save a gpk file this_selectedComplement is (J_cut ?), J_before, J_in, J_root
	@return true: if this_fileContextSaved is true: restore when failure
    */
    private boolean browserOut_gpkFilePath ()
    {
	// System.out.println("DEBUG entering PolychronyWindow::browserOut_gpkFilePath" );
	setVisible (true);
	this_PwindowGraphPanel.revalidate ();	// modif LBesnard.
	if (this_defaultFile != null)
	    {
		if (the_silentSave)
		    {
			save_fileContextVariables ();
			this_currentFileString = this_defaultFileString;
			// System.out.println("DEBUG  exiting PolychronyWindow::browserOut_gpkFilePath true YES" );
			return true;
		    }
		if (this_selectedComplement == UserActions.J_root)
		    {
			int value = JOptionPane.showConfirmDialog (this, "Confirm save " + this_defaultFileString, "Polychrony save", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
			if (value == JOptionPane.YES_OPTION)
			    {
				save_fileContextVariables ();
				this_hasBeenChanged = false;	// PLG if load succeed, if the root has been loaded
				// System.out.println("DEBUG  exiting PolychronyWindow::browserOut_gpkFilePath true YES" );
				return true;
			    }
			if (value == JOptionPane.CANCEL_OPTION)
			    {
				// System.out.println("DEBUG  exiting PolychronyWindow::browserOut_gpkFilePath false CANCEL" );                     
				return false;
			    }
			// value == JOptionPane.NO_OPTION
		    }
	    }
	// System.out.println("DEBUG      in PolychronyWindow::browserOut_gpkFilePath as browserSet_gpkFile " );
	if (browserSet_gpkFile ())
	    return check_gpkOut ();
	return false;
    }

    public boolean browserSet_signalFile (boolean openDialog)
    {
	JFileChooser chooser;
	int Val;
	// System.out.println("DEBUG entering PolychronyWindow::browserSet_signalFile" );
	chooser = create_fileChooser ();
	set_SignalFileChooserFilter (chooser, openDialog);
	Val = chooser.showOpenDialog (this);
	this_currentFile = chooser.getSelectedFile ();
	if (Val != JFileChooser.APPROVE_OPTION)
	    {
		// System.out.println("DEBUG  exiting PolychronyWindow::browserSet_signalFile false NOT APPROVE" );
		return false;
	    }
	if (this_currentFile == null)
	    {
		// System.out.println("DEBUG  exiting PolychronyWindow::browserSet_signalFile false null selected file" );
		return false;
	    }
	if (openDialog)
	    {
		if (this_currentFile.canRead ())
		    {
			this_currentFileString = this_currentFile.getPath ();
			return true;
		    }
		displayFileError (Constants.J_ERR_readableFile);
	    }
	else
	    {
		if (check_fileOut (Constants.J_ERR_fileSaving))
		    {
			this_currentFileString = this_currentFile.getPath ();
			// System.out.println("DEBUG  exiting PolychronyWindow::browserSet_signalFile true" );
			return true;
		    }
	    }
	return false;
    }

    /** PLG
	assume this_defaultDirectoryString, this_defaultFileString (empty file name string), this_currentFileString defined
	@return false if the operation is cancelled or the file does not exist
	@return true: if this_fileContextSaved is true: restore when failure
    */
    private boolean startBrowser_loadInitialFile (boolean isGraphic)
    {
	// load or parse
	// System.out.println("DEBUG entering PolychronyWindow::startBrowser_loadInitialFile " +this_currentFileString);
	this_currentFile = new File (this_currentFileString);
	if (check_gpkIn ())
	    {   // if this_fileContextSaved is true: restore when failure
		int value = JOptionPane.showConfirmDialog (the_mainPwindow, "Confirm load " + this_currentFileString, "Polychrony load", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
		if (value == JOptionPane.YES_OPTION)
		    {
			// System.out.println("DEBUG  exiting PolychronyWindow::startBrowser_loadInitialFile true" );
			return true;
		    }
		if (value == JOptionPane.CANCEL_OPTION)
		    {
			restore_fileContextVariables ();
			refresh ();
			return false;
		    }
		if (browserSet_gpkFile ()) return true;
		restore_fileContextVariables ();
		refresh ();
	    }
	// System.out.println("DEBUG  exiting PolychronyWindow::startBrowser_loadInitialFile false" );
	return false;
    }

    /** PLG
	@return 0 when operation is canceled or errored
	@return -1 when a file name is provided and the current action is load and current complement is J_before
	@return 1 when a file name is provided
    */
    public int browser_directory_file_path (boolean isGraphic, boolean openDialog)
    {
	// System.out.println("DEBUG entering PolychronyWindow::browser_directory_file_path");
	this_actionSafe = true;
	if (openDialog)
	    {				// load or parse
		if (isGraphic)
		    {			// load a gpk file
			if (this_selectedComplement == UserActions.J_start)
			    {
				if (startBrowser_loadInitialFile (isGraphic))
				    {
					// System.out.println("DEBUG  exiting 1 PolychronyWindow::browser_directory_file_path");
					return 1;
				    }
				restore_fileContextVariables ();
				refresh ();
				// System.out.println("DEBUG  exiting 0 PolychronyWindow::browser_directory_file_path");
				return 0;
			    }
			if (browserIn_gpkFilePath ())
			    {
				if (this_selectedComplement == UserActions.J_before)
				    {		//PLG TODO if this_fileContextSaved is true: restore
					// System.out.println("DEBUG  exiting PolychronyWindow::browser_directory_file_path openDialog isGraphic -1" );
					return -1;
				    }
				// System.out.println("DEBUG  exiting PolychronyWindow::browser_directory_file_path openDialog isGraphic 1" );
				return 1;
			    }
			// System.out.println("DEBUG  exiting PolychronyWindow::browser_directory_file_path openDialog isGraphic 0" );
			return 0;
		    }
		// parse a Signal (SSME,SIG) file
		if (browserSet_signalFile (true))
		    {
			if (this_selectedComplement == UserActions.J_before)
			    {
				// System.out.println("DEBUG  exiting PolychronyWindow::browser_directory_file_path openDialog parse -1" );
				return -1;
			    }
			// System.out.println("DEBUG  exiting PolychronyWindow::browser_directory_file_path openDialog parse 1" );
			return 1;
		    }
		// System.out.println("DEBUG  exiting PolychronyWindow::browser_directory_file_path openDialog parse 0" );
		return 0;
	    }
	// save or unparse
	if (isGraphic)
	    {				// save a gpk file
		if (this_selectedAction == UserActions.J_save)
		    {			// other: J_saveAS
			if (this_PwindowTree.isSelected_JTreeRoot () || this_PwindowTree.isSelected_ntreesRoot ())
			    if (this_currentFileString.endsWith (".gpk"))
				// System.out.println("DEBUG  exiting PolychronyWindow::browser_directory_file_path NOTopenDialog J_save isGraphic 1" );
				return 1;
		    }
		if (browserOut_gpkFilePath ())
		    {
			// System.out.println("DEBUG  exiting PolychronyWindow::browser_directory_file_path NOTopenDialog J_saveAS isGraphic 1" );
			return 1;
		    }
		// System.out.println("DEBUG  exiting PolychronyWindow::browser_directory_file_path NOTopenDialog isGraphic 0" );
		return 0;
	    }
	// unparse a Signal (SSME,SIG) file
	if (browserSet_signalFile (false))
	    {
		// System.out.println("DEBUG  exiting PolychronyWindow::browser_directory_file_path NOTopenDialog unparse 1" );
		return 1;
	    }
	// System.out.println("DEBUG  exiting PolychronyWindow::browser_directory_file_path unparse 0" );
	return 0;
    }

    private int get_depth ()
    {
	if (this_isRwindow)
	    return 0;
	return 1 + this_PwindowAncestor.get_depth ();
    }

    public String getgifHtmlDir ()
    {
	return this_gifHtmlDir;
    }

    public String getBrowserFileName ()
    {

	// System.out.println("DEBUG entering/exiting PolychronyWindow::getBrowserFileName "+this_currentFileString);
	return (this_currentFileString);
    }

    public void displayFont ()
    {
	new ChooseFont (new Font ("Arial", Font.PLAIN, 12), this_GEwindow);
    }

    private boolean showConfirmYesNo (String title, String mess)
    {
	int value;
	value = JOptionPane.showConfirmDialog (this, mess, title, JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
	return (value == JOptionPane.YES_OPTION);
    }

    public void displayParsingError (String errorString)
    {
	JOptionPane.showMessageDialog (this, errorString, "Polychrony alert 5", JOptionPane.ERROR_MESSAGE);
	this_GEwindow.repaint ();	// after an error, it may be necessary to refresh the canvas
    }

    /** @return false if this is a non empty window that may have changed and the reset action is cancelled
	@return false otherwise
    */
    private boolean continue_deletePwindowContent (String action)
    {
	if (has_thisNonEmptyPwindowChanged ())
	    if (!showConfirmYesNo (action + " a window", "Some window may have changed. Really " + action + " ?"))
		return false;
	return true;
    }

    /** @return false if this is a non empty window that may have changed and the reset action is cancelled
	@return false otherwise
    */
    private boolean continue_deleteMwindowContent (String action)
    {
	if (has_someNonEmptyRWindowChanged () || has_thisNonEmptyPwindowChanged ())
	    if (!showConfirmYesNo (action + " main window", "Some window may have changed. Really " + action + " ?"))
		return false;
	return true;
    }

    public void reset_Pwindow ()
    {
	if (the_mainPwindow == this)
	    {
		if (!continue_deleteMwindowContent ("reset"))
		    return;
		doClose_theRwindowList (false);
	    }
	else
	    {
		if (!continue_deletePwindowContent ("reset"))
		    return;
	    }
	doReset_Pwindow (false);
    }

    public void reset_polychrony ()
    {
	if (!continue_deleteMwindowContent ("reset"))
	    return;
	doClose_theRwindowList (false);
	the_mainPwindow.doReset_Pwindow (false);
	this_C_interface.SignalRaz ();
    }

    /** only with J_raz
     */
    public void displayConfirm (int action)
    {
	the_mainPwindow.reset_polychrony ();
    }

    /**
     * 	Method called when the user clicks on the button "exit Polychrony"
     */
    public void exit_Polychrony ()
    {
	// show_DEBUG("entering exit_Polychrony");
	reset_selectedAction ();
	if (!continue_deletePwindowContent ("exit"))
	    {
		CloseOperation ();
		return;
	    }
	doClose_theRwindowList (false);
	doClose_innerWindows (false);
	this_C_interface.SignalExit ();
	System.exit (0);
	CloseOperation ();
	// show_DEBUG(" exiting exit_Polychrony *");
    }

    // Method to close a window
    public void close_Pwindow ()
    {
	if (the_mainPwindow == this)
	    {
		exit_Polychrony ();
		return;
	    }
	if (!continue_deletePwindowContent ("close"))
	    {
		CloseOperation ();
		return;
	    }
	doClose_innerWindows (false);
	if (this_isRwindow)
	    the_RwindowList[this_PwindowIndex] = null;
	else
	    {
		this_PwindowAncestor.cancel_aCwindow (this_PwindowIndex);
	    }
	setVisible (false);
	Destroy ();			// destroys and unallocate the window
    }

    public void displayCurrentAction (int action)
    {
	String label;

	this_selectedAction = action;
	if (action == UserActions.J_nop)
	    resetCurrentActionMenuItem ();
	else
	    {
		switch (action)
		    {
			/* Graph EditMenu ======================================================================
			 */
		    case UserActions.J_creer:
			label = "create";
			break;
		    case UserActions.J_cut_graph:
			label = "cut";
			break;
		    case UserActions.J_copy_graph:
			label = "copy";
			break;
		    case UserActions.J_paste_graph:
			label = "paste";
			break;
		    case UserActions.J_encaps:
			label = "glue";
			break;
		    case UserActions.J_decaps:
			label = "break";
			break;
		    case UserActions.J_bte_map:
			label = "box mapping";
			break;
		    case UserActions.J_link_map:
			label = "link mapping";
			break;
			/* Graph DrawMenu
			 */
		    case UserActions.J_justif:
			label = "justify";
			break;
		    case UserActions.J_resize:
			label = "resize";
			break;
		    case UserActions.J_move:
			label = "move";
			break;
			/* Graph ViewMenu
			 */
		    case UserActions.J_selection:
			label = "select";
			break;
		    case UserActions.J_halo:
			label = "halo";
			break;

			/* Text window ==============================================
			 */
		    case UserActions.J_confirm:
			label = "click portnames";
			break;

			/* supervis_sig_g.c ??? ==============================================
			 */
		    case UserActions.J_embed:
			label = "embedded compil";
			break;
			/* LiaisonSignal.java, supervis_sig_g.c ??? ==============================================
			 */
		    case UserActions.J_copy_Icon:
			label = "icon copy";
			break;
			/* LiaisonSignal.java ??? ==============================================
			 */
		    case UserActions.J_info_Icon:
			label = "icon info";
			break;
			/* supervis_sig_g.c ??? ==============================================
			 */
		    case UserActions.J_not_impl:
		    default:
			return;
		    }
		setCurrentActionMenuItem (label);
	    }
    }

    public int getIndexLibrary (String fname)
    {
	int n = this_LibBarList.size ();
	for (int i = 0; i < n; i++)
	    {
		LibBar x = (LibBar) (this_LibBarList.get (i));
		if (x.getFilename ().equals (fname))
		    return i;
	    }
	return -1;
    }

    public boolean isLoadedLibrary (String fname)
    {
	return (getIndexLibrary (fname) >= 0);
    }

    public void createLibBar (long signalRef, String fname, String fnamews, String LibName)
    {
	LibBar barre = new LibBar (signalRef, this_GEwindow, this_C_interface, fname, fnamews,
				   this_gifHtmlDir, LibName);
	// Adds the lib to the libs list
	if (biblioBar != null)
	    {
		biblioBar.setEnabled (true);
		biblioBar.setState (true);
		biblioBarreState = true;
	    }
	this_LibBarList.add (barre);
	this_scrollPanelBib.setVisible (true);
	panelBib.add (barre);
	this_PwindowGraphPanel.revalidate ();
    }

    public boolean removeLibrary (String fname)
    {
	int n = this_LibBarList.size ();
	for (int i = 0; i < n; i++)
	    {
		LibBar x = (LibBar) (this_LibBarList.get (i));
		if (x.getFilename ().equals (fname))
		    {
			System.err.println ("Removed");
			removeIthLibrary (i);
			return true;
		    }
	    }
	return false;
    }

    // It reloads the vindex-th library, the new is defined by the tree vtree.
    public void install_library (long vtree, int vindex)
    {
	LibBar currentLibBar = (LibBar) (this_LibBarList.get (vindex));
	currentLibBar.install (vtree, this_GEwindow, this_C_interface);
    }


    /* Creations requires this_defaultDirectoryString, this_C_interface
     */
    private void create_Rwindow ()
    {
	int depth = get_depth ();
	create_thisTwindowList ();	// initializes the children PolychronyWindows list
	create_thisCwindowList ();	// initializes the children PolychronyWindows list
	// requires this_C_interface
	createTopPanel (20 * (this_PwindowIndex + 2 + depth), 30 * (depth + 1), true);
	this_GEwindow.init ();	// this method cannot be called sooner, otherwise it crashes.
	this_compilerPopup = new CompilerWin (this_C_interface, this, 20, 20);
	create_initialTextWindows ();
	// created this_signalExpressionTwindow, this_signalDeclarationTwindow, this_parameterDeclarationTwindow
	// this_C_interface.Sav(this, this_PwindowTree, this_signalExpressionTwindow canvasText, 
	//                      this_signalDeclarationTwindow this_hierarTwindow, this_compilerPopup, this_parameterDeclarationTwindow paramText, this_docHtmlDir);
	this_C_interface.Sav (this, this_compilerPopup, this_docHtmlDir);
	addWindowListener ();
    }

    /* Creations requires this_defaultDirectoryString, this_C_interface
     */
    private void create_Cwindow ()
    {
	int depth = get_depth ();
	create_thisTwindowList ();	// initializes the children PolychronyWindows list
	create_thisCwindowList ();	// initializes the children PolychronyWindows list

	// requires this_C_interface
	createTopPanel (20 * (this_PwindowIndex + 2 + depth), 30 * (depth + 1), false);
	this_GEwindow.init ();	// this method cannot be called sooner, otherwise it crashes.
	this_compilerPopup = new CompilerWin (this_C_interface, this, 20, 20, this_PwindowAncestor.this_C_interface.getCompilerWin ());
	create_initialTextWindows ();
	// created this_signalExpressionTwindow, this_signalDeclarationTwindow, this_parameterDeclarationTwindow
	this_C_interface.Sav (this, this_compilerPopup, this_docHtmlDir);
	addWindowListener ();
	refresh (); // PLG ??
    }

    /** set this Pwindow as a (child) Cwindow
	apply common inheritred window initialization
    */
    private void set_CwindowVariables (PolychronyWindow PwindowSource)
    {
	this_isRwindow = false;
	this_PwindowAncestor = PwindowSource;
	this_sourceSTNode = this_PwindowAncestor.getTreeWin ().get_selectedJNode ();
	this_sourceSTNode.set_dedicatedWindow (this);
	this_gifHtmlDir = this_PwindowAncestor.this_gifHtmlDir;
	this_docHtmlDir = this_PwindowAncestor.this_docHtmlDir;

	new_fileContextVariables (new String (this_PwindowAncestor.this_defaultDirectoryString + File.separator + this_PwindowAncestor.getSelectedObjectName () + ".gpk"));
	this_initialDirectoryString = this_defaultDirectoryString;

	this_portNameVisibility = this_PwindowAncestor.this_portNameVisibility;
	this_textFullVisibility = this_PwindowAncestor.this_textFullVisibility;
    }

    /** create the Rwindow list
	apply common root window initialization
    */
    private void create_C_interface (boolean localInit)
    {
	this_C_interface = new LiaisonSignal ();
	if (localInit)
	    this_C_interface.localInit (); // Initialization for SIGNAL
	else
	    this_C_interface.globalInit (); // Initialization for SIGNAL
    }

    public boolean newPolychronyWindowCompletion (boolean changed)
    {
	this_PwindowGraphPanel.revalidate ();
	this_selectedAction = UserActions.J_nop;
	refresh (changed);
	return this_C_interface.sig_iterate ();
    }


    /** Initial file loading in this root window:
	--> if fileName exits: sig_iterate with J_load/J_parse action and  J_start complement is called
	--> sig_iterate call Signal then C LOADING
	this default file is reset, this Pwindow becomes not changed (SWING)
    */
    public boolean initialLoadRoot (String fileName, boolean graphic, boolean bcompleteName)
    {
	if (!bcompleteName) bcompleteName = (fileName.indexOf ("this_defaultFileString", 0) == -1); // added LBesnard. (Eclipse calls polychrony with a complete pathname.)
	if (bcompleteName) this_currentFileString = fileName; else this_currentFileString = this_defaultFileString + fileName;
      
	this_actionSafe = true;
	this_currentFile = new File (this_currentFileString);
	if (check_gpkIn ())
	    {
		this_clickedAction = (graphic ? UserActions.J_load : UserActions.J_parse);
		this_selectedComplement = UserActions.J_start;

		if (this_C_interface.sig_iterate ());
		if (this_actionSafe)
		    refresh (true);
		else
		    {
			restore_fileContextVariables ();
			refresh ();
		    }
		return true;
	    }
	return false;
    }

    /**
       Constructor of a child PolychronyWindow built from an another one. 
       success: Call PolychronyWindow::refresh changed, then sig_iterate
    */
    public PolychronyWindow (PolychronyWindow PwindowSource)
    {
	// System.out.println("DEBUG entering PolychronyWindow(PolychronyWindow) " );
	this_PwindowIndex = PwindowSource.register_aCwindow (this);
	if (this_PwindowIndex >= 0)
	    {
		set_CwindowVariables (PwindowSource);
		create_C_interface (true);
		create_Cwindow ();
		if (newPolychronyWindowCompletion (true));
	    }
	// System.out.println("DEBUG  exiting PolychronyWindow(...) " );
    }

    /**
       Constructor of a secondary root PolychronyWindow built from an another one.
       success: Call PolychronyWindow::refresh unchanged, then sig_iterate
    */
    public PolychronyWindow (PolychronyWindow PwindowSource, String WDPath, String imageDirectory, String docDirectory)
    {
	// System.out.println("DEBUG entering PolychronyWindow(PolychronyWindow, String, String, String) " );
	if (register_thisRwindow ())
	    {
		set_RwindowVariables (WDPath, PwindowSource.this_gifHtmlDir, docDirectory);	// set this_defaultDirectoryString, this_defaultFileString, this_defaultFile (null)
		create_C_interface (true);
		// requires this_defaultDirectoryString, this_C_interface
		create_Rwindow ();
		if (newPolychronyWindowCompletion (false));
	    }
	// System.out.println("DEBUG  exiting PolychronyWindow(...) " );
    }

    /**
       Constructor of a secondary root PolychronyWindow at start
       assume defined the main PolychronyWindow 
       success: PolychronyWindow::initial loading (if any) has to be called  from outside (using initialLoadRoot)
       PolychronyWindow::refresh(unchanged), and then sig_iterate have to be called from outside (using newPolychronyWindowCompletion)
    */
    public PolychronyWindow (String WDPath)
    {
	// System.out.println("DEBUG entering PolychronyWindow(String) " );
	if (register_thisRwindow ())
	    {
		set_RwindowVariables (WDPath, the_mainPwindow.this_gifHtmlDir, the_mainPwindow.this_docHtmlDir);	// set this_defaultDirectoryString, this_defaultFileString, this_defaultFile (null)
		create_C_interface (true);
		// requires this_defaultDirectoryString, this_C_interface
		create_Rwindow ();
		// refresh(); // PLG ??
	    }
	// System.out.println("DEBUG  exiting PolychronyWindow(...) " );
    }

    /**
       Call to the constructor of a secondary root PolychronyWindow at start
       assume defined the main PolychronyWindow */
    public static boolean new_PolychronyWindow (String WDPath, String filename, boolean isGraphic)
    {
	PolychronyWindow newROOT = new PolychronyWindow (WDPath);
	if (newROOT.initialLoadRoot (new String (filename), isGraphic, false))
	    if (newROOT.newPolychronyWindowCompletion (false)) return true;
	newROOT.close_Pwindow ();
	return false;
    }

    /**
       Constructor of the empty main Polychrony window 
       Create the SWING main Polychrony window and global variables (set_PolychronySessionVariables)
       Create root window variables
       PolychronyWindow::initial loading (if any) has to be called  from outside (using initialLoadRoot)
       PolychronyWindow::refresh(unchanged), and then sig_iterate have to be called from outside (using newPolychronyWindowCompletion)
    */
    public PolychronyWindow (String WDPath, String imageDirectory, String docDirectory)
    {
	System.out.println ("Welcome to SWING polychrony toolset");
	set_PolychronySessionVariables ();
	set_RwindowVariables (WDPath, imageDirectory, docDirectory); // set this_defaultDirectoryString, this_defaultFileString, this_defaultFile (null)
	create_C_interface (false);
	// requires this_defaultDirectoryString, this_C_interface
	create_Rwindow ();
	refresh ();
    }

}
