 /*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */

import javax.swing.tree.DefaultMutableTreeNode;

public class SignalTreeNode extends DefaultMutableTreeNode {
    private static final long serialVersionUID = 1L;
    private long this_ntrees = 0; // Contains the reference to the ntrees Signal model or module
    private LiaisonSignal this_C_interface = null;
    private boolean this_isErrored = false;
    private boolean this_isModule;
    private PolychronyWindow this_dedicatedWindow;

    public boolean hasError() {
        return this_isErrored;
    }

    public void set_dedicatedWindow(PolychronyWindow dedicatedWindow) {
	this_dedicatedWindow = dedicatedWindow;
    }

    public void detach_dedicatedWindow() {
	this_dedicatedWindow = null;
    }

    public boolean has_dedicatedWindow() {
	return (this_dedicatedWindow != null);
    }

    public PolychronyWindow get_dedicatedWindow() {
	return this_dedicatedWindow;
    }

    public void setError(boolean error) {
        this_isErrored = error;
        if (this.getParent() != null) {
            SignalTreeNode node = (SignalTreeNode) this.getParent();
            node.setError(error);
        }
    }

    /** Constructor:  creation SignalTreeNode  named "modeulName" associated with the ntress aNtrees
	if isAmodule then  aNtrees is a module otherwise it is a model 
    */
// 1,2 called in new_JModel, new_JModule
    public SignalTreeNode(LiaisonSignal LS, String modeulName, long aNtrees, boolean isAmodule) {
    // public SignalTreeNode(LiaisonSignal LS, String modeulName, long aNtrees, long category, long index) {
        super(modeulName);
	this_ntrees = aNtrees;
        this_C_interface = LS;
	this_isModule = isAmodule;
	this_dedicatedWindow = null;
    }

    /** Constructor : insertion of a subprocess (for example file loading). The
	Signal subprocess already exists, we only need to save its reference
	(SignalTree)*/
// 3 called in innerCloneTreeNode(cannot be a module),  reset_theJTreeClipboard(can be a module), pasteSpecial(cannot be a module), insertObject should not
    public SignalTreeNode(LiaisonSignal LS, long SignalTree) {
        super(LS.getNameProc(SignalTree));
        this_ntrees = SignalTree;
        this_C_interface = LS;
	this_isModule = this_C_interface.isModul(this_ntrees);
	this_dedicatedWindow = null;
    }

    /** Constructor called for the root : we create a Signal associated process
	that won't be visible */
// 4 called to build super root this_isModule doesn't matter
    public SignalTreeNode(LiaisonSignal LS, String name) {
        super(name);
        this_C_interface = LS;
        this_ntrees = LS.SignalRootCreate(name);
	this_dedicatedWindow = null;
    }

    /** Builds a new Signal root; call after a click on the menu button "reset all" */
    public void setNewSignalRoot(LiaisonSignal LS, String name) {
        this_ntrees = LS.SignalRootCreate(name);
    }

    /** returns the process child number index of the current process */
    public long getSignalChildRef(LiaisonSignal LS, long index) {
        return (LS.getSignalChildRef(this_ntrees, index));
    }

    public long getSignalTreeArity(LiaisonSignal LS) {
        return (LS.getSignalTreeArity(this_ntrees));
    }

    public void copySignalTreeNode(LiaisonSignal LS) {
        this_ntrees = LS.SignalTreeCopy(this_ntrees);
    }

    public void set_ntreeValue(long x) {
        this_ntrees = x;
    }

    public long get_ntreeValue() {
        return this_ntrees;
    }

    public boolean isModul() {
        return this_isModule;
    }

    public boolean isProc() {
        return (this_C_interface.isProcess(this_ntrees));
    }

    public boolean isFunc() {
        return (this_C_interface.isFunc(this_ntrees));
    }

    public boolean isNode() {
        return (this_C_interface.isNode(this_ntrees));
    }

    public boolean isAction() {
        return (this_C_interface.isAction(this_ntrees));
    }

}
