/*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */
/*--------------------------------------------------------------------------

                          MODUL graphic2Text

   This modul builds the Signal tree equivalent to the graphical tree
   in order to compile it or in order to build its textual representation
   (in a file).
   The treatment consists on :
        adding affectations which correspond to the graphical links; 
	adding new signal names when necessary, to avoid conflict;
	adding the declarations of local signals when they have been forgotten;
	changing the processes which have empty body to external processes;
	replacing all the empty lists (declaration of parameters, signals...)
	by NILTREE, otherwise the Signal text
	is not good.
   This module modifies the graphical tree; all sub-trees that have been
   added are marked. When the compiling or the textual decompiling is
   achieved, all the marked sub-trees are deleted.
 -------------------------------------------------------------------------- */


class graph_text{
  
#include "graphic2Text_m.h"
#include "graphic2Text_g.h"
  
  private :
    
  ptGraphicTrees the_graphicalTrees;
  ptGraphicTablesManager the_graphicTablesManager;
  ntrees proc_cour;
  ntrees namesList;/* list of the signal names, useful to know if a name already exists */
  int namesCount; /* allows to build new unique signal names */
  ntrees proc_encours; /* current process tree which is set by the method "completer" */
  int boite_proc_encours; /* number of the box which is attached to the highest level 
			     expression of proc_encours */
  
};
